// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
*     DATE:        06/03/2023                                          *
*                                                                $     *
*     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
*                                                                $     *
*     PROJET:      Herezh++                                            *
*                                                                $     *
************************************************************************
*     BUT:   Algorithme combiné, l'objectif est de mettre en oeuvre    *
*           plusieurs algorithme existants, suivant une stratégie      *
*           que l'utilisateur impose au travers de fonction nD         *
*           particulières.                                             *
*                                                                $     *
*     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
*                                                                      *
*     VERIFICATION:                                                    *
*                                                                      *
*     !  date  !   auteur   !       but                          !     *
*     ------------------------------------------------------------     *
*     !        !            !                                    !     *
*                                                                $     *
*     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
*     MODIFICATIONS:                                                   *
*     !  date  !   auteur   !       but                          !     *
*     ------------------------------------------------------------     *
*                                                                $     *
************************************************************************/
#ifndef AGORICOMBINER_T
#define AGORICOMBINER_T


#include "Algori.h"
#include "Assemblage.h"
#include "MatLapack.h"

/// @addtogroup Les_algorithmes_de_resolutions_globales
///  @{
///

///     BUT:   Algorithme combiné, l'objectif est de mettre en oeuvre
///           plusieurs algorithme existants, suivant une stratégie
///           que l'utilisateur impose au travers de fonction nD
///           particulières.

class AlgoriCombine : public Algori
{
  public :
    // CONSTRUCTEURS :
    AlgoriCombine () ; // par defaut
    
    // constructeur en fonction du type de calcul
    // du sous type (pour les erreurs, remaillage etc...)
    // il y a ici lecture des parametres attaches au type
    AlgoriCombine (const bool avec_typeDeCal
                   ,const list <EnumSousTypeCalcul>& soustype
                   ,const list <bool>& avec_soustypeDeCal
                   ,UtilLecture& entreePrinc);
    // constructeur de copie
    AlgoriCombine (const AlgoriCombine& algo);
    
    // constructeur de copie à partie d'une instance indifférenciée
    Algori * New_idem(const Algori* algo) const
     {// on vérifie qu'il s'agit bien d'une instance
      if (algo->TypeDeCalcul() != COMBINER)
        { cout << "\n *** erreur lors de la creation par copie d'un algo COMBINER "
               << " l'algo passe en parametre est en fait : " << Nom_TypeCalcul(algo->TypeDeCalcul())
               << " arret !! " << flush;
          Sortie(1);
        }
      else
        { AlgoriCombine* inter = (AlgoriCombine*) algo;
          return ((Algori *) new AlgoriCombine(*inter));
        };
     };

    // DESTRUCTEUR :
     ~AlgoriCombine () ;  
    
    // METHODES PUBLIQUES :
    //lecture des parametres de controle
    // peut-être modifié dans le cas d'un algo particulier
    virtual void Lecture(UtilLecture & entreePrinc,ParaGlob & paraGlob,LesMaillages& lesMail);

    // execution de l'algorithme dans le cas non dynamique
    void Execution(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D* ,LesFonctions_nD*
                  ,VariablesExporter* varExpor,LesLoisDeComp*
                  ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );
    //------- décomposition en 3 du calcul d'équilibre -------------
    // a priori   : InitAlgorithme  et FinCalcul ne s'appellent qu'une fois,
    // par contre : CalEquilibre peut s'appeler plusieurs fois, le résultat sera différent si entre deux calcul
    //              certaines variables ont-été changés
    
    // initialisation
    void InitAlgorithme(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                        ,LesFonctions_nD* ,VariablesExporter*
                        ,LesLoisDeComp*
                        ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );
    // mise à jour
    void MiseAJourAlgo(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                       ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                       ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );
    // calcul de l'équilibre
    // si tb_combiner est non null -> un tableau de 2 fonctions
    //  - la première fct dit si on doit valider ou non le calcul à convergence ok,
    //  - la seconde dit si on doit sortir de la boucle ou non à convergence ok
    //
    // si la validation est effectuée, la sauvegarde pour le post-traitement est également effectuée
    //                                 en fonction de la demande de sauvegard,
    // sinon pas de sauvegarde pour le post-traitement à moins que l'on a demandé un mode debug
    //   qui lui fonctionne indépendamment
    void CalEquilibre(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                      ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                      ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats*
                      ,Tableau < Fonction_nD* > * tb_combiner);
    // dernière passe
    void FinCalcul(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                   ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                   ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );
                        

    // sortie du schemaXML: en fonction de enu
    void SchemaXML_Algori(ofstream& ,const Enum_IO_XML ) const ;
 
    // sortie des temps cpu cumulées de tous les algos
    void AutreSortieTempsCPU(ofstream& sort,const int cas) const;
    
    // sortie pour info de la liste des énumérés de tous les sous-algo
    list <EnumTypeCalcul> List_Sous_Algo() const;
    
    // sortie sur fichier des temps cpu
    // idem la forme générique de la classe mère
    // + des infos partielles pour chaque sous-algorithme
    virtual void Sortie_temps_cpu(const LesCondLim& lesCondLim
                          , const Charge& charge,const LesContacts & contact);
    // une méthode qui a pour objectif de terminer tous les comptages, utile
    // dans le cas d'un arrêt impromptu
    virtual void Arret_du_comptage_CPU();

  private :
    // variables privées
    Tableau <Algori *> tab_algo; // pointeur sur les différents algos
    
    // le type de calcul principal et sous type etc. sont stockés dans paraglob
    // et des pointeurs définis dans Algori, permettent d'accèder à l'info sans
    // la dupliquer
    
    // mais il faut également stocker les infos particulières pour les algo internes
    // du coup on crée des listes locales qui seront utilisées par pointeurs, via Algori
    // pour chaque algo interne (sinon les grandeurs ne sont pas pérennes !!)
    // l'association est faite dans AlgoriCombine::Lecture_algo_interne
    // au moment de la définition des algos internes
    // NB: important: les listes sont rangées : la première -> le dernier algo, etc.
    list <list<EnumSousTypeCalcul> >  list_sousTypeCalcul;
    list <list<bool> > list_avec_sousCalcul;
    
    //-- choix de l'algo en cours ---
    string nom_choix_algo; // le nom éventuel de la fonction du choix de l'algo
    Fonction_nD* choix_algo; // la fonction du choix de l'algo si le pointeur est non nul
    //-- gestion de la sauvegarde de chaque algo
    string nom_gestion_sauvegarde; // le nom éventuel de la fonction qui gère la sauvegarde
    Fonction_nD*  gestion_sauvegarde; // si non NULL, gestion de la sauvegarde de chaque algo
    int nb_dernier_algo_qui_a_fait_une_sauvegarde; //
    //-- gestion de la sortie à convergence de chaque algo
    string nom_gestion_sortie_a_convergence; // le nom éventuel de la fonction qui gère la sortie
    Fonction_nD*  gestion_sortie_a_convergence; // si non NULL, gestion de la sortie de chaque algo

    Tableau < Fonction_nD* > * tb_combiner; // le tableau des 3 fctnD de gestion que l'on passe aux algo
    
    // deux fonctions par défaut qui servent pour tb_combiner
    Fonction_nD* cst_0; // une fonction Cst qui retourne 0
    Fonction_nD* cst_1; // une fonction Cst qui retourne 1

    // METHODES PROTEGEES :
//    // sauvegarde sur base info
//    // cas donne le niveau de sauvegarde
//    // = 0 : initialisation de la sauvegarde -> c'est-à-dire de la sortie base info
//    // = 1 : on sauvegarde tout
//    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
//    // incre : numero d'incrément auquel on sauvegarde
//    // éventuellement est définit de manière spécifique pour chaque algorithme
//    // dans les classes filles
//    virtual void Ecriture_base_info
//             (ofstream& sort,const int cas);
//    // cas de la lecture spécifique à l'algorithme dans base_info
//    virtual void Lecture_base_info(ifstream& ent,const int cas);

    // lecture des paramètres du calcul
    void lecture_Parametres(UtilLecture& entreePrinc);
    
    void Creation_fct_cst(); // création de cst_0 et cst_1
    
    // écriture des paramètres dans la base info
    // = 1 : on écrit tout
    // = 2 : on écrot uniquement les données variables (supposées comme telles)
    void Ecrit_Base_info_Parametre(UtilLecture& entreePrinc,const int& cas);
    // lecture des paramètres dans la base info
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
    // choix = true  : fonctionnememt normal
    // choix = false : la méthode ne doit pas lire mais initialiser les données à leurs valeurs par défaut
    //                 car la lecture est impossible
    void Lecture_Base_info_Parametre(UtilLecture& entreePrinc,const int& cas,bool choix);
    // création d'un fichier de commande: cas des paramètres spécifiques
    void Info_commande_parametres(UtilLecture& entreePrinc);

    // ------- concernant les algo internes --------
    // lecture des algorithmes internes
    void Lecture_algo_interne(const bool avec_typeDeCal
                             ,const list <EnumSousTypeCalcul>& soustype
                             ,const list <bool>& avec_soustypeDeCal
                             ,UtilLecture& entreePrinc);
    
    //---- gestion des commndes interactives --------------
    // écoute et prise en compte d'une commande interactive
    // ramène true tant qu'il y a des commandes en cours
    bool ActionInteractiveAlgo();

 };
 /// @}  // end of group

#endif  
