// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        8/06/06                                             *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Algorithme de calcul dynamique, méthode de Runge Kutta,  *
 *             pour de la mecanique du solide.                          *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef AGORIRUNGEKUTTA_T
#define AGORIRUNGEKUTTA_T


#include "Algori.h"
#include "Assemblage.h"
#include "Algo_edp.h"

/// @addtogroup Les_algorithmes_de_resolutions_globales
///  @{
///

///     BUT:    Algorithme de calcul dynamique, méthode de Runge Kutta,
///             pour de la mecanique du solide.

class AlgoriRungeKutta : public Algori
{
  public :
    // CONSTRUCTEURS :
    AlgoriRungeKutta () ; // par defaut
    
    // constructeur en fonction du type de calcul
    // du sous type (pour les erreurs, remaillage etc...)
    // il y a ici lecture des parametres attaches au type
    AlgoriRungeKutta (const bool avec_typeDeCal
           ,const list <EnumSousTypeCalcul>& soustype
           ,const list <bool>& avec_soustypeDeCal
           ,UtilLecture& entreePrinc);
    // constructeur de copie
    AlgoriRungeKutta (const AlgoriRungeKutta& algo);
    
    // constructeur de copie à partie d'une instance indifférenciée
    Algori * New_idem(const Algori* algo) const
     {// on vérifie qu'il s'agit bien d'une instance
      if (algo->TypeDeCalcul() != DYNA_RUNGE_KUTTA)
        { cout << "\n *** erreur lors de la creation par copie d'un algo DYNA_RUNGE_KUTTA "
               << " l'algo passe en parametre est en fait : " << Nom_TypeCalcul(algo->TypeDeCalcul())
               << " arret !! " << flush;
          Sortie(1);
        }
      else
        { AlgoriRungeKutta* inter = (AlgoriRungeKutta*) algo;
          return ((Algori *) new AlgoriRungeKutta(*inter));
        };
     };

    // DESTRUCTEUR :
     ~AlgoriRungeKutta () ;  
    
    // METHODES PUBLIQUES :
    // execution de l'algorithme explicite dans le cas  dynamique sans contact
    void Execution(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D* ,LesFonctions_nD*
                  ,VariablesExporter* varExpor,LesLoisDeComp*
                  ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );

    //------- décomposition en 3 du calcul d'équilibre -------------
    // a priori   : InitAlgorithme  et FinCalcul ne s'appellent qu'une fois,
    // par contre : CalEquilibre peut s'appeler plusieurs fois, le résultat sera différent si entre deux calcul
    //              certaines variables ont-été changés
    
    // initialisation
    void InitAlgorithme(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                        ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                        ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );
    // mise à jour
    void MiseAJourAlgo(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                       ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                       ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );

    // calcul de l'équilibre
    // si tb_combiner est non null -> un tableau de 2 fonctions
    //  - la première fct dit si on doit valider ou non le calcul à convergence ok,
    //  - la seconde dit si on doit sortir de la boucle ou non à convergence ok
    //
    // si la validation est effectuée, la sauvegarde pour le post-traitement est également effectuée
    //                                 en fonction de la demande de sauvegard,
    // sinon pas de sauvegarde pour le post-traitement à moins que l'on a demandé un mode debug
    //   qui lui fonctionne indépendamment
    void CalEquilibre(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                      ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                      ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats*
                      ,Tableau < Fonction_nD* > * tb_combiner);
    // dernière passe
    void FinCalcul(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                   ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                   ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );
                        
    // sortie du schemaXML: en fonction de enu
    void SchemaXML_Algori(ofstream& sort,const Enum_IO_XML enu) const ;
    
  protected :  
    // VARIABLES PROTEGEES :
    
    // liste de variables de travail déclarées ici pour éviter le passage de paramètre entre les 
    // méthodes internes à la classe
    // variables modifiées par Modif_transi_pas_de_temps, et Gestion_pas_de_temps
    double delta_t,unsurdeltat,deltatSurDeux,deltat2; 
    
    // -- variables de transferts internes nécessaires pour : Dyna_point -- 
    //  === pointeurs d'instance et classe particulières
    LesMaillages * lesMail_;  
    LesReferences* lesRef_;
    LesCourbes1D* lesCourbes1D_;
    LesFonctions_nD* lesFonctionsnD_;
    Charge*         charge_;
    LesCondLim* lesCondLim_;
    LesContacts* lesContacts_;
    Assemblage * Ass1, * Ass2, * Ass3; // pointeurs d'assemblages
    
    //  === variables scalaires
    double maxPuissExt;   // maxi de la puissance des efforts externes
    double maxPuissInt;   // maxi de la puissance des efforts internes 
    double maxReaction;   // maxi des reactions 
    int inReaction;       // pointeur d'assemblage pour le maxi de reaction
    int inSol;            // pointeur d'assemblage du maxi de variation de ddl      
    double maxDeltaDdl;   // maxi de variation de ddl
    int cas_combi_ddl;   // def combinaison des ddl 
    int icas;        // idem cas_combi_ddl mais pour lesCondlim
    bool erreurSecondMembre; // pour la gestion des erreurs de calcul au second membre
    bool prepa_avec_remont; // comme son nom l'indique
    bool brestart;   // booleen qui indique si l'on est en restart ou pas
    OrdreVisu::EnumTypeIncre type_incre; // pour la visualisation au fil du calcul
    
    //  === vecteurs
    Vecteur vglobin; // puissance interne : pour ddl accélération
    Vecteur vglobex; // puissance externe 
    Vecteur vglobaal; // puissance totale
    Vecteur vcontact; // puissance des forces de contact
    Vecteur X_Bl,V_Bl,G_Bl; // stockage transitoirement des X V GAMMA <-> CL
    Vecteur forces_vis_num; // forces visqueuses d'origines numériques

    //  === les listes
    list <LesCondLim::Gene_asso> li_gene_asso; // tableaux d'indices généraux des ddl bloqués
    //  === les tableaux
    Tableau <Nb_assemb> t_assemb; // tableau globalisant les numéros d'assemblage de X V gamma
    Tableau <Enum_ddl> tenuXVG; // les enum des inconnues
    //  === les matrices
    Mat_abstraite* mat_masse,* mat_masse_sauve; // choix de la matrice de masse
    Mat_abstraite* mat_C_pt; // matrice visqueuse numérique
    
    // ... partie relative à une résolution de l'avancement par une intégration de l'équation différentielle
    Algo_edp   alg_edp;
  	 int cas_kutta; // indique le type de runge_kutta que l'on veut utiliser
	   double erreurAbsolue,erreurRelative; // précision absolue et relative que l'on désire sur le calcul Xtdt et vitessetdt
	   double erreur_maxi_global; // l'erreur obtenue      
    int nbMaxiAppel; // nombre maxi d'appel de la fonction dérivée
    bool pilotage_un_step; // indique s'il y a pilotage ou pas
    Vecteur estime_erreur;
    Vecteur val_fonc_initiale,der_val_fonc_initiale,val_fonc,der_val_fonc;
    Vecteur val_fonc_final,der_val_fonc_final;
    double scale_fac; // facteur d'homogénéisation des vecteurs _val_fonc_

    // METHODES PROTEGEES :

    // lecture des paramètres du calcul
    void lecture_Parametres(UtilLecture& entreePrinc);
    // écriture des paramètres dans la base info
    // = 1 : on écrit tout
    // = 2 : on écrot uniquement les données variables (supposées comme telles)
    void Ecrit_Base_info_Parametre(UtilLecture& entreePrinc,const int& cas);
    // lecture des paramètres dans la base info
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
    // choix = true  : fonctionnememt normal
    // choix = false : la méthode ne doit pas lire mais initialiser les données à leurs valeurs par défaut
    //                 car la lecture est impossible
    void Lecture_Base_info_Parametre(UtilLecture& entreePrinc,const int& cas,bool choix);
    // création d'un fichier de commande: cas des paramètres spécifiques
    void Info_commande_parametres(UtilLecture& entreePrinc);
    
    // gestion et vérification du pas de temps et modif en conséquence si nécessaire
    // cas = 1: initialisation du pas de temps et vérif / au pas de temps critique
    // cas = 2: vérif / au pas de temps critique, et division par nbstep
    //          ceci pour garantir que l'on fait le calcul avec 1 step
    // en entrée: modif_pas_de_temps: indique qu'il y a eu par ailleurs (via Charge->Avance())
    //   une modification du pas de temps depuis le dernier appel
    // retourne vrai s'il y a une modification du pas de temps, faux sinon
    bool Gestion_pas_de_temps(bool modif_pas_de_temps,LesMaillages * lesMail,int cas,int nbstep);
    
    // modification transitoire du pas de temps et modif en conséquence si nécessaire
    // utilisée en continu par RK
    // delta_tau : nouveau pas de temps transitoire imposé
    void Modif_transi_pas_de_temps(const double & delta_tau);
    
    //---- gestion des commndes interactives --------------
    // écoute et prise en compte d'une commande interactive
    // ramène true tant qu'il y a des commandes en cours
    bool ActionInteractiveAlgo();
    // ------- pour RK : calcul du vecteur dérivée -----------------
    //  calcul de l'expression permettant d'obtenir la dérivée temporelle du problème
    // utilisée dans la résolution de l'équation d'équilibre dynamique par la méthode RK
    // en entrée: 
    // tau:  temps courant
    // val_fonc: qui contient à la suite X et X_point à tau
    // en sortie:
    // der_val_fonc : qui contient à la suite: X_point et gamma
    // erreur : =0: le calcul est licite, si diff de 0, indique qu'il y a eu une erreur
    //          =1:  la norme de sigma est supérieure à la valeur limite de saturation
    Vecteur&  Dyna_point(const double & tau, const Vecteur & val_fonc
                    ,Vecteur & der_val_fonc,int& erreur);
    // vérification de l'intégrité du résultat calculé
    // erreur : =0: le calcul est licite, si diff de 0, indique qu'il y a eu une erreur
    //          =1:  la norme de sigma est supérieure à la valeur limite de saturation
    void Verif_integrite_Solution(const double & tau, const Vecteur & val_fonc,int & erreur);
 
    
 };
 /// @}  // end of group

#endif  
