// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        29/09/2001                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Algorithme de calcul dynamique, pour de la mecanique     *
 *             du solide déformable en coordonnees materielles          *
 *             entrainees, en utilisant la discrétisation temporelle de *
 *             newmark   .                                              *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef AGORINEWMARK_T
#define AGORINEWMARK_T


#include "Algori.h"
#include "Assemblage.h"

/// @addtogroup Les_algorithmes_de_resolutions_globales
///  @{
///

///     BUT:    Algorithme de calcul dynamique, pour de la mecanique
///             du solide déformable en coordonnees materielles
///             entrainees, en utilisant la discrétisation temporelle de
///             newmark   .

class AlgoriNewmark : public Algori
{
  public :
    // CONSTRUCTEURS :
    AlgoriNewmark () ; // par defaut
    
    // constructeur en fonction du type de calcul
    // du sous type (pour les erreurs, remaillage etc...)
    // il y a ici lecture des parametres attaches au type
    AlgoriNewmark (const bool avec_typeDeCal
           ,const list <EnumSousTypeCalcul>& soustype
           ,const list <bool>& avec_soustypeDeCal
           ,UtilLecture& entreePrinc);
    // constructeur de copie
    AlgoriNewmark (const AlgoriNewmark& algo);
    
    // constructeur de copie à partie d'une instance indifférenciée
    Algori * New_idem(const Algori* algo) const
     {// on vérifie qu'il s'agit bien d'une instance
      if (algo->TypeDeCalcul() != DYNA_IMP)
        { cout << "\n *** erreur lors de la creation par copie d'un algo DYNA_IMP "
               << " l'algo passe en parametre est en fait : " << Nom_TypeCalcul(algo->TypeDeCalcul())
               << " arret !! " << flush;
          Sortie(1);
        }
      else
        { AlgoriNewmark* inter = (AlgoriNewmark*) algo;
          return ((Algori *) new AlgoriNewmark(*inter));
        };
     };

    // DESTRUCTEUR :
     ~AlgoriNewmark () ;  
    
    // METHODES PUBLIQUES :
    // execution de l'algorithme dans le cas non dynamique
    void Execution(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D* ,LesFonctions_nD*
                  ,VariablesExporter* varExpor,LesLoisDeComp*
                  ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );

    //------- décomposition en 3 du calcul d'équilibre -------------
    // a priori   : InitAlgorithme  et FinCalcul ne s'appellent qu'une fois,
    // par contre : CalEquilibre peut s'appeler plusieurs fois, le résultat sera différent si entre deux calcul
    //              certaines variables ont-été changés
    
    // initialisation
    void InitAlgorithme(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                        ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                        ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );
    // mise à jour
    void MiseAJourAlgo(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                       ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                       ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );
    // calcul de l'équilibre
    // si tb_combiner est non null -> un tableau de 2 fonctions
    //  - la première fct dit si on doit valider ou non le calcul à convergence ok,
    //  - la seconde dit si on doit sortir de la boucle ou non à convergence ok
    //
    // si la validation est effectuée, la sauvegarde pour le post-traitement est également effectuée
    //                                 en fonction de la demande de sauvegard,
    // sinon pas de sauvegarde pour le post-traitement à moins que l'on a demandé un mode debug
    //   qui lui fonctionne indépendamment
    void CalEquilibre(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                      ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                      ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats*
                      ,Tableau < Fonction_nD* > * tb_combiner);
    // dernière passe
    void FinCalcul(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                   ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                   ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );
                        
    // sortie du schemaXML: en fonction de enu
    void SchemaXML_Algori(ofstream& sort,const Enum_IO_XML enu) const {};
        
  private :  
    // VARIABLES PROTEGEES :
    // paramètre de newmark
    double* beta_newmark;
    double* gamma_newmark;
    double * hht; // paramètre de hilbert hught taylor
    bool stabilite; // indique si le calcul est inconditionnellement stable ou pas
 
    // -- cas où on veut faire du masse scaling avec éventuellement du pilotage
    bool avec_masse_scaling;
    bool et_pilotage_masse_scaling_par_maxRatioForce_inertieSurStatique;
    bool ou_pilotage_masse_scaling_par_maxRatioForce_inertieSurStatique;
    double ratioForce_inertieSurStatique;
    bool et_pilotage_masse_scaling_par_maxRatioDiag_masseSurRaideur;
    bool ou_pilotage_masse_scaling_par_maxRatioDiag_masseSurRaideur;
    double ratioDiag_masseSurRaideur;
 
    // indicateur disant s'il faut calculer les conditions limites à chaque itération on non
    int cL_a_chaque_iteration; // par défaut, non, == uniquement à chaque début d'incrément

    // paramètres intermédiaires de calcul
       // relatif au temps
    double delta_t; // pas de temps
    double delta_t_2,unSurBetaDeltaTcarre,betaDelta_t_2; 
       // def de coeffs simplificateurs
    double betaMoinsGammaSurBeta,unSurGammaDeltat,unMoinGamma;
    double unMoinGammasurgamma,zero5deltatcarreUnMoinDeuxBeta,zero5deltatcarredeuxBeta;
    double deltatUnMoinGamma,deltatGamma,unsurbetadeltat,unmoinsdeuxbetasurdeuxbeta;
    double unSurBetaDeltaTcarre_o;
       // cas de l'amortissement numérique
    double nuAphaPrime,coef_masse,nuBetaPrime,coef_raideur;
    double nuAphaPrimeunsurcoef_masse,unsurcoef_masse; 
    
    // VARIABLES PROTEGEES :
    // --------------------------------------------------------------------------------------
    // -- variables de transferts internes  entre: InitAlgorithme, CalEquilibre, FinCalcul -- 
    // --------------------------------------------------------------------------------------

    //  === pointeurs d'instance et classe particulières
    Assemblage * Ass1_, * Ass2_, * Ass3_; // pointeurs d'assemblages
    
    //  === variables scalaires
    int cas_combi_ddl;   // def combinaison des ddl 
    int icas;        // idem cas_combi_ddl mais pour lesCondlim
    bool prepa_avec_remont; // comme son nom l'indique
    bool brestart;   // booleen qui indique si l'on est en restart ou pas
    OrdreVisu::EnumTypeIncre type_incre; // pour la visualisation au fil du calcul
    
    //  === vecteurs
    Vecteur vglobin; // puissance interne : pour ddl accélération
    Vecteur vglobex; // puissance externe 
    Vecteur vcontact; // puissance des forces de contact
    Vecteur vglobaal; // puissance totale qui ecrase vglobin
    Vecteur vglobal_n,vglobal_n_inter; // pour HHT
	   Vecteur delta_prec_X;  // les positions
    Vecteur inter_tdt ; // accélérations et un vecteur intermédiaire de travail
    Vecteur X_Bl,V_Bl,G_Bl; // stockage transitoirement des X V GAMMA <-> CL
    Vecteur forces_vis_num; // forces visqueuses d'origines numériques

    //  === les matrices
    Mat_abstraite* matglob; // choix de la matrice de raideur
    // --- cas où l'on a des matrices secondaires pour switcher si la première matrice
    // ne fonctionne pas bien, par défaut matglob, matsauve et Tab_matmoysauve sont les premiers
    // éléments des tableaux
    // choix des matrices de raideur de sustitution éventuelles : par défaut matglob = tab_mato(1)
    Tableau < Mat_abstraite*> tab_mato;

    void (Assemblage::* assembMat)   // un pointeur de fonction d'assemblage
                    (Mat_abstraite & matglob,const Mat_abstraite & matloc,
                    const DdlElement& tab_ddl,const Tableau<Noeud *>&tab_noeud);
    // dans le cas où l'on fait du line search on dimensionne des vecteurs globaux supplémentaires
    Vecteur * sauve_deltadept,*sauve_dept_a_tdt,*Vres,*v_travail;

    //  === les listes
    list <LesCondLim::Gene_asso> li_gene_asso; // tableaux d'indices généraux des ddl bloqués
    //  === les tableaux
    Tableau <Nb_assemb> t_assemb; // tableau globalisant les numéros d'assemblage de X V gamma
    Tableau <Enum_ddl> tenuXVG; // les enum des inconnues
    //  === les matrices
    Mat_abstraite* mat_masse; // choix de la matrice de masse
    Mat_abstraite* mat_C_pt; // matrice visqueuse numérique

    // ------------------------------------------------------------------------------------------
    // -- fin variables de transferts internes  entre: InitAlgorithme, CalEquilibre, FinCalcul -- 
    // ------------------------------------------------------------------------------------------

    // METHODES PROTEGEES :

    // lecture des paramètres du calcul
    void lecture_Parametres(UtilLecture& entreePrinc);
    // écriture des paramètres dans la base info
    // = 1 : on écrit tout
    // = 2 : on écrot uniquement les données variables (supposées comme telles)
    void Ecrit_Base_info_Parametre(UtilLecture& entreePrinc,const int& cas);
    // lecture des paramètres dans la base info 
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
    // choix = true  : fonctionnememt normal
    // choix = false : la méthode ne doit pas lire mais initialiser les données à leurs valeurs par défaut
    //                 car la lecture est impossible
    void Lecture_Base_info_Parametre(UtilLecture& entreePrinc,const int& cas,bool choix);
    // création d'un fichier de commande: cas des paramètres spécifiques
    void Info_commande_parametres(UtilLecture& entreePrinc);
    
    // gestion et vérification du pas de temps et modif en conséquence si nécessaire
    // cas = 1: initialisation du pas de temps et de l'amortissement numérique si nécessaire
    //          ceci pour le temps t=0
    // cas = 2: initialisation du pas de temps 
    //          ceci pour le temps t
    void Gestion_pas_de_temps(LesMaillages * lesMail,int cas);
 
    // pilotage éventuel du masse scaling
    void Pilotage_masse_scaling(const double& moy_diag_masse,const double& moy_diag_K
                                ,const int& sauve_indic_convergence);
    
    //---- gestion des commndes interactives --------------
    // écoute et prise en compte d'une commande interactive
    // ramène true tant qu'il y a des commandes en cours
    bool ActionInteractiveAlgo();

 };
 /// @}  // end of group

#endif  
