// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Algorithme de calcul non dynamique, pour de la mecanique *
 *             du solide déformable en coordonnees materielles          *
 *             entrainees.                                              *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef AGORINONDYNA_T
#define AGORINONDYNA_T


#include "Algori.h"
#include "Assemblage.h"
#include "MatLapack.h"

/// @addtogroup Les_algorithmes_de_resolutions_globales
///  @{
///

///     BUT:    Algorithme de calcul non dynamique, pour de la mecanique
///             du solide déformable en coordonnees materielles
///             entrainees.

class AlgoriNonDyna : public Algori
{
  public :
    // CONSTRUCTEURS :
    AlgoriNonDyna () ; // par defaut
    
    // constructeur en fonction du type de calcul
    // du sous type (pour les erreurs, remaillage etc...)
    // il y a ici lecture des parametres attaches au type
    AlgoriNonDyna (const bool avec_typeDeCal
           ,const list <EnumSousTypeCalcul>& soustype
           ,const list <bool>& avec_soustypeDeCal
           ,UtilLecture& entreePrinc);
    // constructeur de copie
    AlgoriNonDyna (const AlgoriNonDyna& algo);
     
    // constructeur de copie à partie d'une instance indifférenciée
    Algori * New_idem(const Algori* algo) const
      {// on vérifie qu'il s'agit bien d'une instance
      if (algo->TypeDeCalcul() != NON_DYNA)
        { cout << "\n *** erreur lors de la creation par copie d'un algo NON_DYNA "
               << " l'algo passe en parametre est en fait : " << Nom_TypeCalcul(algo->TypeDeCalcul())
               << " arret !! " << flush;
          Sortie(1);
        }
      else
        { AlgoriNonDyna* inter = (AlgoriNonDyna*) algo;
          return ((Algori *) new AlgoriNonDyna(*inter));
        };
     };

    // DESTRUCTEUR :
     ~AlgoriNonDyna () ;  
    
    // METHODES PUBLIQUES :
    // execution de l'algorithme dans le cas non dynamique
    void Execution(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D* ,LesFonctions_nD*
                  ,VariablesExporter* varExpor,LesLoisDeComp*
                  ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );

    //------- décomposition en 3 du calcul d'équilibre -------------
    // a priori   : InitAlgorithme  et FinCalcul ne s'appellent qu'une fois,
    // par contre : CalEquilibre peut s'appeler plusieurs fois, le résultat sera différent si entre deux calcul
    //              certaines variables ont-été changés
    
    // initialisation
    void InitAlgorithme(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                        ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                        ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );
    // mise à jour
    void MiseAJourAlgo(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                       ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                       ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );
    // calcul de l'équilibre
    // si tb_combiner est non null -> un tableau de 2 fonctions
    //  - la première fct dit si on doit valider ou non le calcul à convergence ok,
    //  - la seconde dit si on doit sortir de la boucle ou non à convergence ok
    //
    // si la validation est effectuée, la sauvegarde pour le post-traitement est également effectuée
    //                                 en fonction de la demande de sauvegard,
    // sinon pas de sauvegarde pour le post-traitement à moins que l'on a demandé un mode debug
    //   qui lui fonctionne indépendamment
    void CalEquilibre(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                      ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                      ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats*
                      ,Tableau < Fonction_nD* > * tb_combiner);
    // dernière passe
    void FinCalcul(ParaGlob * ,LesMaillages *,LesReferences*,LesCourbes1D*
                   ,LesFonctions_nD* ,VariablesExporter* ,LesLoisDeComp*
                   ,DiversStockage*,Charge*,LesCondLim*,LesContacts*,Resultats* );

    // sortie du schemaXML: en fonction de enu
    void SchemaXML_Algori(ofstream& ,const Enum_IO_XML ) const ;
        
  private :  
  
    // VARIABLES PROTEGEES :
    // pilotage d'un newton modifié éventuel
    int deb_newton_modifie; // NB iter de debut d'utilisation de NM
    int fin_newton_modifie; // NB iter de fin de la méthode et retour N classique
    int nb_iter_NM; // NB d'iter en NM
    // pilotage de l'utilisation d'une matrice moyenne
    int deb_raideur_moyenne; // NB iter de debut
    int fin_raideur_moyenne; // NB iter de fin de la méthode
    int nb_raideur_moyenne; // NB de matrice pour la moyenne
    
    // indicateur disant s'il faut calculer les conditions limites à chaque itération on non
    int cL_a_chaque_iteration; // par défaut, non, == uniquement à chaque début d'incrément
 
    // --------------------------------------------------------------------------------------
    // -- variables de transferts internes  entre: InitAlgorithme, CalEquilibre, FinCalcul -- 
    // --------------------------------------------------------------------------------------

    //  === pointeurs d'instance et classe particulières
    Assemblage * Ass_; // pointeur d'assemblage
    
    //  === variables scalaires
    int cas_combi_ddl;   // def combinaison des ddl 
    int icas;        // idem cas_combi_ddl mais pour lesCondlim
    int compteur; // compteur d'itération
 
    bool prepa_avec_remont; // comme son nom l'indique
    bool brestart;   // booleen qui indique si l'on est en restart ou pas
    OrdreVisu::EnumTypeIncre type_incre; // pour la visualisation au fil du calcul
    
    //  === vecteurs
    Vecteur vglobin; // puissance interne : pour ddl accélération
    Vecteur vglobex; // puissance externe
    Vecteur vcontact; // puissance des forces de contact
    Vecteur vglobaal; // puissance totale qui ecrase vglobin
    Vecteur delta_prec_X;  // les positions
    Vecteur forces_vis_num; // forces visqueuses d'origines numériques

    //  === les matrices
    Mat_abstraite* matglob; // choix de la matrice de raideur
    Mat_abstraite* matsauve; // sauvegarde pour le newton modifié
    Tableau <Mat_abstraite*>* tab_matmoysauve; // sauvegarde pour les moyennes de raideur
 
    // --- cas où l'on a des matrices secondaires pour switcher si la première matrice
    // ne fonctionne pas bien, par défaut matglob, matsauve et Tab_matmoysauve sont les premiers
    // éléments des tableaux
    // choix des matrices de raideur de sustitution éventuelles : par défaut matglob = tab_mato(1)
    Tableau < Mat_abstraite*> tab_mato;
    // matrices de sustitution pour le newton modifié: par défaut matsauve = tab_matsauve(1)
    Tableau < Mat_abstraite*> tab_matsauve;
    // tab de matrices de sustitution pour le niveau de substitution (i)
    // c-a-d que l'on boucle sur j pour la moyenne avec (*tab_tab_matmoysauve(j))(i)
    // -> j est le premier indice !!
    Tableau < Tableau <Mat_abstraite*> * > tab_tab_matmoysauve; // sauvegarde pour les moyennes de raideur
 
    void (Assemblage::* assembMat)   // un pointeur de fonction d'assemblage
                    (Mat_abstraite & matglob,const Mat_abstraite & matloc,
                    const DdlElement& tab_ddl,const Tableau<Noeud *>&tab_noeud);
    // dans le cas où l'on fait du line search on dimensionne des vecteurs globaux supplémentaires
    Vecteur * sauve_deltadept,*sauve_dept_a_tdt,*Vres,*v_travail;

    // ------------------------------------------------------------------------------------------
    // -- fin variables de transferts internes  entre: InitAlgorithme, CalEquilibre, FinCalcul -- 
    // ------------------------------------------------------------------------------------------
    
    // --- accélération de convergence --------- 
    Tableau <Vecteur> Vi,Yi; // les vecteurs projection et accroissement d'une itération à l'autre
    Vecteur coef_ai; // coefficients de la série 
    MatLapack mat_ai; // matrice pour le calcul des coef_ai 
    Vecteur Sm_ai; // second membre de mat_ai 
    Vecteur X_extrapol; // la position extrapolée
    Vecteur S_0; // la solution initiale
    int nb_vec_cycle; // nombre de vecteur maxi pour le cycle
    int cas_acceleration_convergence; // classe les différents cas: 
               // = 1: les vecteurs de projection sont les positions Xtdt
               // = 2: les vecteurs de projection sont les résidus
    bool acceleration_convergence; // indique si oui ou non on veut une accélération de convergence 
    Vecteur sauve_sol; // sauvegarde de la solution de la résolution de l'équilibre                                              
    
    // METHODES PROTEGEES :

    // lecture des paramètres du calcul
    void lecture_Parametres(UtilLecture& entreePrinc);
    // écriture des paramètres dans la base info
    // = 1 : on écrit tout
    // = 2 : on écrot uniquement les données variables (supposées comme telles)
    void Ecrit_Base_info_Parametre(UtilLecture& entreePrinc,const int& cas);
    // lecture des paramètres dans la base info
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
    // choix = true  : fonctionnememt normal
    // choix = false : la méthode ne doit pas lire mais initialiser les données à leurs valeurs par défaut
    //                 car la lecture est impossible
    void Lecture_Base_info_Parametre(UtilLecture& entreePrinc,const int& cas,bool choix);
    // création d'un fichier de commande: cas des paramètres spécifiques
    void Info_commande_parametres(UtilLecture& entreePrinc);

            
    // Calcul de l'équilibre de la pièce avec pilotage de longueur d'arc
    void Calcul_Equilibre_longueur_arc
           (ParaGlob * paraGlob,LesMaillages * lesMail,
            LesReferences* lesRef,LesCourbes1D* lesCourbes1D,LesFonctions_nD*  lesFonctionsnD
            ,VariablesExporter* varExpor,LesLoisDeComp* lesLoisDeComp,
            DiversStockage* diversStockage,Charge* charge,
            LesCondLim* lesCondLim,LesContacts* lesContacts,Resultats* resultats);

    // accélération de convergence, suivant une méthode MMPE  (modified minimal polynomial extrapolation)
    // accélération de convergence, suivant une méthode MMPE  (modified minimal polynomial extrapolation)
    // compteur : compteur d'itération
    // retour = 0: l'accélération n'est pas active (aucun résultat, aucune action, aucune sauvegarde)
    //        = 1: phase de préparation: sauvegarde d'info, mais aucun résultat
    //        = 2: calcul de l'accélération mais ... pas d'accélération constatée
    //        = 3: calcul de l'accélération et l'accélération est effective
    // dans le cas différent de 0 et 1, le programme a priori, change le stockage des conditions
    // limites, en particulier CL linéaire, mais avec des hypothèses, pour les sorties de réactions
    // etc, il vaut mieux refaire une boucle normale avec toutes les initialisation ad hoc
    // affiche: indique si l'on veut l'affichage ou non des infos
    int AccelerationConvergence(bool affiche,const Vecteur& vres ,LesMaillages * lesMail 
          ,const int& compteur,const Vecteur& X_t,const Vecteur& delta_X,Charge* charge
          ,Vecteur& vglobex,Assemblage& Ass ,const Vecteur& delta_delta_X
          ,LesCondLim* lesCondLim,Vecteur& vglobal 
          ,LesReferences* lesRef,Vecteur & vglobin,const Nb_assemb& nb_casAssemb 
          ,int cas_combi_ddl,LesCourbes1D* lesCourbes1D,LesFonctions_nD*  lesFonctionsnD);
    
    //---- gestion des commndes interactives --------------
    // écoute et prise en compte d'une commande interactive
    // ramène true tant qu'il y a des commandes en cours
    bool ActionInteractiveAlgo();

 };
 /// @}  // end of group

#endif  
