
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Definir  La geometrie d'un segment .                       *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef SEGMENT_H
#define SEGMENT_H

#include"ElemGeomC0.h"

/*
// lieaire -> 2 noeuds
//
//         -1            0           1            KSI
//       ---*------------|-----------*--------->   
//         (1)                     (2)
//
// quadratique -> 3 noeuds
//
//         -1            0           1            KSI
//       ---*------------|-----------*--------->   
//         (1)          (2)         (3)
//
// cubique -> 4 noeuds
//
//         -1              0              1            KSI
//       ---*--------*----------*---------*--------->   
//         (1)      (2)        (3)       (4)
//
// points d'intégration de Gauss: code = nbi
// 1 pt d'integ -> KSI = 0;
// 2 pt d'integ -> KSI = -1/sqrt(3), 1/sqrt(3);
// 3 pt d'integ -> KSI =  -sqrt(3./5.), 0, sqrt(3./5.);
// 4 pt d'integ -> KSI = -T1, -T, T, T1  , T1 et T environ 0.86 et 0.33
//     avec T = sqrt( (3.-S)/7)  et T1 =sqrt((3.+2.*sqrt(6./5.))/7.);
// ce qu'il faut retenir c'est que les points sont a suivre
// points d'intégration de Gauss-Lobatto: code = 2000+nbi
// de 1 à 16 points, qui sont connus uniquement sous forme numérique 
*/
         
/// @addtogroup Les_Elements_de_geometrie
///  @{
///


class GeomSeg : public ElemGeomC0
{
  public :
    // CONSTRUCTEURS :
    // par defaut on considere un segment un point d'integ et a deux noeuds  
    // par défaut ce sont des points de gauss, cependant, il est possible d'utiliser
    // également une intégration de Gauss-lobatto 
    GeomSeg(int code_nbi =1, int nbe = 2,Enum_type_pt_integ type_pti = PTI_GAUSS);
    // de copie
    GeomSeg(const GeomSeg& a);    
    // DESTRUCTEUR :
    ~GeomSeg() {};
    
    // retourne le tableau des derivees secondes des fonctions d'interpolation
    inline Tableau < Mat_pleine >& taD2phi() { return tabD2Phi;};
    
    // création d'élément identiques : cette fonction est analogue à la fonction new
    // elle y fait d'ailleurs appel. l'implantation est spécifique dans chaque classe
    // dérivée
    // pt est le pointeur qui est affecté par la fonction 
    ElemGeomC0 * newElemGeomC0(ElemGeomC0 * pt) ;
    
    //--------- cas de coordonnees locales quelconques ---------------- 
    // retourne les fonctions d'interpolation au point M (en coordonnees locales)
    const Vecteur& Phi(const Coordonnee& M);
    // retourne les derivees des fonctions d'interpolation au point M (en coordonnees locales)
    const Mat_pleine& Dphi(const Coordonnee& M);
    // en fonction de coordonnees locales, retourne true si le point est a l'interieur
    // de l'element, false sinon
    bool Interieur(const Coordonnee& M);
    // en fonction de coordonnees locales, retourne le point local P, maximum intérieur à l'élément, donc sur la frontière
    // dont les coordonnées sont sur la droite GM: c-a-d GP = alpha GM, avec apha maxi et P appartenant à la frontière
    // de l'élément, G étant le centre de gravité, sauf si GM est nul, dans ce cas retour de M
    Coordonnee Maxi_Coor_dans_directionGM(const Coordonnee& M);
    
    // ---------- methodes particuliere----------------------------
      // fourni la coordonnees ksi du point d'integ i
    inline double& KSI(int i) { return ptInteg(i)(1);};
    
  protected :  
    // VARIABLES PROTEGEES :
    Tableau < Mat_pleine > tabD2Phi; // tabD2Phi(ni) = D2phi pour le point d'intégration ni
            // tel que, D2phi(i,r) =
            // valeur de la derivee seconde de la fonction phi(r) par rapport a la coordonnee`
            // locale i = 1   
 
    // variables de stockage transitoire, locales pour éviter de les reconstruire à chaque appel
    Vecteur phi_M; // le tableau phi au point M(en coordonnees locales)
    Mat_pleine dphi_M; //les derivees des fonctions d'interpolation au point M(en coordonnees locales)

    // METHODES PROTEGEES :
    // constitution du tableau Extrapol
    void Calcul_extrapol(int nbi);
 
    // concernant l'intégration à l'aide des points et poids de Gauss_Lobatto
	 static Tableau <Tableau <double > > ksi_gl; 
	 static Tableau <Tableau <double > > wi_gl; 
	 class Construire_Gauss_Lobatto {public: Construire_Gauss_Lobatto();};
	 static Construire_Gauss_Lobatto construire_gauss_lobatto; 
    
        
 };
 /// @}  // end of group

#endif  
