
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Definir  La geometrie du quadrangle .                      *
 *                    les points sur la surface sont calcules par       *
 *                    la methode produit.                         $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef GEOMQUADRANGLE_H
#define GEOMQUADRANGLE_H

#include"ElemGeomC0.h"

/*
// description de la numerotation employee
//
// cas des noeuds: 
//
//    lineaire                      quadratique               lineaire/quadratique     cubique complet
//
//   (4)          (3)          (4)    (7)    (3)               (4)    (6)    (3)     
//    *------------*            *------*------*                 *------*------*       (4)--(10)--(9)--(3)
//    |            |            |             |                 |             |        |     |    |    |
//    |            |            |             |                 |             |       (11)-(16)--(15)-(8)  
//    |            |         (8)*     (9)     * (6)             |             |        |     |    |    |        
//    |            |            |             |                 |             |       (12)-(13)--(14)-(7)
//    |            |            |             |                 |             |        |     |    |    |
//    *------------*            *------*------*                 *------*------*       (1)---(5)--(6)--(2)
//   (1)          (2)          (1)    (5)    (2)               (1)    (5)    (2)
// 
//  dans le cas d'un quadratique incomplet, nbne = 8, le noeud (9) est supprime
//
// face 1 : noeuds de l'élément 
//  on attribue le même nombre de points d'integration pour la  face que pour l'élément
//
// pour les aretes on suis le fichier Elmail, 4 aretes
//  1) pour le quadrangle bilinéaire
//   1 2   2 3   3 4   4 1
//  2) pour le quadrangle quadratique complet ou incomplet
//   1 5 2   2 6 3   3 7 4   4 8 1
//  3) pour le quadrangle cubique complet 
//   1 5 6 2   2 7 8 3   3 9 10 4   4 11 12  1
//
//   on attribue la racine carré du nombre de point d'intégration de l'élément pour l'arrête
//   d'ou en général : 1 point d'integration par arete pour les bilinéaires et 2 points pour
//   les quadratiques
//  
//
// cas des points d'integration
//          ^ y                                 ^ y
//          |                                   |
//          |                                   |
//         1 point                           4 points           
//    *------------*                      *-------------*  
//    |            |                      | (3)    (4)  |
//    |            |                      |             |
//    |    (1)     |                      |             |   
//    |            |                      |             |
//    |            |                      | (1)    (2)  |
//    *------------*                      *-------------*
//          
//        9 points                             16 points
//    *-------------*                      *--------------------*  
//    | (7) (8) (9) |                      | (13)(14) (15) (16) | 
//    |             |                      |                    |
//    | (4) (5) (6) | --> x                | (9) (10) (11) (12) |   --> x
//    |             |                      |                    |
//    | (1) (2) (3) |                      | (5)  (6)  (7)  (8) |
//    *-------------*                      |                    |
//                                         | (1)  (2)  (3)  (4) |
//                                         *--------------------*
//
//   concernant la triangulation linéaire de l'élément :
//    - pour l'élément linéaire : décomposé en 2 éléments triangulaires de connexion :
//          1e : 1 2 3,  2e : 1 3 4
//    - pour l'élément quadratique incomplet : décomposé en 6 éléments triangulaires de connexion :
//          1e : 1 5 8,  2e : 8 5 6, 3e : 5 2 6, 4e : 6 3 7, 5e : 8 6 7, 6e : 8 7 4
//    - pour l'élément quadratique complet : décomposé en 8 éléments triangulaires de connexion :
//          1e : 8 5 9,  2e : 5 6 9, 3e : 9 6 7, 4e : 8 9 7, 5e : 1 5 8, 6e :5 2 6, 7e : 6 3 7, 8e :8 7 4
//    - pour l'élément linéaire/quadratique : décomposé en 4 éléments triangulaires de connexion :
//          1e : 1 6 4,  2e : 1 5 6, 3e : 5 3 6, 4e : 5 2 3
//    - pour l'élément cubique complet : décomposé en 18 éléments triangulaires de connexion :
//          1e  : 1 5 12,    2e  : 5 13 12,  3e  : 5 6 13,   4e  : 6 14 13
//          5e  : 6 2 14,    6e  : 2 7 14,   7e  : 12 13 11, 8e  : 13 16 11
//          9e  : 13 14 16,  10e : 14 15 16, 11e : 14 7 15,  12e :  7 8 15
//          13e : 11 16 4,   14e : 16 10 4,  15e : 16 15 10, 16e : 15 9 10
//          17e : 15 8 9,    18e : 8 3 9
//
*/

         
/// @addtogroup Les_Elements_de_geometrie
///  @{
///


class GeomQuadrangle : public ElemGeomC0
{
  public :
    // CONSTRUCTEURS :
    // par defaut on a 4 pt d'integ et 4 noeuds     
    // sans_extrapole : cas particulier pour lequel on ne construit pas le
    // tableau d'extrapolation: nécessaire pour justement définir ce tableau pour
    // des éléments particuliers (ex: hexaèdre) en évitant une boucle récurcive
    // via l'utilisation de sous-éléments particuliers (quadrangle, hexaèdre etc.)
    GeomQuadrangle(int nbi = 4, int nbne = 4,int sans_extrapole = 0);
    // de copie
    GeomQuadrangle(const GeomQuadrangle& a);    
    // DESTRUCTEUR :
    ~GeomQuadrangle();
    
    // création d'élément identiques : cette fonction est analogue à la fonction new
    // elle y fait d'ailleurs appel. l'implantation est spécifique dans chaque classe
    // dérivée
    // pt est le pointeur qui est affecté par la fonction 
    ElemGeomC0 * newElemGeomC0(ElemGeomC0 * pt) ;
    
    //--------- cas de coordonnees locales quelconques ---------------- 
    // retourne les fonctions d'interpolation au point M (en coordonnees locales)
    const Vecteur& Phi(const Coordonnee& M);
    // retourne les derivees des fonctions d'interpolation au point M (en coordonnees locales)
    const Mat_pleine& Dphi(const Coordonnee& M);
    // en fonction de coordonnees locales, retourne true si le point est a l'interieur
    // de l'element, false sinon
    bool Interieur(const Coordonnee& M);
    // en fonction de coordonnees locales, retourne le point local P, maximum intérieur à l'élément, donc sur la frontière
    // dont les coordonnées sont sur la droite GM: c-a-d GP = alpha GM, avec apha maxi et P appartenant à la frontière
    // de l'élément, G étant le centre de gravité, sauf si GM est nul, dans ce cas retour de M
    Coordonnee Maxi_Coor_dans_directionGM(const Coordonnee& M);       
    
  protected :  
 
    // variables de stockage transitoire, locales pour éviter de les reconstruire à chaque appel
    Vecteur phi_M; // le tableau phi au point M(en coordonnees locales)
    Mat_pleine dphi_M; //les derivees des fonctions d'interpolation au point M(en coordonnees locales)

    // METHODES PROTEGEES :
    // constitution du tableau Extrapol
    void Calcul_extrapol(int nbi);
 };
 /// @}  // end of group

#endif  
