
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Definir  les info qui sont communs aux elements        *
 *           triangle.                                                  *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef GEOMTRIANGLE_H
#define GEOMTRIANGLE_H

#include "ElemGeomC0.h"

/*
// les fonctions d'interpolation sont definis pour lineaire quadratique  cubique
//
// lineaire:                      quadratique :                  (3)      cubique  et cubique incomplet
//       (3) |\                     (3) |\                        |\                    = 9 premiers noeuds
//           | \                        | \                       | \
//           |  \                   (6) |  \ (5)             (8)  |  \ (7)
//       (1) |___\ (2)                  |   \                     |   \
//                                  (1) |____\ (2)           (9)  |(10)\ (6)      
//                                       (4)                      |     \
//                                                                |      \ 
//   concernant les aretes                                    (1) --------- (2)
//  elles sont lineaire  quadratique ou cubique comme l'element    (4) (5) 
//  numerotation des arrêtes : 
//  en linéaire :     1 : 1 2,        2 : 2 3,      3 : 3 1
//  en quadratique :  1 : 1 4 2,      2 : 2 5 3,    3 : 3 6 1
//  en cubique:       1 : 1 4 5 2     2: 2 6 7 3    3: 3 8 9 1
//  leur nombre de point d'integration nbil depend du nombre nbi de l'element
//    nbi = 1 -> nbil = 1
//    nbi =  3 ou 4    -> nbil = 2
//   concernant la triangulation linéaire de l'élément :
//    - pour l'élément linéaire : décomposé en lui même
//    - pour l'élément quadratique : décomposé en 4 éléments triangulaires de connexion :
//          1e : 6 4 5,  2e : 1 4 6, 3e : 4 2 5, 4e : 6 5 3
//    - pour l'élément cubique : décomposé en 9 éléments triangulaires de connexion
//          1e : 1 4 9   2e : 10 9 4  3e: 4 5 10  4e: 6 10 5  
//          5e : 5 2 6   6e : 9 10 8  7e: 7 8 10  8e: 10 6 7  9e: 8 7 3
//
//   concernant les points d'intégrations
//
//   un point -> au centre de gravité
//    
//     3 points                   3 points (code 1003)            4 points             6 points
//
//     | \                            | \                           | \                  | \
//     |  \                           |  \                          |  \                 |  \                
//     |    \                         | 3  \                        | 4  \               | 6  \               
//     |     \                        |     \                       |     \              |     \             
//     2       1                      |       \           	  	      |       \            |       \           
//     |        \    	                |        \    	               |      1 \    	 	    |  2   1  \    	    
//     |          \                   |          \                  |          \         |          \       	      
//     |           \                  |           \                 |           \        |           \        
//     |             \                | 1         2 \               | 2        3  \      | 4    3    5 \       
//     |______3_______\               |______________\              |______________\     |______________\       
//
//     7 points                        12 points 
//
//      | \                             |\                         
//      |   \                           | \                        
//      | 7  \                          |3  \                      
//      |      \                        |     \                    
//      |        \                      |12  11 \           	  	  
//      |3      2 \    	                |  6      \    	           
//      |     1     \                   |8        9 \                    
//      |            \                  |  4   5     \              
//      | 5         6  \                |1   7   10  2 \             
//      |_______4_______\               |_______________\             
																	
*/

																		    	 
																		        
																		        
																		        
                                   
/// @addtogroup Les_Elements_de_geometrie
///  @{
///


class GeomTriangle : public ElemGeomC0
{
  public :
    // CONSTRUCTEURS :
    // par defaut on suppose un point d'integration et
    // un triangle lineaire a 3 noeuds
    GeomTriangle( int nbi = 1, int nbne = 3);
    // de copie
    GeomTriangle(const GeomTriangle& a);    
    // DESTRUCTEUR :
    ~GeomTriangle();
    // methodes particuliere
    
    // création d'élément identiques : cette fonction est analogue à la fonction new
    // elle y fait d'ailleurs appel. l'implantation est spécifique dans chaque classe
    // dérivée
    // pt est le pointeur qui est affecté par la fonction 
    ElemGeomC0 * newElemGeomC0(ElemGeomC0 * pt) ;

    //--------- cas de coordonnees locales quelconques ---------------- 
    // retourne les fonctions d'interpolation au point M (en coordonnees locales)
    const Vecteur& Phi(const Coordonnee& M);
    // retourne les derivees des fonctions d'interpolation au point M (en coordonnees locales)
    const Mat_pleine& Dphi(const Coordonnee& M);
    // en fonction de coordonnees locales, retourne true si le point est a l'interieur
    // de l'element, false sinon
    bool Interieur(const Coordonnee& M);
    // en fonction de coordonnees locales, retourne le point local P, maximum intérieur à l'élément, donc sur la frontière
    // dont les coordonnées sont sur la droite GM: c-a-d GP = alpha GM, avec apha maxi et P appartenant à la frontière
    // de l'élément, G étant le centre de gravité, sauf si GM est nul, dans ce cas retour de M
    Coordonnee Maxi_Coor_dans_directionGM(const Coordonnee& M);       

  protected :  
 
    // variables de stockage transitoire, locales pour éviter de les reconstruire à chaque appel
    Vecteur phi_M; // le tableau phi au point M(en coordonnees locales)
    Mat_pleine dphi_M; //les derivees des fonctions d'interpolation au point M(en coordonnees locales)

    // METHODES PROTEGEES :
      // fourni la coordonnees ksi du point d'integ i
    inline double& KSI(int i) { return ptInteg(i)(1);};
    inline double& ETA(int i) { return ptInteg(i)(2);};
    // constitution du tableau Extrapol
    void Calcul_extrapol(int nbi);
        
 };
 /// @}  // end of group

#endif  
