
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        19/12/99                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Definir  Les éléments communs aux géométrie pentaèdrique.  *
 *           Poids et  points d'integration etc                         *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef GEOMPENTACOM_H
#define GEOMPENTACOM_H

#include"ElemGeomC0.h"

/*
// ***********************************************************************
//                                                                       *
//     ELEMENT DE REFERENCE , POINTS D'INTEGRATION:                      *
//                                                                       *
// ----------------------------------------------------------------------*
//
//                         |zeta
//                         |
//                         4---------6
//                        /|       * |
//                       / |     *   |
//                      /  |   *     |
//                     /   |-*-------|----- eta
//                    /   /|         |
//                   /   * |         |
//                  / * /  |         |
//                 5   /   1---------3
//                 |  /   /       *
//                 | /   /      *
//                 |/   /     *
//                 |   /    *  
//                /|  /   *   
//              xi | /  *     
//                 |/ *      
//                 2  
//                
//              
// 
//                pentaèdre trilinéaire
//           
//
//
// Points d'integration (voir triangle et segment) 
//
//  cas du trilinéaire -> description des faces , puis des arêtes
// face 1 : noeud 1 3 2, face 2 : noeud 1 4 6 3,
// face 3 : noeud 1 2 5 4, face 4 : noeud 4 5 6,
// face 5 : noeud 2 3 6 5
//  les normales sortent des faces des elements
//  
// par défaut: on attribue un point d'intégration dans le plan // aux triangles
//  et 2 points dans l'épaisseur
//  
// pour les aretes on suis le fichier Elmail, 9 aretes
// 1-> 1 2   2->2 3   3->3 1   
// 4-> 1 4   5->2 5   6->3 6   
// 7-> 4 5   8->5 6   9->6 4
//
//  cas du triquadratique -> description des faces 
// face 1 : noeud 9  3  8  2  7  1, face 2 : noeud 9  1 10  4 15  6  12  3,
// face 3 : noeud 7  2 11  5 13  4  10  1, face 4 : noeud 15  6 14  5 13  4,
// face 5 : noeud 8  3 12  6 14  5  11  2, 
//  les normales sortent des faces des elements, puis des arêtes
//
// pour les aretes on suit le fichier Elmail, 9 aretes
//1->1 7 2    2->2 8 3    3->3 9 1   
//4->1 10 4   5->2 11 5   6->3 12 6   
//7->4 13 5   8->5 14 6   9->6 15 4
//
//  
// on attribue 3 points d'intégration suivant les plans // aux triangles
//  et 2 points dans l'épaisseur
//
// ***********************************************************************
//                                                                       *
//     ELEMENT DE REFERENCE en quadratique , POINTS D'INTEGRATION:       *
//                                                                       *
// ----------------------------------------------------------------------*
//
//                         |zeta
//                         |
//                         4---15----6
//                        /|       * |
//                       / |     *   |
//                      /  |   *     |
//                    13  10-14------12----- eta
//                    /   / *        |
//                   /   * |         |
//                  / * /  |         |
//                 5   /   1----9----3
//                 |  /   /       *
//                 | /   /      *
//                 |/   /     *
//                11   7    8  
//                /|  /   *   
//              xi | /  *     
//                 |/ *      
//                 2  
//               
//              
// 
//                pentaèdre triquadratique incomplet
//           
//
// Points d'integration (voir triangle et segment) 
//
//  cas du triquadratique -> description des faces 
// face 1 : noeud 9  3  8  2  7  1, face 2 : noeud 9  1 10  4 15  6  12  3,
// face 3 : noeud 7  2 11  5 13  4  10  1, face 4 : noeud 15  6 14  5 13  4,
// face 5 : noeud 8  3 12  6 14  5  11  2, 
//  les normales sortent des faces des elements, puis des arêtes
//
// pour les aretes on suit le fichier Elmail, 9 aretes
//1->1 7 2    2->2 8 3    3->3 9 1   
//4->1 10 4   5->2 11 5   6->3 12 6   
//7->4 13 5   8->5 14 6   9->6 15 4
//
//  
// on attribue 3 points d'intégration suivant les plans // aux triangles
//  et 2 points dans l'épaisseur  -> valeurs par défaut
//
//
//  concernant la triangulation de chaque face elle est réalisée à l'aide
//  de la triangulation implantée sur l'élément de référence de la face
//  
//
// ************************************************************************
//                                                                       *
//     ELEMENT DE REFERENCE 18 noeuds, POINTS D'INTEGRATION:             *
//                                                                       *
// ----------------------------------------------------------------------*
//
//                         |zeta
//                         |
//                         4----15---6
//                        /|        *|
//                       / |      *  |
//                      /  |    *    |
//                    13  10-14-18---12----- eta
//                    /  / *         |
//                   /   * |         |
//                  / * /  | 17      |
//                 5  16   1----9----3
//                 |  /   /        *
//                 | /   /       *
//                 |/   /      *
//                11   7    8  
//                /|  /   *   
//              xi | /  *     
//                 |/ *      
//                 2  
//               
//              
// 
//                pentaèdre triquadratique complet
//           
//
// Points d'integration (voir triangle et segment) 
//
//  cas du triquadratique -> description des faces 
// face 1 : noeud 1  3  2  9  8  7, face 2 : noeud 1  4  6  3 10  15  12  9 18,
// face 3 : noeud 1  2  5  4  7 11  13  10 16, face 4 : noeud 4  5   6  13 14  15,
// face 5 : noeud 2  3  6  5  8  12  14  11 17, 
//  les normales sortent des faces des elements, puis des arêtes
//
// pour les aretes on suit le fichier Elmail, 9 aretes
//1->1 7 2    2->2 8 3    3->3 9 1   
//4->1 10 4   5->2 11 5   6->3 12 6   
//7->4 13 5   8->5 14 6   9->6 15 4
//
//  
// on attribue 3 points d'intégration suivant les plans // aux triangles
//  et 2 points dans l'épaisseur  -> valeurs par défaut
//
//
//  concernant la triangulation de chaque face elle est réalisée à l'aide
//  de la triangulation implantée sur l'élément de référence de la face
//  
//
// ************************************************************************
*/
         
/// @addtogroup Les_Elements_de_geometrie
///  @{
///


class GeomPentaCom : public ElemGeomC0
{
  public :
    // CONSTRUCTEURS :
    // le constructeur par défaut ne doit pas être utilisé
    GeomPentaCom();
    
    // il y a nbi points d'integration et nbn noeuds  
    // l'interpolation est donné par les classes dérivées   
    GeomPentaCom(int nbi , int nbn, Enum_interpol interpol);
    
    // de copie
    GeomPentaCom(const GeomPentaCom& a);    
    // DESTRUCTEUR :
    ~GeomPentaCom();
    
    //--------- cas de coordonnees locales quelconques ---------------- 
    // en fonction de coordonnees locales, retourne true si le point est a l'interieur
    // de l'element, false sinon
    bool Interieur(const Coordonnee& M);
    // en fonction de coordonnees locales, retourne le point local P, maximum intérieur à l'élément, donc sur la frontière
    // dont les coordonnées sont sur la droite GM: c-a-d GP = alpha GM, avec apha maxi et P appartenant à la frontière
    // de l'élément, G étant le centre de gravité, sauf si GM est nul, dans ce cas retour de M
    Coordonnee Maxi_Coor_dans_directionGM(const Coordonnee& M);
 
  protected :  
    // METHODES PROTEGEES :
    // constitution du tableau Extrapol
    void Calcul_extrapol(int nbi);
 };
 /// @}  // end of group

#endif  
