
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:         4/12/99                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Definir  Les éléments communs aux géométrie pentaèdrique   *
 *           quadratique incomplet :                                    *
 *           Fonction d'interpolation, points d'integration etc         *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef GEOMPENTAQ_H
#define GEOMPENTAQ_H

#include"GeomPentaCom.h"

/*
// ***********************************************************************
//                                                                       *
//     ELEMENT DE REFERENCE , POINTS D'INTEGRATION:                      *
//                                                                       *
// ----------------------------------------------------------------------*
//
//                         |zeta
//                         |
//                         4---15----6
//                        /|       * |
//                       / |     *   |
//                      /  |   *     |
//                    13  10-14------12----- eta
//                    /  / *         |
//                   /   * |         |
//                  / * /  |         |
//                 5   /   1----9----3
//                 |  /   /       *
//                 | /   /      *
//                 |/   /     *
//                11   7    8  
//                /|  /   *   
//              xi | /  *     
//                 |/ *      
//                 2  
//               
//              
// 
//                pentaèdre triquadratique incomplet
//           
//
// Points d'integration (voir triangle et segment) 
//
//  cas du triquadratique -> description des faces 
// face 1 : noeud 1  3  2  9  8  7, face 2 : noeud 1  4  6  3 10  15  12  9,
// face 3 : noeud 1  2  5  4  7 11  13  10, face 4 : noeud 4  5   6  13 14  15,
// face 5 : noeud 2  3  6  5  8  12  14  11, 
//  les normales sortent des faces des elements, puis des arêtes
//
// pour les aretes on suis le fichier Elmail, 9 aretes
//1->1 7 2    2->2 8 3    3->3 9 1   
//4->1 10 4   5->2 11 5   6->3 12 6   
//7->4 13 5   8->5 14 6   9->6 15 4
//
//  
// on attribue 3 points d'intégration suivant les plans // aux triangles
//  et 2 points dans l'épaisseur  -> valeurs par défaut
//
//
//  concernant la triangulation de chaque face elle est réalisée à l'aide
//  de la triangulation implantée sur l'élément de référence de la face
//  
//
// ************************************************************************
*/
         
/// @addtogroup Les_Elements_de_geometrie
///  @{
///


class GeomPentaQ : public GeomPentaCom

{
  public :
    // CONSTRUCTEURS :
    // il y a 2 points d'integration par défaut     
    GeomPentaQ(int nbi = 6);
    // de copie
    GeomPentaQ(const GeomPentaQ& a);    
    // DESTRUCTEUR :
    ~GeomPentaQ();
    
    // création d'élément identiques : cette fonction est analogue à la fonction new
    // elle y fait d'ailleurs appel. l'implantation est spécifique dans chaque classe
    // dérivée
    // pt est le pointeur qui est affecté par la fonction 
    ElemGeomC0 * newElemGeomC0(ElemGeomC0 * pt) ;

    //--------- cas de coordonnees locales quelconques ---------------- 
    // retourne les fonctions d'interpolation au point M (en coordonnees locales)
    const Vecteur& Phi(const Coordonnee& M);
    // retourne les derivees des fonctions d'interpolation au point M (en coordonnees locales)
    const Mat_pleine& Dphi(const Coordonnee& M);
        
  protected :  
 
    // variables de stockage transitoire, locales pour éviter de les reconstruire à chaque appel
    Vecteur phi_M; // le tableau phi au point M(en coordonnees locales)
    Mat_pleine dphi_M; //les derivees des fonctions d'interpolation au point M(en coordonnees locales)

    // METHODES PROTEGEES :
    inline double& DPHI(int i,int j,int k) { return tabDPhi(k)(i,j);};
    // constitution du tableau Extrapol
    void Calcul_extrapol(int nbi);
        
 };
 /// @}  // end of group

#endif  
