
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Definir  La geometrie des tétraèdres.                      *
 *           Fonction d'interpolation, points d'integration etc         *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef GEOMTETRA_H
#define GEOMTETRA_H

#include"ElemGeomC0.h"

// ***********************************************************************
//                                                                       *
//     ELEMENT DE REFERENCE , POINTS D'INTEGRATION:                      *
//                                                                       *
//     Source : Dhatt et Touzot p 130, 131, 132 pour les fonctions       *
//              d'interpolation. Pour la numérotation : Modulef          *
// ----------------------------------------------------------------------*
//
//               ^                                
//               |zeta
//               |
//               4
//              /|\
//             / | \
//            /  |  \
//           /   |   \
//          /    |    \
//         /     1-----3 - - - > eta
//        /   '      '    
//       / '   .  
//      2 
//     /
//    xi     tetraèdre linéaire                   
//               ^                                
//               |zeta
//               |
//               4
//              /|\
//             / | \
//            /  10 9
//           8   |   \
//          /    |    \
//         /     1- 7- 3 - - - > eta
//        /  5 '   6 '    
//       / '  .    
//      2 
//     /
//    xi      tetraèdre quadratique                   
//
// Points d'integration 
// a=1/racine(3) 
// Pt1 (a,a,a)  ; Pt2 (a,a,-a)  ; Pt3 (a,-a,a)  ; Pt4 (a,-a,-a)
// Pt5 (-a,a,a) ; Pt6 (-a,a,-a) ; Pt7 (-a,-a,a) ; Pt8 (-a,-a,-a)
//
//------------------------------
// pour le tetraèdre linéaire : 
//------------------------------
// face 1 : noeud 1 3 2 , face 2 : noeud 1 4 3,
// face 3 : noeud 1 2 4, face 4 : noeud 2 3 4, 
//  les normales sortent des faces des elements
//  on attribue 1 points d'integration par face
//
// pour les aretes on suit le fichier Elmail, 6 aretes
// A1-> 1 2  A2-> 2 3   A3-> 3 1   
// A4-> 1 4  A5-> 2 4   A6-> 3 4
//
//   on attribue 1 point d'integration par arete
//
//------------------------------
// pour le tetraèdre quadratique : 
//------------------------------
// face 1 : noeud 7  3  6  2  5  1 , face 2 : noeud 5  2  9  4  8  1,
// face 3 : noeud 7  1  8  4 10  3 , face 4 : noeud 6  3 10  4  9  2, 
//  les normales sortent des faces des elements
//  on attribue 3 points d'integration par face
//
// pour les aretes on suit le fichier Elmail, 6 aretes
// A1-> 1 5 2  A2-> 2 6 3   A3-> 3 7 1   
// A4-> 1 8 4  A5-> 2 9 4   A6-> 3 10 4
//
//   on attribue 2 point d'integration par arete
//  
//
// ************************************************************************
         
/// @addtogroup Les_Elements_de_geometrie
///  @{
///


class GeomHexalin : public ElemGeomC0
{
  public :
    // CONSTRUCTEURS :
    // il y a 8 points d'integration et 8 noeuds     
    GeomHexalin();
    // de copie
    GeomHexalin(const GeomHexalin& a);    
    // DESTRUCTEUR :
    ~GeomHexalin();
 
    //--------- cas de coordonnees locales quelconques ---------------- 
    // retourne les fonctions d'interpolation au point M (en coordonnees locales)
    const Vecteur& Phi(const Coordonnee& M);
    // retourne les derivees des fonctions d'interpolation au point M (en coordonnees locales)
    const Mat_pleine& Dphi(const Coordonnee& M);
    // en fonction de coordonnees locales, retourne true si le point est a l'interieur
    // de l'element, false sinon
    bool Interieur(const Coordonnee& M);
        
  protected :  
 
    // variables de stockage transitoire, locales pour éviter de les reconstruire à chaque appel
    Vecteur phi_M; // le tableau phi au point M(en coordonnees locales)
    Mat_pleine dphi_M; //les derivees des fonctions d'interpolation au point M(en coordonnees locales)

    // METHODES PROTEGEES :
    inline double& DPHI(int i,int j,int k) { return tabDPhi(k)(i,j);};
    inline double& PHI(int i,int j) {return tabPhi(j)(i); }; 
    // because les routine de calcul de phi et dphi aux pt d'integ sont trop grandes
    // on en fait des routines 
    void Phiphi();
    void DphiDphi();
        
 };
 /// @}  // end of group

#endif  
