// FICHIER : ElemPoint.h
// CLASSE : ElemPoint

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *    UNIVERSITE DE BRETAGNE SUD (UBS)  --- I.U.P/I.U.T. DE LORIENT     *
 ************************************************************************
 *           LABORATOIRE DE GENIE MECANIQUE ET MATERIAUX (LG2M)         *
 * Centre de Recherche Rue de Saint Maudé - 56325 Lorient cedex         *
 * tel. 02.97.87.45.70 fax. 02.97.87.45.72 http://www-lg2m.univ-ubs.fr  *
 ************************************************************************
 *     DATE:        24/02/2005                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerard.rio@univ-ubs.fr)             *
 *                  Tel 0297874571   fax : 02.97.87.45.72               *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  La classe ElemPoint permet de declarer des elements        *
 *     ElemPoint.                                                       *
 *     L'élément comporte un noeud qui est identique au point d'inté-   *
 *     gration.                                                         *
 *                                                                      *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef ELEM_POINT_H
#define ELEM_POINT_H

#include "ParaGlob.h"
#include "ElemMeca.h"
//#include "Loi_comp_abstraite.h"
#include "Met_abstraite.h"
#include "Met_ElemPoint.h"
#include "Noeud.h"
#include "UtilLecture.h"
#include "Tenseur.h"
#include "NevezTenseur.h"
#include "Deformation.h"
#include "ElFrontiere.h"
#include "GeomPoint.h"
#include "ParaAlgoControle.h"
#include "UmatAbaqus.h" 

//class ConstrucElemPoint;

class ElemPoint : public ElemMeca
{
		
	public :
	
		// CONSTRUCTEURS :
		// les tenseurs on par défaut (dimension==-1) la dimension de l'espace
		// sinon il ont la dimension donnée
		// Constructeur par defaut
		ElemPoint (int dimension=-1);
		// Constructeur fonction  d'un numero de maillage et d'identification  
		ElemPoint (int num_mail,int num_id,int dimension=-1);
 
 
		// Constructeur de copie
		ElemPoint (const ElemPoint& elem);
		
		
		// DESTRUCTEUR :
		virtual ~ElemPoint ();
				
		// création d'un élément de copie: utilisation de l'opérateur new et du constructeur de copie 
		// méthode virtuelle
		Element* Nevez_copie() const { Element * el= new ElemPoint(*this); return el;}; 

		// Surcharge de l'operateur = : realise l'egalite entre deux instances de ElemPoint
		ElemPoint& operator= (ElemPoint& biel);
				
		// METHODES :
// 1) derivant des virtuelles pures
		// Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture *,Tableau<Noeud  *> * );
		
		// Calcul du residu local et de la raideur locale,
		//  pour le schema implicite
		Element::ResRaid  Calcul_implicit (const ParaAlgoControle & pa);
		
		// Calcul du residu local a t
		// pour le schema explicit par exemple 
		Vecteur* CalculResidu_t (const ParaAlgoControle & pa)
		  { return ElemPoint::CalculResidu(false,pa);};
		
		// Calcul du residu local a tdt
		// pour le schema explicit par exemple 
		Vecteur* CalculResidu_tdt (const ParaAlgoControle & pa)
		  { return ElemPoint::CalculResidu(true,pa);};
		
  // Calcul de la matrice masse pour l'élément
  Mat_pleine * CalculMatriceMasse (Enum_calcul_masse id_calcul_masse) ;

  // --------- calcul dynamique ---------
  // calcul  de la longueur d'arrête de l'élément minimal
  // divisé par la célérité  la plus rapide dans le matériau
  double Long_arrete_mini_sur_c(Enum_dure )
       { return 0.;};

  //------- calcul d'erreur, remontée des contraintes -------------------
    // 1)calcul du résidu et de la matrice de raideur pour le calcul d'erreur
  Element::Er_ResRaid ContrainteAuNoeud_ResRaid();
    // 2) remontée aux erreurs aux noeuds
  Element::Er_ResRaid ErreurAuNoeud_ResRaid();
                                 
		// retourne les tableaux de ddl associés aux noeuds, gere par l'element
		// ce tableau et specifique a l'element
		const DdlElement & TableauDdl() const ;
			
		// Libere la place occupee par le residu et eventuellement la raideur
		// par l'appel de Libere de la classe mere et libere les differents tenseurs
		// intermediaires cree pour le calcul et les grandeurs pointee
		// de la raideur et du residu
		void Libere ();
		
		// acquisition  d'une loi de comportement
		void DefLoi (LoiAbstraiteGeneral * NouvelleLoi);

       // test si l'element est complet
       // = 1 tout est ok, =0 element incomplet
		int TestComplet();
		
		// procesure permettant de completer l'element apres
		// sa creation avec les donnees du bloc transmis
		// peut etre appeler plusieurs fois
		Element* Complete(BlocGen & bloc,LesFonctions_nD*  lesFonctionsnD);                         
		// Compléter pour la mise en place de la gestion de l'hourglass
		Element* Complet_Hourglass(LoiAbstraiteGeneral * NouvelleLoi, const BlocGen & bloc) {return this;};

		// ramene l'element geometrique 
		ElemGeomC0& ElementGeometrique() const  { return unefois->doCoMemb->ptpoint;};
		// ramene l'element geometrique en constant
		const ElemGeomC0& ElementGeometrique_const() const { return unefois->doCoMemb->ptpoint;};
		
		// calcul d'un point dans l'élément réel en fonction des coordonnées dans l'élément de référence associé
		// temps: indique si l'on veut les coordonnées à t = 0, ou t ou tdt
		// 1) cas où l'on utilise la place passée en argument
		Coordonnee & Point_physique(const Coordonnee& c_int,Coordonnee & co,Enum_dure temps);
		// 3) cas où l'on veut les coordonnées aux 1, 2 ou trois temps selon la taille du tableau t_co
		void Point_physique(const Coordonnee& c_int,Tableau <Coordonnee> & t_co);

      // affichage dans la sortie transmise, des variables duales "nom"
      // dans le cas ou nom est vide, affichage de "toute" les variables
		void AfficheVarDual(ofstream& sort, Tableau<string>& nom);
        
	   // affichage d'info en fonction de ordre
	   // ordre = "commande" : affichage d'un exemple d'entree pour l'élément
		void Info_com_Element(UtilLecture * entreePrinc,string& ordre,Tableau<Noeud  *> * tabMaillageNoeud) 
         { return Element::Info_com_El(2,entreePrinc,ordre,tabMaillageNoeud);};                         
        
		// retourne un numero d'ordre d'un point le plus près ou est exprimé la grandeur enum
		// par exemple un point d'intégration, mais n'est utilisable qu'avec des méthodes particulières
		// par exemple CoordPtInteg, ou Valeur_a_diff_temps
		// car le numéro d'ordre peut-être différent du numéro d'intégration au sens classique 
		// temps: dit si c'est à 0 ou t ou tdt
		int PointLePlusPres(Enum_dure ,Enum_ddl , const Coordonnee& ) 
		    { return 1;}; // ici c'est uniquement le point de l'élément
		         
        // recuperation des coordonnées du point de numéro d'ordre iteg pour 
        // la grandeur enu
		// temps: dit si c'est à 0 ou t ou tdt
  // si erreur retourne erreur à true
  Coordonnee CoordPtInteg(Enum_dure temps,Enum_ddl enu,int iteg,bool& erreur);
  
  // récupération des  valeurs au numéro d'ordre  = iteg pour
  // les grandeur enu
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  Tableau <double> Valeur_a_diff_temps(bool absolue,Enum_dure enu_t,const List_io<Ddl_enum_etendu>& enu,int iteg) ;
                  	
  // récupération des valeurs au numéro d'ordre = iteg pour les grandeurs enu
  // ici il s'agit de grandeurs tensorielles, le retour s'effectue dans la liste
  // de conteneurs quelconque associée
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  void ValTensorielle_a_diff_temps(bool absolue,Enum_dure enu_t,List_io<TypeQuelconque>& enu,int iteg);

		// ramene vrai si la surface numéro ns existe pour l'élément
		// dans le cas de la ElemPoint il n'y a pas de surface
		bool SurfExiste(int ) const 
		  { return false;};
		
		// ramene vrai si l'arête numéro na existe pour l'élément
		// dans le cas de la ElemPoint il n'y a pas d'arrête
		bool AreteExiste(int ) const
		  { return false;};
    //============= spécifique à l'interface abaqus ============
	   // on associe un noeud à l'élément, remplace le noeud existant, s'il existe déjà
		virtual void Associer_noeud (Noeud  *  noeu);
		
		// une classe de travail spécifiques
		class inNeNpti 
			{ public: int* incre; // numéro d'incrément
			          int* step; // numéro du step
			          int* nbe;   // numéro d'élément
			          int* nbpti; // numéro du point d'intégration
			          double* temps_tdt; // temps courant
			          double* delta_t; // incrément courant de temps
			          string* nom_loi; // nom de la loi
			};
    
  // lecture des grandeurs umat transmises par abaqus sur le pipe
  static const ElemPoint::inNeNpti& Lecture_Abaqus(bool utilisation_umat_interne)
     {unefois_Point.doCoMemb->umatAbaqus.LectureDonneesUmat
        (utilisation_umat_interne,ParaGlob::NiveauImpression());return unefois_Point.doCoMemb->inne;};
  // récup du inNeNpti en cours
  static const ElemPoint::inNeNpti& IncreElemPtint_encours() {return unefois_Point.doCoMemb->inne;};
  // écriture du résultat sur le pipe
  void Ecriture_Abaqus(bool utilisation_umat_interne)
     {unefois->doCoMemb->umatAbaqus.EcritureDonneesUmat
         (utilisation_umat_interne,ParaGlob::NiveauImpression());};
	 // calcul de l'UMat pour abaqus
	 void CalculUmatAbaqus(ParaAlgoControle & pa);
	 // initialisation éventuelle: ajout de point d'intégration si nécessaire
  // les tenseurs ont la dimension 3
  virtual void InitialisationUmatAbaqus()
     {InitialisationUmatAbaqus_interne(3);};
  // récup du  nombre de fois où l'élément est appelé depuis le début
  // de l'incrément. Correspond environ au nombre d'itération 
  int  NbIteration() const {return nb_appelsCalculUmat;};         
	    
	//============= lecture écriture dans base info ==========
	
	 // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info
	     (ifstream& ent,const Tableau<Noeud  *> * tabMaillageNoeud,const int cas) ;
     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info(ofstream& sort,const int cas) ;
 
  // METHODES VIRTUELLES:
    // --------- calculs utils dans le cadre de la recherche du flambement linéaire		
		// Calcul de la matrice géométrique et initiale
		ElemMeca::MatGeomInit MatricesGeometrique_Et_Initiale (const ParaAlgoControle & pa) ;

  // inactive les ddl du problème primaire de mécanique
  inline void Inactive_ddl_primaire()
      {ElemMeca::Inact_ddl_primaire(unefois->doCoMemb->tab_ddl);};
  // active les ddl du problème primaire de mécanique
  inline void Active_ddl_primaire()
      {ElemMeca::Act_ddl_primaire(unefois->doCoMemb->tab_ddl);};
  // ajout des ddl de contraintes pour les noeuds de l'élément
  inline void Plus_ddl_Sigma() 
      {ElemMeca::Ad_ddl_Sigma(unefois->doCoMemb->tab_ddlErr);};
  // inactive les ddl du problème de recherche d'erreur : les contraintes
  inline void Inactive_ddl_Sigma()
      {ElemMeca::Inact_ddl_Sigma(unefois->doCoMemb->tab_ddlErr);};
  // active les ddl du problème de recherche d'erreur : les contraintes
  inline void Active_ddl_Sigma()
      {ElemMeca::Act_ddl_Sigma(unefois->doCoMemb->tab_ddlErr);};
  // active le premier ddl du problème de recherche d'erreur : SIGMA11
  inline void Active_premier_ddl_Sigma() 
      {ElemMeca::Act_premier_ddl_Sigma();};
	 
  // lecture de données diverses sur le flot d'entrée
  void LectureContraintes(UtilLecture * entreePrinc); 
  
  // retour des contraintes en absolu retour true si elle existe sinon false
  bool ContraintesAbsolues(Tableau <Vecteur>& tabSig);
               

  // 2) derivant des virtuelles 
	 // retourne un tableau de ddl element, correspondant à la 
	 // composante de sigma -> SIG11, pour chaque noeud qui contiend
	 // des ddl de contrainte
	 // -> utilisé pour l'assemblage de la raideur d'erreur
  inline   DdlElement&  Tableau_de_Sig1() const
                         {return unefois->doCoMemb->tab_Err1Sig11;} ;
                         
  // actualisation des ddl et des grandeurs actives de t+dt vers t      
  void TdtversT();
  // actualisation des ddl et des grandeurs actives de t vers tdt      
  void TversTdt();

  // calcul de l'erreur sur l'élément. Ce calcul n'est disponible
  // qu'une fois la remontée aux contraintes effectuées sinon aucune
  // action. En retour la valeur de l'erreur sur l'élément
  // type indique le type de calcul d'erreur :
  void ErreurElement(int type,double& errElemRelative
                     ,double& numerateur, double& denominateur); 
		
     // ========= définition et/ou construction des frontières ===============
     			
		// Calcul des frontieres de l'element
		//  creation des elements frontieres et retour du tableau de ces elements
		// la création n'a lieu qu'au premier appel
		// ou lorsque l'on force le paramètre force a true
		// dans ce dernier cas seul les frontière effacées sont recréée
		Tableau <ElFrontiere*> const & Frontiere(bool force = false);
		
  // ramène la frontière point
  // éventuellement création des frontieres points de l'element et stockage dans l'element 
  // si c'est la première fois  sinon il y a seulement retour de l'elements
  // a moins que le paramètre force est mis a true
  // dans ce dernier cas la frontière effacéee est recréée
  // num indique le numéro du point à créer (numérotation EF)
//        ElFrontiere* const  Frontiere_points(int num,bool force = false);

		// ramène la frontière linéique
		// éventuellement création des frontieres linéique de l'element et stockage dans l'element 
		// si c'est la première fois et en 3D sinon il y a seulement retour de l'elements
		// a moins que le paramètre force est mis a true
		// dans ce dernier cas la frontière effacéee est recréée
		// num indique le numéro de l'arête à créer (numérotation EF)
//		ElFrontiere* const  Frontiere_lineique(int num,bool force = false);
		
		// ramène la frontière surfacique
		// éventuellement création des frontieres surfacique de l'element et stockage dans l'element 
		// si c'est la première fois sinon il y a seulement retour de l'elements
		// a moins que le paramètre force est mis a true
		// dans ce dernier cas la frontière effacéee est recréée
        // num indique le numéro de la surface à créer (numérotation EF) 
        // ici normalement la fonction ne doit pas être appelée
//		ElFrontiere* const  Frontiere_surfacique(int ,bool force = false);

// 3) methodes propres a l'element
		
  // ajout du tableau specific de ddl des noeuds de la  ElemPoint
  // la procedure met a jour les ddl(relatif a l'element, c-a-d Xi)
  // des noeuds constituants l'element
  void ConstTabDdl();
                
 protected:
    		
	  // ==== >>>> methodes virtuelles dérivant d'ElemMeca ============  
		// ramene la dimension des tenseurs contraintes et déformations de l'élément
		int Dim_sig_eps() const {return 1;}; 

 // -------------------- calcul de frontières en protected -------------------
    
  //  --- fonction nécessaire pour la construction des Frontières linéiques ou surfaciques particulière à l'élément 
  // adressage des frontières linéiques et surfacique
  // définit dans les classes dérivées, et utilisées pour la construction des frontières
  virtual ElFrontiere* new_frontiere_lin(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
    { return NULL;}; // il n'y a pas de ligne possible
  virtual ElFrontiere* new_frontiere_surf(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
    {return NULL;} // il n'y a pas de surface possible

 
 public:
          
  class DonneeCommune
	      { public :
	        DonneeCommune (GeomPoint& pteg,DdlElement& tab,DdlElement&  tabErr,DdlElement&  tab_Err1Sig,
	                       Met_ElemPoint&  met_point, 
	                       Tableau <Vecteur *> & resEr,Mat_pleine& raidEr,
	                       GeomPoint& pteEr,Vecteur&  residu_int,Mat_pleine&  raideur_int,
	                       Tableau <Vecteur* > & residus_extN,Tableau <Mat_pleine* >&  raideurs_extN,
	                       Mat_pleine&  mat_masse ,GeomPoint& pteMa,UmatAbaqus& umatAbaqus
	                       ,int dimension ,int nbi );
	        DonneeCommune(DonneeCommune& a);
	        ~DonneeCommune();    
         // variables
         GeomPoint ptpoint ; // element geometrique correspondant
         DdlElement  tab_ddl; // tableau des degres 
                     //de liberte des noeuds de l'element commun a tous les
                     // elements
         Met_ElemPoint     met_ElemPoint;  
		       Mat_pleine  matGeom ; // matrice géométrique
         Mat_pleine  matInit  ; // matrice initiale
         Tableau <TenseurBB *> d_epsBB;  // place pour la variation des def
         Tableau <TenseurHH *> d_sigHH;  // place pour la variation des contraintes
         Tableau < Tableau2 <TenseurBB *> > d2_epsBB; // variation seconde des déformations
                 // calcul d'erreur  
         DdlElement  tab_ddlErr; // tableau des degres servant pour le calcul
	                // d'erreur : contraintes
         DdlElement  tab_Err1Sig11;  // tableau du ddl SIG11 pour chaque noeud,  
	         //servant pour le calcul d'erreur : contraintes, en fait pour l'assemblage
         Tableau <Vecteur *>  resErr;  // residu pour le calcul d'erreur
         Mat_pleine  raidErr; // raideur pour le calcul d'erreur 
         GeomPoint pteEr; // contiend les fonctions d'interpolation et
	                           // les derivees  pour le calcul du hessien dans 
	                           //la résolution de la fonctionnelle d'erreur             
	         // -------- calcul de résidus, de raideur : interne ou pour les efforts extérieurs ----------
	         // on utilise des pointeurs pour optimiser la place (même place pointé éventuellement)
         Vecteur  residu_interne;    
         Mat_pleine  raideur_interne; 
         Tableau <Vecteur* >  residus_externeN;   // pour les noeuds
         Tableau <Mat_pleine* >  raideurs_externeN;  // pour les noeuds 
         // ------ données concernant la dynamique --------
         Mat_pleine matrice_masse;
         GeomPoint pteMas; // contiend les fonctions d'interpolation et les dérivées
              // pour les calculs relatifs au calcul de la masse
         // -- particularités pour la routine Umat pour Abaqus
         UmatAbaqus umatAbaqus; 
         inNeNpti  inne; // indices pointant directement sur des données de umatAbaqus
       };
         

	  // classe contenant tous les indicateurs statique qui sont modifiés une seule fois
	  // et un pointeur sur les données statiques communes 
	  // la classe est interne, toutes les variables sont publique. Un pointeur sur une instance de la 
	  // classe est défini. Son allocation est effectuée dans les classes dérivées
	     class UneFois
	      { public :
	         UneFois () ; // constructeur par défaut
	         ~UneFois () ; // destructeur
	   
	        // VARIABLES :
	        public : 
          DonneeCommune * doCoMemb;
        
          // incicateurs permettant de dimensionner seulement au premier passage
          // utilise dans "CalculResidu" et "Calcul_implicit"
          int CalResPrem_t; int CalResPrem_tdt; // à t ou à tdt
          int  CalimpPrem;
          int  dualSortbiel; // pour la sortie des valeurs au pt d'integ
          int  CalSMlin_t; // pour les seconds membres  concernant les arretes
          int  CalSMlin_tdt; // pour les seconds membres  concernant les arretes
          int  CalSMRlin; // pour les seconds membres  concernant les arretes
          int  CalDynamique; // pour le calcul de la matrice de masse
          int  CalPt_0_t_tdt; // pour le calcul de point à 0 t et tdt
			       // ---------- sauvegarde du nombre d'élément en cours --------
			       int nbelem_in_Prog;
      };

  // ------------------------------------------------------------------------------------

	protected :
      // VARIABLES PROTÉGÉES :
  // pour minimiser la place on définit une classe des contraintes et déformations locale
	 // dans le cas de l'umat, l'instance de la classe n'est pas affectée
	  
	 // grandeurs aux points d'intégration: contraintes, déformations, vitesses de def etc.
	 LesPtIntegMecaInterne  lesPtMecaInt;
	 		
	 
  // le nombre de point d'intégration pour le calcul mécanique
  int nbi;  // ce nombre peux évoluer pendant le calcul, il n'est donc pas
            // commun à tous les éléments
	    //-- cas de l'utilisation Umat				  
  int nb_appelsCalculUmat; // stocke le nombre de fois où l'élément est appelé depuis le début
          // de l'incrément. Correspond environ au nombre d'itération          
		
  // place memoire commune a tous les elements ElemPoints      
  static DonneeCommune * doCo_Point;
 
  // idem mais pour les indicateurs qui servent pour l'initialisation
  static UneFois  unefois_Point;
 
  // on utilise une variable intermédiaire car on a des classes dérivées
  // du coup pour différencier les données communes, on utilise une variable spécifique
  // qui donc ne peut pas être static
  UneFois * unefois;

  // type structuré pour construire les éléments
  class NombresConstruire
	     { public:
	       NombresConstruire();
	       int nbne; // le nombre de noeud de l'élément
	       int nbiEr; // le nombre de point d'intégration pour le calcul d'erreur
        int nbiMas; // le nombre de point d'intégration pour le calcul de la matrice masse consistante
      };
  static NombresConstruire nombre_V; // les nombres propres à l'élément

  // --- fonctions protégées
  
        // fonction d'initialisation servant  au niveau du constructeur
		// dim_tenseur: = la dimension des tenseurs
  void Init (int dim_tenseur);
  // construction de données communes: correspond à un new
  DonneeCommune* Def_DonneeCommune(int dim_tenseur);
  // destructions de certaines grandeurs pointées, créées  au niveau de l'initialisation
  void Destruction();
  // changement du nombre de point d'intégration
  void ChangeNombrePtinteg(int nevez_nbi, int dim_tens);
  // pour l'ajout d'element dans la liste : listTypeElemen, geree par la class Element
  class ConstrucElemPoint : public ConstrucElement
       { public :  ConstrucElemPoint () 
               { NouvelleTypeElement nouv(POINT,CONSTANT,MECA_SOLIDE_DEFORMABLE,this);
                 if (ParaGlob::NiveauImpression() >= 4)
                    cout << "\n initialisation ElemPoint" << endl;
                 Element::listTypeElement.push_back(nouv);
               };
          Element * NouvelElement(int num_maill,int num) // un nouvel élément sans rien
               {Element * pt;
                pt  = new ElemPoint (num_maill,num) ;
                return pt;};
             // ramene true si la construction de l'element est possible en fonction
             // des variables globales actuelles: ex en fonction de la dimension	
          bool Element_possible() {return true;};
       }; 
  static ConstrucElemPoint construcElemPoint;        
  // Calcul du residu local a t ou tdt en fonction du booleen
  Vecteur* CalculResidu (bool atdt,const ParaAlgoControle & pa);
 
  //------- pour des classes dérivées ------
  // CONSTRUCTEURS :
  // les tenseurs on par défaut (dimension==-1) la dimension de l'espace
  // sinon il ont la dimension donnée
  ElemPoint (ElemPoint::UneFois& unefois, Enum_geom nouveau_id, int dimension=-1);
  // Constructeur fonction  d'un numero de maillage et d'identification
  ElemPoint (ElemPoint::UneFois& unefois, Enum_geom nouveau_id, int num_mail,int num_id,int dimension=-1);
 
  // initialisation éventuelle: ajout de point d'intégration si nécessaire
  // les tenseurs on par défaut (dimension==-1) la dimension de l'espace
  // utilisation par Point et les classes dérivées
  void InitialisationUmatAbaqus_interne(int dimension = -1);

        
};
#endif
	
	
		

