
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        26/11/2006                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT: Classe pour stocker l'ensemble des informations aux points  *
 *          d'intégration mécanique (plutôt puissance interne)          *
 *     Les données sont totalement, et volontairement non encapsulées,  *
 *     l'accés est direct, l'encapsulage s'effectue à l'étage supérieur.*
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef LESPTINTEGMECAINTERNE_H
#define LESPTINTEGMECAINTERNE_H

#include "PtIntegMecaInterne.h"

/// @addtogroup Groupe_concernant_les_points_integration
///  @{
///


class  LesPtIntegMecaInterne

{   // surcharge de l'operator de lecture
    friend istream & operator >> (istream &, LesPtIntegMecaInterne &);
    // surcharge de l'operator d'ecriture
    friend ostream & operator << (ostream &, const LesPtIntegMecaInterne &);

  public :
    // CONSTRUCTEURS :
    // contructeur par défaut
    LesPtIntegMecaInterne(); 
    // contructeur fonction du nombre de points d'intégration et de la dimension de tenseurs
    LesPtIntegMecaInterne(int nbpti, int dimtens);     
    // contructeur de copie
    LesPtIntegMecaInterne(const LesPtIntegMecaInterne& lespti);     
    
    // DESTRUCTEUR :
    ~LesPtIntegMecaInterne();
    
    // METHODES PUBLIQUES :
	   // Surcharge de l'operateur = 
    LesPtIntegMecaInterne& operator= ( const LesPtIntegMecaInterne& lespti);
    
    // le tableau des grandeurs aux points d'intégration
    // en lecture écriture
    Tableau <PtIntegMecaInterne>& TabPtIntMeca()  {return tabPtInt;};
    // l'élément PtIntegMecaInterne de numéro i
    PtIntegMecaInterne& operator () (int i)  {return tabPtInt(i);};
    
    
    // les tableaux des contraintes
	   Tableau <TenseurHH *>& TabSigHH() {return tabSigHH;}; // contrainte finale
	   Tableau <TenseurHH *>& TabSigHH_t() {return tabSigHH_t;}; // contrainte au début de l'incrément
	   // nombre de points d'intégration
	   int NbPti() const {return tabPtInt.Taille();};
	   // changement de taille donc de nombre de points d'intégration
	   // fonction du nombre de points d'intégration et de la dimension de tenseurs
    // attention: il s'agit d'un dimentionnement pas défaut (les activations diverses
    // sont ensuite à faire: par exemple pour les invariants)
	   void Change_taille_PtIntegMeca(int nbpti, int dimtens);
	   // idem, mais les instances ajoutées ou retirer ont la même dimension de tenseur que celles
	   // qui existent déjà
	   void Change_taille_PtIntegMeca(int nbpti);
	   // retour la dimension des tenseurs gérés
	   int DimTens() const; 
    // actualisation des grandeurs actives de t+dt vers t, pour celles qui existent 
    // sous ces deux formes    
    void TdtversT();
    // actualisation des grandeurs actives de t vers tdt, pour celles qui existent 
    // sous ces deux formes          
    void TversTdt();
    
    // ramène la compressibilité moyenne sur tous les points d'intégration
    double CompressibiliteMoyenne() const;
    
	//============= lecture écriture dans base info ==========
    // cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
	   void Lecture_base_info (ifstream& ent,const int cas);
    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	   void Ecriture_base_info(ofstream& sort,const int cas);
    
  protected:
    // données protégées
	   // grandeurs aux points d'intégration
	   Tableau <PtIntegMecaInterne> tabPtInt;
	   // contraintes groupées sous forme de tableau, qui pointent sur celles de tabPtMecaInt
	   Tableau <TenseurHH *> tabSigHH; // contrainte finale
	   Tableau <TenseurHH *> tabSigHH_t; // contrainte au début de l'incrément

 };
 /// @}  // end of group

#endif  
