// FICHIER : TetraQ_cm15pti.h

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        06/03/2023                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   La classe TetraQ_15pti permet de declarer des elements    *
 * Tetraedriques Triquadratiques complets avec 15 pti et de realiser    *
 * le calcul du residu local et de la raideur locale pour une loi de    *
 * comportement donnee. La dimension de l'espace pour un tel element    *
 * est 3.                                                               *
 *                                                                      *
 * l'interpolation est Triquadratiques complet a 10 noeuds,le nombre    *
 * de point d'integration est  de 15.                                   *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
// -----------classe pour un calcul de mecanique---------



#ifndef TETRAQ_CM15PTI_H
#define TETRAQ_CM15PTI_H

#include "ParaGlob.h"
#include "ElemMeca.h"
#include "Met_abstraite.h"
#include "GeomQuadrangle.h"
#include "Noeud.h"
#include "UtilLecture.h"
#include "Tenseur.h"
#include "NevezTenseur.h"
#include "Deformation.h"
#include "TetraMemb.h"
#include "FrontTriaQuad.h"
#include "FrontSegQuad.h"


/// @addtogroup groupe_des_elements_finis
///  @{
///

class TetraQ_15pti : public TetraMemb
{
		
	public :
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut 
		TetraQ_15pti ();
		
		// Constructeur  fonction  d'un numero
		// d'identification 
		TetraQ_15pti (int num_mail,int num_id);
		
		// Constructeur fonction  d'un numero de maillage et d'identification et
		// du tableau de connexite des noeuds 
		TetraQ_15pti (int num_mail,int num_id,const Tableau<Noeud *>& tab);
		
		// Constructeur de copie
		TetraQ_15pti (const TetraQ_15pti& tetra);
		
		
		// DESTRUCTEUR :
		~TetraQ_15pti ();
				
		// création d'un élément de copie: utilisation de l'opérateur new et du constructeur de copie 
		// méthode virtuelle
        Element* Nevez_copie() const { Element * el= new TetraQ_15pti(*this); return el;};

		// Surcharge de l'operateur = : realise l'egalite entre deux instances de TetraQ_15pti
		TetraQ_15pti& operator= (TetraQ_15pti& tetra);
		
		// METHODES :
// 1) derivant des virtuelles pures
                         
        // affichage dans la sortie transmise, des variables duales "nom"
        // aux differents points d'integration
        // dans le cas ou nom est vide, affichage de "toute" les variables
        void AfficheVarDual(ofstream& sort, Tableau<string>& nom);                 	
        
	protected :
	  
        // adressage des frontières linéiques et surfacique
        // définit dans les classes dérivées, et utilisées pour la construction des frontières
        ElFrontiere* new_frontiere_lin(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontSegQuad(tab,ddelem)));};
        ElFrontiere* new_frontiere_surf(int ,Tableau <Noeud *> & tab, DdlElement& ddelem) 
          { return ((ElFrontiere*) (new FrontTriaQuad(tab,ddelem)));};

      // VARIABLES PRIVEES :
        // place memoire commune a tous les elements TetraQ_15pti
        static TetraMemb::DonnComTetra * doCoTetraQ_15pti;
        // idem mais pour les indicateurs qui servent pour l'initialisation
        static TetraMemb::UneFois  uneFois;

	    class  NombresConstruireTetraQ_15pti : public NombresConstruire
	     {  public: NombresConstruireTetraQ_15pti();
	      };
	    static NombresConstruireTetraQ_15pti nombre_V; // les nombres propres à l'élément
	     		
        // GESTION AUTOMATIQUE D'AJOUT D'ELEMENT DANS LE PROGRAMME
        //ajout de l'element dans la liste : listTypeElemen, geree par la class Element
        class ConsTetraQ_15pti : public ConstrucElement
          { public :  ConsTetraQ_15pti ()
               { NouvelleTypeElement nouv(TETRAEDRE,QUADRACOMPL,MECA_SOLIDE_DEFORMABLE,this,"_cm15pti");
                 if (ParaGlob::NiveauImpression() >= 4)
                    cout << "\n initialisation TetraQ_15pti" << endl;
                 Element::listTypeElement.push_back(nouv);
                };
            Element * NouvelElement(int num_maill,int num) // un nouvel élément sans rien
               {Element * pt;
                pt  = new TetraQ_15pti (num_maill,num) ;
                return pt;};	   
            // ramene true si la construction de l'element est possible en fonction
            // des variables globales actuelles: ex en fonction de la dimension	
            bool Element_possible() { if (ParaGlob::Dimension() == 3) return true; else return false;};	
           }; 
        static ConsTetraQ_15pti consTetraQ_15pti;
        static int bidon; // a virer lorsque l'on n'aura plus a declarer une instance dans herezh
};
/// @}  // end of group
#endif
	
	
		

