// FICHIER : QuadQCom.h
// CLASSE : QuadQCom

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   La classe QuadQCom permet de declarer des elements        *
 * Quadrangulaire membranne et de realiser                              *
 * le calcul du residu local et de la raideur locale pour une loi de    *
 * comportement donnee. La dimension de l'espace pour un tel element    *
 * est 2.                                                               *
 *                                                                      *
 * l'interpolation est quadratique complete 9 noeuds, le nombre de      *
 * point d'integration est  de 9.                                       *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
// -----------classe pour un calcul de mecanique---------



#ifndef QUADQCOM_H
#define QUADQCOM_H

#include "ParaGlob.h"
#include "ElemMeca.h"
#include "Met_abstraite.h"
#include "GeomQuadrangle.h"
#include "Noeud.h"
#include "UtilLecture.h"
#include "Tenseur.h"
#include "NevezTenseur.h"
#include "Deformation.h"
#include "QuadraMemb.h"
#include "FrontQuadQC.h"
#include "FrontSegQuad.h"


/// @addtogroup groupe_des_elements_finis
///  @{
///


class QuadQCom : public QuadraMemb
{
		
	public :
	
		// CONSTRUCTEURS :
		// Constructeur par defaut
		QuadQCom ();
		
		// Constructeur fonction d'une epaisseur et eventuellement d'un numero
		// d'identification 
		QuadQCom (double epaiss,int num_mail=0,int num_id=-3);
		
		// Constructeur fonction  d'un numero de maillage et d'identification  
		QuadQCom (int num_mail,int num_id);
		
		// Constructeur fonction d'une epaisseur, d'un numero d'identification,
		// du tableau de connexite des noeuds 
		QuadQCom (double epaiss,int num_mail,int num_id,const Tableau<Noeud *>& tab);
		
		// Constructeur de copie
		QuadQCom (const QuadQCom& quadra);
		
		
		// DESTRUCTEUR :
		~QuadQCom ();
				
		// création d'un élément de copie: utilisation de l'opérateur new et du constructeur de copie 
		// méthode virtuelle
        Element* Nevez_copie() const { Element * el= new QuadQCom(*this); return el;}; 

		// Surcharge de l'operateur = : realise l'egalite entre deux instances de QuadQCom
		QuadQCom& operator= (QuadQCom& quadra);
		
		// METHODES :
// 1) derivant des virtuelles pures
                         
        // renseignement d'un élément complet à partir d'un élément incomplet de même type
        // retourne les nouveaux noeuds construit à partir de l'interpolation incomplète.
        // dans le cas l'élément n'est pas concerné, retourne une liste vide
        // ramène également une liste de même dimension contenant les bornes en numéros de noeuds
        // entre lesquelles il faut définir les nouveaux numéros de noeuds si l'on veut conserver
        // une largeur de bande optimisée du même type
        // nbnt+1: est le premier numéro de noeud utilisable pour les nouveaux noeuds
        virtual list <Noeud *> Construct_from_imcomplet(const Element & elem,list <DeuxEntiers> & li_bornes,int nbnt);

        // affichage dans la sortie transmise, des variables duales "nom"
        // aux differents points d'integration
        // dans le cas ou nom est vide, affichage de "toute" les variables
        void AfficheVarDual(ofstream& sort, Tableau<string>& nom);                 	
        
// 2) derivant des virtuelles 
// 3) methodes propres a l'element
        
	protected :
        
        // adressage des frontières linéiques et surfacique
        // définit dans les classes dérivées, et utilisées pour la construction des frontières
        ElFrontiere* new_frontiere_lin(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontSegQuad(tab,ddelem)));};
        ElFrontiere* new_frontiere_surf(int ,Tableau <Noeud *> & tab, DdlElement& ddelem)
          { return ((ElFrontiere*) (new FrontQuadQC(tab,ddelem)));};
	  
      // VARIABLES PRIVEES :
        // place memoire commune a tous les elements TriaMembL1 
        static QuadraMemb::DonnComQuad * doCoQuadQCom;
        // idem mais pour les indicateurs qui servent pour l'initialisation
        static QuadraMemb::UneFois  uneFoisQCom;

	    class  NombresConstruireQuadQCom : public NombresConstruire
	     {  public: NombresConstruireQuadQCom(); 
	      };
	    static NombresConstruireQuadQCom nombre_V; // les nombres propres à l'élément
	     		
        // GESTION AUTOMATIQUE D'AJOUT D'ELEMENT DANS LE PROGRAMME
        //ajout de l'element dans la liste : listTypeElemen, geree par la class Element
        class ConsQuadQCom : public ConstrucElement
          { public :  ConsQuadQCom () 
               { NouvelleTypeElement nouv(QUADRANGLE,QUADRACOMPL,MECA_SOLIDE_DEFORMABLE,this);
                 if (ParaGlob::NiveauImpression() >= 4)
                    cout << "\n initialisation QuadQCom" << endl;
                 Element::listTypeElement.push_back(nouv);
                };
            Element * NouvelElement(int num_maill,int num) // un nouvel élément sans rien
               {Element * pt;
                pt  = new QuadQCom (num_maill,num) ;
                return pt;};	   
            // ramene true si la construction de l'element est possible en fonction
            // des variables globales actuelles: ex en fonction de la dimension	
            bool Element_possible() { if (ParaGlob::Dimension() >= 2) return true; else return false;};	
           }; 
        static ConsQuadQCom consQuadQCom;
};
/// @}  // end of group
#endif
	
	
		

