
// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

#include "Assemblage.h" 
 
// constructeur
Assemblage::Assemblage () :
  nb_casAssemb() 
   { // ne numéro d'assemblage doit être défini, on ne doit donc pas utiliser 
     // le constructeur par défaut
     cout << "\n ***************************************************** "
          << " \n erreur : le constructeur par défaut de la classe assemblage"
          << " ne doit pas être utilisé, car le numéro d'assemblage doit être "
          << " fixé !!" 
          << "\n Assemblage::Assemblage ()";
     Sortie(1);
    };

    // assemblage du second membre
     // vecglob : second membre global
     // vecloc : contribution de l'element au second membre
     // tab_ddl : ddl de l'element,tab_noeud : tableau de connection de l'element       
void Assemblage::AssemSM (Vecteur& vecglob,const Vecteur& vecloc,const DdlElement& tab_ddl,
                   const Tableau<Noeud *>&tab_noeud)
        { int  iloc, inoeud ; // differents indices
          // boucle sur les noeuds de l'element
          int inoeud_max = tab_noeud.Taille();
          for ( inoeud=1,iloc=1; inoeud<= inoeud_max; inoeud++)
            { Noeud& noe = *(tab_noeud(inoeud)); 
              // boucle sur les ddl du noeudElement
              int ni_nbddl = tab_ddl.NbDdl(inoeud); // nbddl du noeud de l'element
              for (int i = 1;i<= ni_nbddl ;i++,iloc++)
                { // on assemble que si le ddl est actif
                  Enum_ddl enu = tab_ddl(inoeud,i);
                  if (noe.En_service(enu) && noe.UneVariable(enu))
                   { int iglob = noe.Pointeur_assemblage(enu,nb_casAssemb.n);
                     #ifdef MISE_AU_POINT
                      if ( iglob == -1 )
                       { cout << "\nErreur : ddl " << enu
                              << " inexistant  pour le cas de charge " << nb_casAssemb.n
                              <<  '\n'
                              << "Assemblage::AssemSM ( un seul second membre\n";
                          Sortie(1);
                        };
                      #endif
                     vecglob(iglob) +=  vecloc(iloc); // assemblage
                   }
                 } 
            }
        };
     

     // assemblage de plusieurs  second membre en parrallèle
     // vecglob : les seconds membres globaux
     // vecloc : contributions de l'element aux seconds membres
     // tab_ddl : ddl de l'element,tab_noeud : tableau de noeud de l'element
     // a l'aide du tab_ddl et de tab_noeud on recupere le pointeur
     // d'assemblage de chaque noeud      
void Assemblage::AssemSM (Tableau<Vecteur>& vecglob,const Tableau<Vecteur*>& vecloc,
                           const DdlElement& tab_ddl,const Tableau<Noeud *>&tab_noeud)
        { int  iloc, inoeud ; // differents indices
          int nbSM = vecglob.Taille(); // nb de second membre
          
          // boucle sur les noeuds de l'element
          int inoeud_max = tab_noeud.Taille();
          for ( inoeud=1,iloc=1; inoeud<= inoeud_max; inoeud++)
            { Noeud& noe = *(tab_noeud(inoeud)); 
              // boucle sur les ddl du noeudElement
              int ni_nbddl = tab_ddl.NbDdl(inoeud);
              for (int i = 1;i<= ni_nbddl ;i++,iloc++)
                { // on assemble que si le ddl est actif
                  Enum_ddl enu = tab_ddl(inoeud,i);
                  if (noe.En_service(enu) && noe.UneVariable(enu))
                   for (int iSM = 1; iSM<= nbSM; iSM++)
                    { int iglob = noe.Pointeur_assemblage(enu,nb_casAssemb.n);
                    #ifdef MISE_AU_POINT	 	 
	                 if ( iglob == -1 )
			           { cout << "\nErreur : ddl " << enu 
			                  << " inexistant  pour le cas de charge " << nb_casAssemb.n
			                  <<  '\n'
			                  << "Assemblage::AssemSM ( plusieurs second membres\n";
			              Sortie(1);
			            };
                    #endif  
                      vecglob(iSM)(iglob) +=  (*vecloc(iSM))(iloc); // assemblage
                     }
                    
                 }
              }   
         };

       
    // assemblage des matrices  symetriques
    // seul la moitiee supérieure de la matrice est assemblée
    // matglob : matrice globale
    //  matloc : matrice locale ,
    // tab_ddl : ddl de l'element,tab_noeud : tqbleau de connection de l'element  
 void Assemblage::AssembMatSym (Mat_abstraite & matglob,const Mat_abstraite & matloc,
                const DdlElement& tab_ddl,const Tableau<Noeud *>&tab_noeud)
   {int i, iloc, inoeud ; // differents indices de lignes
    int j, jloc, jnoeud ; // differents indices de colonnes
         
//--------------- lignes -------------------------------------
    // boucle sur les noeuds de l'element
    int inoeud_max = tab_noeud.Taille(); 
    for ( inoeud=1,iloc=1; inoeud<= inoeud_max; inoeud++)
      {Noeud& noei = *(tab_noeud(inoeud)); 
       // boucle sur les ddl du noeudElement
       int ni_nbddl = tab_ddl.NbDdl(inoeud);
       for (i = 1;i<= ni_nbddl ;i++,iloc++)
        { // on assemble que si le ddl est actif
          Enum_ddl enui = tab_ddl(inoeud,i);
          if (noei.En_service(enui) && noei.UneVariable(enui))
          { int iglob = noei.Pointeur_assemblage(enui,nb_casAssemb.n);
            #ifdef MISE_AU_POINT
            if ( iglob == -1 )
             { cout << "\nErreur : ddl " << enui
                    << " inexistant  pour le cas de charge " << nb_casAssemb.n
                    <<  '\n'
                    << "Assemblage::AssembMatSym ( \n";
               Sortie(1);
             };
            #endif
      // ------------------ colonne -----------------------
            // boucle sur les noeuds de l'element
            int jnoeud_max = tab_noeud.Taille();
            for ( jnoeud=1,jloc=1; jnoeud<= jnoeud_max; jnoeud++)
              {Noeud& noej = *(tab_noeud(jnoeud)); 
               // boucle sur les ddl du noeudElement
               int nj_nbddl = tab_ddl.NbDdl(jnoeud);
               for (j = 1;j<= nj_nbddl ;j++,jloc++)
               { // on assemble que si le ddl est actif
                 Enum_ddl enuj = tab_ddl(jnoeud,j);
                 if (noej.En_service(enuj) && noej.UneVariable(enuj))
                 { int jglob = noej.Pointeur_assemblage(enuj,nb_casAssemb.n);
                   #ifdef MISE_AU_POINT
                   if ( jglob == -1 )
                    { cout << "\nErreur : ddl " << enuj
                           << " inexistant  pour le cas de charge " << nb_casAssemb.n
                           <<  '\n'
                           << "Assemblage::AssembMatSym ( \n";
                      Sortie(1);
                    };
                   #endif
                   if  (jglob >= iglob)
                    { 
//          --------------------------------- assemblage ----------------------                                   
//                           matglob(ipt+ii,jpt+jj) = matglob(ipt+ii,jpt+jj) +
//                                                    matloc(iloc,jloc); 
//   le test qui suit en assemblant que les parties non nulles, permet d'adresser
//   dans la matrice globale " que les éléments qui deviennent non nulles"
//   de cette manière, on prend en compte automatiquement une structure qui peut être creuse pour
//   la matrice locale (à condition toutefois que pour la matrice globale, les termes nulles n'ont pas 
//   été aloués, ou même une partie, ce qui permet un gain de place. 
                      if ((matloc(jloc,iloc) != 0.) || (matloc(iloc,jloc) != 0.))
                          matglob(iglob,jglob) += 0.5*( matloc(jloc,iloc)+
                                                    matloc(iloc,jloc)); 
							 // dans le cas d'une matrice qui contient toute la matrice, on renseigne la partie symétrique
							 // sinon, la ligne qui suit ne sert à rien, mais elle ne pose pas de pb non plus
							                  matglob(jglob,iglob) = matglob(iglob,jglob);										 
//          --------------------------------- fin assemblage ----------------------
                     }
                   } 
                 }
               } 
     // ------------------ fin colonne ------------------- 
           }      
         }
       }
//--------------- fin lignes -------------------------------------
  };
       
     // assemblage des matrices non symetriques
     // matglob : matrice globale
     //  matloc : matrice locale ,
     // tab_ddl : ddl de l'element,tab_noeud : tqbleau de connection de l'element  
 void Assemblage::AssembMatnonSym (Mat_abstraite & matglob,const Mat_abstraite & matloc,
                const DdlElement& tab_ddl,const Tableau<Noeud *>&tab_noeud)
   {int i, iloc, inoeud ; // differents indices de lignes
    int j, jloc, jnoeud ; // differents indices de colonnes
         
//--------------- lignes -------------------------------------
         
    // boucle sur les noeuds de l'element
    int inoeudmax = tab_noeud.Taille();
    for ( inoeud=1,iloc=1; inoeud<= inoeudmax; inoeud++)
      {Noeud& noei = *(tab_noeud(inoeud)); 
       // boucle sur les ddl du noeudElement
       int imax = tab_ddl.NbDdl(inoeud);
       for (i = 1;i<= imax ;i++,iloc++)
        { // on assemble que si le ddl est actif
          Enum_ddl enui = tab_ddl(inoeud,i);
          if (noei.En_service(enui) && noei.UneVariable(enui))
          { int iglob = noei.Pointeur_assemblage(enui,nb_casAssemb.n);
          #ifdef MISE_AU_POINT	 	 
	        if ( iglob == -1 )
			    { cout << "\nErreur : ddl " << enui
			           << " inexistant  pour le cas de charge " << nb_casAssemb.n
			           <<  '\n'
			           << "Assemblage::AssembMatnonSym ( \n";
			      Sortie(1);
			       };
          #endif  
      // ------------------ colonne -----------------------
            // boucle sur les noeuds de l'element
            int jnoeudmax = tab_noeud.Taille();
            for ( jnoeud=1,jloc=1; jnoeud<= jnoeudmax; jnoeud++)
              {Noeud& noej = *(tab_noeud(jnoeud)); 
               // boucle sur les ddl du noeudElement
               int jmax = tab_ddl.NbDdl(jnoeud);
               for (j = 1;j<= jmax ;j++,jloc++)
               { // on assemble que si le ddl est actif
                 Enum_ddl enuj = tab_ddl(jnoeud,j);
                 if (noej.En_service(enuj) && noej.UneVariable(enuj))
                  { int jglob = noej.Pointeur_assemblage(enuj,nb_casAssemb.n);
                #ifdef MISE_AU_POINT	 	 
	             if ( jglob == -1 )
			       { cout << "\nErreur : ddl " << enuj 
			              << " inexistant  pour le cas de charge " << nb_casAssemb.n
			              <<  '\n'
			              << "Assemblage::AssembMatnonSym ( \n";
			         Sortie(1);
			       };
                #endif  
//          --------------------------------- assemblage ----------------------                                   

//   le test qui suit en assemblant que les parties non nulles, permet d'adresser
//   dans la matrice globale " que les éléments qui deviennent non nulles"
//   de cette manière, on prend en compte automatiquement une structure qui peut être creuse pour
//   la matrice locale (à condition toutefois que pour la matrice globale, les termes nulles n'ont pas 
//   été aloués, ou même une partie, ce qui permet un gain de place. 
                      if (matloc(iloc,jloc) != 0.)
                        matglob(iglob,jglob) += matloc(iloc,jloc);

//          --------------------------------- fin assemblage ----------------------
                   }
                  } 
                }
     // ------------------ fin colonne ------------------- 
            }      
          }
        }
//--------------- fin lignes -------------------------------------
  };

// assemblage uniquement de la diagonale de matloc dans vecglob
// vecglob : second membre global
//  matloc : matrice locale ,
// tab_ddl : ddl de l'element,tab_noeud : tqbleau de noeud de l'element  
void Assemblage::AssembDiagonale (Vecteur& vecglob,const Mat_abstraite & matloc,
                       const DdlElement& tab_ddl,const Tableau<Noeud *>&tab_noeud)
        { int  iloc, inoeud ; // differents indices
          // boucle sur les noeuds de l'element
          int inoeud_max = tab_noeud.Taille();
          for ( inoeud=1,iloc=1; inoeud<= inoeud_max; inoeud++)
            { Noeud& noe = *(tab_noeud(inoeud)); 
              // boucle sur les ddl du noeudElement
              int ni_nbddl = tab_ddl.NbDdl(inoeud); // nbddl du noeud de l'element
              for (int i = 1;i<= ni_nbddl ;i++,iloc++)
                { // on assemble que si le ddl est actif
                  Enum_ddl enu = tab_ddl(inoeud,i);
                  if (noe.En_service(enu) && noe.UneVariable(enu))
                   { int iglob = noe.Pointeur_assemblage(enu,nb_casAssemb.n);
                     #ifdef MISE_AU_POINT
                     if ( iglob == -1 )
                      { cout << "\nErreur : ddl " << enu
                             << " inexistant  pour le cas de charge " << nb_casAssemb.n
                             <<  '\n'
                             << "Assemblage::AssembDiagonale( .... \n";
                         Sortie(1);
                      };
                     #endif
                     vecglob(iglob) +=  matloc(iloc,iloc); // assemblage
                   };
                };
            };
        };
							  

 // assemblage diagonale, = une majoration de la matrice des valeurs propre des matloc
 // vecglob : second membre global
 //  matloc : matrice locale ,
 // tab_ddl : ddl de l'element,tab_noeud : tqbleau de noeud de l'element  
void Assemblage::AssembDiagoMajorValPropre (Vecteur& vecglob,const Mat_abstraite & matloc,
                       const DdlElement& tab_ddl,const Tableau<Noeud *>&tab_noeud) 

   {int i, iloc, inoeud ; // differents indices de lignes
    int j, jloc, jnoeud ; // differents indices de colonnes
         
//--------------- lignes -------------------------------------
    // boucle sur les noeuds de l'element
    int inoeud_max = tab_noeud.Taille(); 
    for ( inoeud=1,iloc=1; inoeud<= inoeud_max; inoeud++)
      {Noeud& noei = *(tab_noeud(inoeud)); 
       // boucle sur les ddl du noeudElement
       int ni_nbddl = tab_ddl.NbDdl(inoeud);
       for (i = 1;i<= ni_nbddl ;i++,iloc++)
        { // on assemble que si le ddl est actif
          Enum_ddl enui = tab_ddl(inoeud,i);
          if (noei.En_service(enui) && noei.UneVariable(enui))
          { int iglob = noei.Pointeur_assemblage(enui,nb_casAssemb.n);
            #ifdef MISE_AU_POINT
            if ( iglob == -1 )
             { cout << "\nErreur : ddl " << enui
                    << " inexistant  pour le cas de charge " << nb_casAssemb.n
                    <<  '\n'
                    << "Assemblage::AssembDiagoMajorValPropre ( \n";
               Sortie(1);
             };
            #endif
      // ------------------ colonne -----------------------
            // boucle sur les noeuds de l'element
            int jnoeud_max = tab_noeud.Taille();
            for ( jnoeud=1,jloc=1; jnoeud<= jnoeud_max; jnoeud++)
             { Noeud& noej = *(tab_noeud(jnoeud));
               // boucle sur les ddl du noeudElement
               int nj_nbddl = tab_ddl.NbDdl(jnoeud);
               for (j = 1;j<= nj_nbddl ;j++,jloc++)
                { // on assemble que si le ddl est actif
                  Enum_ddl enuj = tab_ddl(jnoeud,j);
                  if (noej.En_service(enuj) && noej.UneVariable(enuj))
                  { int jglob = noej.Pointeur_assemblage(enuj,nb_casAssemb.n);
                    #ifdef MISE_AU_POINT
                    if ( jglob == -1 )
                     { cout << "\nErreur : ddl " << enuj
                            << " inexistant  pour le cas de charge " << nb_casAssemb.n
                            <<  '\n'
                            << "Assemblage::AssembDiagoMajorValPropre ( \n";
                       Sortie(1);
                     };
                    #endif
                     {
//          --------------------------------- assemblage ----------------------                                   
//                          on fait la somme de toutes les valeurs absolues de la ligne
//   le test qui suit en assemblant que les parties non nulles,  on prend en compte automatiquement une structure qui peut être creuse pour
//   la matrice locale. 
                       if (matloc(iloc,jloc) != 0.)
                         vecglob(iglob) +=  Dabs(matloc(iloc,jloc)); // assemblage
 //          --------------------------------- fin assemblage ----------------------
                     };
                  };
                };
             };
     // ------------------ fin colonne ------------------- 
          };
        };
      };
//--------------- fin lignes -------------------------------------
  };
       





