// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

// fichier : Fonc_scal_combinees_nD.h
// classe  : Fonc_scal_combinees_nD


/************************************************************************
 *     DATE:        01/06/2016                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Classe permettant le calcul d'une fonction scalaire nD   *
 *     correspondant à une combinaison d'autres fonctions scalaire nD.  *
 *     La combinaison est définie de manière littérale.                 *
 *     On utilise pour cela la bibliothèque MuParser                    *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
#ifndef FONCTION_SCAL_COMBINEES_ND_H
#define FONCTION_SCAL_COMBINEES_ND_H

#include "Fonction_nD.h"
#include "muParser.h"

/// @addtogroup Les_fonctions_nD
///  @{
///

/**
*
*     BUT:    Classe permettant le calcul d'une fonction scalaire nD
*     correspondant à une combinaison d'autres fonctions scalaire nD.
*     La combinaison est définie de manière littérale.
*     On utilise pour cela la bibliothèque MuParser
*
*
* \author    Gérard Rio
* \version   1.0
* \date      01/06/2016
* \brief       Classe permettant le calcul d'une fonction scalaire nD
*     correspondant à une combinaison d'autres fonctions scalaire nD.
*
*/

class Fonc_scal_combinees_nD  : public Fonction_nD
{
  public :

    // CONSTRUCTEURS :
    // par défaut
    Fonc_scal_combinees_nD(string nom = "");
    // constructeur avec plus d'info
    Fonc_scal_combinees_nD(const Tableau <string >& var, string nom = "");
    // de copie
    Fonc_scal_combinees_nD(const Fonc_scal_combinees_nD& Co);
    // de copie à partir d'une instance générale
    Fonc_scal_combinees_nD(const Fonction_nD& Coo);
    // DESTRUCTEUR :
    virtual ~Fonc_scal_combinees_nD() ;
    
    // METHODES PUBLIQUES :
    
    // --------- virtuelles ---------
 
    // Surcharge de l'operateur = : realise l'egalite de deux fonctions
    Fonction_nD& operator= (const Fonction_nD& elt);
 
    // affichage de la Fonction
    // = 0, 1 ou 2 (le plus précis)
    void Affiche(int niveau = 0) const ;

    // vérification que tout est ok, pres à l'emploi
    // ramène true si ok, false sinon
    bool Complet_Fonction(bool affichage = true)const;
 
 
    // ramène le nombre de composantes de la fonction
    int NbComposante() const {return 1;};

    // Lecture des donnees de la classe sur fichier
    // le nom passé en paramètre est le nom de la Fonction
    // s'il est vide c-a-d = "", la methode commence par lire le nom sinon
    // ce nom remplace le nom actuel
    void LectDonnParticulieres_Fonction_nD(const string& nom, UtilLecture * );
 
    // mise à jour des variables globales: en fonction de l'apparition de nouvelles variables
    // globales en cours de calcul
    virtual void Mise_a_jour_variables_globales();
 
    // établir le lien entre la Fonction et des Fonctions déjà existantes dont
    // on connait que le nom
    // permet ainsi de complèter la Fonction
    // 1) renseigne si la Fonction dépend d'autre Fonction ou non
    bool DependAutreFoncCourbes() const;
    // 2) retourne une liste de nom correspondant aux noms de courbes dont dépend *this
    list <string>& ListDependanceCourbes(list <string>& lico) const;
    // 3) retourne une liste de nom correspondant aux noms de Fonction dont dépend *this
    list <string>& ListDependanceFonctions(list <string>& lico) const;
    // 4) établit la connection entre la demande de *this et les Fonctions passées en paramètres
    void Lien_entre_fonc_courbe(list <Fonction_nD *>&  liptfonc,list <Courbe1D *>&  liptco);
    
    // def info fichier de commande
    void Info_commande_Fonctions_nD(UtilLecture & entreePrinc);

	   //----- lecture écriture de restart -----
	   // cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
	   void Lecture_base_info(ifstream& ent,const int cas);
    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	   void Ecriture_base_info(ofstream& sort,const int cas);
    // sortie du schemaXML: en fonction de enu 
    void SchemaXML_Fonctions_nD(ofstream& sort,const Enum_IO_XML enu);
     
     
  protected :
    
    // VARIABLES PROTEGEES :
//-------------------------------------------------------
    // on considère que l'utilisateur donne une liste d'arguments
    // qui sont dédiés uniquement à l'expression globale,
    // car les variables des fonctions internes sont définies
    // au moment de la définition (lecture) des lois internes
    // en lecture, il y a détection des variables globales
    //  celles-ci sont rangées dans le tableau enu_variables défini
    // dans Fonction_nD.h
    //
    // par contre au moment de l'appel de la fonction, toutes les variables
    // nécessaires sont stockés en entrée dans le tableau tab_fVal
    // NB: ne sont pas incluses les variables globales par ce qu'elles
    //    sont accessibles directement
    //
    // l'ordre des variables est le suivant
    // 1) les variables internes à chaque fonction membre, en suivant
    //    l'ordre d'apparition des fonctions
    // 2) les variables de la fonction générale
//-------------------------------------------------------
 
    // 1) la fonction globale
    string expression_fonction; // l'expression littérale de la fonction
    mu::Parser p; // définition d'une instance de parser
    // ici on fait une différence entre :
 
    // a) les variables spécifique à "expression_fonction"
    //    ces variables sont dans l'ordre:
    //      . l'identificateur de chaque fonction. Cette
    //        identificateur intervient comme une variable
    //        dans l'appel de l'expression
    //      . les variables patentées de passage
    //      . les variables patentées globales
    //
    // du coup:
    //    . les premières valeurs de tab_fVal_int
    //      correspondent aux retours des fonctions
    //    . les dernières aux valeurs des variables patentées
    Tableau <double> tab_fVal_int; // les variables internes dont
             // le nombre doit être égal à celui du tableau nom_variables_int
             // + les variables globales (enum et nom)
    Tableau <string > nom_variables_int; // nom fct int + variables de la fonction
                                          // "sans" les variables globales !!
 
    Tableau <Enum_GrandeurGlobale > enu_variables_globale_int; //tableau des énumérés
      // de variables globales correspondant à la fonction globale
      // éventuellement vide s'il ne sert pas
    Tableau <string > nom_variables_globales_int; //tableau des noms en string
      // de variables globales correspondant à la fonction globale
      // éventuellement vide s'il ne sert pas


    // b) l'ensemble des variables de la fonction, vu de l'extérieur
    //    qui agglomère dans l'ordre:
    //    . les variables de chaque fonction interne
    //    . + les variables internes à la fonction globale
    Tableau <double> tab_fVal; // l'ensemble des variables dont le nombre est
                   // égal à celui du tableau nom_variables
    Tableau <double> tab_ret; // le tableau final de retour
 

    // 2) les fonctions internes
    Tableau <Fonction_nD* > Fi; //  fonctions de base
 
    // variable intermédiaires pour la lecture en deux temps
    // servent également ensuite pour dire si Fi est interne ou pas
    // si elle est  interne -> nom_fonctioni="i_interne_i",
    // sinon ="e_externe_e" tant que le nom de la fonction n'est pas définie
    // après : LectDonnParticulieres_Fonction_nD :
    //   nom_fonctioni(i) = soit "i_interne_i" sinon le nom de la fonction, qu'elle
    //                      soit ou nom définie
    Tableau <string > nom_fonctioni;
    // dans le cas où on a affaire à une fonction interne
    // pour pouvoir l'utiliser dans l'expression littérale
    // on définit un identificateur associé: ident_interne(i)
    // s'il s'agit d'une fonction externe : ident_interne(i) = le nom de la fonction;
    Tableau <string> ident_interne;
    //--- maintenant les variables de passage
    Tableau < Tableau <double> > tab_fVal_Fi; // les variables internes pour chaque fonction interne
    Tableau < Tableau <double> > tab_ret_Fi; // le tableau de retour

 
    int indice_precedant; // position précédente
    
    // METHODES PROTEGEES :
    // dans le cas où les fonctions membres sont des  fonctions externes
    // fonction pour les définir
    // les fonctions sont défini en interne que si les fonctions argument sont elles même
    // des fonctions locales. c'est-à-dire si FFi(i)->NomFonction() ="_" alors on recrée une fonction
    // interne avec new pour Fi(i), sinon Fi(i)=FFi(i) et pas de création; 
    // dans le cas où FFi(i) est NULL on passe, pas de traitement pour ce pointeur
    void DefFonctionsMembres(Tableau <Fonction_nD* >& FFi);
 
    // mise à jour des variables de la classe mère en fonction des fonctions membres
    void Mise_a_jour_variables_nD();
 
    // initialisation de la fonction analytique
    void Init_fonction_analytique();

    // calcul  des valeurs de la fonction, retour d'un tableau de scalaires
    virtual Tableau <double> & Valeur_FnD_interne(Tableau <double >* xi);

    // calcul des valeurs de la fonction, dans le cas où les variables
    // sont des grandeurs globales
    virtual Tableau <double> & Valeur_pour_variables_globales_interne();
 

 };
 /// @}  // end of group

#endif  
