// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

// fichier : Poly_Lagrange.h
// classe  : Poly_Lagrange


/************************************************************************
 *     DATE:        30/03/2008                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Classe permettant le calcul d'un polynôme 1D             *
 *             à l'aide d'une interpolation de Lagrange                 *
 *             ainsi qu'un certain nombre d'information                 *
 *             supplémentaires telles que dérivées.                     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
#ifndef COURBEINTERPOLAGRANGE_1_D_H
#define COURBEINTERPOLAGRANGE_1_D_H

#include "Courbe1D.h"
#include "Tableau_T.h"
#include "Coordonnee2.h"
/// @addtogroup Les_courbes_1D
///  @{
///

/**
*
*     BUT:    Classe permettant le calcul d'un polynôme 1D
*             à l'aide d'une interpolation de Lagrange
*             ainsi qu'un certain nombre d'information
*             supplémentaires telles que dérivées.
*
*
* \author    Gérard Rio
* \version   1.0
* \date      19/01/2001
* \brief       Classe permettant le calcul d'un polynôme 1D à l'aide d'une interpolation de Lagrange 
*
*/

class Poly_Lagrange : public Courbe1D
{
  public :

    // CONSTRUCTEURS :
    // par défaut
    Poly_Lagrange(string nom = "");
    // fonction d'un tableau de points
    Poly_Lagrange(Tableau <Coordonnee2>& pt,string nom = "");
    
    // de copie
    Poly_Lagrange(const Poly_Lagrange& Co);
    Poly_Lagrange(const Courbe1D& Co);
    
    // DESTRUCTEUR :
    ~Poly_Lagrange();
    
    // METHODES PUBLIQUES :
    
    // --------- virtuelles ---------
    
    // affichage de la courbe
	void Affiche() const;
    // ramène true si ok, false sinon
    bool Complet_courbe()const;

    // Lecture des donnees de la classe sur fichier
    // le nom passé en paramètre est le nom de la courbe
    // s'il est vide c-a-d = "", la methode commence par lire le nom sinon
    // ce nom remplace le nom actuel
    void LectDonnParticulieres_courbes(const string& nom, UtilLecture * );
    
    // def info fichier de commande
    void Info_commande_Courbes1D(UtilLecture & entreePrinc);
        
    // ramène la valeur 
    
    double Valeur(double x)  ;

    // ramène la valeur et la dérivée en paramètre
    Courbe1D::ValDer Valeur_Et_derivee(double x)  ;
    
    // ramène la dérivée 
    double Derivee(double x)  ;
    
    // ramène la valeur et les dérivées première et seconde en paramètre
    Courbe1D::ValDer2 Valeur_Et_der12(double x);
        
    // ramène la dérivée seconde
    double Der_sec(double x);
    
    // ramène la valeur si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y
    // si supérieur au x maxi , ramène le valeur maximale possible de y
    Courbe1D::Valbool  Valeur_stricte(double x)  ;
    
    // ramène la valeur et la dérivée si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y et Y' correspondant
    // si supérieur au x maxi , ramène le valeur maximale possible de y et Y' correspondant
    Courbe1D::ValDerbool  Valeur_Et_derivee_stricte(double x)  ;
    
    // méthode pour changer le tableau de points associé
    void Change_tabPoints(Tableau <Coordonnee2>& pt);
    
	//----- lecture écriture de restart -----
	// cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
   	void Lecture_base_info(ifstream& ent,const int cas);
    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	   void Ecriture_base_info(ofstream& sort,const int cas);
    // sortie du schemaXML: en fonction de enu 
    virtual void SchemaXML_Courbes1D(ofstream& sort,const Enum_IO_XML enu) ;
     
    // ------------ méthodes particulières à la fonction poly-linéaire ---------
    // ramène le nombre de point de la polyligne
    int NombrePoint() {return points.Taille();};
    // ramène le x et y du point i
    const Coordonnee2& Pt_nbi(int i) {return points(i);};
    
  protected :  
    // VARIABLES PROTEGEES :
    Tableau <Coordonnee2> points; // les points de la courbe
    double der_init; // dérivée initiale
    double der_finale; // dérivée finale
    int indice_precedant; // position précédente
    

    // METHODES PROTEGEES :

 };
 /// @}  // end of group
#endif  
