// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

#include "SixpodeCos3phi.h"
#include "Sortie.h"
#include "ConstMath.h"
#include "MathUtil.h"
#include "ParaGlob.h"

    // CONSTRUCTEURS :
SixpodeCos3phi::SixpodeCos3phi(string nom) :
 Courbe1D(nom,COURBE_SIXPODECOS3PHI)
 ,xn(1.),gamma(0.)
  {};
    
    // de copie
SixpodeCos3phi::SixpodeCos3phi(const SixpodeCos3phi& Co) :
 Courbe1D(Co)
 ,xn(Co.xn),gamma(Co.gamma)
  {};
    // de copie à partir d'une instance générale
SixpodeCos3phi::SixpodeCos3phi(const Courbe1D& Coo) :
 Courbe1D(Coo)
  { if (Coo.Type_courbe() != COURBE_SIXPODECOS3PHI)
      { cout << "\n erreur dans le constructeur de copie pour une courbe COURBE_SIXPODECOS3PHI "
             << " à partir d'une instance générale ";
        cout << "\n SixpodeCos3phi::SixpodeCos3phi(const Courbe1D& Co) ";
           Sortie(1);          
          };
    // définition des données
    SixpodeCos3phi & Co = (SixpodeCos3phi&) Coo;
    xn = Co.xn;gamma = Co.gamma;
  };

    // DESTRUCTEUR :
SixpodeCos3phi::~SixpodeCos3phi() 
  {};
    
    // METHODES PUBLIQUES :
    
    // --------- virtuelles ---------
    
    // affichage de la courbe
void SixpodeCos3phi::Affiche() const 
  { cout << "\n SixpodeCos3phi: nom_ref= " << nom_ref;
    cout << "\n  n= " << xn << " gamma= " << gamma;
    cout << " f(x) = 1./(1.+gamma*(cos(3*x))^2)^n ";
   };

    // vérification que tout est ok, pres à l'emploi
    // ramène true si ok, false sinon
bool SixpodeCos3phi::Complet_courbe()const
 { bool ret = Complet_var(); // on regarde du coté de la classe mère tout d'abord
   // puis les variables propres
   if (!ret && (ParaGlob::NiveauImpression() >0))
      { cout << "\n ***** la courbe n'est pas complete ";
        this->Affiche();
        };
   return ret;
  } ;

    // Lecture des donnees de la classe sur fichier
    // le nom passé en paramètre est le nom de la courbe
    // s'il est vide c-a-d = "", la methode commence par lire le nom sinon
    // ce nom remplace le nom actuel
void SixpodeCos3phi::LectDonnParticulieres_courbes(const string& nom,UtilLecture * entreePrinc)
  {  if (nom == "")  { *(entreePrinc->entree) >> nom_ref;}
     else {nom_ref=nom;};
     entreePrinc->NouvelleDonnee();  // lecture d'une nouvelle ligne
     // on lit l'entête
     if(strstr(entreePrinc->tablcar,"n=")==0)
         { cout << "\n erreur en lecture du coefficient n ";
           cout << "\n SixpodeCos3phi::LectureDonneesParticulieres "
                << "(UtilLecture * entreePrinc) " << endl ;
           entreePrinc->MessageBuffer("**erreur1, SixpodeCos3phi::LectureDonneesParticulieres **");
           throw (UtilLecture::ErrNouvelleDonnee(-1));
           Sortie(1);          
          }
     if(strstr(entreePrinc->tablcar,"gamma=")==0)
         { cout << "\n erreur en lecture du coefficient gamma ";
           cout << "\n SixpodeCos3phi::LectureDonneesParticulieres "
                << "(UtilLecture * entreePrinc) " << endl ;
           entreePrinc->MessageBuffer("**erreur2, SixpodeCos3phi::LectureDonneesParticulieres **");
           throw (UtilLecture::ErrNouvelleDonnee(-1));
           Sortie(1);          
          }
     // lecture des coeffs
     string toto;
     *(entreePrinc->entree) >> toto >> gamma >> toto >> xn ; 
         
   };
    
// def info fichier de commande
void SixpodeCos3phi::Info_commande_Courbes1D(UtilLecture & entreePrinc) 
  {  
     ofstream & sort = *(entreePrinc.Commande_pointInfo()); // pour simplifier
     sort << "\n#............................................"
          << "\n# exemple de definition d'une courbe COURBE_SIXPODECOS3PHI "
          << " ( f(x) = 1./(1.+gamma*(cos(3*x))^2)^n = (1.+gamma*(cos(3*x))^2)^(-n) "
          << "\n      # def des coeff de la courbe COURBE_SIXPODECOS3PHI "
          << "\n      gamma= 0.9  n= 0.1 "
          << endl;
   };
        
    // ramène la valeur 
double SixpodeCos3phi::Valeur(double x)  
  { return pow((1.+gamma*(cos(3*x))*(cos(3*x))),-xn);
  };

    // ramène la valeur et la dérivée en paramètre
Courbe1D::ValDer SixpodeCos3phi::Valeur_Et_derivee(double x)  
  { ValDer ret;
    double X = 1.+gamma*(cos(3*x))*(cos(3*x));
    ret.valeur = pow(X,-xn);
    ret.derivee =  (6.*xn*gamma*sin(3.*x)*cos(3.*x)/X) * ret.valeur;
    return ret;
  };
    
    // ramène la dérivée 
double SixpodeCos3phi::Derivee(double x)  
  { double X = 1.+gamma*(cos(3*x))*(cos(3*x));
    return ( (6.*xn*gamma*sin(3.*x)*cos(3.*x)/X) * pow(X,-xn));
  };
        
    // ramène la valeur et les dérivées première et seconde en paramètre
Courbe1D::ValDer2 SixpodeCos3phi::Valeur_Et_der12(double x)
  { ValDer2 ret;
    double X = 1.+gamma*(cos(3*x))*(cos(3*x));
    ret.valeur = pow(X,-xn);
    ret.derivee =  (6.*xn*gamma*sin(3.*x)*cos(3.*x)/X) * ret.valeur;
    ret.der_sec = 18.*xn *ret.valeur / X * ( sin(6. * x) *(xn-1.)/(X*X)- cos(6.*x));
    return ret;
  };
        
    // ramène la dérivée seconde
double SixpodeCos3phi::Der_sec(double x)
  { double X = 1.+gamma*(cos(3*x))*(cos(3*x));
    double valeur = pow(X,-xn);
    return  18.*xn * valeur / X * ( sin(6. * x) *(xn-1.)/(X*X)- cos(6.*x));
  };
    
    // ramène la valeur si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y
    // si supérieur au x maxi , ramène le valeur maximale possible de y
Courbe1D::Valbool  SixpodeCos3phi::Valeur_stricte(double x)  
  { Valbool ret; // def de la valeur de retour
    // ici toujours ok
    ret.valeur = pow((1.+gamma*(cos(3*x))*(cos(3*x))),-xn);
    ret.dedans = true;
    return ret;
   }; 
    
    // ramène la valeur et la dérivée si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y et Y' correspondant
    // si supérieur au x maxi , ramène le valeur maximale possible de y et Y' correspondant
Courbe1D::ValDerbool  SixpodeCos3phi::Valeur_Et_derivee_stricte(double x)  
  { ValDerbool ret; // def de la valeur de retour
    double X = 1.+gamma*(cos(3*x))*(cos(3*x));
    ret.valeur = pow(X,-xn);
    ret.derivee =  (6.*xn*gamma*sin(3.*x)*cos(3.*x)/X) * ret.valeur;
    // ici toujours ok
    ret.dedans = true;
    return ret;
   }; 
    
	//----- lecture écriture de restart -----
	// cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
void SixpodeCos3phi::Lecture_base_info(ifstream& ent,const int cas)
  { // on n'a que des grandeurs constantes
    if (cas == 1)
      { string nom;
        // lecture et vérification de l'entête
        ent >> nom;
        if (nom != "SixpodeCos3phi")
         { cout << "\n erreur dans la vérification du type de courbe lue ";    
           cout << "\n SixpodeCos3phi::Lecture_base_info(... ";
           Sortie(1); 
           } 
        // lecture des infos
        ent >> nom >> gamma >> nom >> xn ;
       }
   };

    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
void SixpodeCos3phi::Ecriture_base_info(ofstream& sort,const int cas)
  { // on n'a que des grandeurs constantes
    if (cas == 1)
      { sort << " SixpodeCos3phi ";
        sort << "  gamma= " << gamma  << "  n= " << xn << " ";
       }
   };
         
// sortie du schemaXML: en fonction de enu 
void SixpodeCos3phi::SchemaXML_Courbes1D(ofstream& ,const Enum_IO_XML enu)
  {
	switch (enu)
	{ case XML_TYPE_GLOBAUX :
		{
		 break;
		}
		case XML_IO_POINT_INFO :
		{
		 break;
		}
		case XML_IO_POINT_BI :
		{
		 break;
		}
		case XML_IO_ELEMENT_FINI :
		{
		 break;
		}
	};		
  };
     
