/*! \file MathUtil.h
    \brief Utilitaires divers de calculs élémentaires
*/

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        23/01/97                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Utilitaires divers mathematiques                           *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef MATHUTIL_H
#define MATHUTIL_H

#include <iostream>
#include <math.h>
#include <stdlib.h>
#include "Sortie.h"
#include "Vecteur.h"
#include "ConstMath.h" 


/// minimum de deux doubles
double MiN(double a,double b);
/// maximum de deux doubles
double MaX(double a,double b);
/// minimum de deux entiers
int MiN(int a,int b);
/// maximum de deux entiers
int MaX(int a,int b);

/// minimum des valeurs absolues de deux doubles
double DabsMiN(double a,double b);
/// maximum des valeurs absolues de deux doubles
double DabsMaX(double a,double b);
/// maximum des valeurs absolues de 3 doubles
double DabsMaX(double a,double b,double c);
/// minimum des valeurs absolues de deux entiers
int DabsMiN(int a,int b);
/// maximum des valeurs absolues de deux entiers
int DabsMaX(int a,int b);

/// maximum des valeurs absolu d'un tableau de n réels
double DabsMaxiTab(double * tab, int n) ;
/// maximum des valeurs absolu d'un tableau de n entier relatifs
int DabsMaxiTab(int  * tab, int n) ;
/// valeur absolu d'un reel
inline double Dabs(double a) { return( a>=0. ? a : -a); }; 
inline double Abs(double a) { return( a>=0. ? a : -a); };
inline int Abs(int a) { return( a>=0. ? a : -a); };
/// ramène 1 ou -1 suivant que a est positif ou non
inline int Signe(double a) { return( a>=0. ? 1 : -1); };
/// idem mais ramène en double  1. ou -1. suivant que a est positif ou non
inline double DSigne(double a) { return( a>=0. ? 1. : -1.); };
/// ramène b ou -b suivant que a est positif ou non
inline double Signe(double a,double b) { return( a>=0. ? b : -b); };


/// calcul la difference entre deux nombres pondéré par la valeur du max des deux nombres + 1
/// c-a-d ramene vrai si |x-y| > epsilon*(1+max(|x|,|y|)) , faux sinon, epsimon = ConstMath::pasmalpetit
inline bool diffpetit(double x,double y) 
 { return((Dabs(x-y) <= ConstMath::petit * (1. + MaX(Dabs(x),Dabs(y)) )) ? false : true); };
/// idem mais avec un epsilon plus petit = ConstMath::trespetit
inline bool difftrespetit(double x,double y) 
 { return((Dabs(x-y) <= ConstMath::pasmalpetit * (1. + MaX(Dabs(x),Dabs(y)) )) ? false : true); };
/// un autre nom plus explicite
inline bool EgaleApeuPres(double x,double y) 
 { return((Dabs(x-y) <= ConstMath::pasmalpetit * (1. + MaX(Dabs(x),Dabs(y)) )) ? true : false); };
/// calcul la difference entre deux nombres (les deux premiers paramètres) pondéré par la valeur
/// du dernier paramètre z
/// c-a-d ramene vrai si |x-y| > epsilon*(|z|) , faux sinon, epsimon = ConstMath::pasmalpetit
inline bool diffpetit(double x,double y,double z) 
 { return((Dabs(x-y) <= ConstMath::petit * (Dabs(z))) ? false : true); };
/// idem mais avec un epsilon plus petit = ConstMath::trespetit
inline bool difftrespetit(double x,double y,double z) 
 { return((Dabs(x-y) <= ConstMath::pasmalpetit * ( Dabs(z))) ? false : true); };
/// idem mais avec un epsilon que l'on choisit en paramètre
inline bool diffpourcent(double x,double y,double z,const double epsilo) 
 { return((Dabs(x-y) <= epsilo * ( Dabs(z))) ? false : true); };
/// eleve au carre
template <class T1>
inline T1  Sqr(T1 a)
   { return (a * a);};
/// eleve à la puissance n
template <class T1>
inline T1  PUISSN(T1 a, const int n)
   { 
    #ifdef MISE_AU_POINT
      if ( n < 1) 
         { cout << "\n erreur, exposant négatif dans la fonction PUISSN(T1 a, const int n) ";
           Sortie(1);
          } 
    #endif
	int m=n;
    return ((m > 1) ? (a * PUISSN(a,--m))  : a);
    };
   
   
#ifndef MISE_AU_POINT
  #include "MathUtil.cc"
  #define  MATHUTIL_H_deja_inclus
#endif

        
#endif  
