

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        6/12/2007                                           *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   La classe Hart_Smith permet de calculer la contrainte     *
 *            et ses derivees pour une loi isotrope hyper élastique     *
 *            de type Hart Smith  en 3D. Ici on  considère              *
 *            la variation de volume.                                   *
 *     Il s'agit d'une classe derivee de la classe Loi_comp_abstraite.  *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     * 
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef HART_SMITH_3D_H
#define HART_SMITH_3D_H




#include "Loi_comp_abstraite.h"
#include "Courbe1D.h"
#include "MathUtil.h"
#include "Hyper_W_gene_3D.h"
#include "CourbePolyLineaire1D.h"


/// @addtogroup Les_lois_hyperelastiques
///  @{
///


class Hart_Smith3D : public Hyper_W_gene_3D
{


	public :
	
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		Hart_Smith3D ();
		
		
		// Constructeur de copie
		Hart_Smith3D (const Hart_Smith3D& loi) ;
		
		// DESTRUCTEUR :
		
		~Hart_Smith3D ();
		
	    // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();
	 
	 //----- lecture écriture de restart -----
	 // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info_loi(ofstream& sort,const int cas);
        
  // calcul d'un module d'young équivalent à la loi, 
  // c'est sans doute complètement débile mais c'est pour pouvoir avancer !!
  // Annexe B, formule B12
  double Module_young_equivalent(Enum_dure temps,const Deformation & ,SaveResul *)  {return 6.* C1;};
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const {return ConstMath::tresgrand;};
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new Hart_Smith3D(*this)); };
       
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);
		  	       
	protected :
	   // donnée de la loi
	   double C1,C2,C3,K; // 4 coeffs
	   Courbe1D* C1_temperature; // courbe éventuelle d'évolution de C10 en fonction de la température
	   Courbe1D* C2_temperature; // courbe éventuelle d'évolution de C10 en fonction de la température
	   Courbe1D* C3_temperature; // courbe éventuelle d'évolution de C01 en fonction de la température
	   Courbe1D* K_temperature; // courbe éventuelle d'évolution de K en fonction de la température
	   int type_pot_vol; // indique le type de potentiel volumique, par défaut 2
    bool avec_courbure; // indique s'il y a un potentiel de courbure ou non
    double a_courbure; // para a utilisé que dans le cas avec courbure
    double r_courbure; // para r utilisé que dans le cas avec courbure
	   Courbe1D* a_temperature; // courbe éventuelle d'évolution de a en fonction de la température
	   Courbe1D* r_temperature; // courbe éventuelle d'évolution de r en fonction de la température
	   
	   // définition de la fonction énergie
	   CourbePolyLineaire1D int_J1; // intégrale de la partie relative à J1
	   
	   
	   double W_d,W_v; // le potentiel: partie déviatorique, partie sphérique
	   Vecteur W_r; // dérivées premières du potentiel par rapport aux J_r
	   double W_d_J1,W_d_J2; // dérivées premières du potentiel déviatoire par rapport aux J_1 et J_2
	   double W_d_J1_2,W_d_J1_J2,W_d_J2_2; // dérivées secondes
	   double W_v_J3,W_v_J3J3; // dérivées premières et seconde du potentiel volumique / J3
	   Tableau2 <double> W_rs; // dérivées secondes du potentiel par rapport aux J_r
    // cas éventuel de potentiel additionel de raidissement: variables intermédiaires de passage
    double W_c,W_c_J1,W_c_J3,W_c_J1_2,W_c_J3_2,W_c_J1_J3; // potentiel et dérivées 1 et 2

	   
    // codage des METHODES VIRTUELLES  protegees:
    // calcul des contraintes a t+dt
    void Calcul_SigmaHH (TenseurHH& sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl,
             TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H,TenseurBB & epsBB_,
             TenseurBB & delta_epsBB,
             TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_,
             double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	        ,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	        ,const Met_abstraite::Expli_t_tdt& ex);
        
    // calcul des variations des contraintes a t+dt
    void Calcul_DsigmaHH_tdt (TenseurHH& sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
            ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t,
            BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt,
            TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB,
            TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt,
		          Tableau <TenseurBB *>& d_gijBB_tdt,
		  	       Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien,
		  	       Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	       ,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	       ,const Met_abstraite::Impli& ex);

    // calcul des contraintes et ses variations  par rapport aux déformations a t+dt
    // en_base_orthonormee:  le tenseur de contrainte en entrée est  en orthonormee
    //                  le tenseur de déformation et son incrémentsont également en orthonormees
    //                 si = false: les bases transmises sont utilisées
    // ex: contient les éléments de métrique relativement au paramétrage matériel = X_(0)^a
    virtual void Calcul_dsigma_deps (bool en_base_orthonormee, TenseurHH & sigHH_t,TenseurBB& DepsBB
            ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	       ,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
		  	       ,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	       ,const Met_abstraite::Umat_cont& ex) ;

    // calcul de grandeurs de travail aux points d'intégration via la def
    // fonction surchargée dans les classes dérivée si besoin est
    virtual void CalculGrandeurTravail
         (const PtIntegMecaInterne&
         ,const Deformation & ,Enum_dure,const ThermoDonnee&
         ,const Met_abstraite::Impli* ex_impli
         ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
         ,const Met_abstraite::Umat_cont* ex_umat
         ,const List_io<Ddl_etendu>* exclure_dd_etend
         ,const List_io<const TypeQuelconque *>* exclure_Q) {};

	private:
	   // calcul du potentiel et de ses dérivées premières / aux invariants J_r
	   void  Potentiel_et_var(double & module_compressibilite);
	   // calcul du potentiel et de ses dérivées premières et secondes / aux invariants J_r
	   void Potentiel_et_var2(double & module_compressibilite);
       // définition de la courbe représentant l'évolution de l'énergie en fonction de J1
       void Calcul_courbe_evolW_J1();
	   // calcul de la dérivée numérique de la contrainte
	   void Cal_dsigma_deps_num (const TenseurBB & gijBB_0,const TenseurHH & gijHH_0
		                       ,const TenseurBB & gijBB_tdt,const TenseurHH & gijHH_tdt
		                       ,const double& jacobien_0,const double& jacobien
		                       ,Tenseur3HHHH& dSigdepsHHHH);
	   // calcul de la contrainte avec le minimum de variable de passage, utilisé pour le numérique
	   void Cal_sigma_pour_num(const TenseurBB & gijBB_0,const TenseurHH & gijHH_0
		                       ,const TenseurBB & gijBB_tdt,const TenseurHH & gijHH_tdt
		                       ,const double& jacobien_0,const double& jacobien,TenseurHH & sigHH_);	  		  	
	   // idem avec la variation 
	   void Cal_sigmaEtDer_pour_num(const TenseurBB & gijBB_0,const TenseurHH & gijHH_0
		                       ,const TenseurBB & gijBB_tdt,const TenseurHH & gijHH_tdt
		                       ,const double& jacobien_0,const double& jacobien
		                       ,TenseurHH & sigHH_,Tenseur3HHHH& dSigdepsHHHH);		  		  	
};
/// @}  // end of group


#endif		

		

