

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        06/03/2023                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
  ************************************************************************
 *     BUT:   La classe Hyper_externe_W permet de calculer la contrainte*
 *            et ses derivees pour une loi isotrope hyper élastique     *
 *            de type extension de la loi de Mooney Rivlin  en 3D       *
 *            à l'aide de l'utilisation d'une fonction externe nD.      *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef HYPER_EXTERNE_W_H
#define HYPER_EXTERNE_W_H




#include "Loi_comp_abstraite.h"
#include "Courbe1D.h"
#include "MathUtil.h"
#include "Hyper_W_gene_3D.h"


/// @addtogroup Les_lois_hyperelastiques
///  @{
///


class Hyper_externe_W : public Hyper_W_gene_3D
{


	public :
	
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		Hyper_externe_W ();
		
		
		// Constructeur de copie
		Hyper_externe_W (const Hyper_externe_W& loi) ;
		
		// DESTRUCTEUR :
		
		~Hyper_externe_W ();
  
// initialise les donnees particulieres a l'elements
// de matiere traite ( c-a-dire au pt calcule)
// Il y a creation d'une instance de SaveResul particuliere
// a la loi concernee
// la SaveResul classe est remplie par les instances heritantes
// le pointeur de SaveResul est sauvegarde au niveau de l'element
// c'a-d que les info particulieres au point considere sont stocke
// au niveau de l'element et non de la loi.
class SaveResulHyper_externe_W: public SaveResulHyper_W_gene_3D
  { public :

      SaveResulHyper_externe_W(); // constructeur par défaut
      SaveResulHyper_externe_W(int sortie_post); // avec init ou pas
      SaveResulHyper_externe_W(const SaveResulHyper_externe_W& sav); // de copie
      virtual ~SaveResulHyper_externe_W() ; // destructeur
      
      // définition d'une nouvelle instance identique
      // appelle du constructeur via new
      SaveResul * Nevez_SaveResul() const{return (new SaveResulHyper_externe_W(*this));};

      // affectation
      virtual SaveResul & operator = ( const SaveResul & a);

      //============= lecture écriture dans base info ==========
         // cas donne le niveau de la récupération
         // = 1 : on récupère tout
         // = 2 : on récupère uniquement les données variables (supposées comme telles)
      void Lecture_base_info (ifstream& ent,const int cas);

         // cas donne le niveau de sauvegarde
         // = 1 : on sauvegarde tout
         // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
      void Ecriture_base_info(ofstream& sort,const int cas);

      // mise à jour des informations transitoires
      void TdtversT();
      void TversTdt();

      // affichage à l'écran des infos
      void Affiche() const;

      //changement de base de toutes les grandeurs internes tensorielles stockées
      // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
      // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
      // gpH(i) = gamma(i,j) * gH(j)
      virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma) {};

      // procedure permettant de completer éventuellement les données particulières
      // de la loi stockées
      // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
      // completer est appelé apres sa creation avec les donnees du bloc transmis
      // peut etre appeler plusieurs fois
      virtual SaveResul* Complete_SaveResul(const BlocGen & bloc
                                   , const Tableau <Coordonnee>& tab_coor
                                   ,const Loi_comp_abstraite* loi) {return NULL;};


      //-------------------------------------------------------------------
      // données
      //-------------------------------------------------------------------
      // le potentiel total est stocké dans la classe mère: SaveResulHyper_W_gene_3D
      double module_compressibilite,module_compressibilite_t;
      // stockage éventuel des valeurs de la fct nD
      Vecteur * W_poten,* W_poten_t;
      // stockage éventuel des invariants
      Vecteur * inv_B, * inv_B_t;
      Vecteur * inv_J, * inv_J_t;

    };

  // def d'une instance de données spécifiques, et initialisation
  SaveResul * New_et_Initialise()
   { return (new SaveResulHyper_externe_W(this->sortie_post));};

  // affichage des donnees particulieres a l'elements
  // de matiere traite ( c-a-dire au pt calcule)
  void AfficheDataSpecif(ofstream& ,SaveResul * a) const
   { SaveResulHyper_externe_W& sav = *((SaveResulHyper_externe_W*) a);
     sav.Affiche();
   };

	    // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();
	 
	 //----- lecture écriture de restart -----
	 // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info_loi(ofstream& sort,const int cas);
        
   // récupération des grandeurs particulière (hors ddl )
   // correspondant à liTQ
   // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
   void Grandeur_particuliere
         (bool absolue,List_io<TypeQuelconque>& ,Loi_comp_abstraite::SaveResul *  ,list<int>& decal) const;
   // récupération de la liste de tous les grandeurs particulières
   // ces grandeurs sont ajoutées à la liste passées en paramètres
   // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
   void ListeGrandeurs_particulieres(bool absolue,List_io<TypeQuelconque>& ) const;
            
   // calcul d'un module d'young équivalent à la loi, ceci pour un
   // chargement nul
   double Module_young_equivalent(Enum_dure temps,const Deformation & ,SaveResul * saveResul);
  
   // récupération d'un module de compressibilité équivalent à la loi, ceci pour un chargement nul
   // il s'agit ici de la relation -pression = sigma_trace/3. = module de compressibilité * I_eps
   // >>> en fait ici il s'agit du dernier module tangent calculé !!
   double Module_compressibilite_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul) ;

   // récupération de la variation relative d'épaisseur calculée: h/h0
   //  cette variation n'est utile que pour des lois en contraintes planes
   // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
   // - pour les lois 2D def planes: retour de 0
   // les infos nécessaires à la récupération , sont stockées dans saveResul
   // qui est le conteneur spécifique au point où a été calculé la loi
   virtual double HsurH0(SaveResul * saveResul) const {return ConstMath::tresgrand;};
  

  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new Hyper_externe_W(*this)); };
       
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);
		  	       
  // calcul de grandeurs de travail aux points d'intégration via la def et autres
  // ici éventuellement permet de récupérer la compressibilité
  virtual void CalculGrandeurTravail
                (const PtIntegMecaInterne& ,const Deformation &
                 ,Enum_dure,const ThermoDonnee&
                 ,const Met_abstraite::Impli* ex_impli
                 ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
                 ,const Met_abstraite::Umat_cont* ex_umat
                 ,const List_io<Ddl_etendu>* exclure_dd_etend
                 ,const List_io<const TypeQuelconque *>* exclure_Q
                 ) {};

            
	protected :
	   // données de la loi
    // la fonction potentielle
    Fonction_nD* W_potentiel;
 
	   double W_d,W_v; // le potentiel: partie déviatorique, partie sphérique
	   double W_d_J1,W_d_J2; // dérivées premières du potentiel déviatoire par rapport aux J_1 et J_2
	   double W_d_J1_2,W_d_J1_J2,W_d_J2_2; // dérivées secondes
	   double W_v_J3,W_v_J3J3; // dérivées premières et seconde du potentiel volumique / J3
    
    // variables de travail
    List_io<TypeQuelconque> invar; // contient INVAR_B,INVAR_J
    List_io<Ddl_enum_etendu> exclure_ddenum; // pour exclure le calcul des invariants
                                   // qui est en fait calculé localement

	   
    // --- codage des METHODES VIRTUELLES  protegees:
 // calcul des contraintes a t+dt
        // calcul des contraintes 
 void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
     ,TenseurBB & delta_epsBB_
     ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
     ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Expli_t_tdt& ex);

       // calcul des contraintes et de ses variations  a t+dt
 void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
     ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
     ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
     ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
     ,Tableau <TenseurBB *>& d_gijBB_tdt
		  	,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
		  	,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Impli& ex);
		  	
        // calcul des contraintes et ses variations  par rapport aux déformations a t+dt
        // en_base_orthonormee:  le tenseur de contrainte en entrée est  en orthonormee
        //                  le tenseur de déformation et son incrémentsont également en orthonormees
        //                 si = false: les bases transmises sont utilisées
        // ex: contient les éléments de métrique relativement au paramétrage matériel = X_(0)^a
 void Calcul_dsigma_deps (bool en_base_orthonormee, TenseurHH & sigHH_t,TenseurBB& DepsBB
     ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Umat_cont& ex) ; //= 0;

	private:

	   // calcul du potentiel et de ses dérivées premières et secondes / aux invariants J_r
	   void Potentiel_et_var2(const Met_abstraite::Impli* ex_impli
                           ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
                           ,const Met_abstraite::Umat_cont* ex_umat);
	   // calcul de la dérivée numérique de la contrainte
	   void Cal_dsigma_deps_num (const TenseurBB & gijBB_0,const TenseurHH & gijHH_0
		                       ,const TenseurBB & gijBB_tdt,const TenseurHH & gijHH_tdt
		                       ,const double& jacobien_0,const double& jacobien
		                       ,Tenseur3HHHH& dSigdepsHHHH
                         ,const Met_abstraite::Impli& ex );
	   // calcul de la contrainte avec le minimum de variable de passage, utilisé pour le numérique
	   void Cal_sigma_pour_num(const TenseurBB & gijBB_0,const TenseurHH & gijHH_0
		                       ,const TenseurBB & gijBB_tdt,const TenseurHH & gijHH_tdt
		                       ,const double& jacobien_0,const double& jacobien,TenseurHH & sigHH_
                         ,const Met_abstraite::Impli& ex);
	   // idem avec la variation 
	   void Cal_sigmaEtDer_pour_num(const TenseurBB & gijBB_0,const TenseurHH & gijHH_0
		                       ,const TenseurBB & gijBB_tdt,const TenseurHH & gijHH_tdt
		                       ,const double& jacobien_0,const double& jacobien
		                       ,TenseurHH & sigHH_,Tenseur3HHHH& dSigdepsHHHH
                         ,const Met_abstraite::Impli& ex);
		                       

};
/// @}  // end of group


#endif		

		

