// FICHIER : Hypo_hooke3D.h
// CLASSE : Hypo_hooke3D


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        28/06/2004                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   La classe Hypo_hooke3D definit une loi 3D hypo_élastique  *
 *            qui sous forme intégrée peut-être équivalente à hooke.    *
 *            viscosité non linéaire éventuelle.                        *
 *            On a donc :                                               *
 *                 S_point = mu  D_b                                    *
 *                 I_point_sigma = K  I_D_b                             *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef HYPO_HOOKE_3D_H
#define HYPO_HOOKE_3D_H


#include "Loi_comp_abstraite.h"



/// @addtogroup Les_lois_hypoelastiques
///  @{
///

class Hypo_hooke3D : public Loi_comp_abstraite
{


	public :
	
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		Hypo_hooke3D ();
		
		
		// Constructeur de copie
		Hypo_hooke3D (const Hypo_hooke3D& loi) ;
		
		// DESTRUCTEUR :
		
		~Hypo_hooke3D ();
		
  // initialise les donnees particulieres a l'elements
  // de matiere traite ( c-a-dire au pt calcule)
  // Il y a creation d'une instance de SaveResul particuliere
  // a la loi concernee
  // la SaveResul classe est remplie par les instances heritantes
  // le pointeur de SaveResul est sauvegarde au niveau de l'element
  // c'a-d que les info particulieres au point considere sont stocke
  // au niveau de l'element et non de la loi.
  class SaveResulLoi_Hypo3D: public SaveResul
    { public :

      SaveResulLoi_Hypo3D(); // constructeur par défaut (a ne pas utiliser)
      // le constructeur courant
      SaveResulLoi_Hypo3D(SaveResul* l_des_SaveResul);
      // de copie
      SaveResulLoi_Hypo3D(const SaveResulLoi_Hypo3D& sav): // de copie
        Kc(sav.Kc),Kc_t(sav.Kc_t),mu(sav.mu),mu_t(sav.mu_t)
        ,eps_cumulBB(sav.eps_cumulBB),eps_cumulBB_t(sav.eps_cumulBB_t)
        {};
      virtual ~SaveResulLoi_Hypo3D() {}; // destructeur
      // définition d'une nouvelle instance identique
      // appelle du constructeur via new
      SaveResul * Nevez_SaveResul() const{return (new SaveResulLoi_Hypo3D(*this));};
      // affectation
      virtual SaveResul & operator = ( const SaveResul & a)
        { SaveResulLoi_Hypo3D& sav = *((SaveResulLoi_Hypo3D*) &a);
          Kc=sav.Kc;Kc_t=sav.Kc_t;mu=sav.mu;mu_t=sav.mu_t;
          eps_cumulBB=sav.eps_cumulBB;eps_cumulBB_t=sav.eps_cumulBB_t;
          return *this;
        };
      //============= lecture écriture dans base info ==========
         // cas donne le niveau de la récupération
         // = 1 : on récupère tout
         // = 2 : on récupère uniquement les données variables (supposées comme telles)
      void Lecture_base_info (ifstream& ent,const int cas);

         // cas donne le niveau de sauvegarde
         // = 1 : on sauvegarde tout
         // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
      void Ecriture_base_info(ofstream& sort,const int cas);
        
      // mise à jour des informations transitoires
      void TdtversT()
       {Kc_t = Kc; mu_t=mu; eps_cumulBB_t = eps_cumulBB;};
      void TversTdt()
       {Kc = Kc_t; mu=mu_t;eps_cumulBB = eps_cumulBB_t;};
      
      // affichage à l'écran des infos
      void Affiche() const
       { cout <<"\n  Kc= "<< Kc << " mu= " << mu << " eps_cumulBB= " << eps_cumulBB
              << " ";
       };
      
      //changement de base de toutes les grandeurs internes tensorielles stockées
      // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
      // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
      // gpH(i) = gamma(i,j) * gH(j)
      virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma);
      
      // procedure permettant de completer éventuellement les données particulières
      // de la loi stockées
      // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
      // completer est appelé apres sa creation avec les donnees du bloc transmis
      // peut etre appeler plusieurs fois
      virtual SaveResul* Complete_SaveResul(const BlocGen & bloc
                                   , const Tableau <Coordonnee>& tab_coor
                                   ,const Loi_comp_abstraite* loi) {return NULL;};


      //-------------------------------------------------------------------
      // données
      //-------------------------------------------------------------------
      double Kc,Kc_t; // les paramètres matériaux réellement utilisés
      double mu,mu_t;
      Tenseur3BB eps_cumulBB,eps_cumulBB_t; // déformation cumulée associée à la loi

    };
  // def d'une instance de données spécifiques, et initialisation
  SaveResul * New_et_Initialise();

  // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D
                                    ,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();
	 
  //----- lecture écriture de restart -----
  // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
  void Lecture_base_info_loi(ifstream& ent,const int cas
                             ,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                             ,LesFonctions_nD& lesFonctionsnD);
     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
  void Ecriture_base_info_loi(ofstream& sort,const int cas);

  // récupération des grandeurs particulière (hors ddl )
  // correspondant à liTQ
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  void Grandeur_particuliere
        (bool absolue,List_io<TypeQuelconque>& ,Loi_comp_abstraite::SaveResul *  ,list<int>& decal) const;
  // récupération de la liste de tous les grandeurs particulières
  // ces grandeurs sont ajoutées à la liste passées en paramètres
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  void ListeGrandeurs_particulieres(bool absolue,List_io<TypeQuelconque>& ) const;
           
  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure temps,const Deformation & ,SaveResul * saveResul);
 
  // récupération d'un module de compressibilité équivalent à la loi, ceci pour un chargement nul
  // il s'agit ici de la relation -pression = sigma_trace/3. = module de compressibilité * I_eps
  // >>> en fait ici il s'agit du dernier module tangent calculé !!
  double Module_compressibilite_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);

  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const {return ConstMath::tresgrand;};
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new Hypo_hooke3D(*this)); };
	
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);  
  // calcul de grandeurs de travail aux points d'intégration via la def et autres
  // ici permet de récupérer la compressibilité
  // fonction surchargée dans les classes dérivée si besoin est
  virtual void CalculGrandeurTravail
            (const PtIntegMecaInterne& ptintmeca
             ,const Deformation & def,Enum_dure temps,const ThermoDonnee& dTP
             ,const Met_abstraite::Impli* ex_impli
             ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
             ,const Met_abstraite::Umat_cont* ex_umat
             ,const List_io<Ddl_etendu>* exclure_dd_etend
             ,const List_io<const TypeQuelconque *>* exclure_Q
            )
    {if (compress_thermophysique) Kc = 3./dTP.Compressibilite(); };
	
	protected :
	   // donnée de la loi
	   double mu; // coef de proportionalité entre S_point et D_barre
	   Courbe1D* mu_temperature; // courbe éventuelle d'évolution de mu en fonction de la température
	   Courbe1D* mu_IIeps; // courbe éventuelle d'évolution de mu en fonction du deuxième invariant d'epsilon
    Fonction_nD* mu_nD; // fonction nD éventuelle pour mu
    
	   double Kc; // coefficient de compressibilité instantané
	   Courbe1D* Kc_temperature; // courbe éventuelle d'évolution de Kc en fonction de la température
	   Courbe1D* Kc_IIeps; // courbe éventuelle d'évolution de Kc en fonction du deuxième invariant d'epsilon
    Fonction_nD * Kc_nD; // fonction nD éventuelle pour Kc

    bool compress_thermophysique; // indique si oui ou non la compressibilité est calculée par une loi
                         // thermophysique et donc
                         // récupéré par la fonction "CalculGrandeurTravail"
	   int type_derive; // type de dérivée objective utilisée pour sigma
        // -1: dérivée de Jauman (par défaut)
        // 0 : dérivée deux fois covariante
        // 1 : dérivée deux fois contravariante
	   short int cas_calcul; // indique le choix entre différents types de calcul possible
	                         // = 0 : calcul normal
	                         // = 1 : calcul seulement déviatorique (la partie sphérique est mise à zéro)
	                         // = 2 : calcul seulement sphérique (la partie déviatorique est mise à zéro)

    // on introduit un certain nombre de tenseur du quatrième ordre, qui vont nous servir pour
    // Calcul_dsigma_deps, dans le cas où on n'est pas en orthonormee
    Tenseur3HHHH  I_x_I_HHHH,I_xbarre_I_HHHH,I_x_eps_HHHH,I_x_D_HHHH,I_xbarre_D_HHHH,d_sig_t_HHHH;
    Tenseur3HHHH  d_spherique_sig_t_HHHH;
	   
       // codage des METHODES VIRTUELLES  protegees:
 // calcul des contraintes a t+dt
        // calcul des contraintes 
 void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
     ,TenseurBB & delta_epsBB_
     ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
     ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Expli_t_tdt& ex);

       // calcul des contraintes et de ses variations  a t+dt
 void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
     ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
     ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
     ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
     ,Tableau <TenseurBB *>& d_gijBB_tdt
		  	,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
		  	,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Impli& ex);
		  	
        // calcul des contraintes et ses variations  par rapport aux déformations a t+dt
        // en_base_orthonormee:  le tenseur de contrainte en entrée est  en orthonormee
        //                  le tenseur de déformation et son incrémentsont également en orthonormees
        //                 si = false: les bases transmises sont utilisées
        // ex: contient les éléments de métrique relativement au paramétrage matériel = X_(0)^a
 void Calcul_dsigma_deps (bool en_base_orthonormee, TenseurHH & sigHH_t,TenseurBB& DepsBB
     ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Umat_cont& ex) ; //= 0;


};
/// @}  // end of group


#endif		

		

