// FICHIER : Projection_anisotrope_3D.h
// CLASSE : Projection_anisotrope_3D


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


/************************************************************************
 *     DATE:        11/06/2019                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:                                                             *
 *  La classe Projection_anisotrope_3D permet de calculer la contrainte *
 *   et ses variations pour une loi initialement isotrope 3D puis       *
 *   ensuite projetée de manière anisotrope en 3D dans l'espace réel.   *
 *   Plusieurs projections sont envisagées.                             *
 *   1) La première implantée concerne une homotétie des composantes de *
 *   contraintes par rapport à un repère particulier orthonormé ce qui  *
 *   conduit initialement à une loi orthotrope. L'opérateur d'homotétie *
 *   est représenté par un tenseur du quatrième ordre, diagonal dans    *
 *   son repère initiale de définition. Ce repère est donc principal.   *
 *   Ensuite ce tenseur est convecté matériellement pour prendre en     *
 *   compte les déplacements solides et les variations d'angles entre   *
 *   les directions principales du tenseur du 4ième ordre.              *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
/** @defgroup Les_lois_anisotropes
*
*     BUT:   groupe des lois anisotropes
*
*
* \author    Gérard Rio
* \version   1.0
* \date       11/06/2019
* \brief       Définition des  lois anisotropes
*
*/


#ifndef PROJECTION_ANISOTROPE_3D_H
#define PROJECTION_ANISOTROPE_3D_H
#include "Base3D3.h"
#include "Coordonnee2.h"
#include "Ponderation.h"


#include "Loi_comp_abstraite.h"
#include "Enum_proj_aniso.h"
#include "TenseurQ3gene.h"

/// @addtogroup Les_lois_anisotropes
///  @{
///


class Projection_anisotrope_3D : public Loi_comp_abstraite
{


	public :
 
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		Projection_anisotrope_3D ();

		// Constructeur de copie
		Projection_anisotrope_3D (const Projection_anisotrope_3D& loi) ;
		
		// DESTRUCTEUR :
		~Projection_anisotrope_3D ();
 
  // initialise les donnees particulieres a l'elements
  // de matiere traite ( c-a-dire au pt calcule)
  // Il y a creation d'une instance de SaveResul particuliere
  // a la loi concernee
  // la SaveResul classe est remplie par les instances heritantes
  // le pointeur de SaveResul est sauvegarde au niveau de l'element
  // c'a-d que les info particulieres au point considere sont stocke
  // au niveau de l'element et non de la loi.
  class SaveResulProjection_anisotrope_3D: public SaveResul
   { public :
         SaveResulProjection_anisotrope_3D(); // constructeur par défaut (a ne pas utiliser)
         // le constructeur courant
         SaveResulProjection_anisotrope_3D(SaveResul* l_SaveResul,TenseurHH*  l_siHH
                                          ,TenseurHH*  l_siHH_t
                                          ,EnergieMeca  l_energ_,EnergieMeca  l_energ_t_
                                          ,bool avec_ponderation);
         SaveResulProjection_anisotrope_3D(const SaveResulProjection_anisotrope_3D& sav); // de copie
         virtual ~SaveResulProjection_anisotrope_3D(); // destructeur
         // définition d'une nouvelle instance identique
         // appelle du constructeur via new
         SaveResul * Nevez_SaveResul() const{return (new SaveResulProjection_anisotrope_3D(*this));};
         // affectation
         virtual SaveResul & operator = ( const SaveResul & a);
         //============= lecture écriture dans base info ==========
            // cas donne le niveau de la récupération
            // = 1 : on récupère tout
            // = 2 : on récupère uniquement les données variables (supposées comme telles)
         void Lecture_base_info (ifstream& ent,const int cas);
            // cas donne le niveau de sauvegarde
            // = 1 : on sauvegarde tout
            // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
         void Ecriture_base_info(ofstream& sort,const int cas);
    
         // mise à jour des informations transitoires
         void TdtversT();
         void TversTdt();
    
         // affichage à l'écran des infos
         void Affiche() const;
    
         //changement de base de toutes les grandeurs internes tensorielles stockées
         // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
         // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
         // gpH(i) = gamma(i,j) * gH(j)
         virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma);
    
         // procedure permettant de completer éventuellement les données particulières
         // de la loi stockées
         // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
         // completer est appelé apres sa creation avec les donnees du bloc transmis
         // peut etre appeler plusieurs fois
         SaveResul* Complete_SaveResul(const BlocGen & bloc, const Tableau <Coordonnee>& tab_coor
                                       ,const Loi_comp_abstraite* loi);

         // ---- méthodes spécifiques
         // initialise les informations de travail concernant le pas de temps en cours
         void Init_debut_calcul();
    
         //-------------------------------------------------------------------
         // données
         //-------------------------------------------------------------------
         // - partie repère d'orthotropie
         // 1) le repère lui-même
         // soit O_B est non nul et O_H est NULL
         //  soit l'inverse,
         // c'est celui qui est non nul, qui indique le type de convection
         BaseB * O_B; // définit éventuellement les coordonnées covariante
                      //       convectées donc fixes, de O, dans gi_H_tdt
         BaseH * O_H; // définit éventuellement les coordonnées contravariante
                      //       convectées donc fixes, de O, dans gi_B_tdt
         // 2) le repère obtenu par convection -> O', exprimé
         BaseH  Op_H,Op_H_t; // définit les coordonnées contravariante
                       //  de la base transportée

         // 2) les tenseurs intermédiaires
         TenseurHH* eps_loc_HH; // def local dans le repère O'_a
         TenseurHH* sig_loc_HH; // contrainte local dans le repère O'_a
    
         // les 6 paramètres de la loi dans l'ordre suivant
         // double A1,A2,A3,B12,B13,B23; // paramètres de la loi
         Vecteur *para_loi;
    
         // données protégées
         // les données protégées de la loi interne
         SaveResul* SaveResul_interne;
         // les contraintes initiales particulières de la loi
         TenseurHH*  l_sigoHH,* l_sigoHH_t; // valeur courante, et valeur sauvegardée au pas précédent
         //  énergies pour la loi interne
         EnergieMeca  l_energ,l_energ_t; // valeur courante, et valeur sauvegardée au pas précédent
         //  fonctions de pondération
         double f_ponder,f_ponder_t;  // le résultat des fonctions de pondérations
           // = 1 si pas de pondération
    
   };

  SaveResul * New_et_Initialise();
 
  friend class SaveResulProjection_anisotrope_3D;
 
	    // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();
          
  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure temps,const Deformation & ,SaveResul * saveResul);
  // récupération d'un module de compressibilité équivalent à la loi pour un chargement non nul
  // il s'agit ici de la relation -pression = sigma_trace/3. = module de compressibilité * I_eps
  double Module_compressibilite_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const {return ConstMath::tresgrand;};

  // activation des données des noeuds et/ou elements nécessaires au fonctionnement de la loi
  // exemple: mise en service des ddl de température aux noeuds
  virtual void Activation_donnees(Tableau<Noeud *>& tabnoeud,bool dilatation,LesPtIntegMecaInterne& lesPtMecaInt);
  // récupération des grandeurs particulière (hors ddl )
  // correspondant à liTQ
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void Grandeur_particuliere
        (bool absolue,List_io<TypeQuelconque>& liTQ,Loi_comp_abstraite::SaveResul * saveDon,list<int>&) const;
  // récupération de la liste de tous les grandeurs particulières
  // ces grandeurs sont ajoutées à la liste passées en paramètres
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void ListeGrandeurs_particulieres(bool absolue,List_io<TypeQuelconque>& liTQ) const;
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new Projection_anisotrope_3D(*this)); };
 
  // indique le type Enum_comp_3D_CP_DP_1D correspondant à une  loi de comportement
  // la fonction est simple dans le cas d'une loi basique, par contre dans le cas
  // d'une loi combinée, la réponse dépend des lois internes donc c'est redéfini
  // dans les classes dérivées
  virtual Enum_comp_3D_CP_DP_1D  Comportement_3D_CP_DP_1D();
 
  //----- lecture écriture de restart -----
	 // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info_loi(ofstream& sort,const int cas);
            
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);
 
  // récupe du nom de repère
  const string& NomRepere() const {return nom_repere;};
 
  // récupe du type de transport
  const int& Type_transport() const {return type_transport;}; 

	protected :

   Enum_proj_aniso type_projection; // le type de projection de la loi

  // donnees protegees
  //------------ premier type de projection: PROJ_ORTHO ---------
  // - coef de la loi
	 double A1,A2,A3,B12,B13,B23; // paramètres de la loi
  Tableau2 <double*> hij; // idem sous forme de tableau
	 Tableau <Fonction_nD* > fct_para; // fonction nD éventuelle d'évolution des paramètres
  bool Bll_fct_para; // indicateur pour le cas particulier où il n'y a aucune fct_para
  string nom_repere; // le nom du repère d'anisotropie associé
	 int cas_calcul; // indique le choix entre différents types de calcul possible
	                       // = 0 : calcul normal
	                       // = 1 : calcul seulement déviatorique (la partie sphérique est mise à zéro)
	                       // = 2 : calcul seulement sphérique (la partie déviatorique est mise à zéro)
  double ratio_inf_module_compressibilite; // indique le ratio mini / au module initial
  Mat_pleine inv_loi; // matrice pour inverser la relation eps_jj = mat * sig_ii
  // type de transport
  int type_transport; // = 0 : par défaut: transport de type contravariant
                      // = 1 : transport de type covariant
 
  BaseB  Op_B;  // coordonnée de la base de travail : correspond à la base O'_i actuelle exprimée dans g^j
                // correspond au transport de type covariant
 
  BaseB  d_Op_B; // variation du repère: c'est une grandeur de travail
  BaseB  pO_B; // les vecteurs O_i non normés: grandeur de travail
  BaseH  d_Op_H; //  idem
  BaseH  pO_H; // les vecteurs O_i non normés: grandeur de travail

  BaseH alpha_H; // coordonnées locales de O_a:    O_a = alpha_a^{.i} * g_i

  Mat_pleine beta; // matrice de passage de g_i à O'_i: O'_i = beta_i^{.j} * g_i
  Mat_pleine gamma; // matrice de passage de g^i à O'^i: O'^i = gamma^i^_{.j} * g^j
  Mat_pleine beta_transpose;
  Mat_pleine gamma_transpose;

  Mat_pleine  beta_inv; // l'inverse

  // ----- controle de la sortie des informations
// -> maintenant définit dans LoiAbstraiteGeneral
//  int permet_affichage; // pour permettre un affichage spécifique dans les méthodes,
                        // pour les erreurs et des warnings
                        
  int sortie_post; // permet de stocker et ensuite d'accéder en post-traitement à certaines données
        // = 0 par défaut,
        // = 1 : on stocke toutes les grandeurs et elles sont disponibles en sortie

  // on introduit un certain nombre de tenseur du quatrième ordre, qui vont nous servir pour
  // Calcul_dsigma_deps, dans le cas où on n'est pas en orthonormee
		Tenseur3HHHH  I_x_I_HHHH,I_xbarre_I_HHHH,I_x_eps_HHHH,Ixbarre_eps_HHHH;
  TenseurQ3geneHHHH dsig_ijkl_HHHH; // tenseur de travail pour Calcul_dsigma_deps
 

  // cas d'un point d'intégration locale (méthode CalculGrandeurTravail par exemple)
  PtIntegMecaInterne ptintmeca;

  //------------------------------ lois internes --------------------
  // un type énuméré pour faciliter la lecture
  enum Enumcompletudecalcul { CONTRAINTE_ET_TANGENT =0, CONTRAINTE_UNIQUEMENT, TANGENT_UNIQUEMENT};
  // donnees protegees
  Loi_comp_abstraite * loi_interne; // la  loi constitutive de l'espace de référence
  Enumcompletudecalcul completude_calcul; // pour savoir si on utilise tout ou une partie

  //-- partie optionnelle
 
  // une fonction nD via un objet: Ponderation_TypeQuelconque
  // ici un pointeur nulle indique qu'il n'y a pas de fct
  // les grandeurs quelconque sont celles de la loi, elles doivent donc être renseignées
  Ponderation_TypeQuelconque* ponderation_nD_quelconque;

  // ---- tableau de travail
  Tableau <TenseurHH *> d_sigRef_HH;
  // tenseur du 4ième orde de travail
  TenseurHHHH* d_sigma_deps_inter;
  Tenseur3HHHH d_sig_deps_3D_HHHH;
  Tenseur3HHBB H_HHBB; // le tenseur H dans la base de travail

	   		
  // codage des METHODES VIRTUELLES  protegees:
  // calcul des contraintes a t+dt
         // calcul des contraintes
  void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
      ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
      ,TenseurBB & delta_epsBB_
      ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
      ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
      ,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
      ,const Met_abstraite::Expli_t_tdt& ex);

        // calcul des contraintes et de ses variations  a t+dt
  void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
      ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
      ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
      ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
      ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
      ,Tableau <TenseurBB *>& d_gijBB_tdt
      ,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
      ,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
      ,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
      ,const Met_abstraite::Impli& ex);
 
         // calcul des contraintes et ses variations  par rapport aux déformations a t+dt
         // en_base_orthonormee:  le tenseur de contrainte en entrée est  en orthonormee
         //                  le tenseur de déformation et son incrémentsont également en orthonormees
         //                 si = false: les bases transmises sont utilisées
         // ex: contient les éléments de métrique relativement au paramétrage matériel = X_(0)^a
  void Calcul_dsigma_deps (bool en_base_orthonormee, TenseurHH & sigHH_t,TenseurBB& DepsBB
      ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
      ,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
      ,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
      ,const Met_abstraite::Umat_cont& ex) ;
 
  // fonction surchargée dans les classes dérivée si besoin est
  virtual void CalculGrandeurTravail(const PtIntegMecaInterne&
              ,const Deformation & ,Enum_dure,const ThermoDonnee&
              ,const Met_abstraite::Impli* ex_impli
              ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
              ,const Met_abstraite::Umat_cont* ex_umat
              ,const List_io<Ddl_etendu>* exclure_dd_etend
              ,const List_io<const TypeQuelconque *>* exclure_Q
              ) ;

  // fonction interne utilisée par les classes dérivées de Loi_comp_abstraite
  // pour répercuter les modifications de la température
  // ici utiliser pour modifier la température des lois élémentaires
  // l'Enum_dure: indique quel est la température courante : 0 t ou tdt
  void RepercuteChangeTemperature(Enum_dure temps);
 
  // vérification et préparation de l'acces aux grandeurs locales
  void  Verif_et_preparation_acces_grandeurs_locale();
};
/// @}  // end of group


#endif		

		

