// FICHIER : Hysteresis1D.h
// CLASSE : Hysteresis1D


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        10/02/2004                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   La classe Hysteresis1D permet de calculer la contrainte   *
 *     et ses derivees pour une loi d'hysteresis 1D,  type              *
 *     celle développée par Guelin, Favier, Pegon.                      *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef HYSTERESIS_1D_H
#define HYSTERESIS_1D_H


#include "Loi_comp_abstraite.h"
#include "Courbe1D.h"
#include "TypeConsTens.h"
#include "Algo_zero.h"
#include "TenseurQ1gene.h"
#include "Algo_edp.h"

/** @defgroup Les_lois_hysteresis
*
*     BUT:   groupe des lois de type hystérésis
*
*
* \author    Gérard Rio
* \version   1.0
* \date       10/02/2004
* \brief       Définition des  lois de type hystérésis
*
*/

/// @addtogroup Les_lois_hysteresis
///  @{
///


class Hysteresis1D : public Loi_comp_abstraite
{
  public :
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		Hysteresis1D ();
		
		
		// Constructeur de copie
		Hysteresis1D (const Hysteresis1D& loi) ;
		
		// DESTRUCTEUR :
		
		~Hysteresis1D ();
		
		// initialise les donnees particulieres a l'elements
		// de matiere traite ( c-a-dire au pt calcule)
		// Il y a creation d'une instance de SaveResul particuliere
		// a la loi concernee
		// la SaveResul classe est remplie par les instances heritantes
		// le pointeur de SaveResul est sauvegarde au niveau de l'element
		// c'a-d que les info particulieres au point considere sont stocke
		// au niveau de l'element et non de la loi.
		class SaveResulHysteresis1D: public SaveResul 
		 { public : 
	        SaveResulHysteresis1D(); // constructeur par défaut :
	        SaveResulHysteresis1D(const SaveResulHysteresis1D& sav); // de copie
	        ~SaveResulHysteresis1D(){}; // destructeur
         // définition d'une nouvelle instance identique
         // appelle du constructeur via new 
         SaveResul * Nevez_SaveResul() const{return (new SaveResulHysteresis1D(*this));};
         // affectation
         virtual SaveResul & operator = ( const SaveResul & a);
	        //============= lecture écriture dans base info ==========
            // cas donne le niveau de la récupération
            // = 1 : on récupère tout
            // = 2 : on récupère uniquement les données variables (supposées comme telles)
	        void Lecture_base_info (ifstream& ent,const int cas);
            // cas donne le niveau de sauvegarde
            // = 1 : on sauvegarde tout
            // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	        void Ecriture_base_info(ofstream& sort,const int cas);
	        
	        // mise à jour des informations transitoires 
	        void TdtversT();
	        void TversTdt(); 
			  
         // affichage à l'écran des infos
			      void Affiche() const;
    
         //changement de base de toutes les grandeurs internes tensorielles stockées
         // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
         // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
         // gpH(i) = gamma(i,j) * gH(j)
         virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma);
    
         // procedure permettant de completer éventuellement les données particulières
         // de la loi stockées
         // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
         // completer est appelé apres sa creation avec les donnees du bloc transmis
         // peut etre appeler plusieurs fois
         SaveResul* Complete_SaveResul(const BlocGen & bloc, const Tableau <Coordonnee>& tab_coor
                                       ,const Loi_comp_abstraite* loi) {};

	        // ---- méthodes spécifiques 
	        // initialise les informations de travail concernant le pas de temps en cours 
	        void Init_debut_calcul();
	        
	        // données protégées	        
	        Tenseur1BH sigma_barre_BH_t; // dernière contrainte en BH  à t 
	        Tenseur1BH sigma_barre_BH_tdt; // contrainte en cours BH  à tdt
	        double fonction_aide_t; // dernière valeur de la fonction d'aide 
	        double fonction_aide_tdt; // valeur de la fonction d'aide en cours
	        double wprime_t,wprime_tdt; // paramètre de masing
	        List_io <double>::iterator ip2; // adresse éventuelle du 2 éléments de fct_aide
	        
	        // --- informations de travail concernant le pas de temps en cours ---
	        int modif; // = 0 rien de changé, =1 coincidence(s), =2  inversion(s), =3 coin et inver
	        // liste des nouvelles contraintes de référence, qui sont apparu
	        List_io <Tenseur1BH> sigma_barre_BH_R_t_a_tdt;
	        int nb_coincidence; // nombre de coincidence durant le pas de temps 
	        List_io <double> fct_aide_t_a_tdt; // liste des valeurs de la fonction d'aide durant le pas
	                                           // de temps
	        List_io <bool> indic_coin; // liste d'indicateurs indiquant la suite des coincidences et 
	                      // inversion,  = true -> indique que c'est une coincidence, sinon inversion
	        // --- fin informations de travail concernant le pas de temps en cours ---
	        
	        // --- informations de mémorisation discrète de 0 à t
	        // le dernier élément rangé est en .begin() (c-a-d front())
	        List_io <Tenseur1BH> sigma_barre_BH_R; // liste des contraintes de référence
	        List_io <double> fct_aide; // liste des valeurs de la fonction d'aide
    
         // 5) --- tableau d'indicateur de la résolution, éventuellement vide
         //     cela dépend de sortie_post
         //  indicateurs_resolution(1) : nb d'incrément utilisé pour la résolution de l'équation linéarisée
         //                        (2) : nb total d'itération "   "       "                 "      "
         //                        (3) : cas Runge Kutta: nb d'appel de la fonction
         //                        (4) : cas Runge Kutta: nb de step de calcul
         //                        (5) : cas Runge Kutta: erreur globale de la résolution
         Tableau <double> indicateurs_resolution,indicateurs_resolution_t;
    
   };

		SaveResul * New_et_Initialise() 
		    { SaveResulHysteresis1D * pt = new SaveResulHysteresis1D();
		      return pt;};
		
  // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();
          
  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure temps,const Deformation & def ,SaveResul * saveResul);
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const
    { cout << "\n Hysteresis1D::HsurH0(.. , methode non implante pour l'instant ";
      Sortie(1);
    };
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new Hysteresis1D(*this)); };
		  	  
	 //----- lecture écriture de restart -----
	 // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);

     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info_loi(ofstream& sort,const int cas);
          
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);

  // récupération des grandeurs particulière (hors ddl )
  // correspondant à liTQ
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void Grandeur_particuliere
       (bool absolue,List_io<TypeQuelconque>& ,Loi_comp_abstraite::SaveResul * ,list<int>& decal) const ;
  // récupération de la liste de tous les grandeurs particulières
  // ces grandeurs sont ajoutées à la liste passées en paramètres
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void ListeGrandeurs_particulieres(bool absolue,List_io<TypeQuelconque>& ) const;
		  	       
 
	protected :

     // donnees protegees
     // ---- paramètres matériaux ----
     double xnp; // paramètre de Prager
	    Courbe1D* xnp_temperature; // courbe éventuelle d'évolution de xnp en fonction de la température
     //    int cas_prager; // indique si xnp =2 (=1), ou est compris entre 2 et 3 (=2), ou est sup à 3 (=3)
     double Qzero; // limite de plasticité du critère de von mises
	    Courbe1D* Qzero_temperature; // courbe éventuelle d'évolution de Qzero en fonction de la température
     double xmu; // paramètre de lame
	    Courbe1D* xmu_temperature; // courbe éventuelle d'évolution de xmu en fonction de la température
	    
	    // ---- paramètres de l'algorithme de Newton -----
	    double  tolerance_residu; // tolérance absolu
	    double  tolerance_residu_rel; // tolérance relative
     double maxi_delta_var_sig_sur_iter_pour_Newton; // le maxi de variation que l'on tolère d'une itération à l'autre
	    double  tolerance_coincidence; // tolérance sur la précision de la coincidence
	    int nb_boucle_maxi; // le maximum d'itération de plasticité permis
	    int nb_sous_increment; // le maxi de sous incrément prévu
     int type_resolution_equa_constitutive; // linéarisation ou kutta par exemple
     int nb_maxInvCoinSurUnPas; // nombre maximum d'inversion ou de coïncidence sur un pas

     // ----- controle de la sortie des informations
// -> maintenant définit dans LoiAbstraiteGeneral
//     int permet_affichage; // pour permettre un affichage spécifique dans les méthodes, pour les erreurs et warning
     int sortie_post; // permet d'accèder au nombre d'itération, d'incrément, de précision etc. des résolutions
           // = 0 par défaut,
           // = 1 : on stocke toutes les grandeurs et elles sont disponibles en sortie
 
 
	    // variables de travail pour l'échange entre les différentes méthodes en internes
	    Tenseur1BH sigma_t_barre_tdt; // sigma barre finale
	    Tenseur1BH sigma_i_barre_BH;   // sigma barre de début de calcul (à t au début)      	    
	    Tenseur1BH sigma_barre_BH_R;   // sigma barre de Référence  en cours
	    Tenseur1BH delta_sigma_barre_BH_Rat;  // deltat sigma barre de R a t
	    Tenseur1BH delta_sigma_barre_BH_Ratdt;  // deltat sigma barre de R a tdt
	    Tenseur1BH delta_sigma_barre_tdt_BH;  // delta sigma barre de t à tdt
	    Tenseur1BH residuBH;
	    Tenseur1BH delta_barre_epsBH;  // delta_barre epsilon totale
	    Tenseur1BH delta_barre_alpha_epsBH;  // delta_barre epsilon intermediaire (avec alpha de 0 à 1)
	    double wprime; // paramètre de masing	    
	    Vecteur residu;  // résidu de l'équation pour la résolution de l'équation constitutive
	    Mat_pleine derResidu; // dérivé du résidu de l'équation pour la résolution de l'équation constitutive	    
     Algo_zero  alg_zero; // algo pour la recherche de zero	
     // ... partie relative à une résolution de l'avancement par une intégration de l'équation différentielle
     Algo_edp   alg_edp;
     int cas_kutta; // indique le type de runge_kutta que l'on veut utiliser
	    double erreurAbsolue,erreurRelative; // précision absolue et relative que l'on désire sur le calcul de sig_tdt      
     int nbMaxiAppel; // nombre maxi d'appel de la fonction dérivée
	    Vecteur sig_point; // vitesse de sig: version vecteur de sigma_point
	    Tenseur1BH sigma_pointBH; // idem mais en tenseur
	    Tenseur1BH sigma_tauBH; // valeur de sigma pour le temps tau
	    Vecteur sigma_tau; // valeur de sigma pour le temps tau, en vecteur	
     Tenseur1BH delta_sigma_barre_R_a_tauBH; // delta sigma de R à tau
     Tenseur1BH betaphideltasigHB,deuxmudeltaepsHB; // pour le calcul de la dérivée
     
     // -- variables de travail internes à Residu_constitutif() et Mat_tangente_constitutif()
     // définit ici pour éviter de les définir à chaque passage ds la méthode,
     // ne doivent pas être utilisée en dehors de ces deux routines
	    Tenseur1BH rdelta_sigma_barre_BH_Ratdt;  // deltat sigma barre de R a tdt
	    Tenseur1BH rdelta_sigma_barre_tdt_BH;  // delta sigma barre de t à tdt
	    
        // --------------- méthodes internes ---------------:
	    // affinage d'un point de coincidence
	    // ramène true si le traitement est exactement terminé, sinon false, ce qui signifie qu'il
	    // faut encore continuer à utiliser l'équation d'évolution
	    // premiere_charge : indique si c'est oui ou non une coincidence avec la première charge
	    // pt_sur_principal : indique si oui ou non les pointeurs iafct et iatens pointent sur les listes
	    // principales
	    // iatens_princ et iafct_princ: pointeurs sur les listes principales
	    bool Coincidence(double& unSur_wprimeCarre,bool premiere_charge
	                    ,SaveResulHysteresis1D & save_resul,double& W_a
	                    ,List_io <Tenseur1BH>::iterator& iatens,List_io <double>::iterator& iafct
	                    ,bool& pt_sur_principal,List_io <Tenseur1BH>::iterator& iatens_princ
	                    ,List_io <double>::iterator& iafct_princ,double& delta_W_a);
	    
	
     // --------------- codage des METHODES VIRTUELLES  protegees ---------------:
 // calcul des contraintes a t+dt
 // calcul des contraintes a t+dt
        // calcul des contraintes 
 void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
     ,TenseurBB & delta_epsBB_
     ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
     ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Expli_t_tdt& ex);

       // calcul des contraintes et de ses variations  a t+dt
 void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
     ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
     ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
     ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
     ,Tableau <TenseurBB *>& d_gijBB_tdt
		  	,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
		  	,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Impli& ex);
		  	
 
	
     // fonction surchargée dans les classes dérivée si besoin est
     virtual void CalculGrandeurTravail
                   (const PtIntegMecaInterne& ,const Deformation &
                    ,Enum_dure,const ThermoDonnee&
                    ,const Met_abstraite::Impli* ex_impli
                    ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
                    ,const Met_abstraite::Umat_cont* ex_umat
                    ,const List_io<Ddl_etendu>* exclure_dd_etend
                    ,const List_io<const TypeQuelconque *>* exclure_Q
                    ) {};

     // initialisation éventuelle des variables thermo-dépendantes
     void Init_thermo_dependance();
     // méthode permettant le calcul de sigma à tdt par différente méthodes: linéarisation
     // ou kutta
     void CalculContrainte_tdt(Tableau<double>& indicateurs_resolution);
     // calcul de l'avancement temporel sur 1 pas,
     // utilisé par les 3 programmes principaux: 
     // Calcul_SigmaHH, Calcul_DsigmaHH_tdt, Calcul_dsigma_deps,
     void Avancement_temporel(const Tenseur1BB & delta_epsBB,const Tenseur1HH & gijHH
                                 ,SaveResulHysteresis1D & save_resul
                                 ,Tenseur1HH & sigHH);
   public:
     // calcul de la fonction résidu de la résolution de l'équation constitutive
     // l'argument test ramène
     //     . 1 si le calcul a été ok, -1 s'il y a eu un pb, mais on peut continuer, 0 s'il y a eu un pb
     //       fatal, qui invalide le calcul du résidu
     Vecteur& Residu_constitutif (const double & alpha, const Vecteur & x, int& test);
     // calcul de la matrice tangente de la résolution de l'équation constitutive
     // l'argument test ramène
     //     . 1 si le calcul a été ok, -1 s'il y a eu un pb, mais on peut continuer, 0 s'il y a eu un pb
     //       fatal, qui invalide le calcul du résidu et de la dérivée
     Mat_abstraite& Mat_tangente_constitutif(const double & alpha,const Vecteur & x, Vecteur& resi, int& test);
     // calcul de l'opérateur tangent : dsigma/depsilon
     TenseurQ1geneBHBH& Dsig_depsilon(TenseurQ1geneBHBH& dsig_deps);	
     // calcul de l'expression permettant d'obtenir la dérivée temporelle de la contrainte
     // à un instant tau quelconque, en fait il s'agit de l'équation constitutive
     // utilisée dans la résolution explicite (runge par exemple) de l'équation constitutive
     // erreur : =0: le calcul est licite, si diff de 0, indique qu'il y a eu une erreur
     //          =1:  la norme de sigma est supérieure à la valeur limite de saturation
     Vecteur& Sigma_point(const double & tau, const Vecteur & sigma_tau,Vecteur& sig_point,int & erreur);
     // vérification de l'intégrité du sigma calculé
     // erreur : =0: le calcul est licite, si diff de 0, indique qu'il y a eu une erreur
     //          =1:  la norme de sigma est supérieure à la valeur limite de saturation
     void Verif_integrite_Sigma(const double & tau, const Vecteur & sigma_tau,int & erreur);
	
};
/// @}  // end of group


#endif		

		

