// FICHIER : Hysteresis3D.h
// CLASSE : Hysteresis3D


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        30/06/2004                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   La classe Hysteresis3D permet de calculer la contrainte   *
 *     et ses derivees pour une loi d'hysteresis 3D,  type              *
 *     celle développée par Guelin, Favier, Pegon.                      *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef HYSTERESIS_3D_H
#define HYSTERESIS_3D_H

#include "Vecteur.h"
#include "Loi_comp_abstraite.h"
#include "Courbe1D.h"
#include "TypeConsTens.h"
#include "Algo_zero.h"
#include "TenseurQ1gene.h"
#include "MatLapack.h"
#include "Algo_edp.h"
#include "TenseurQ3gene.h"


/// @addtogroup Les_lois_hysteresis
///  @{
///

class Hysteresis3D : public Loi_comp_abstraite
{
 public :
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		Hysteresis3D ();
		
		
		// Constructeur de copie
		Hysteresis3D (const Hysteresis3D& loi) ;
		
		// DESTRUCTEUR :
		
		~Hysteresis3D ();
		
		// initialise les donnees particulieres a l'elements
		// de matiere traite ( c-a-dire au pt calcule)
		// Il y a creation d'une instance de SaveResul particuliere
		// a la loi concernee
		// la SaveResul classe est remplie par les instances heritantes
		// le pointeur de SaveResul est sauvegarde au niveau de l'element
		// c'a-d que les info particulieres au point considere sont stocke
		// au niveau de l'element et non de la loi.
		class SaveResulHysteresis3D: public SaveResul 
		 { public : 
	        SaveResulHysteresis3D(); // constructeur par défaut :
	        SaveResulHysteresis3D(const SaveResulHysteresis3D& sav); // de copie
	        virtual ~SaveResulHysteresis3D(){}; // destructeur
		       // définition d'une nouvelle instance identique
		       // appelle du constructeur via new 
		       SaveResul * Nevez_SaveResul() const{return (new SaveResulHysteresis3D(*this));};		    
         // affectation
         virtual SaveResul & operator = ( const SaveResul & a);
	        //============= lecture écriture dans base info ==========
            // cas donne le niveau de la récupération
            // = 1 : on récupère tout
            // = 2 : on récupère uniquement les données variables (supposées comme telles)
	        void Lecture_base_info (ifstream& ent,const int cas);
            // cas donne le niveau de sauvegarde
            // = 1 : on sauvegarde tout
            // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	        void Ecriture_base_info(ofstream& sort,const int cas);
	        
	        // mise à jour des informations transitoires 
	        void TdtversT();
	        void TversTdt(); 
			  
			      // affichage à l'écran des infos
			      void Affiche() const;
    
         //changement de base de toutes les grandeurs internes tensorielles stockées
         // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
         // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
         // gpH(i) = gamma(i,j) * gH(j)
         virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma);
    
         // procedure permettant de completer éventuellement les données particulières
         // de la loi stockées
         // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
         // completer est appelé apres sa creation avec les donnees du bloc transmis
         // peut etre appeler plusieurs fois
         SaveResul* Complete_SaveResul(const BlocGen & bloc, const Tableau <Coordonnee>& tab_coor
                                       ,const Loi_comp_abstraite* loi) {};

	        // ---- méthodes spécifiques
	        // initialise les informations de travail concernant le pas de temps en cours 
	        void Init_debut_calcul();
         // vérif de la cohérence des centres et références: les rayons associés doivent être de taille
         // décroissante:  programme utilisé uniquement en  débug 
         void Verif_centre_reference(const int& permet_affichage);
	        
	        //-------------------------------------------------------------------
	        // données
	        //-------------------------------------------------------------------
	   
	        // 1) ====== valeurs courantes (que l'on pourra peut-être économiser par la suite)
	        
	        Tenseur3BH sigma_barre_BH_t; // dernière contrainte en BH  à t
	        Tenseur3BH sigma_barre_BH_tdt; // contrainte en cours BH  à tdt
	        // les contraintes sigma_barre_BH_t et sigma_barre_BH_tdt sont celles obtenues
	        // à partir de l'équation constitutive en mixte BH, 
	        // ce n'est donc pas la contrainte finale (qui utilise la dérivée de jauman) 
	        double fonction_aide_t; // dernière valeur de la fonction d'aide 
	        double fonction_aide_tdt; // valeur de la fonction d'aide en cours
	        List_io <double>::iterator ip2; // adresse éventuelle du 2 éléments de fct_aide
    
	        Tenseur3BH oc_BH_t; // dernière valeur du centre de référence
	        Tenseur3BH oc_BH_tdt; // valeur du centre de référence en cours
	        int wBase_t,wBase_tdt; // paramètre de masing
	        double def_equi_at;      // valeur de la def équivalente à t 
	        double def_equi_atdt;    // valeur de la def équivalente à tdt 
	        // fonction d'aide
//	        double fonction_aide_t; // dernière valeur de la fonction d'aide 
//	        double fonction_aide_tdt; // valeur de la fonction d'aide en cours
//	        List_io <double>::iterator ip2; // adresse éventuelle du 2 éléments de fct_aide
	        
	        // 2)  a) ====== informations de travail concernant le pas de temps en cours, c-a-d de t à t+dt
	        int modif; // = 0 rien de changé, 
	                   // = 1 une ou plusieurs coincidence(s) (sans inversion), 
	                   // = 2 une ou plusieurs inversion(s) (sans coïncidence), 
	                   // = 3 un ensemble d'une ou plusieurs coïncidences de inversions
	        // liste des nouvelles contraintes de référence, qui sont apparu
	        List_io <Tenseur3BH> sigma_barre_BH_R_t_a_tdt;
	        int nb_coincidence; // nombre de coincidence durant le pas de temps 
	        // liste des centres (contraintes) de référence qui sont apparues
	        List_io <Tenseur3BH> oc_BH_t_a_tdt; 
	        List_io <double> def_equi_t_a_tdt; // liste des def équivalentes
	               //aux  points d'inversions durant le pas de temps
	        List_io <double> fct_aide_t_a_tdt; // liste des valeurs de la fonction d'aide durant le pas
	                                           // de temps
//	        // 2)  b) ====== sauvegarde des variable de 2) a) pour pouvoir revenir en arrière : de tdt vers t
//         //        ces variables ne sont pas sauvegardées dans les .BI
//	        int modif_sauve; 	        // liste des nouvelles contraintes de référence, qui sont apparu
//	        List_io <Tenseur3BH> sigma_barre_BH_R_sauve;
//         int nb_ref_new_sauve;
//	        int nb_coincidence_sauve; // nombre de coincidence durant le pas de temps
//	        // liste des centres (contraintes) de référence qui sont apparues
//	        List_io <Tenseur3BH> oc_BH_R_sauve;
//         int nb_centre_new_sauve;
//	        List_io <double> def_equi_sauve; // liste des def équivalentes
//	               //aux  points d'inversions durant le pas de temps
	   
	        // 3) --- informations de mémorisation discrète de 0 à t ----------------------------------
	        // le dernier élément rangé est en .begin() (c-a-d front())
	        List_io <Tenseur3BH> sigma_barre_BH_R; // liste des contraintes de référence
	        List_io <Tenseur3BH> oc_BH_R; // liste des centres (contraintes) de référence
	        // il y a un élément de moins dans la liste oc_BH_R que dans la liste sigma_barre_BH_R
	        // car sur la première charge, le centre c'est 0, et la ref de sigma = 0 aussi
	        // sur la deuxième charge: le centre c'est toujours 0, et il y a un sigma de ref  non nulle
	        // sur une autre charge : on a un nouveau centre et 2 sig de ref, puis etc par la suite
	        List_io <double> def_equi; // liste des def équivalentes aux points d'inversions
	        List_io <double> fct_aide; // liste des valeurs de la fonction d'aide
    
	        // 4) --- sauvegarde des informations de mémorisation discrète de 0 à t avant transport -----
         //    le transport sur toute la mémorisation tensorielle à lieu à chaque calcul !!
         //    donc il faut pouvoir revenir en arrière
         //  ces grandeurs ne sont pas sauvegardées, elles sont utilisée pour t->tdt uniquement
	        List_io <Tenseur3BH> sigma_barre_BH_R_atrans; // liste des contraintes de référence
	        List_io <Tenseur3BH> oc_BH_R_atrans; // liste des centres (contraintes) de référence
    
         // 5) --- tableau d'indicateur de la résolution, éventuellement vide
         //     cela dépend de sortie_post
         //  indicateurs_resolution(1) : nb d'incrément utilisé pour la résolution de l'équation linéarisée
         //                        (2) : nb total d'itération "   "       "                 "      "
         //                        (3) : cas Runge Kutta: nb d'appel de la fonction
         //                        (4) : cas Runge Kutta: nb de step de calcul
         //                        (5) : cas Runge Kutta: erreur globale de la résolution
         Tableau <double> indicateurs_resolution,indicateurs_resolution_t;
	   
   };

		SaveResul * New_et_Initialise() 
		  { SaveResulHysteresis3D * pt = new SaveResulHysteresis3D();
		    return pt;
    };
		
  friend class SaveResulHysteresis3D;      
                     
	 // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();
    
  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const {return ConstMath::tresgrand;};
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new Hysteresis3D(*this)); };
			  
	 //----- lecture écriture de restart -----
	 // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);

     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info_loi(ofstream& sort,const int cas);
          
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);  

  // récupération des grandeurs particulière (hors ddl )
  // correspondant à liTQ
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void Grandeur_particuliere
       (bool absolue,List_io<TypeQuelconque>& ,Loi_comp_abstraite::SaveResul * ,list<int>& decal) const ;
  // récupération de la liste de tous les grandeurs particulières
  // ces grandeurs sont ajoutées à la liste passées en paramètres
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void ListeGrandeurs_particulieres(bool absolue,List_io<TypeQuelconque>& ) const;
		  	       
	protected :

  // donnees protegees
  // ---- paramètres matériaux -------
  double xnp,xnp_lue; // paramètre de Prager
	 Courbe1D* xnp_temperature; // courbe éventuelle d'évolution de xnp en fonction de la température
	 Courbe1D* xnp_phase; // courbe éventuelle d'évolution de xnp en fonction de la phase
	 Courbe1D* xnp_defEqui; // courbe éventuelle d'évolution de xnp en fonction de la déformation équivalente
     double Qzero,Qzero_lue; // limite de plasticité du critère de von mises
	 Courbe1D* Qzero_temperature; // courbe éventuelle d'évolution de Qzero en fonction de la température
	 Courbe1D* Qzero_phase; // courbe éventuelle d'évolution de Qzero en fonction de la phase
	 Courbe1D* Qzero_defEqui; // courbe éventuelle d'évolution de Qzero en fonction de la déformation équivalente
     double xmu,xmu_lue; // paramètre de lame
	 Courbe1D* xmu_temperature; // courbe éventuelle d'évolution de xmu en fonction de la température
	 Courbe1D* xmu_phase; // courbe éventuelle d'évolution de xmu en fonction de la phase
	 // deux types de dépendance à la phase: soit Delta_S_Oi_tdt, soit la déformation totale, ceci pour xnp,Qzero,xmu
	 // 1 - prise en compte éventuelle de la phase (de Delta_S_Oi_tdt par rapport aux axes principaux dans le plan déviat)
  bool avec_phaseDeltaSig; // indique si oui ou non on a de la phase
  // 2 - prise en compte éventuelle de la phase de la déformation
  bool avec_phaseEps;
  // dépendance éventuelle à la déformation équivalente
  bool avec_defEqui;
  // dépendance éventuelle à la phase de la déformation totale pour un  calcul de val_coef_paradefEqui, 
  // s'emploie  en conjonction avec defEaui 
  bool avec_phase_paradefEqui;
  Courbe1D* coef_paradefEqui; 
  double val_coef_paradefEqui; // valeur courante du coef
  double mini_QepsilonBH; // le minimun de Qsig pour le calcul de la phase de delta_sigma_Oi_tdt
 
	    
	    // ------ paramètres de l'algorithme -------
	 double  tolerance_residu; // tolérance absolu sur la résolution de la plasticité
	 double  tolerance_residu_rel; // tolérance relative sur la résolution de la plasticité
  double maxi_delta_var_sig_sur_iter_pour_Newton; // le maxi de variation que l'on tolère d'une itération à l'autre
  double  tolerance_coincidence; // tolérance sur la précision de la coincidence
	 double  tolerance_centre; // tolérance sur la précision du calcul des centres
	 int nb_boucle_maxi; // le maximum d'itération de plasticité permis
	 int nb_dichotomie; // le maxi de dichotomie prévu pour l'équation de Newton
	 double mini_rayon; // mini en dessous duquel on considère les rayons et accroissements nuls
  int type_resolution_equa_constitutive; // linéarisation ou kutta par exemple
  int type_calcul_comportement_tangent; // indique le type de calcul du comportement tangent
  bool avecVarWprimeS; // prise en compte ou non des variations du cos et du cosPoint pour le calcul tangent
  int nb_maxInvCoinSurUnPas; // nombre maximum d'inversion ou de coïncidence sur un pas
  double min_angle_trajet_neutre; // le minimum du cos(d'angle) en dessous duquel le trajet est considéré neutre
  bool possibilite_cosAlphaNegatif; // indique si oui ou non on tolère transitoirement un cosAlpha négatif (près d'une inversion par exe)
  double mini_Qsig_pour_phase_sigma_Oi_tdt; // le minimun de Qsig pour le calcul de la phase de delta_sigma_Oi_tdt
  double force_phi_zero_si_negatif; // para pour forcer le phi à 0 si ça valeur négative devient sup
  double depassement_Q0; // valeur en relatif de dépassement permis sur la saturation: devrait = 1, mais dans les faits il faut laisser une marge (1.2 par exe), par défaut très grand pour l'instant, je ne sais pas pourquoi !!
  // def du type de transport des grandeurs mémorisée: = 0, transport en mixte (historique)
  int type_de_transport_memorisation;  //=-1 : transport cohérent avec la dérivée de Jauman
  // def le niveau (entre 0 et 1) du rayon par rapport au rayon maxi de Q0 à partir duquel on utilise
  double niveau_bascule_fct_aide; // la fonction d'aide
 
  // ----- controle de la sortie des informations
// -> maintenant définit dans LoiAbstraiteGeneral
//	 int permet_affichage; // pour permettre un affichage spécifique dans les méthodes, pour les erreurs et warning
  int sortie_post; // permet d'accèder au nombre d'itération, d'incrément, de précision etc. des résolutions
        // = 0 par défaut,
        // = 1 : on stocke toutes les grandeurs et elles sont disponibles en sortie
 
	 // variables de travail pour l'échange entre les différentes méthodes en internes
	 Tenseur3BH sigma_t_barre_tdt; // sigma barre finale
	 Tenseur3BH sigma_i_barre_BH;   // sigma barre de début de calcul (à t au début)      	    
	 Tenseur3BH sigma_barre_BH_R;   // sigma barre de Référence  en cours
	 double     def_equi_R; // déformation équivalente à R
	 double     def_i_equi; // déformation équivalente de de début de calcul (comme sigma_i_barre_BH) 
  double     defEqui_maxi; // defEqui maxi sur la courbe de première charge
	 double     def_equi_atdt; // déformation équivalente  à tdt
	 Tenseur3BH delta_sigma_barre_BH_Rat;  // deltat sigma barre de R a t
	 Tenseur3BH delta_sigma_barre_BH_Ra_i;  // deltat sigma barre de R a i
	 Tenseur3BH delta_sigma_barre_BH_Ratdt;  // deltat sigma barre de R a tdt
	 Tenseur3BH delta_sigma_barre_tdt_BH;  // delta sigma barre de t à tdt	    	    
	 
	 Tenseur3BH delta_sigma_barre_BH_OiaR; // delta sigma barre de Oi à R
	 double Q_OiaR; // norme de delta_sigma_barre_BH_OiaR
	 double QdeltaSigmaRatdt; // norme de rdelta_sigma_barre_BH_Ratdt
	 Tenseur3BH oc_BH_R;
	    
	    // -- variables internes à Calcul_SigmaHH et Calcul_DsigmaHH_tdt
  //    algo de newton
  Vecteur val_initiale; // val initiale 
  Vecteur racine;       // racine finale
  MatLapack der_at_racine; // dérivée du résidu locale
	    
	 bool centre_initial; // pour gérer le fait qu'au début le centre initial = tenseur nul
	 bool aucun_pt_inversion; // pour gérer le fait qu'au début il n'y a pas de pt d'inversion !
	 
	 Tenseur3BH residuBH;
	 Tenseur3BH delta_epsBH; // delta epsilon totale
	 Tenseur3BH delta_barre_epsBH;  // delta_barre epsilon totale
	 Tenseur3BH delta_barre_alpha_epsBH;  // delta_barre epsilon intermediaire (avec alpha de 0 à 1)
	 int wBase; // paramètre de masing
	 double wprime,wprime_point;	// paramètre de masing modifié et sa dérivée temporelle
	 bool trajet_neutre;         // indique si oui ou non le trajet est neutre: associé au calcul de wprime
	 Tenseur3BH d_wprime;       // variation de wprime par rapport à  deltat sigma barre de R a tdt  
	 Tenseur3BH d_wprime_point; // variation de wprime_point par rapport à  deltat sigma barre de R a tdt  
	 Vecteur residu;  // résidu de l'équation pour la résolution de l'équation constitutive
	 MatLapack derResidu; // dérivé du résidu de l'équation pour la résolution de l'équation constitutive	
	                          // correspond également à la dérivée du résidu à déformation BH constante    
  MatLapack derResidu_adefConstant; // dérivé du résidu de l'équation constitutive, à contrainte cte
  Algo_zero  alg_zero; // algo pour la recherche de zero

  // ... partie relative à une résolution de l'avancement par une intégration de l'équation différentielle
  Algo_edp   alg_edp;
  int cas_kutta; // indique le type de runge_kutta que l'on veut utiliser
  double erreurAbsolue,erreurRelative; // précision absolue et relative que l'on désire sur le calcul de sig_tdt      
  int nbMaxiAppel; // nombre maxi d'appel de la fonction dérivée
	 Vecteur sig_point; // vitesse de sig: version vecteur de sigma_point
	 Tenseur3BH sigma_pointBH; // idem mais en tenseur
	 Tenseur3BH sigma_tauBH; // valeur de sigma pour le temps tau
	 Vecteur sigma_tau; // valeur de sigma pour le temps tau, en vecteur	
  Tenseur3BH delta_sigma_barre_R_a_tauBH; // delta sigma de R à tau
	 Tenseur3BH delta_sigma_barre_t_a_tauBH;  // delta sigma barre de t à tau
  Tenseur3BH betaphideltasigHB,deuxmudeltaepsHB; // pour le calcul de la dérivée
  Vecteur sig_initiale,dersig_initiale; // pour CalculContrainte_tdt
  Vecteur sig_finale,dersig_finale;     //    "         "
  Vecteur estime_erreur;
 
  // -- variables de travail internes à Residu_constitutif() et Mat_tangente_constitutif()
  // définit ici pour éviter de les définir à chaque passage ds la méthode,
  // ne doivent pas être utilisée en dehors de ces deux routines
	 Tenseur3BH rdelta_sigma_barre_BH_Ratdt;  // deltat sigma barre de R a tdt
	 Tenseur3BH rdelta_sigma_barre_tdt_BH;  // delta sigma barre de t à tdt
	 
	 // variable de travail commune à Mat_tangente_constitutif() et Dsig_d_ddl et Calcul_DsigmaHH_tdt
	 bool calcul_dResi_dsig; // indique si oui ou non le calcul de la matrice a été effectué
	 
	 // -- variables de travail pour la méthode: Dsig_d_ddl
	 MatLapack dRdsigMoinsun; // inverse de der_at_racine
	 MatLapack dRdeps; // variation du résidu constitutif / aux déformations
	 MatLapack MPC; // produit de dRdsigMoinsun et de dRdeps
  // -- variable de travail pour Hysteresis3D::Dsig_depsilon
  TenseurQ3geneHHHH dsig_ojef_HHHH;

	    
	 // -- variables internes à la méthode Dsig_d_Runge
	 MatLapack matriceH, matriceM,matHmoinsun; 
 
     // --------------- méthodes internes ---------------:
	 // 1)  affinage d'un point de coincidence à l'aide des rayons
	 // ramène true si le traitement est exactement terminé, sinon false, ce qui signifie qu'il
	 // faut encore continuer à utiliser l'équation d'évolution
	 // pt_sur_principal : indique si oui ou non le pointeur iatens pointe sur sa liste principale
	 // oi_sur_principal : indique si oui ou non le pointeur iaOi pointe sur sa liste principale
	 // iatens_princ et iafct_princ: pointeurs sur les listes principales
  // force_coincidence : on force la coincidence à la position actuelle
  //                    cas très particulier où l'algo normal n'aboutie pas, et que le rayon finale est 
  //                    plus grand que le dernier rayon de référence
  // le rayon peut-être modifié dans le cas particulier d'une coincidence à la précision près
	 bool Coincidence(bool bascule_fct_aide,double& unSur_wBaseCarre
                   ,const Tenseur3HH & gijHH,const Tenseur3BB & gijBB
	                  ,SaveResulHysteresis3D & save_resul,double& W_a
	                  ,List_io <Tenseur3BH>::iterator& iatens
                   ,List_io <double>::iterator& iadef_equi
                   ,List_io <double>::iterator& iafct
                   ,bool& pt_sur_principal
                   ,List_io <Tenseur3BH>::iterator& iaOi,bool& oi_sur_principal
                   ,List_io <Tenseur3BH>::iterator& iatens_princ
                   ,List_io <double>::iterator& iadef_equi_princ
                   ,List_io <Tenseur3BH>::iterator& iaOi_princ
                   ,List_io <double>::iterator& iafct_princ
                   ,const double& rayon_tdt,bool force_coincidence);
 
	 // 2) affinage d'un point de coincidence à l'aide de la fonction d'aide
	 // ramène true si le traitement est exactement terminé, sinon false, ce qui signifie qu'il
	 // faut encore continuer à utiliser l'équation d'évolution
	 // premiere_charge : indique si c'est oui ou non une coincidence avec la première charge
	 // pt_sur_principal : indique si oui ou non les pointeurs iafct et iatens pointent sur les listes
	 // principales
	 // iatens_princ et iafct_princ: pointeurs sur les listes principales
	 bool Coincidence(bool bascule_fct_aide, double& unSur_wprimeCarre
                   ,const Tenseur3HH & gijHH,const Tenseur3BB & gijBB
                   ,SaveResulHysteresis3D & save_resul,double& W_a
	                  ,List_io <Tenseur3BH>::iterator& iatens
                   ,List_io <double>::iterator& iadef_equi
                   ,List_io <double>::iterator& iafct
	                  ,bool& pt_sur_principal
                   ,List_io <Tenseur3BH>::iterator& iaOi,bool& oi_sur_principal
                   ,List_io <Tenseur3BH>::iterator& iatens_princ
                   ,List_io <double>::iterator& iadef_equi_princ
                   ,double& position_coincidence
                   ,List_io <Tenseur3BH>::iterator& iaOi_princ
	                  ,List_io <double>::iterator& iafct_princ,double& delta_W_a,bool force_coincidence);
 
	
	
  // --------------- codage des METHODES VIRTUELLES  protegees ---------------:
 // calcul des contraintes a t+dt
        // calcul des contraintes 
 void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
     ,TenseurBB & delta_epsBB_
     ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
     ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Expli_t_tdt& ex);

       // calcul des contraintes et de ses variations  a t+dt
 void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
     ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
     ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
     ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
     ,Tableau <TenseurBB *>& d_gijBB_tdt
		  	,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
		  	,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Impli& ex);
		  	
        // calcul des contraintes et ses variations  par rapport aux déformations a t+dt
        // en_base_orthonormee:  le tenseur de contrainte en entrée est  en orthonormee
        //                  le tenseur de déformation et son incrémentsont également en orthonormees
        //                 si = false: les bases transmises sont utilisées
        // ex: contient les éléments de métrique relativement au paramétrage matériel = X_(0)^a
 void Calcul_dsigma_deps (bool en_base_orthonormee, TenseurHH & sigHH_t,TenseurBB& DepsBB
     ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Umat_cont& ex) ; //= 0;


  // calcul de grandeurs de travail aux points d'intégration via la def
  // fonction surchargée dans les classes dérivée si besoin est
  virtual void CalculGrandeurTravail
                (const PtIntegMecaInterne& ,const Deformation &
                 ,Enum_dure,const ThermoDonnee&
                 ,const Met_abstraite::Impli* ex_impli
                 ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
                 ,const Met_abstraite::Umat_cont* ex_umat
                 ,const List_io<Ddl_etendu>* exclure_dd_etend
                 ,const List_io<const TypeQuelconque *>* exclure_Q
                 ) {};

public:		  			  	
		// calcul de la fonction résidu de la résolution de l'équation constitutive
  // l'argument test ramène
  //     . 1 si le calcul a été ok, -1 s'il y a eu un pb, mais on peut continuer, 0 s'il y a eu un pb
  //       fatal, qui invalide le calcul du résidu
		Vecteur& Residu_constitutif (const double & alpha, const Vecteur & x, int& test);
		// calcul de la matrice tangente de la résolution de l'équation constitutive
  // l'argument test ramène
  //     . 1 si le calcul a été ok, -1 s'il y a eu un pb, mais on peut continuer, 0 s'il y a eu un pb
  //       fatal, qui invalide le calcul du résidu et de la dérivée
		Mat_abstraite& Mat_tangente_constitutif(const double & alpha,const Vecteur & x, Vecteur& resi, int& test);
  // calcul de l'opérateur tangent : dsigma/depsilon, dans le cas d'un repère ortho-normé ou non
  TenseurHHHH& Dsig_depsilon(bool en_base_orthonormee,const Tenseur3HH & gijHH_tdt , TenseurHHHH& dsig_deps
                             ,TenseurHH & sigHH);
		// calcul de l'opérateur tangent : dsigma/d_ddl
		void Dsig_d_ddl(Tableau <TenseurHH *>& d_sigHH,TenseurBB & epsBB_tdt
               ,TenseurHH & gijHH_tdt,Tableau <TenseurBB *>& d_gijBB_tdt
               ,Tableau <TenseurHH *>& d_gijHH_tdt,Tableau <TenseurBB *>& d_epsBB );
		// calcul de l'expression permettant d'obtenir la dérivée temporelle de la contrainte
		// à un instant tau quelconque, en fait il s'agit de l'équation constitutive
		// utilisée dans la résolution explicite (runge par exemple) de l'équation constitutive
  //          sinon (diff de 0) indique que les valeurs calculées ne sont pas licite
  // erreur : =0: le calcul est licite, si diff de 0, indique qu'il y a eu une erreur
  //          =1:  la norme de sigma est supérieure à la valeur limite de saturation
		Vecteur& Sigma_point(const double & tau, const Vecteur & sigma_tau,Vecteur& sig_point,int& erreur);
  // vérification de l'intégrité du sigma calculé
  // erreur : =0: le calcul est licite, si diff de 0, indique qu'il y a eu une erreur
  //          =1:  la norme de sigma est supérieure à la valeur limite de saturation
  void Verif_integrite_Sigma(const double & tau, const Vecteur & sigma_tau,int & erreur);
  // ramène le type de transport
  int Type_de_transport_tenseur() const {return type_de_transport_memorisation;};
	
protected:
  // --- définition de différentes fonctions utiles 	pour le calcul final
  // calcul de wprime en fonction de grandeurs supposées déjà calculées
  // utilise : Q_OiaR, QdeltaSigmaRatdt, 
  //           delta_sigma_barre_BH_Ratdt, delta_sigma_barre_BH_OiaR
  //           wBase
  // trajet_neutre: indique si oui ou non, on est dans un chargement neutre à la précision près
  void Cal_wprime(const double& QdeltaSiRatdt,const Tenseur3BH& delta_sig_barre_BH_Ratdt
                        ,const double& Q__OiaR,const Tenseur3BH& delta_sig_barre_BH_OiaR 
                        ,const double& w_Base,Tenseur3BH& rdelta_sigma_barre_tdt_BH
                        , double& w_prime, double& w_prime_point,bool& trajet_neutre) const;
  // calcul de wprime et de sa variation par rapport aux coordonnées de delta sigma barre
  // trajet_neutre: indique si oui ou non, on est dans un chargement neutre à la précision près
  void Cal_wprime_et_der(const double& QdeltaSiRatdt,const Tenseur3BH& delta_sig_barre_BH_Ratdt
                        ,const double& Q__OiaR,const Tenseur3BH& delta_sig_barre_BH_OiaR 
                        ,const double& w_Base,Tenseur3BH& rdelta_sigma_barre_tdt_BH
                        ,double& w_prime, Tenseur3BH& d_w_prime
                        ,double& w_prime_point,Tenseur3BH& d_wprime_point,bool& trajet_neutre) const;
  // calcul du centre d'inversion avec les données actuelles
  // ramène un indicateur informant comment l'opération s'est passé et si elle est valide (= 0)
  int CentreCercle(Tenseur3BH& sig0i_BH,const Tenseur3BH& oc_BH_R
                          ,const Tenseur3BH & delta_sigma_barre_BH_OiaR
                          ,const Tenseur3BH& delta_sigma_barre_BH_Rat);
  // gestion d'un dépilement des pointeurs dans les listes, dans le cas d'une coïncidence
  // met à jour les booléens interne à l'instance : aucun_pt_inversion et centre_initial
  void Gestion_pointeur_coincidence(double& unSur_wBaseCarre
	                    ,SaveResulHysteresis3D & save_resul,double& W_a
	                    ,List_io <Tenseur3BH>::iterator& iatens
	                    ,List_io <double>::iterator& iadef_equi
                     ,List_io <double>::iterator& iafct
                     ,bool& pt_sur_principal
	                    ,List_io <Tenseur3BH>::iterator& iaOi,bool& oi_sur_principal
	                    ,List_io <Tenseur3BH>::iterator& iatens_princ
	                    ,List_io <double>::iterator& iadef_equi_princ
	                    ,List_io <Tenseur3BH>::iterator& iaOi_princ
                     ,List_io <double>::iterator& iafct_princ);
  // gestion du paramètre "modif" de saveresult
  // inversion = true: on met à jour après une inversion
  //           = false : on met à jour après une coïncidence
  void Gestion_para_Saveresult_Modif
                 (const bool& pt_sur_principal,SaveResulHysteresis3D & save_resul
                  ,const bool& inversion
                 );

	 // traitement des cas où des autres coincidences existent à la précision près
	 // ramène true si effectivement il y a une nouvelle coincidence               
	 bool Autre_coincidence_a_la_tolerance_pres(bool bascule_fct_aide
                      ,const Tenseur3HH & gijHH,const Tenseur3BB & gijBB
                      ,const bool& pt_sur_principal,const bool& oi_sur_principal
	                     ,List_io <Tenseur3BH>::iterator& iatens
                      ,List_io <Tenseur3BH>::iterator& iaOi);
  // calcul de la première phase de l'opérateur tangent, est utilisé par Dsig_d_ddl et
  // Dsig_depsilon
  // -> calcul de la matrice M
  void Dsig_d_(MatLapack& MPC);
  // idem que Dsig_d_, mais pour le cas où l'on calcul l'avancement à l'aide d'un runge_Kutta
  // -> calcul de la matrice M
  void Dsig_d_Runge(MatLapack& MPC);
  
  // calcul de l'avancement temporel sur 1 pas,
  // utilisé par les 3 programmes principaux: 
  // Calcul_SigmaHH, Calcul_DsigmaHH_tdt, Calcul_dsigma_deps,
  // int cas: =1 : normal, on symétrise le tenseur des contraintes en fonction de la dérivée
  //               de Jauman
  // int cas = 2 : pas de symétrisation du tenseur des contraintes à la fin du calcul
  void Avancement_temporel(const Tenseur3BB & delta_epsBB,const Tenseur3HH & gijHH
                           ,const Tenseur3BB & gijBB,int cas,SaveResulHysteresis3D & save_resul
                           ,Tenseur3HH & sigHH,const EnergieMeca & energ_t,EnergieMeca & energ);
        
  // initialisation éventuelle des variables thermo-dépendantes
  void Init_thermo_dependance();		  	
		// méthode permettant le calcul de sigma à tdt par différente méthodes: linéarisation
		// ou kutta
        // en sortie calcul de : 
        //    sigma_t_barre_tdt, delta_sigma_barre_tdt_BH, delta_sigma_barre_BH_Ratdt
        //    également les grandeurs: QdeltaSigmaRatdt, et wprime,wprime_point, calculées à tdt
        // ramène en retour la valeur de phi_2_dt
		void  CalculContrainte_tdt(double & phi_2_dt,Tableau<double>& indicateurs_resolution);
		// update des différentes énergies sur le pas de temps ou la partie de pas de temps
		// effectué,  et calcul de defEqui finalisée
  void UpdateEnergieHyste(EnergieMeca & energ
		       ,const Tenseur3BH & sigma_deb_BH,const Tenseur3BH & sigma_fin_BH
		       ,const Tenseur3BH & delta_sigma_sur_tau_BH
		       ,const Tenseur3BH & delta_sigma_R_tau_BH
		       ,const double& phi_2_dt
		       ,const Tenseur3BH & delta_eps_sur_deltaTau_BH,double& QdeltaSigmaRatdt
		       ,const double& wprime,const double& w_prime_point,const bool& trajet_neutre
		       ,const double& def_equi_R,const double& def_i_equi, double& def_equi);
		// calcul de la déformation cumulée, et de la déformation maxi à prendre en compte 
		void  DefEqui_et_maxi(double& QdeltaSigmaRatdt,const bool& trajet_neutre,double& defEquiMaxi
		                    ,const double& def_equi_R,const double& def_i_equi
								  , double& def_equi,const double & phi_2_dt,const double& wprime);				 
		// calcul des paramètres matériaux dépendants de la phase de deltaSig_Oi^tdt
		// en entrée: les paramètres sans phase = paramètres lues ou fonction de la température
		// en sortie: les paramètres en tenant compte de la phase de delta_sigma_Oi_tdt
		void ParaMateriauxAvecPhaseDeltaSig(double & xnp,const Tenseur3BH&  delta_sigma_barre_BH_OiaR
		                           ,const Tenseur3BH& delta_sig_barre_BH_Ratdt
		                           , double & xmu, double& Qzero );
		// calcul des paramètres matériaux dépendants de la déformation équivalente
		// en entrée: les paramètres sans dépendance = paramètres lues ou fonction de la température ou phase
		// en sortie: les paramètres en tenant compte de la dépendance 
		void ParaMateriauxAvecDefEqui(double & xnp,const double& QdeltaSigmaRatdt,const double& phi_2_dt
		                           ,const double& wprime,const double& def_equi_R
		                           ,const bool& trajet_neutre,const double& def_i_equi,const double& defEqui_maxi);
		// calcul des paramètres matériaux dépendants de la phase de la déformation totale
		// en entrée: les paramètres sans phase = paramètres lues ou fonction de la température
		// en sortie: les paramètres en tenant compte de la phase de epsBH
		void ParaMateriauxAvecPhaseEpsilon(const Tenseur3HH&  gijHH,const Tenseur3BB&  epsBB
                                           ,double & xnp, double & xmu, double& Qzero );
        
  // affichage des informations pour le débug
  void Affiche_debug(double& unSur_wBaseCarre,SaveResulHysteresis3D & save_resul
        ,List_io <Tenseur3BH>::iterator& iatens,List_io <double>::iterator& iadef_equi,bool& pt_sur_principal
        ,List_io <Tenseur3BH>::iterator& iaOi,bool& oi_sur_principal,const List_io <Tenseur3BH>::iterator& iatens_princ
	       ,const List_io <double>::iterator& iadef_equi_princ,const List_io <Tenseur3BH>::iterator& iaOi_princ);
  // vérif de la coïncidence: prog de mise au point
  // premiere_fois: comme il y a une récursion, pour arrêter la boucle infinie        
  void Verif_coincidence(const Tenseur3BB & delta_epsBB,const Tenseur3HH & gijHH
                         ,const Tenseur3BB & gijBB,int cas,SaveResulHysteresis3D & save_resul
                         ,Tenseur3HH & sigHH,const EnergieMeca & energ_t,EnergieMeca & energ
                         ,bool premiere_fois);
  // Transport des différentes grandeurs en fonction de la variable : type_de_transport_memorisation
  // coordonnées initiales du tenseur en mixte: aBH,
  // coordonnées finales: rBH : donc correspondant au tenseur transporté en l'état actuel
  void Transport_grandeur(const Tenseur3BB&  gijBB, const Tenseur3HH&  gijHH
                        ,const Tenseur3BH& aBH, Tenseur3BH & rBH);
  // transport éventuel de toutes les grandeurs tensorielles mémorisées dans save_resul
  void Transport_grandeur(const Tenseur3BB&  gijBB, const Tenseur3HH&  gijHH,SaveResulHysteresis3D & save_resul);
 
 
};
/// @}  // end of group


#endif		

		

