// FICHIER : Hysteresis_bulk.h
// CLASSE : Hysteresis_bulk


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        06/03/2023                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *   BUT:   La classe Hysteresis_bulk permet de calculer une contrainte *
 *   avec une évolution hystérétique non visqueuse  et ses derivees     *
 *   pour comportement purement sphérique.                              *
 * (cf celle développée par Guelin, Favier, Pegon pour le déviatorique.)*
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     * 
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef HYSTERESIS_BULK_H
#define HYSTERESIS_BULK_H


#include "Loi_comp_abstraite.h"
#include "Courbe1D.h"
#include "TypeConsTens.h"
#include "Algo_zero.h"
#include "TenseurQ1gene.h"
#include "Algo_edp.h"


/// @addtogroup Les_lois_hysteresis
///  @{
///

class Hysteresis_bulk : public Loi_comp_abstraite
{
    public :
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		Hysteresis_bulk ();
		
		
		// Constructeur de copie
		Hysteresis_bulk (const Hysteresis_bulk& loi) ;
		
		// DESTRUCTEUR :
		
		~Hysteresis_bulk ();
		
		// initialise les donnees particulieres a l'elements
		// de matiere traite ( c-a-dire au pt calcule)
		// Il y a creation d'une instance de SaveResul particuliere
		// a la loi concernee
		// la SaveResul classe est remplie par les instances heritantes
		// le pointeur de SaveResul est sauvegarde au niveau de l'element
		// c'a-d que les info particulieres au point considere sont stocke
		// au niveau de l'element et non de la loi.
		class SaveResulHysteresis_bulk: public SaveResul
		 { public : 
	        SaveResulHysteresis_bulk(); // constructeur par défaut :
	        SaveResulHysteresis_bulk(const SaveResulHysteresis_bulk& sav); // de copie
	        ~SaveResulHysteresis_bulk(); // destructeur
		       // définition d'une nouvelle instance identique
		       // appelle du constructeur via new
		       SaveResul * Nevez_SaveResul() const{return (new SaveResulHysteresis_bulk(*this));};
         // affectation
         virtual SaveResul & operator = ( const SaveResul & a);
	        //============= lecture écriture dans base info ==========
            // cas donne le niveau de la récupération
            // = 1 : on récupère tout
            // = 2 : on récupère uniquement les données variables (supposées comme telles)
	        void Lecture_base_info (ifstream& ent,const int cas);
            // cas donne le niveau de sauvegarde
            // = 1 : on sauvegarde tout
            // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	        void Ecriture_base_info(ofstream& sort,const int cas);
	        
	        // mise à jour des informations transitoires 
	        void TdtversT();
	        void TversTdt(); 
			  
			      // affichage à l'écran des infos
			      void Affiche() const;
   
         //changement de base de toutes les grandeurs internes tensorielles stockées
         // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
         // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
         // ici il n'y a pas de données tensorielles donc rien n'a faire
         // gpH(i) = gamma(i,j) * gH(j)
         virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma){};
    
         // procedure permettant de completer éventuellement les données particulières
         // de la loi stockées
         // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
         // completer est appelé apres sa creation avec les donnees du bloc transmis
         // peut etre appeler plusieurs fois
         SaveResul* Complete_SaveResul(const BlocGen & bloc, const Tableau <Coordonnee>& tab_coor
                                       ,const Loi_comp_abstraite* loi) {};

	        // ---- méthodes spécifiques
	        // initialise les informations de travail concernant le pas de temps en cours 
	        void Init_debut_calcul();
	        
	        // données protégées	        
	        double MPr_t; // dernière MPr à t
	        double MPr_tdt; // MPr en cours à tdt
	        double fonction_aide_t; // dernière valeur de la fonction d'aide 
	        double fonction_aide_tdt; // valeur de la fonction d'aide en cours
	        double wprime_t,wprime_tdt; // paramètre de masing
	        List_io <double>::iterator ip2; // adresse éventuelle du 2 éléments de fct_aide
	        
	        // --- informations de travail concernant le pas de temps en cours ---
	        int modif; // = 0 rien de changé, =1 coincidence(s), =2  inversion(s), =3 coin et inver
	        // liste des nouvelles MPrs de référence, qui sont apparu
	        List_io <double> MPr_R_t_a_tdt;
	        int nb_coincidence; // nombre de coincidence durant le pas de temps 
	        List_io <double> fct_aide_t_a_tdt; // liste des valeurs de la fonction d'aide durant le pas
	                                           // de temps
	        List_io <int> indic_coin; // liste d'indicateurs indiquant la suite des coincidences et inversions,
	                      // = 1 -> indique que c'est une coincidence avec un seul dépilage
                       // = 2 -> indique que c'est une coincidence avec deux dépilages
                       // = 0 -> c'est une inversion
	        // --- fin informations de travail concernant le pas de temps en cours ---
	        
	        // --- informations de mémorisation discrète de 0 à t
	        // le dernier élément rangé est en .begin() (c-a-d front())
	        List_io <double> MPr_R; // liste des MPrs de référence
	        List_io <double> fct_aide; // liste des valeurs de la fonction d'aide
    
         // 5) --- tableau d'indicateur de la résolution, éventuellement vide
         //     cela dépend de sortie_post
         //  indicateurs_resolution(1) : nb d'incrément utilisé pour la résolution de l'équation linéarisée
         //                        (2) : nb total d'itération "   "       "                 "      "
         //                        (3) : cas Runge Kutta: nb d'appel de la fonction
         //                        (4) : cas Runge Kutta: nb de step de calcul
         //                        (5) : cas Runge Kutta: erreur globale de la résolution
         Tableau <double> indicateurs_resolution,indicateurs_resolution_t;

         // --- gestion d'une map de grandeurs quelconques éventuelles ---
         // une map de grandeurs quelconques particulière qui peut servir aux classes appelantes
         // il s'agit ici d'une map interne qui a priori ne doit servir qu'aux class loi de comportement
         // un exemple d'utilisation est une loi combinée qui a besoin de grandeurs spéciales définies
         // -> n'est pas sauvegardé, car a priori il s'agit de grandeurs redondantes
         map <  EnumTypeQuelconque , TypeQuelconque, std::less < EnumTypeQuelconque> > map_type_quelconque;

         // récupération des type quelconque sous forme d'un arbre pour faciliter la recherche
         const map <  EnumTypeQuelconque , TypeQuelconque, std::less < EnumTypeQuelconque> >* Map_type_quelconque()
               const {return &map_type_quelconque;};
       private:
         void Mise_a_jour_map_type_quelconque();
    
         // ---- fin gestion d'une liste de grandeurs quelconques éventuelles ---
    
   };

		SaveResul * New_et_Initialise();
 
	    // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();
 
  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);
  // récupération d'un module de compressibilité équivalent à la loi, ceci pour un chargement nul
  // il s'agit ici de la relation - MPr = sigma_trace/3. = module de compressibilité * I_eps
  virtual double Module_compressibilite_equivalent(Enum_dure temps,const Deformation & def);
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const
   { cout << "\n Hysteresis_bulk::HsurH0(.. , methode non implante pour l'instant ";
      Sortie(1);
    };
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new Hysteresis_bulk(*this)); };
		  	  
	 //----- lecture écriture de restart -----
	 // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);

     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info_loi(ofstream& sort,const int cas);
          
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);

  // récupération des grandeurs particulière (hors ddl )
  // correspondant à liTQ
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void Grandeur_particuliere
       (bool absolue,List_io<TypeQuelconque>& ,Loi_comp_abstraite::SaveResul * ,list<int>& decal) const ;
  // récupération de la liste de tous les grandeurs particulières
  // ces grandeurs sont ajoutées à la liste passées en paramètres
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void ListeGrandeurs_particulieres(bool absolue,List_io<TypeQuelconque>& ) const;
 
  // activation du stockage de grandeurs quelconques qui pourront ensuite être récupéré
  // via le conteneur SaveResul, si la grandeur n'existe pas ici, aucune action
		virtual void Activation_stockage_grandeurs_quelconques(list <EnumTypeQuelconque >& listEnuQuelc);
 
  // insertion des conteneurs ad hoc concernant le stockage de grandeurs quelconques
  // passée en paramètre, dans le save result: ces conteneurs doivent être valides
  // c-a-d faire partie de listdeTouslesQuelc_dispo_localement
  virtual void Insertion_conteneur_dans_save_result(SaveResul * saveResul);
 
	protected :

     // donnees protegees
     // ---- paramètres matériaux ----
     double xnp; // paramètre de Prager
	    Courbe1D* xnp_temperature; // courbe éventuelle d'évolution de xnp en fonction de la température
     //    int cas_prager; // indique si xnp =2 (=1), ou est compris entre 2 et 3 (=2), ou est sup à 3 (=3)
     double Qzero; // limite de plasticité du critère de von mises
	    Courbe1D* Qzero_temperature; // courbe éventuelle d'évolution de Qzero en fonction de la température
     double xmu; // pente à l'origine
	    Courbe1D* xmu_temperature; // courbe éventuelle d'évolution de xmu en fonction de la température
 
	    // ---- paramètres de l'algorithme -----
	    double  tolerance_residu; // tolérance absolu sur la résolution de la plasticité
	    double  tolerance_residu_rel; // tolérance relative sur la résolution de la plasticité
     double maxi_delta_var_sig_sur_iter_pour_Newton; // le maxi de variation que l'on tolère d'une itération à l'autre
	    double  tolerance_coincidence; // tolérance sur la précision de la coincidence
	    int nb_boucle_maxi; // le maximum d'itération de plasticité permis
	    int nb_sous_increment; // le maxi de sous incrément prévu
     int type_resolution_equa_constitutive; // linéarisation ou kutta par exemple
     int nb_maxInvCoinSurUnPas; // nombre maximum d'inversion ou de coïncidence sur un pas
     double depassement_Q0; // valeur en relatif de dépassement permis sur la saturation: devrait = 1, mais dans les faits il faut laisser une marge (1.2 par exe)
 
     // ----- controle de la sortie des informations
// -> maintenant définit dans LoiAbstraiteGeneral
//     int permet_affichage; // pour permettre un affichage spécifique dans les méthodes, pour les erreurs et warning
     int sortie_post; // permet d'accèder au nombre d'itération, d'incrément, de précision etc. des résolutions
           // = 0 par défaut,
           // = 1 : on stocke toutes les grandeurs et elles sont disponibles en sortie

     // on introduit un certain nombre de tenseur du quatrième ordre, qui vont nous servir pour
     // Calcul_dsigma_deps, dans le cas où on n'est pas en orthonormee
		   Tenseur3HHHH  I_x_I_HHHH,I_xbarre_I_HHHH,I_x_eps_HHHH,Ixbarre_eps_HHHH;
 
	    // variables de travail pour l'échange entre les différentes méthodes en internes
	    double MPr_t___tdt; // - pression finale
	    double MPr_i___;   // - pression de début de calcul (à t au début)
	    double MPr_R;   // - pression de Référence  en cours
	    double delta_MPr_Rat;  // deltat (- pression) de R a t
	    double delta_MPr_Ratdt;  // deltat ( - pression) de R a tdt
	    double delta_MPr_tatdt;  // delta ( - pression) de t à tdt
	    double residuBH;
	    double delta_V;  // delta V totale
	    double delta__alpha_V;  // delta V intermediaire (avec alpha de 0 à 1)
	    double wprime; // paramètre de masing	    
	    Vecteur residu;  // résidu de l'équation pour la résolution de l'équation constitutive
	    Mat_pleine derResidu; // dérivé du résidu de l'équation pour la résolution de l'équation constitutive	    
     Algo_zero  alg_zero; // algo pour la recherche de zero	
     // ... partie relative à une résolution de l'avancement par une intégration de l'équation différentielle
     Algo_edp   alg_edp;
     int cas_kutta; // indique le type de runge_kutta que l'on veut utiliser
	    double erreurAbsolue,erreurRelative; // précision absolue et relative que l'on désire sur le calcul de sig_tdt      
     int nbMaxiAppel; // nombre maxi d'appel de la fonction dérivée
	    Vecteur MPr_point_; // vitesse de - pression: version vecteur de - pression_point
	    double MPr_point; // idem mais en scalaire
	    double MPr_tau; // valeur de la MPr pour le temps tau
	    Vecteur MPr_tau_vect; // valeur de la MPr pour le temps tau, en vecteur
     double delta_MPr_R_a_tau; // delta MPr de R à tau
 
     // -- variables de travail internes à Residu_constitutif() et Mat_tangente_constitutif()
     // définit ici pour éviter de les définir à chaque passage ds la méthode,
     // ne doivent pas être utilisée en dehors de ces deux routines
	    double rdelta_MPr_Ratdt;  // deltat MPr de R a tdt
	    double rdelta_MPr_tatdt;  // delta MPr de t à tdt
 
	    bool aucun_pt_inversion; // pour gérer le fait qu'au début il n'y a pas de pt d'inversion !
 
        // --------------- méthodes internes ---------------:
	    // affinage d'un point de coincidence
	    // ramène true si le traitement est exactement terminé, sinon false, ce qui signifie qu'il
	    // faut encore continuer à utiliser l'équation d'évolution
	    // premiere_charge : indique si c'est oui ou non une coincidence avec la première charge
	    // pt_sur_principal : indique si oui ou non les pointeurs iafct et iatens pointent sur les listes
	    // principales
	    // iatens_princ et iafct_princ: pointeurs sur les listes principales
	    bool Coincidence(bool & aucun_pt_inversion,double& unSur_wprimeCarre,bool premiere_charge
	                    ,SaveResulHysteresis_bulk & save_resul,double& W_a
	                    ,List_io <double>::iterator& iatens,List_io <double>::iterator& iafct
	                    ,bool& pt_sur_principal,List_io <double>::iterator& iatens_princ
	                    ,List_io <double>::iterator& iafct_princ,double& delta_W_a
                     ,bool force_coincidence,const double& MPr_tdt );
	    // cas particulier d'une inversion et coïncidence : affinage d'un point de coincidence
	    // ramène true si le traitement est exactement terminé, sinon false, ce qui signifie qu'il
	    // faut encore continuer à utiliser l'équation d'évolution
	    // premiere_charge : indique si c'est oui ou non une coincidence avec la première charge
	    // pt_sur_principal : indique si oui ou non les pointeurs iafct et iatens pointent sur les listes
	    // principales
	    // iatens_princ et iafct_princ: pointeurs sur les listes principales
	    bool Inversion_et_Coincidence(bool & aucun_pt_inversion,double& unSur_wprimeCarre
	                    ,SaveResulHysteresis_bulk & save_resul,double& W_a
	                    ,List_io <double>::iterator& iatens,const double& delta_MPr_tatdt
                     ,List_io <double>::iterator& iafct
	                    ,bool& pt_sur_principal,List_io <double>::iterator& iatens_princ
                     ,List_io <double>::iterator& iafct_princ,double& delta_W_a
                     ,bool force_coincidence,const double& MPr_tdt );

     // gestion d'un dépilement des pointeurs dans les listes, dans le cas d'une coïncidence
     // met à jour les booléens interne à l'instance : aucun_pt_inversion et centre_initial
     void Gestion_pointeur_coincidence(double& unSur_wprimeCarre
	                    ,SaveResulHysteresis_bulk & save_resul,double& W_a
                     ,bool & aucun_pt_inversion
	                    ,List_io <double>::iterator& iatens
                     ,List_io <double>::iterator& iafct
                     ,bool& pt_sur_principal
	                    ,List_io <double>::iterator& iatens_princ
                     ,List_io <double>::iterator& iafct_princ
                     ,const double& MPr_tdt);


     // gestion d'un dépilement des pointeurs dans les listes, dans le cas particulier
     // d'une inversion - coïncidence
     // met à jour les booléens interne à l'instance : aucun_pt_inversion et centre_initial
     void Gestion_pointeur_Inversion_et_Coincidence(double& unSur_wprimeCarre
	                    ,SaveResulHysteresis_bulk & save_resul,double& W_a
                     ,bool & aucun_pt_inversion
	                    ,List_io <double>::iterator& iatens
                     ,List_io <double>::iterator& iafct
                     ,bool& pt_sur_principal
	                    ,List_io <double>::iterator& iatens_princ
                     ,List_io <double>::iterator& iafct_princ
                     ,const double& MPr_tdt);

     // gestion du paramètre "modif" de saveresult
     // inversion = true: on met à jour après une inversion
     //           = false : on met à jour après une coïncidence
     void Gestion_para_Saveresult_Modif
                    (const bool& pt_sur_principal,SaveResulHysteresis_bulk & save_resul
                     ,const bool& inversion
                    );
	
     // --------------- codage des METHODES VIRTUELLES  protegees ---------------:
 // calcul des contraintes a t+dt
        // calcul des contraintes 
 void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
     ,TenseurBB & delta_epsBB_
     ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
     ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Expli_t_tdt& ex);

       // calcul des contraintes et de ses variations  a t+dt
 void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
     ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
     ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
     ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
     ,Tableau <TenseurBB *>& d_gijBB_tdt
		  	,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
		  	,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Impli& ex);
		  	
        // calcul des contraintes et ses variations  par rapport aux déformations a t+dt
        // en_base_orthonormee:  le tenseur de contrainte en entrée est  en orthonormee
        //                  le tenseur de déformation et son incrémentsont également en orthonormees
        //                 si = false: les bases transmises sont utilisées
        // ex: contient les éléments de métrique relativement au paramétrage matériel = X_(0)^a
 void Calcul_dsigma_deps (bool en_base_orthonormee, TenseurHH & sigHH_t,TenseurBB& DepsBB
     ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Umat_cont& ex) ; //= 0;


	
     // fonction surchargée dans les classes dérivée si besoin est
     virtual void CalculGrandeurTravail
                   (const PtIntegMecaInterne& ,const Deformation &
                    ,Enum_dure,const ThermoDonnee&
                    ,const Met_abstraite::Impli* ex_impli
                    ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
                    ,const Met_abstraite::Umat_cont* ex_umat
                    ,const List_io<Ddl_etendu>* exclure_dd_etend
                    ,const List_io<const TypeQuelconque *>* exclure_Q
                    ) {};

     // initialisation éventuelle des variables thermo-dépendantes
     void Init_thermo_dependance();
     // méthode permettant le calcul de la MPr à tdt par différente méthodes: linéarisation
     // ou kutta
     void CalculPression_tdt(Tableau<double>& indicateurs_resolution);
 
     // calcul de l'avancement temporel sur 1 pas,
     // utilisé par les 3 programmes principaux: 
     // Calcul_SigmaHH, Calcul_DsigmaHH_tdt, Calcul_dsigma_deps,
     // int cas: =1 : normal, on symétrise le tenseur des contraintes en fonction de la dérivée
     //               de Jauman
     // int cas = 2 : pas de symétrisation du tenseur des contraintes à la fin du calcul
     void Avancement_temporel(const Tenseur3HH & gijHH
                              ,const Tenseur3BB & gijBB,int cas,SaveResulHysteresis_bulk & save_resul
                              ,Tenseur3HH & sigHH,const EnergieMeca & energ_t,EnergieMeca & energ);
   public:
     // calcul de la fonction résidu de la résolution de l'équation constitutive
     // l'argument test ramène
     //     . 1 si le calcul a été ok, -1 s'il y a eu un pb, mais on peut continuer, 0 s'il y a eu un pb
     //       fatal, qui invalide le calcul du résidu
     Vecteur& Residu_constitutif (const double & alpha, const Vecteur & x, int& test);
     // calcul de la matrice tangente de la résolution de l'équation constitutive
     // l'argument test ramène
     //     . 1 si le calcul a été ok, -1 s'il y a eu un pb, mais on peut continuer, 0 s'il y a eu un pb
     //       fatal, qui invalide le calcul du résidu et de la dérivée
     Mat_abstraite& Mat_tangente_constitutif(const double & alpha,const Vecteur & x, Vecteur& resi, int& test);
     // calcul de l'opérateur tangent : dsigma/depsilon, dans le cas d'un repère ortho-normé ou non
     // T_d_pres_d_V: représente la variation de la MPr par rapport à V
     // dsig_deps : version 3D
     void Dsig_depsilon(double& T_d_pres_d_V,bool en_base_orthonormee,const Tenseur3HH & gijHH_tdt
                        , TenseurHHHH* dsig_deps,const double & V_tdt);
     // calcul de l'expression permettant d'obtenir la dérivée temporelle de la MPr
     // à un instant tau quelconque, en fait il s'agit de l'équation constitutive
     // utilisée dans la résolution explicite (runge par exemple) de l'équation constitutive
     // erreur : =0: le calcul est licite, si diff de 0, indique qu'il y a eu une erreur
     //          =1:  la norme de sigma est supérieure à la valeur limite de saturation
     Vecteur& Pression_point(const double & tau, const Vecteur & sigma_tau,Vecteur& sig_point,int & erreur);
     // vérification de l'intégrité de la MPr calculée
     // erreur : =0: le calcul est licite, si diff de 0, indique qu'il y a eu une erreur
     //          =1:  la norme de la MPr est supérieure à la valeur limite de saturation
     void Verif_integrite_Pression(const double & tau, const Vecteur & MPr_tau,int & erreur);
        
     // affichage des informations pour le débug
     void Affiche_debug(double& unSur_wBaseCarre,SaveResulHysteresis_bulk & save_resul
                       ,List_io <double>::iterator& iatens,bool& pt_sur_principal
                       ,const List_io <double>::iterator& iatens_princ
	                      );
	
};
/// @}  // end of group


#endif		

		

