

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        06/03/2023                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  La loi est 1D et est associée à une loi 3D quelconque      *
 *           L'objectif est de transformer une loi 3D en 1D contraintes *
 *           planes dans les 2 autres directions.			        												*
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
// FICHIER : LoiContraintesPlanesDouble.h
// CLASSE : LoiContraintesPlanesDouble

#ifndef LOICONTRAINTESPLANESDOUBLE_H
#define LOICONTRAINTESPLANESDOUBLE_H

#include "Loi_comp_abstraite.h"
#include "Enum_contrainte_mathematique.h"
#include "Algo_edp.h"
#include "MatLapack.h"
#include "LoiContraintesPlanes.h"


/// @addtogroup Les_lois_combinees
///  @{
///


class LoiContraintesPlanesDouble : public Loi_comp_abstraite
{
	public :
  friend class LoiContraintesPlanes;
  friend class LoiCritere;
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		LoiContraintesPlanesDouble ();
 
  // constructeur à partir d'une instance de contraintes planes
  // à condition que cette loi s'appuie sur un comportement 3D
  // les deux lois étant proche, les paramètres semblables sont mis en commun
  LoiContraintesPlanesDouble (LoiContraintesPlanes& loiCP_de3D
                       ,bool calcul_en_3D_via_direction_quelconque_);
 
		// Constructeur de copie
		LoiContraintesPlanesDouble (const LoiContraintesPlanesDouble& loi) ;
 
		// DESTRUCTEUR :
		~LoiContraintesPlanesDouble ();
		
		
		
		// initialise les donnees particulieres a l'elements
		// de matiere traite ( c-a-dire au pt calcule)
		// Il y a creation d'une instance de SaveResul particuliere
		// a la loi concernee
		// la SaveResul classe est remplie par les instances heritantes
		// le pointeur de SaveResul est sauvegarde au niveau de l'element
		// c'a-d que les info particulieres au point considere sont stocke
		// au niveau de l'element et non de la loi.
		class SaveResul_LoiContraintesPlanesDouble: public LoiContraintesPlanes::SaveResul_LoiContraintesPlanes
		 { public : 
	        SaveResul_LoiContraintesPlanesDouble(); // constructeur par défaut (a ne pas utiliser)
	        // le constructeur courant
	        SaveResul_LoiContraintesPlanesDouble(SaveResul* l_des_SaveResul);
	        // constructeur de copie 
	        SaveResul_LoiContraintesPlanesDouble(const SaveResul_LoiContraintesPlanesDouble& sav );
	        // destructeur
	        ~SaveResul_LoiContraintesPlanesDouble();
		       // définition d'une nouvelle instance identique
		       // appelle du constructeur via new 
		       SaveResul * Nevez_SaveResul() const {return (new SaveResul_LoiContraintesPlanesDouble(*this));};
         // affectation
         virtual SaveResul & operator = ( const SaveResul & a);
	        //============= lecture écriture dans base info ==========
            // cas donne le niveau de la récupération
            // = 1 : on récupère tout
            // = 2 : on récupère uniquement les données variables (supposées comme telles)
	        void Lecture_base_info (ifstream& ent,const int cas);
            // cas donne le niveau de sauvegarde
            // = 1 : on sauvegarde tout
            // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	        void Ecriture_base_info(ofstream& sort,const int cas);
	        
	        // mise à jour des informations transitoires en définitif s'il y a convergence 
	        // par exemple (pour la plasticité par exemple)
	        void TdtversT() ;
	        void TversTdt() ;
			  
			      // affichage à l'écran des infos
			      void Affiche() const;
    
         //changement de base de toutes les grandeurs internes tensorielles stockées
         // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
         // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
         // *** les métriques locales ne sont pas concernées: pour les modifier, il faut utiliser la méthode
         // *** Affectation_metriques_locales, et Recuperation_metriques_locales
         // gpH(i) = gamma(i,j) * gH(j)
         virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma);
    
         // procedure permettant de completer éventuellement les données particulières
         // de la loi stockées
         // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
         // completer est appelé apres sa creation avec les donnees du bloc transmis
         // peut etre appeler plusieurs fois
         SaveResul* Complete_SaveResul(const BlocGen & bloc, const Tableau <Coordonnee>& tab_coor
                                       ,const Loi_comp_abstraite* loi);

	        // ---- récupération d'information: spécifique à certaine classe dérivée
	        double Deformation_plastique() ;
    
         // transfert des infos d'épaisseur et de largeur d'une instance de même type
         void Transfert_var_hetb(const SaveResul_LoiContraintesPlanesDouble& sav)
           {bsurb0 = sav.bsurb0;b_tsurb0 = sav.b_tsurb0;
            LoiContraintesPlanes::SaveResul_LoiContraintesPlanes::Transfert_var_h(sav);
           };
    
         // mise à jour des métriques locales
         void Affectation_metriques_locale(Deformation::Stmet& met_ref_00
                                          ,Deformation::Stmet& met_ref_t
                                          ,Deformation::Stmet& met_ref_tdt)
             {meti_ref_00 = met_ref_00;meti_ref_t = met_ref_t;meti_ref_tdt=met_ref_tdt;};
    
         // mise à 0 des variations de largeur
         void Zero_var_largeur() {bsurb0=0.; };    
	        
	        // données protégées
         //1) toutes celles de LoiContraintesPlanes
    
         // -- infos relatives au repère de référence 3D pour l'élément 1D associé
         //    les repères sont orthogonaux, la direction 1 est celle de l'élément 1D
         Deformation::Stmet meti_ref_00,meti_ref_t,meti_ref_tdt; // les infos à 0 à t et à tdt
    
         // def méca dans un repère ortho de traction: 1 axe de traction, 2 épaisseur, 3 largeur
         // ordonnée suivant
         // def_P(1) -> eps_P(2,2), def_P(2) -> eps_P(3,3), def_P(3) -> eps_P(1,2)
         Vecteur def_P,def_P_t; // si taille nulle n'est pas alimenté
                   // est alimenté que si : calcul_en_3D_via_direction_quelconque == true

         // les élongations suivant la largeur:
         // valeur courante et valeur sauvegardée au pas précédent
         double bsurb0,b_tsurb0; // largeur
 //        Vecteur d_hsurh0;
          // vecteur de taille éventuellement nulle, contenant les variations de h / au ddl
                           // c'est un vecteur de travail, il n'est pas sauvegardé d'un incrément à l'autre
                           // il sert à mémoriser les choses d'une itération à l'autre
         Vecteur d_bsurb0; // idem pour la largeur
   };

        // def d'une instance de données spécifiques, et initialisation
		SaveResul * New_et_Initialise() ;
 
  friend class SaveResul_LoiContraintesPlanesDouble;

  // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
 
  // Lecture des paramètres particuliers de l'objet sur fichier
  // cette lecture est utilisée lorsque l'objet a été déjà défini
  // il s'agit donc d'une lecture à l'intérieur d'une autre loi par exemple
  void LectureParametres_controles (UtilLecture * ,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();

  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);
  // récupération d'un module de compressibilité équivalent à la loi, ceci pour un chargement nul
  // il s'agit ici de la relation -pression = sigma_trace/3. = module de compressibilité * I_eps
  double Module_compressibilite_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);
 
 
  // récupération de la dernière déformation d'épaisseur calculée: cette déformaion n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération de la def, sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double Eps33BH(SaveResul * saveResul) const
    { // retour de la def
      return eps_BB_3D(3,3);
    };
 
  // récupération de la dernière déformation de largeur calculée: cette déformaion n'est utile que pour des lois en contraintes doublement planes
  // - pour les lois 3D et 2D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // les infos nécessaires à la récupération de la def, sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double Eps22BH(SaveResul * saveResul) const
    { // retour de la def
      return eps_BB_3D(2,2);
    };

  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const 
    { // récup du conteneur spécifique
      SaveResul_LoiContraintesPlanesDouble & save_resul = *((SaveResul_LoiContraintesPlanesDouble*) saveResul);
      // retour de la variation relative
      return save_resul.hsurh0;
    };
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  // et de sa variation par rapport aux ddls la concernant: d_hsurh0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double d_HsurH0(SaveResul * saveResul,Vecteur & d_hsurh0) const
    { // récup du conteneur spécifique
      SaveResul_LoiContraintesPlanesDouble & save_resul = *((SaveResul_LoiContraintesPlanesDouble*) saveResul);
      d_hsurh0 = save_resul.d_hsurh0;
      // retour de la variation relative
      return save_resul.hsurh0;
    };
 
  // récupération de la variation relative de largeur calculée: b/b0
  //  cette variation n'est utile que pour des lois en contraintes planes double
  // - pour les lois 3D et 2D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double BsurB0(SaveResul * saveResul) const
    { // récup du conteneur spécifique
      SaveResul_LoiContraintesPlanesDouble & save_resul = *((SaveResul_LoiContraintesPlanesDouble*) saveResul);
      // retour de la variation relative
      return save_resul.bsurb0;
    };
 
  // récupération de la variation relative de largeur calculée: b/b0
  // et de sa variation par rapport aux ddls la concernant: d_bsurb0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D et 2D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double d_BsurB0(SaveResul * saveResul,Vecteur & d_bsurb0) const
    { // récup du conteneur spécifique
      SaveResul_LoiContraintesPlanesDouble & save_resul = *((SaveResul_LoiContraintesPlanesDouble*) saveResul);
      d_bsurb0 = save_resul.d_bsurb0;
      // retour de la variation relative
      return save_resul.bsurb0;
    };
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new LoiContraintesPlanesDouble(*this)); };
		  	
		// activation des données des noeuds et/ou elements nécessaires au fonctionnement de la loi
		// exemple: mise en service des ddl de température aux noeuds
		virtual void Activation_donnees(Tableau<Noeud *>& tabnoeud,bool dilatation,LesPtIntegMecaInterne& lesPtMecaInt);
  // récupération des grandeurs particulière (hors ddl )
  // correspondant à liTQ
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void Grandeur_particuliere
        (bool absolue,List_io<TypeQuelconque>& liTQ,Loi_comp_abstraite::SaveResul * saveDon,list<int>& decal) const;
  // récupération de la liste de tous les grandeurs particulières
  // ces grandeurs sont ajoutées à la liste passées en paramètres
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void ListeGrandeurs_particulieres(bool absolue,List_io<TypeQuelconque>& liTQ) const;
        
		  	  
	 //----- lecture écriture de restart -----
	 // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);

     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info_loi(ofstream& sort,const int cas);
          
      // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);

	protected :
	
  // codage des METHODES VIRTUELLES  protegees:
 // calcul des contraintes a t+dt
        // calcul des contraintes 
 void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
     ,TenseurBB & delta_epsBB_
     ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
     ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Expli_t_tdt& ex);

       // calcul des contraintes et de ses variations  a t+dt
 void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
     ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
     ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
     ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
     ,Tableau <TenseurBB *>& d_gijBB_tdt
		  	,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
		  	,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Impli& ex);
		  	
        // calcul des contraintes et ses variations  par rapport aux déformations a t+dt
        // en_base_orthonormee:  le tenseur de contrainte en entrée est  en orthonormee
        //                  le tenseur de déformation et son incrémentsont également en orthonormees
        //                 si = false: les bases transmises sont utilisées
        // ex: contient les éléments de métrique relativement au paramétrage matériel = X_(0)^a
  void Calcul_dsigma_deps (bool en_base_orthonormee, TenseurHH & sigHH_t,TenseurBB& DepsBB
     ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Umat_cont& ex) ; //= 0;
 
  // calcul des contraintes et ses variations  par rapport aux déformations a t+dt
  // ceci dans le cas où l'axe de traction simple est quelconque
  // ici on suppose que:
  //   - tous les calculs sont en dim 3
  //   - l'axe 3 est normal aux deux autres (ceci pour Vi et pour gi)
  //
  // ViB et ViH : correspond à la base de traction telle que ViB(1) = ViH(1) = l'axe de traction
  //              La base Vi est supposée orthonormée
  // en_base_orthonormee:  le tenseur de contrainte en entrée est  en orthonormee
  //                  le tenseur de déformation et son incrémentsont également en orthonormees
  //                 si = false: les bases transmises sont utilisées
  // ex: contient les éléments de métrique relativement au paramétrage matériel = X_(0)^a
  void Calcul_dsigma_deps (bool en_base_orthonormee,const BaseB * ViB,const BaseH * ViH
     ,const TenseurHH & sigHH_t,const TenseurBB& DepsBB
     ,const TenseurBB & epsBB_tdt,const TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
     ,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
     ,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
     ,const Met_abstraite::Umat_cont& ex) ; //= 0;
 
  // fonction surchargée dans les classes dérivée si besoin est
  virtual void CalculGrandeurTravail
          (const PtIntegMecaInterne& ptintmeca
           ,const Deformation & def,Enum_dure temps,const ThermoDonnee& dTP
           ,const Met_abstraite::Impli* ex_impli
           ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
           ,const Met_abstraite::Umat_cont* ex_umat
           ,const List_io<Ddl_etendu>* exclure_dd_etend
           ,const List_io<const TypeQuelconque *>* exclure_Q
          );

  // permet d'indiquer à la classe à quelle valeur de PtIntegMecaInterne il faut se référer
  // en particulier est utilisé par les lois additives,
  // par contre doit être utilisé avec prudence
  virtual void IndiquePtIntegMecaInterne(const PtIntegMecaInterne * ptintmeca)
    { lois_interne->IndiquePtIntegMecaInterne(ptintmeca);
      // puis la classe mère
      Loi_comp_abstraite::IndiquePtIntegMecaInterne(ptintmeca);
    };


  // fonction interne utilisée par les classes dérivées de Loi_comp_abstraite
  // pour répercuter les modifications de la température
  // ici utiliser pour modifier la température des lois élémentaires
  // l'Enum_dure: indique quel est la température courante : 0 t ou tdt
  void RepercuteChangeTemperature(Enum_dure temps);
 
  // retourne le type de méthode utilisée pour imposer la condition de contrainte plane
  Enum_contrainte_mathematique Type_de_contrainte() const {return type_de_contrainte;};
 
  // permet de changer la valeur de l'indicateur: bool calcul_en_3D_via_direction_quelconque;
  // *** cet indicateur agit sur les conteneurs de stockage, il faut donc le changer dès la construction
  // de l'instance: donc est utile lorsque l'on ne connait pas encore la valeur de
  // calcul_en_3D_via_direction_quelconque au moment de sa création
  void Change_calcul_en_3D_via_direction_quelconque (bool calcul_en_3D_via_direction_quelconque_)
    {calcul_en_3D_via_direction_quelconque = calcul_en_3D_via_direction_quelconque_;}
  
 
public:
            //--- cas de la résolution de l'équation sig33(hsurh0)=0
		// calcul de la fonction résidu de la résolution de l'équation constitutive
  // l'argument test ramène
  //     . 1 si le calcul a été ok, -1 s'il y a eu un pb, mais on peut continuer, 0 s'il y a eu un pb
  //       fatal, qui invalide le calcul du résidu
		Vecteur& Residu_constitutif (const double & alpha, const Vecteur & x, int& test);
		// calcul de la matrice tangente de la résolution de l'équation constitutive
  // l'argument test ramène
  //     . 1 si le calcul a été ok, -1 s'il y a eu un pb, mais on peut continuer, 0 s'il y a eu un pb
  //       fatal, qui invalide le calcul du résidu et de la dérivée
		Mat_abstraite& Mat_tangente_constitutif(const double & alpha,const Vecteur & x, Vecteur& resi, int& test);
            //--- cas de la résolution de l'équation sigij(eps_meca)*V_j{.l}=0, l=2 et 3
  // calcul de la fonction résidu de la résolution de l'équation constitutive
  // l'argument test ramène
  //     . 1 si le calcul a été ok, -1 s'il y a eu un pb, mais on peut continuer, 0 s'il y a eu un pb
  //       fatal, qui invalide le calcul du résidu
  Vecteur& Residu_constitutif_3D (const double & alpha, const Vecteur & x, int& test);
  // calcul de la matrice tangente de la résolution de l'équation constitutive
  // l'argument test ramène
  //     . 1 si le calcul a été ok, -1 s'il y a eu un pb, mais on peut continuer, 0 s'il y a eu un pb
  //       fatal, qui invalide le calcul du résidu et de la dérivée
  Mat_abstraite& Mat_tangente_constitutif_3D(const double & alpha,const Vecteur & x, Vecteur& resi, int& test);

private :
        // donnees protegees
		Enum_contrainte_mathematique type_de_contrainte; // def de la méthode utilisée pour imposer la condition de
		                                                 // de contrainte plane 
		// paramètre qui servent éventuellement pour la contrainte mathématique
  double fac_penal;  // le facteur de pénalisation relative,
		double prec; // précision sur la contrainte
		 																  			
	 Loi_comp_abstraite * lois_interne; // loi 3D  correspondante

  int sortie_post; // permet d'accèder au nombre d'itération, d'incrément, de précision etc. des résolutions
        // = 0 par défaut,
        // = 1 : on stocke toutes les grandeurs et elles sont disponibles en sortie
 
  bool calcul_en_3D_via_direction_quelconque; // indicateur qui permet d'utiliser la seconde méthode
    // de calcul des contraintes planes doubles: par défaut = false
  // --- variables particulières pour le cas ou on utilise une boucle de newton interne pour la contrainte plane
  Algo_zero  alg_zero; // algo pour la recherche de zero
  double maxi_delta_var_eps_sur_iter_pour_Newton; // le maxi de variation que l'on tolère d'une itération à l'autre
  // pilotage éventuel des précisions
  Fonction_nD * fct_tolerance_residu, * fct_tolerance_residu_rel;
 
  Vecteur val_initiale; // on démarre la recherche à la valeur à t
  Vecteur racine;            // dimensionnement init du résultat à 0.
  //Mat_pleine
  MatLapack der_at_racine; // dimensionnement et init de la matrice dérivée à 0.
	 Vecteur residu;  // résidu de l'équation  c'est à dire <sig22,sig33>
	 //Mat_pleine
  MatLapack derResidu; // dérivé du résidu de l'équation c-a-d d<sig22,sig33>/<dbsurb0,dhsurh0>
  double mini_hsurh0; // limitation de la variation de hsurh0
  double maxi_hsurh0; // limitation de la variation de hsurh0
  double mini_bsurb0; // limitation de la variation de bsurb0
  double maxi_bsurb0; // limitation de la variation de bsurb0
  // ----- controle de la sortie des informations
// -> maintenant définit dans LoiAbstraiteGeneral
//	 int permet_affichage; // pour permettre un affichage spécifique dans les méthodes,
                        // pour les erreurs et des warnings
  // -- pour le calcul de d_eps_eg_11
  Mat_pleine d_sig_ef_gh;
  Vecteur d_eps_ef_11;
  //
  double module_compressibilite_3D;
  double  module_cisaillement_3D;
  CoordonneeB  giB_normer_3_tdt_3D_sauve; // sauvegarde du vecteur giB(3), normé
  CoordonneeB  giB_normer_2_tdt_3D_sauve; // sauvegarde du vecteur giB(2), normé
  double sauve_jacobien1D_tdt;
 
  // déclaration des variables internes nécessaires pour les passages 1D - 3D
  // -- on définit des conteneurs pour le stockage des résultats des métriques, dimentionnés par défaut non vide
  // on utilise des pointeurs pour dimentionner après les variables internes
  Met_abstraite::Expli_t_tdt*  expli_3D;
  Met_abstraite::Impli* impli_3D;
  Met_abstraite::Umat_cont*  umat_cont_3D;
  
  // -- variables nécessaires pour la création de expli_3D, impli_3D et umat_cont_3D
  // certaines grandeurs sont associées à un pointeur qui peut soit être nulle soit pointer sur le conteneur
  // l'intérêt est que le fait d'avoir un pointeur nul est parfois utilisé pour éviter un calcul 
  BaseB giB_0_3D; 
  BaseH giH_0_3D; 
  BaseB giB_t_3D;
  BaseH giH_t_3D;
  BaseB  giB_tdt_3D;
  BaseH  giH_tdt_3D; 
  Tenseur3BB  gijBB_0_3D; 
  Tenseur3HH  gijHH_0_3D; 
  Tenseur3BB  gijBB_t_3D;
  Tenseur3HH  gijHH_t_3D;
  Tenseur3BB  gijBB_tdt_3D;
  Tenseur3HH  gijHH_tdt_3D;
  
  TenseurBB * gradVmoyBB_t_3D_P;		Tenseur_ns3BB  gradVmoyBB_t_3D;
  TenseurBB * gradVmoyBB_tdt_3D_P;	Tenseur_ns3BB  gradVmoyBB_tdt_3D;
  TenseurBB * gradVBB_tdt_3D_P;		Tenseur_ns3BB  gradVBB_tdt_3D;
  double jacobien_tdt_3D;double jacobien_t_3D;double jacobien_0_3D;  // pour les jacobiens on considère qu'ils existent toujours
  Vecteur d_jacobien_tdt_3D;
      // pour tous les tableaux de pointeurs, on double le tableau en déclarant un vrai tableau en //
  Tableau <BaseB>  d_giB_tdt_3D;
  Tableau <BaseH>  d_giH_tdt_3D;		
  Tableau <TenseurBB *> d_gijBB_tdt_3D_P;       Tableau <Tenseur3BB > d_gijBB_tdt_3D;
  Tableau2 <TenseurBB *>*  d2_gijBB_tdt_3D_P;    Tableau2 <Tenseur3BB >  d2_gijBB_tdt_3D; // a priori ne sera pas affecté, car ne sert
                                                                                          // dans les lois de comportement
  Tableau <TenseurHH *> d_gijHH_tdt_3D_P;       Tableau <Tenseur3HH > d_gijHH_tdt_3D;  
  Tableau <TenseurBB * >* d_gradVmoyBB_t_3D_P;   Tableau <Tenseur_ns3BB > d_gradVmoyBB_t_3D;
  Tableau <TenseurBB * >* d_gradVmoyBB_tdt_3D_P; Tableau <Tenseur_ns3BB > d_gradVmoyBB_tdt_3D;
  Tableau <TenseurBB * >* d_gradVBB_t_3D_P;      Tableau <Tenseur_ns3BB > d_gradVBB_t_3D;
  Tableau <TenseurBB * >* d_gradVBB_tdt_3D_P;    Tableau <Tenseur_ns3BB > d_gradVBB_tdt_3D;
								  
  // -- on définit les conteneurs pour les passages d'appels entrant de la loi 3D : donc en 3D par défaut
  Tenseur3HH  sig_HH_t_3D, sig_HH_3D ;
  Tenseur3BB  Deps_BB_3D, eps_BB_3D, delta_eps_BB_3D;
  Tableau <TenseurBB *> d_eps_BB_3D_P;          Tableau <Tenseur3BB > d_eps_BB_3D; // le tableau de pointeur puis les vrais grandeurs
  Tableau <TenseurHH *> d_sig_HH_3D_P;          Tableau <Tenseur3HH > d_sig_HH_3D; //  """"
  Tenseur3HHHH d_sigma_deps_3D;
  
  Tenseur1HHHH d_sigma_deps_1D; // un tenseur de travail pour  la méthode Calcul_dsigma_deps (
     // pour les méthodes: Mat_tangente_constitutif et Residu_constitutif
 
  // cas d'un point d'intégration locale (méthode CalculGrandeurTravail par exemple)
  PtIntegMecaInterne ptintmeca;
 
  //--------  spécifiques à Calcul_dsigma_deps hors axes ------------------------
  //grandeurs de travail, pour le passage d'infos en interne aux méthodes
  // Residu_constitutif_3D et Mat_tangente_constitutif_3D, de l'équation
  // sigij(eps_meca)*V_j{.l}=0, l=2 et 3
  const BaseB* ViB_3D; const BaseH* ViH_3D;
  Vecteur val_initiale_3D,racine_3D;
  Vecteur residu_3D;
  MatLapack derResidu_3D; // dérivé du résidu de l'équation c-a-d d<résidu>/<d def méca>
  Tenseur3BB eps_P_BB,Deps_P_BB,delta_eps_P_BB; // def de travail: {epsilon'}_{kl}
  Mat_pleine gamma3D,beta3D,gammaP_3D,betaP_3D; //  matrices de travail pour les changements de base
  const Tenseur3BB* DepsBB_cin; // vitesse cinématique d'entrée
  const Tenseur3BB* epsBB_tdt_cin; // déformation cinématique d'entrée
  const Tenseur3BB* delta_epsBB_cin; // delta def cinématique d'entrée
  Mat_pleine mat_inter; // sert pour le calcul du jacobien mécanique
  Tenseur3BB gij_meca_BB,gij_meca_BB_t; // un tenseur intermédiaire pour le calcul du jacobien mécanique
  //double& jacobien_0,double& jacobien
  Tableau <int> ind,jnd; // tableau d'index pour passer de (2,2) , (3,3) , (1,2) à 1,2,3
     // ind le premier indice et jnd le second indice
     // pour (m,n) = (2,2); (3,3); (1,2),  et pour (g,h) = (2,2); (3,3); et (1,2)
     // on va utiliser des matrices intermédiaires tels que:
     // in = 1 pour (2,2) , 2 pour (3,3) et 3 pour (1,2)
  Tenseur3BB  V1V1_BB;
  Tableau <Tenseur3BB  > VhVg_BB;


  //--------  fin spécifiques à Calcul_dsigma_deps hors axes ------------------------

  //---  méthodes internes
  // passage des grandeurs métriques de l'ordre 1 à 3: cas implicite
  void Passage_metrique_ordre1_vers_3(const Met_abstraite::Impli& ex);
  // passage des grandeurs métriques de l'ordre 1 à 3: cas explicite
  void Passage_metrique_ordre1_vers_3(const Met_abstraite::Expli_t_tdt& ex);
 // passage des grandeurs métriques de l'ordre 1 à 3: cas implicite
  void Passage_metrique_ordre1_vers_3(const Met_abstraite::Umat_cont& ex);
  // passage des informations liées à la déformation  de 1 vers 3, et variation de volume éventuelle
  // si le pointeur d_jacobien_tdt est non nul
  // idem pour d_epsBB
  void Passage_deformation_volume_ordre1_vers_3(TenseurBB& DepsBB
                        ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>* d_epsBB
                        ,TenseurBB & delta_epsBB,const double& jacobien_0,double& jacobien
                        ,Vecteur* d_jacobien_tdt,const double& jacobien_t);
  // mise à jour des informations liées à la déformation  de 1 vers 3
  void Mise_a_jour_deformations_et_Jacobien_en_3D();

  // calcul des déformations d'épaisseur et de largeur, en utilisant la variation de volume et la compressibilité
  // cas de la méthode 1
  void Calcul_eps_trans_parVarVolume1(Tenseur3BH& sigBH_t,double& jaco_1D_0,const double& module_compressibilite,double& jacobien
                    ,Tenseur3BH& sigBH,double& jaco_1D_t);
  // calcul des déformations d'épaisseur et de largeur, en utilisant la variation de volume et la compressibilité
  // cas de la méthode 2
  void Calcul_eps_trans_parVarVolume2(Tenseur3BH& sigBH_t,double& jaco_1D_0,const double& module_compressibilite,double& jacobien
                    ,Tenseur3BH& sigBH,double& jaco_1D_t);
  // calcul des déformations d'épaisseur et de largeur et des variations
  void Calcul_d_eps_trans_parVarVolume(double& jaco_1D_0,const double& module_compressibilite,double& jacobien
                    ,TenseurHH& sigHH_,Vecteur& d_jaco_1D
                    ,Tableau <TenseurHH *>& d_sigHH,Tableau <TenseurBB *>& d_gijBB_tdt
                    ,TenseurBB & gijBB_);
  // calcul de la variation des déformations d'hauteur et de largeur, par rapport à la def_11, à cause
  // des contraintes planes doubles
  void Calcul_d_eps_eg_11();
  // calcul de l'opérateur tangent  (2ième méthode)
  // d sigma^{ij} / d epsilon_{kl}
  void Calcul2_dsigma_deps(TenseurHHHH& d_sigma_deps);
 
  // calcul des invariants de déformation et de vitesse de déformation, et les def cumulées
  // correspondant aux cas 3D
  void Calcul_invariants_et_def_cumul();
 
  // limitation des variations d'épaisseurs et largeurs
  // ramène true s'il y a eu une modif
  bool Limitation_h_b(double& hsurh0, double& bsurb0);
 
  // vérif des grandeurs et calcul de l'épaisseur et de largeur ainsi que calcul du jacobien final
  //   = 0 pas de  modif
  //   = 1 modif due aux maxi permis de h et b, = 2 si def d'entrée incorrecte
  // cas de la 2ième méthode
  int Calcul_et_Limitation2_h_b(double& hsurh0,Tenseur3BB& eps_BB_3D, double& bsurb0
                                 ,double& jacobien_tdt_3DD, const double & jacobien_0_3DD);
 
  // calcul de l'état final, dans le cas de la méthode de perturbation: version 2
  void Cal_avec_perturbation2();

  // passage entre le calcul classique en 1D de Calcul_dsigma_deps et le calcul 3D
  void Passage_Calcul_1D_3D_dsigma_deps (bool en_base_orthonormee, TenseurHH & sigHH_t,TenseurBB& DepsBB
     ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
     ,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
     ,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
     ,const Met_abstraite::Umat_cont& ex) ; //= 0;


  // passage entre le calcul classique en 1D de Calcul_DsigmaHH_tdt et le calcul 3D
  void Passage_Calcul_1D_3D_dsigma_DsigmaHH_tdt
           (TenseurHH& sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
            ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
            ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
            ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
            ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
            ,Tableau <TenseurBB *>& d_gijBB_tdt
            ,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
            ,Vecteur& d_jacobien_tdt,TenseurHH& sigHH_tdt,Tableau <TenseurHH *>& d_sigHH
            ,EnergieMeca & energ,const EnergieMeca &
            ,double& module_compressibilite,double&  module_cisaillement
            ,const Met_abstraite::Impli& ex);

 
};
/// @}  // end of group

#endif		

		

