

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        11/06/2014                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Définir une loi telle que la contrainte résultante intègre *
 *           une ou plusieurs contraintes: ex emdomagement, rupture     *
 *           plissement ...                                             *
 *       La loi contient a minima une loi classique interne.            *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
// FICHIER : LoiCritere.h
// CLASSE : LoiCritere

#ifndef LOICRITERE_H
#define LOICRITERE_H

#include "Loi_comp_abstraite.h"
#include "Enum_Critere_Loi.h"
#include "LoiContraintesPlanes.h"
#include "LoiContraintesPlanesDouble.h"
#include "Coordonnee2.h"
#include "Ponderation.h"


/// @addtogroup Les_lois_combinees
///  @{
///

class LoiCritere : public Loi_comp_abstraite
{
	public :
  friend class LoiContraintesPlanes;
  friend class LoiContraintesPlanesDouble;
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		LoiCritere ();
		
		// Constructeur de copie
		LoiCritere (const LoiCritere& loi) ;
		
		// DESTRUCTEUR :
		~LoiCritere ();
		
				
		// initialise les donnees particulieres a l'elements
		// de matiere traite ( c-a-dire au pt calcule)
		// Il y a creation d'une instance de SaveResul particuliere
		// a la loi concernee
		// la SaveResul classe est remplie par les instances heritantes
		// le pointeur de SaveResul est sauvegarde au niveau de l'element
		// c'a-d que les info particulieres au point considere sont stocke
		// au niveau de l'element et non de la loi.
		class SaveResul_LoiCritere: public SaveResul
		 { public : 
	        SaveResul_LoiCritere(); // constructeur par défaut (a ne pas utiliser)
	        // le constructeur courant
	        SaveResul_LoiCritere(list <SaveResul*>& l_des_SaveResul,list <TenseurHH* >& l_siHH
	                             ,list <TenseurHH* >& l_siHH_t
	                             ,list <EnergieMeca >& l_energ,list <EnergieMeca >& l_energ_t
                              ,bool avec_ponderation,Enum_Critere_Loi type_crite);
	        // constructeur de copie 
	        SaveResul_LoiCritere(const SaveResul_LoiCritere& sav );
	        // destructeur
	        ~SaveResul_LoiCritere();
		       // définition d'une nouvelle instance identique
         // appelle du constructeur via new 
		       SaveResul * Nevez_SaveResul() const {return (new SaveResul_LoiCritere(*this));};
         // affectation
         virtual SaveResul & operator = ( const SaveResul & a);
	        //============= lecture écriture dans base info ==========
            // cas donne le niveau de la récupération
            // = 1 : on récupère tout
            // = 2 : on récupère uniquement les données variables (supposées comme telles)
	        void Lecture_base_info (ifstream& ent,const int cas);
            // cas donne le niveau de sauvegarde
            // = 1 : on sauvegarde tout
            // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	        void Ecriture_base_info(ofstream& sort,const int cas);
	        
	        // mise à jour des informations transitoires en définitif s'il y a convergence 
	        // par exemple (pour la plasticité par exemple)
	        void TdtversT() ;
	        void TversTdt() ;
			  
			      // affichage à l'écran des infos
			      void Affiche() const;
    
         //changement de base de toutes les grandeurs internes tensorielles stockées
         // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
         // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
         // gpH(i) = gamma(i,j) * gH(j)
         virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma);
    
         // procedure permettant de completer éventuellement les données particulières
         // de la loi stockées
         // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
         // completer est appelé apres sa creation avec les donnees du bloc transmis
         // peut etre appeler plusieurs fois
         SaveResul* Complete_SaveResul(const BlocGen & bloc, const Tableau <Coordonnee>& tab_coor
                                       ,const Loi_comp_abstraite* loi);

	        // ---- récupération d'information: spécifique à certaine classe dérivée
	        double Deformation_plastique() ;
	        
	        // données protégées
	        // la liste des données protégées de chaque loi
	        list <SaveResul*> liste_des_SaveResul; 
	        // la liste des contraintes initiales particulières pour chaque loi
	        list <TenseurHH* > l_sigoHH,l_sigoHH_t; // valeur courante, et valeur sauvegardée au pas précédent
	        // la liste des énergies pour chaque loi
	        list <EnergieMeca > l_energ,l_energ_t; // valeur courante, et valeur sauvegardée au pas précédent
			      // listes éventuelles des fonctions de pondération
			      list <double> f_ponder,f_ponder_t;  // le résultat des fonctions de pondérations
    
         // --- pour les plis ---
         // éventuellement les directions principales des contraintes qui sont exprimées dans
         // la base orthonormee ! donc qui n'évoluent pas avec la méthode: ChBase_des_grandeurs
         Tableau <Coordonnee>* V_P_sig;
         Tableau <Coordonnee>* V_P_sig_t;
         Coordonnee2 eps_pli,eps_pli_t; // intensité des def de plis qui sont nulles si pas de pli, c-a-d
            // plis de membrane
                // pour 1 plis: eps_pli(1) = eps22 en orthonormee
                // pour 2 plis: eps_pli(2) = eps11 en orthonormee
            // plis de biel
                // eps_pli(1) = eps11 en orthonormee
         // pour le critère pli, un pointeur sur une grandeur de travail
         LoiContraintesPlanesDouble::SaveResul_LoiContraintesPlanesDouble * save_result_1DCP2;

         Enum_Critere_Loi le_type_critere; // le type de critère de la loi
         double niveau_declenchement_critere; // le niveau de déclenchement du critère
    
         //  -- indicateur de calcul de direction de plis --
         //   =  0 : pas encore de calcul effectué
         //   = -1 : pas de calcul de valeur propre possible en contrainte
         //   =  1 : pas de plis (pas de calcul de nouvelle direction )
         //   = -2 : pas de calcul de valeur propre de déformation
         //   = -3 : plis dans les deux sens, mais pas de calcul de direction propre valide
         //   =  2 : plis dans les deux sens, calcul des directions de plis
         //   = -4 : pas de calcul de vecteurs propres possible pour les contraintes
         //   =  3 : plis dans une seule directions, calcul ok
         int cas_cal_plis,cas_cal_plis_t;
    
         // --- gestion d'une map de grandeurs quelconques éventuelles ---
         // une map de grandeurs quelconques particulière qui peut servir aux classes appelantes
         // il s'agit ici d'une map interne qui a priori ne doit servir qu'aux class loi de comportement
         // un exemple d'utilisation est une loi combinée qui a besoin de grandeurs spéciales définies
         // -> n'est pas sauvegardé, car a priori il s'agit de grandeurs redondantes
         map <  EnumTypeQuelconque , TypeQuelconque, std::less < EnumTypeQuelconque> > map_type_quelconque;

         // récupération des type quelconque sous forme d'un arbre pour faciliter la recherche
         const map <  EnumTypeQuelconque , TypeQuelconque, std::less < EnumTypeQuelconque> >* Map_type_quelconque()
               const {return &map_type_quelconque;};
       private:
         void Mise_a_jour_map_type_quelconque();
    
         // ---- fin gestion d'une liste de grandeurs quelconques éventuelles ---
    

   };

        // def d'une instance de données spécifiques, et initialisation
		SaveResul * New_et_Initialise() ;
 
  friend class SaveResul_LoiCritere;

	    // Lecture des lois de comportement
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();

  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);
  // récupération d'un module de compressibilité équivalent à la loi, ceci pour un chargement nul
  // il s'agit ici de la relation -pression = sigma_trace/3. = module de compressibilité * I_eps
  double Module_compressibilite_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const;

		// activation des données des noeuds et/ou elements nécessaires au fonctionnement de la loi
		// exemple: mise en service des ddl de température aux noeuds
		virtual void Activation_donnees(Tableau<Noeud *>& tabnoeud,bool dilatation,LesPtIntegMecaInterne& lesPtMecaInt);
  // récupération des grandeurs particulière (hors ddl )
  // correspondant à liTQ
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void Grandeur_particuliere
        (bool absolue,List_io<TypeQuelconque>& liTQ,Loi_comp_abstraite::SaveResul * saveDon,list<int>&) const;
  // récupération de la liste de tous les grandeurs particulières
  // ces grandeurs sont ajoutées à la liste passées en paramètres
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void ListeGrandeurs_particulieres(bool absolue,List_io<TypeQuelconque>& liTQ) const;
  
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new LoiCritere(*this)); };

  // indique le type Enum_comp_3D_CP_DP_1D correspondant à une  loi de comportement
  // la fonction est simple dans le cas d'une loi basique, par contre dans le cas
  // d'une loi combinée, la réponse dépend des lois internes donc c'est redéfini
  // dans les classes dérivées
  virtual Enum_comp_3D_CP_DP_1D  Comportement_3D_CP_DP_1D();
		  	  
	 //----- lecture écriture de restart -----
	 // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);

     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info_loi(ofstream& sort,const int cas);
          
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);  
            
  // activation du stockage de grandeurs quelconques qui pourront ensuite être récupéré
  // via le conteneur SaveResul, si la grandeur n'existe pas ici, aucune action
  virtual void Activation_stockage_grandeurs_quelconques(list <EnumTypeQuelconque >& listEnuQuelc);
  
  // insertion des conteneurs ad hoc concernant le stockage de grandeurs quelconques
  // passée en paramètre, dans le save result: ces conteneurs doivent être valides
  // c-a-d faire partie de listdeTouslesQuelc_dispo_localement
  virtual void Insertion_conteneur_dans_save_result(SaveResul * saveResul);

 protected :
 
   // def des différents critères
   Enum_Critere_Loi type_critere; // le type de critère de la loi
 
   // ----- controle de la sortie des informations
// -> maintenant définit dans LoiAbstraiteGeneral
//   int permet_affichage; // pour permettre un affichage spécifique dans les méthodes, pour les erreurs et warning
   // choix entre la première ou deuxième méthode pour le calcul des plis en membrane
   int choix_methode_cal_plis_memb;

   Tableau<int> ordre_criteres; // donne l'ordre d'application des différents critères
   //%%%% liste des paramètres associés à chaque critère %%%%
   //---------------------------------------------------
   // --- 1a) critère de plissement de membranne:
   LoiContraintesPlanes * loi_2DCP_de_3D; // la loi de contrainte plane qui sert de support
                  //  pour le plissement: il s'agit ici d'un pointeur sur la première loi de lois_internes
   LoiContraintesPlanesDouble * loi_1DCP2_de_3D; // la loi de contrainte plane double qui sert de support
                  // il s'agit ici d'une nouvelle loi créée en interne qui sert pour ses méthodes internes
   double niveau_declenchement_critere; // le niveau de déclenchement du critère
   // contrôle éventuel du niveau d'apparition des plis
   bool avecNiveauSigmaI_mini_pour_plis; // oui ou non on a des fonctions
   // a) contrôle via une ou plusieurs grandeurs globales ou locales
   Ponderation_TypeQuelconque* niveauF_fct_nD;
   // b) via éventuellement un ddl étendu
   Ponderation * niveauF_ddlEtendu;
   // c) via éventuellement le temps
   Courbe1D * niveauF_temps;
   // d) contrôle via une ou plusieurs grandeurs consultables
   Ponderation_Consultable* niveauF_grandeurConsultable;
     // un indicateur pour gérer le cas relachement complet pour le critère plis
   // par défaut = 1 : on utilise l'épaisseur initiale -> vrai pour un comportement réversible sinon pas vrai !
   // sera amélioré par la suite
   int choix_calcul_epaisseur_si_relachement_complet;
     // gestion du recalcul des directions des plis
   Fonction_nD* recalcul_dir_plis; // si nulle ==> recalcul tous les itérations
 
   // --- 1b) critère de plissement 1D:
    // la loi qui sert de support est la première loi la liste des lois_internes
    //  pour le plissement
   // sinon on utilise les paramètres du 1a c-a-d des membrannes pour le contrôle
    //---------------------------------------------------
   // --- 2) critère de rupture en def ou contrainte unilatérale :
   LoiContraintesPlanes * loi_2DCP_pour_rupture; // la loi de contrainte plane qui sert de support
                  //  il s'agit ici d'une nouvelle loi créée en interne qui sert pour ses méthodes internes
   LoiContraintesPlanesDouble * loi_1DCP2_pour_rupture; // la loi de contrainte plane double qui sert de support
                  // il s'agit ici d'une nouvelle loi créée en interne qui sert pour ses méthodes internes
 
 
		 //-- partie optionnelle au cas d'une somme pondérée par rapport à des
   //    grandeurs globales
		 bool avec_ponderation_grandeur_globale; // indique si oui ou non il y a des fonctions
		 // pour chaque loi il y a un élément Ponderation_GGlobal associé, qui contient lui-même m fonctions 1D dont le produit
		 // = la fonction finale de ponderation de la loi
		 list <Ponderation_GGlobal > list_ponderation_GGlob; // liste éventuellement vide des fonctions de ponderation_GGlob
		 list <double> fonc_ponder_GGlob;  // le résultat des fonctions de pondérations
 
   //---------------------------------------------------------------------------------------------------------
   // déclaration des variables internes nécessaires pour les passages 2D ou 3D -->  1D
   // -- on définit des conteneurs pour le stockage des résultats des métriques, dimentionnés par défaut non vide
   // on utilise des pointeurs pour dimentionner après les variables internes
   Met_abstraite::Expli_t_tdt*  expli_1D;
   Met_abstraite::Impli* impli_1D;
   Met_abstraite::Umat_cont*  umat_cont_1D;
  
   // -- variables nécessaires pour la création de expli_1D, impli_1D et umat_cont_1D
   // certaines grandeurs sont associées à un pointeur qui peut soit être nul soit pointer sur le conteneur
   // l'intérêt est que le fait d'avoir un pointeur nul est parfois utilisé pour éviter un calcul 
   BaseB giB_0_1D;
   BaseH giH_0_1D;
   BaseB giB_t_1D;
   BaseH giH_t_1D;
   BaseB  giB_tdt_1D;
   BaseH  giH_tdt_1D;
   Tenseur1BB  gijBB_0_1D;
   Tenseur1HH  gijHH_0_1D;
   Tenseur1BB  gijBB_t_1D;
   Tenseur1HH  gijHH_t_1D;
   Tenseur1BB  gijBB_tdt_1D;
   Tenseur1HH  gijHH_tdt_1D;
   
   TenseurBB * gradVmoyBB_t_1D_P;		Tenseur1BB  gradVmoyBB_t_1D;
   TenseurBB * gradVmoyBB_tdt_1D_P;	Tenseur1BB  gradVmoyBB_tdt_1D;
   TenseurBB * gradVBB_tdt_1D_P;		Tenseur1BB  gradVBB_tdt_1D;
   double jacobien_tdt_1D;double jacobien_t_1D;double jacobien_0_1D;  // pour les jacobiens on considère qu'ils existent toujours
   Vecteur d_jacobien_tdt_1D;
       // pour tous les tableaux de pointeurs, on double le tableau en déclarant un vrai tableau en //
   Tableau <BaseB>  d_giB_tdt_1D;
   Tableau <BaseH>  d_giH_tdt_1D;
   Tableau <TenseurBB *> d_gijBB_tdt_1D_P;       Tableau <Tenseur1BB > d_gijBB_tdt_1D;
   Tableau2 <TenseurBB *>*  d2_gijBB_tdt_1D_P;    Tableau2 <Tenseur1BB >  d2_gijBB_tdt_1D; // a priori ne sera pas affecté, car ne sert
                                                                                           // dans les lois de comportement
   Tableau <TenseurHH *> d_gijHH_tdt_1D_P;       Tableau <Tenseur1HH > d_gijHH_tdt_1D;
   Tableau <TenseurBB * >* d_gradVmoyBB_t_1D_P;   Tableau <Tenseur1BB > d_gradVmoyBB_t_1D;
   Tableau <TenseurBB * >* d_gradVmoyBB_tdt_1D_P; Tableau <Tenseur1BB > d_gradVmoyBB_tdt_1D;
   Tableau <TenseurBB * >* d_gradVBB_t_1D_P;      Tableau <Tenseur1BB > d_gradVBB_t_1D;
   Tableau <TenseurBB * >* d_gradVBB_tdt_1D_P;    Tableau <Tenseur1BB > d_gradVBB_tdt_1D;
           
   // -- on définit les conteneurs pour les passages d'appels entrant de la loi 1D : donc en 1D par défaut
   Tenseur1HH  sig_HH_t_1D, sig_HH_1D ;
   Tenseur1BB  Deps_BB_1D, eps_BB_1D, delta_eps_BB_1D;
   Tableau <TenseurBB *> d_eps_BB_1D_P;          Tableau <Tenseur1BB > d_eps_BB_1D; // le tableau de pointeur puis les vrais grandeurs
   Tableau <TenseurHH *> d_sig_HH_1D_P;          Tableau <Tenseur1HH > d_sig_HH_1D; //  """"
   TenseurHHHH* d_sigma_deps_1D_P;              Tenseur1HHHH d_sigma_deps_1D;

   // -- on définit également certains  conteneurs pour les passages d'appels entrant de la loi 2D : donc en 2D par défaut
   Tenseur2BB  eps_BB_2D_t,delta_eps_BB_2D;
   Tenseur2HH eps_HH_2D_t; // tenseur de travail
   // deux bases de travail qui servent dans Deuxieme_type_calcul_en_un_pli
   BaseB  ViB;
   BaseH  ViH;
   // -- on définit les conteneurs pour les passages d'appels entrant de la loi 3D : donc en 3D par défaut
   Tenseur3HH  sig_HH_t_3D, sig_HH_3D ;
   Tenseur3BB  Deps_BB_3D, eps_BB_3D, delta_eps_BB_3D;

   // cas d'un point d'intégration locale (méthode CalculGrandeurTravail par exemple)
   PtIntegMecaInterne ptintmeca;

//----------- la suite du type des lois membres  --------

   // un type énuméré pour faciliter la lecture
   enum Enumcompletudecalcul { CONTRAINTE_ET_TANGENT =0, CONTRAINTE_UNIQUEMENT, TANGENT_UNIQUEMENT};
   // donnees protegees			
	  list <Loi_comp_abstraite *> lois_internes; // liste des lois constitutives
	  list <Enumcompletudecalcul> list_completude_calcul; // pour savoir si on utilise tout ou une partie

   int type_calcul; // indique si l'on travail sur la contrainte ou l'incrément de contrainte
		 //-- partie optionnelle au cas d'une somme pondérée
		 bool avec_ponderation; // indique si oui ou non il y a des fonctions de ponderation
		 // pour chaque loi il y a un élément Ponderation associé, qui contient lui-même m fonctions 1D dont le produit
		 // = la fonction finale de ponderation de la loi
		 list <Ponderation > list_ponderation; // liste éventuellement vide des fonctions de ponderation
		 list <double> fonc_ponder;  // le résultat des fonctions de pondérations
		 
	  // ---- tableau de travail 
	  Tableau <TenseurHH *> d_sigtotalHH;
	  // tenseur du 4ième orde de travail
	  TenseurHHHH* d_sigma_deps_inter;
   Tenseur3HHHH d_sig_deps_3D_HHHH;
	
   // codage des METHODES VIRTUELLES  protegees:
 // calcul des contraintes a t+dt
        // calcul des contraintes 
 void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
     ,TenseurBB & delta_epsBB_
     ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
     ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Expli_t_tdt& ex);

       // calcul des contraintes et de ses variations  a t+dt
 void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
     ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
     ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
     ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
     ,Tableau <TenseurBB *>& d_gijBB_tdt
		  	,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
		  	,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Impli& ex);
		  	
        // calcul des contraintes et ses variations  par rapport aux déformations a t+dt
        // en_base_orthonormee:  le tenseur de contrainte en entrée est  en orthonormee
        //                  le tenseur de déformation et son incrémentsont également en orthonormees
        //                 si = false: les bases transmises sont utilisées
        // ex: contient les éléments de métrique relativement au paramétrage matériel = X_(0)^a
 void Calcul_dsigma_deps (bool en_base_orthonormee, TenseurHH & sigHH_t,TenseurBB& DepsBB
     ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Umat_cont& ex) ; //= 0;

		  			  	
   // fonction surchargée dans les classes dérivée si besoin est
   virtual void CalculGrandeurTravail
           (const PtIntegMecaInterne& ptintmeca
            ,const Deformation & def,Enum_dure temps,const ThermoDonnee& dTP
            ,const Met_abstraite::Impli* ex_impli
            ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
            ,const Met_abstraite::Umat_cont* ex_umat
            ,const List_io<Ddl_etendu>* exclure_dd_etend
            ,const List_io<const TypeQuelconque *>* exclure_Q
           );

   // permet d'indiquer à la classe à quelle valeur de PtIntegMecaInterne il faut se référer
   // en particulier est utilisé par les lois additives,
   // par contre doit être utilisé avec prudence
   virtual void IndiquePtIntegMecaInterne(const PtIntegMecaInterne * ptintmeca)
     { list <Loi_comp_abstraite *>::iterator il,ilfin=  lois_internes.end();
       for ( il = lois_internes.begin();il != ilfin; il++)
         (*il)->IndiquePtIntegMecaInterne(ptintmeca);
       // puis la classe mère
       Loi_comp_abstraite::IndiquePtIntegMecaInterne(ptintmeca);
     };

   // fonction interne utilisée par les classes dérivées de Loi_comp_abstraite
   // pour répercuter les modifications de la température
   // ici utiliser pour modifier la température des lois élémentaires
   // l'Enum_dure: indique quel est la température courante : 0 t ou tdt
   void RepercuteChangeTemperature(Enum_dure temps);
 
   // application d'un critère
   // en retour:
   //  0  : il y a eu un pb que l'on n'a pas peu résoudre, rien n'a été modifié
   //  1  : le critère n'a rien modifié
   //  2  : l'application du critère conduit à une contrainte et un opérateur tangent nul
   //  3  : les données d'entrée ont été modifiées: contraintes, opérateur tangent, module, énergies
   int Critere(bool en_base_orthonormee,TenseurHH & sigHH_t,TenseurBB& DepsBB
            ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	       ,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps_inter
		  	       ,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	       ,bool implicit,const Met_abstraite::Umat_cont& ex) ;
 
   // fonction critère de plissement de membrane
   // en entrée:
   // implicit : si oui on est en implicite, sinon en explicite
   // retour:
   //   = -1 : pas de calcul de valeur propre possible en contrainte
   //   =  1 : pas de plis (pas de calcul de nouvelle direction )
   //   = -2 : pas de calcul de valeur propre de déformation
   //   = -3 : plis dans les deux sens, mais pas de calcul de direction propre valide
   //   =  2 : plis dans les deux sens, calcul des directions de plis
   //   = -4 : pas de calcul de vecteurs propres possible pour les contraintes
   //   =  3 : plis dans une seule directions, calcul ok

   int Critere_plis_membrane(bool en_base_orthonormee,TenseurHH & sigHH_t,TenseurBB& DepsBB
            ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	       ,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps_inter
		  	       ,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	       ,bool implicit,const Met_abstraite::Umat_cont& ex);

   // fonction critère de plissement de biel
   // en entrée:
   // implicit : si oui on est en implicite, sinon en explicite
   // retour:
   //  0  : il y a eu un pb que l'on n'a pas peu résoudre, rien n'a été modifié
   //  1  : le critère n'a rien modifié
   //  2  : l'application du critère conduit à une contrainte et un opérateur tangent nul
   int Critere_plis_biel(bool en_base_orthonormee,TenseurHH & sigHH_t,TenseurBB& DepsBB
            ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	       ,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps_inter
		  	       ,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	       ,bool implicit,const Met_abstraite::Umat_cont& ex);
 
  //---  méthodes internes
  // création du conteneur UMAT a partir des vecteurs propres
  void Creation_metrique_a_partir_vecteurs_propres_pour_Umat1D
         (const Met_abstraite::Umat_cont& ex,const Mat_pleine& gamma);
  // passage des grandeurs métriques de l'ordre 3 ou 2 à 1: cas implicite
  void Passage_metrique_ordre_3_2_vers_1(const Met_abstraite::Umat_cont& ex,const Mat_pleine& gamma);
  // passage des grandeurs métriques de l'ordre 3 ou 2 à 1: cas explicite
//  void Passage_metrique_ordre_3_2_vers_1(const Met_abstraite::Umat_cont& ex,Mat_pleine& gamma);

  // chgt de repère et de dimension pour les variables de passage pour le calcul final de la loi en 1D
  void Passage_3ou2_vers_1(const Mat_pleine& gamma,const TenseurHH & sigHH_t,const Mat_pleine& beta
                           ,const TenseurBB& DepsBB
                           ,const TenseurBB & epsBB_tdt,const TenseurBB & delta_epsBB
                           ,const bool& deux_plis,Coordonnee2& eps_pli);
  // passage inverse: chgt de repère et de dimension pour les variables de passage
  // et stockage des infos pour le prochain appel
  // en entrée: d_sigma_deps_1D : l'opérateur qui a été calculé
  // en sortie: d_sigma_deps_inter
  // l'umat c'est uniquement pour des vérifs
  void Passage_1_vers_3ou2(const Mat_pleine& gamma,TenseurHH & sigHH
                           ,const TenseurHHHH& d_sigma_deps_1D
                           ,const Mat_pleine& beta
                           ,TenseurHHHH& d_sigma_deps_inter,const Met_abstraite::Umat_cont& ex
                           ,const Tableau <Coordonnee2H>& V_Pr_H
                           ,const Tableau <Coordonnee>& V_P_sig);

// calcul d'une nouvelle direction de plis
// en entrée: force: = true par défaut
//            si == false -> pas de calcul de direction, et mise en place
//                des indicateurs en cohérence avec le fait de ne pas faire de calcul
// en retour:
//   = -1 : pas de calcul de valeur propre possible en contrainte
//   =  1 : pas de plis (pas de calcul de nouvelle direction )
//   = -2 : pas de calcul de valeur propre de déformation
//   = -3 : plis dans les deux sens, mais pas de calcul de direction propre valide
//   =  2 : plis dans les deux sens, calcul des directions de plis
//   = -4 : pas de calcul de vecteurs propres possible pour les contraintes
//   =  3 : plis dans une seule directions, calcul ok
//   =  0 : erreur inconnue ??

int Calcul_directions_plis(const TenseurBB & epsBB_tdt,const TenseurHH& sigHH
                           ,Coordonnee2& valPropreEps
                           ,Tableau <Coordonnee2H>& V_Pr_H
                           ,const Met_abstraite::Umat_cont& ex
                           ,Coordonnee2& valPropreSig
                           , bool force = true
                           );
 
 // premier type de calcul dans le cas d'un pli dans une seule direction
void Premie_type_calcul_en_un_pli(const TenseurBB & epsBB_tdt,const TenseurBB & delta_epsBB
                          ,const TenseurHH & sigHH_t
                          ,const double& jacobien_0,const double& jacobien
                          ,EnergieMeca & energ,const EnergieMeca & energ_t
                          ,const TenseurBB& DepsBB
                          ,double& module_compressibilite,double&  module_cisaillement
                          ,const Tableau <Coordonnee2H>& V_Pr_H
                          ,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps_inter
                          ,const Coordonnee2& valPropreSig
                          ,const Met_abstraite::Umat_cont& ex);

 // premier type de calcul dans le cas d'un pli dans une seule direction
void Deuxieme_type_calcul_en_un_pli(const TenseurBB & epsBB_tdt,const TenseurBB & delta_epsBB
                          ,const TenseurHH & sigHH_t
                          ,double& jacobien_0,double& jacobien
                          ,EnergieMeca & energ,const EnergieMeca & energ_t
                          ,const TenseurBB& DepsBB
                          ,double& module_compressibilite,double&  module_cisaillement
                          ,const Tableau <Coordonnee2H>& V_Pr_H
                          ,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps_inter
                          ,const Coordonnee2& valPropreSig
                          ,const Met_abstraite::Umat_cont& ex);

// préparation à l'appel du comportement
//Par exemple dans le cas d'un critère pli (plutôt seconde méthode), l'incrément de déformation
// dépend de la situation précédente: avec pli ou pas
int Pre_Critere
           (const TenseurBB& DepsBB,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien,const Met_abstraite::Expli_t_tdt* ex_expli,const Met_abstraite::Impli* ex_impli,const Met_abstraite::Umat_cont* ex_umat
           );
 
   // fonction pre_critère de plissement de membrane
   void Pre_Critere_plis_membrane
           (TenseurBB & epsBB_tdt_,TenseurBB & delta_epsBB
            ,const Met_abstraite::Expli_t_tdt* ex_expli
            ,const Met_abstraite::Impli* ex_impli
            ,const Met_abstraite::Umat_cont* ex_umat
           );

   // passage des informations liées à la déformation et contrainte de 2 vers 3
   void Passage_deformation_contrainte_ordre2_vers_3
                     (const TenseurBB& DepsBB,const TenseurBB & epsBB_tdt
                      ,const TenseurBB & delta_epsBB,const TenseurHH&  sig_HH_t);
   // passage des informations liées à la nouvelle contrainte de 2 vers 3
   // et à l'opérateur tangent : méthode 2
   // et mise à jour d'eps_pli
   void Passage_contrainte_et_operateur_tangent_ordre2_vers_3
                     (TenseurHH&  sig_HH_tdt,TenseurHHHH& d_sigma_deps_inter
                      ,const bool& deux_plis,Coordonnee2& eps_pli);

};
/// @}  // end of group

#endif		

		

