

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        8/02/2012                                           *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  La loi est 2D_D et est associée à une loi 3D quelconque    *
 *           L'objectif est de transformer une loi 3D en 2D déformations*
 *           planes.																*
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
// FICHIER : LoiDeformationsPlanes.h
// CLASSE : LoiDeformationsPlanes

#ifndef LOIDEFORMATIONSPLANES_H
#define LOIDEFORMATIONSPLANES_H

#include "Loi_comp_abstraite.h"


/// @addtogroup Les_lois_combinees
///  @{
///


class LoiDeformationsPlanes : public Loi_comp_abstraite
{
	public :
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		LoiDeformationsPlanes ();
		
		// Constructeur de copie
		LoiDeformationsPlanes (const LoiDeformationsPlanes& loi) ;
		
		// DESTRUCTEUR :
		~LoiDeformationsPlanes ();
		
		
		
		// initialise les donnees particulieres a l'elements
		// de matiere traite ( c-a-dire au pt calcule)
		// Il y a creation d'une instance de SaveResul particuliere
		// a la loi concernee
		// la SaveResul classe est remplie par les instances heritantes
		// le pointeur de SaveResul est sauvegarde au niveau de l'element
		// c'a-d que les info particulieres au point considere sont stocke
		// au niveau de l'element et non de la loi.
		class SaveResul_LoiDeformationsPlanes: public SaveResul 
		 { public : 
	        SaveResul_LoiDeformationsPlanes(); // constructeur par défaut (a ne pas utiliser)
	        // le constructeur courant
	        SaveResul_LoiDeformationsPlanes(SaveResul* l_des_SaveResul);
	        // constructeur de copie 
	        SaveResul_LoiDeformationsPlanes(const SaveResul_LoiDeformationsPlanes& sav );
	        // destructeur
	        ~SaveResul_LoiDeformationsPlanes();
		       // définition d'une nouvelle instance identique
		       // appelle du constructeur via new 
		       SaveResul * Nevez_SaveResul() const {return (new SaveResul_LoiDeformationsPlanes(*this));};		    
         // affectation
         virtual SaveResul & operator = ( const SaveResul & a);
	        //============= lecture écriture dans base info ==========
            // cas donne le niveau de la récupération
            // = 1 : on récupère tout
            // = 2 : on récupère uniquement les données variables (supposées comme telles)
	        void Lecture_base_info (ifstream& ent,const int cas);
            // cas donne le niveau de sauvegarde
            // = 1 : on sauvegarde tout
            // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	        void Ecriture_base_info(ofstream& sort,const int cas);
	        
	        // mise à jour des informations transitoires en définitif s'il y a convergence 
	        // par exemple (pour la plasticité par exemple)
	        void TdtversT() ;
	        void TversTdt() ;
			  
			      // affichage à l'écran des infos
			      void Affiche() const;
    
         //changement de base de toutes les grandeurs internes tensorielles stockées
         // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
         // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
         // gpH(i) = gamma(i,j) * gH(j)
         virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma);
    
         // procedure permettant de completer éventuellement les données particulières
         // de la loi stockées
         // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
         // completer est appelé apres sa creation avec les donnees du bloc transmis
         // peut etre appeler plusieurs fois
         SaveResul* Complete_SaveResul(const BlocGen & bloc, const Tableau <Coordonnee>& tab_coor
                                       ,const Loi_comp_abstraite* loi);

	        // ---- récupération d'information: spécifique à certaine classe dérivée
	        double Deformation_plastique() ;
	        
	        // données protégées
	        // les données protégées de la loi
	        SaveResul* le_SaveResul; 
	        // les contraintes qui servent d'entrée au calcul de la loi associée 
	        TenseurHH*  l_sigoHH, * l_sigoHH_t; // valeur courante, et valeur sauvegardée au pas précédent
         Vecteur  sigInvar,sigInvar_t; // on sauvegarde les invariants ordre 3 à l'intant t
	        // les énergies pour la loi
	        EnergieMeca  l_energ,l_energ_t; // valeur courante, et valeur sauvegardée au pas précédent
   };

        // def d'une instance de données spécifiques, et initialisation
		SaveResul * New_et_Initialise() ;
 
  friend class SaveResul_LoiDeformationsPlanes;

	    // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();

  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);
  // récupération d'un module de compressibilité équivalent à la loi, ceci pour un chargement nul
  // il s'agit ici de la relation -pression = sigma_trace/3. = module de compressibilité * I_eps
  double Module_compressibilite_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const {return 0.;};
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new LoiDeformationsPlanes(*this)); };
			
		// activation des données des noeuds et/ou elements nécessaires au fonctionnement de la loi
		// exemple: mise en service des ddl de température aux noeuds
		virtual void Activation_donnees(Tableau<Noeud *>& tabnoeud,bool dilatation,LesPtIntegMecaInterne& lesPtMecaInt);
  // récupération des grandeurs particulière (hors ddl )
  // correspondant à liTQ
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void Grandeur_particuliere
        (bool absolue,List_io<TypeQuelconque>& liTQ,Loi_comp_abstraite::SaveResul * saveDon,list<int>& decal) const;
  // récupération de la liste de tous les grandeurs particulières
  // ces grandeurs sont ajoutées à la liste passées en paramètres
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void ListeGrandeurs_particulieres(bool absolue,List_io<TypeQuelconque>& liTQ) const;
  
			  
	   //----- lecture écriture de restart -----
	   // cas donne le niveau de la récupération
      // = 1 : on récupère tout
      // = 2 : on récupère uniquement les données variables (supposées comme telles)
  void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);

      // cas donne le niveau de sauvegarde
      // = 1 : on sauvegarde tout
      // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
  void Ecriture_base_info_loi(ofstream& sort,const int cas);
            
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);  
		  	       
	protected :
        // donnees protegees
	 Loi_comp_abstraite * lois_interne; // loi 3D correspondante
	 // tenseur du 4ième orde de travail
	 TenseurHHHH* d_sigma_deps_inter;
	
	
  // codage des METHODES VIRTUELLES  protegees:
 // calcul des contraintes a t+dt
        // calcul des contraintes 
 void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
     ,TenseurBB & delta_epsBB_
     ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
     ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Expli_t_tdt& ex);

       // calcul des contraintes et de ses variations  a t+dt
 void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
     ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
     ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
     ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
     ,Tableau <TenseurBB *>& d_gijBB_tdt
		  	,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
		  	,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Impli& ex);
		  	
        // calcul des contraintes et ses variations  par rapport aux déformations a t+dt
        // en_base_orthonormee:  le tenseur de contrainte en entrée est  en orthonormee
        //                  le tenseur de déformation et son incrémentsont également en orthonormees
        //                 si = false: les bases transmises sont utilisées
        // ex: contient les éléments de métrique relativement au paramétrage matériel = X_(0)^a
 void Calcul_dsigma_deps (bool en_base_orthonormee, TenseurHH & sigHH_t,TenseurBB& DepsBB
     ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Umat_cont& ex) ; //= 0;

		  	
  // fonction surchargée dans les classes dérivée si besoin est
  virtual void CalculGrandeurTravail
          (const PtIntegMecaInterne& ptintmeca
           ,const Deformation & def,Enum_dure temps,const ThermoDonnee& dTP
           ,const Met_abstraite::Impli* ex_impli
           ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
           ,const Met_abstraite::Umat_cont* ex_umat
           ,const List_io<Ddl_etendu>* exclure_dd_etend
           ,const List_io<const TypeQuelconque *>* exclure_Q
          );

  // permet d'indiquer à la classe à quelle valeur de PtIntegMecaInterne il faut se référer
  // en particulier est utilisé par les lois additives,
  // par contre doit être utilisé avec prudence
  virtual void IndiquePtIntegMecaInterne(const PtIntegMecaInterne * ptintmeca)
    { lois_interne->IndiquePtIntegMecaInterne(ptintmeca);
      // puis la classe mère
      Loi_comp_abstraite::IndiquePtIntegMecaInterne(ptintmeca);
    };


  // fonction interne utilisée par les classes dérivées de Loi_comp_abstraite
  // pour répercuter les modifications de la température
  // ici utiliser pour modifier la température des lois élémentaires
  // l'Enum_dure: indique quel est la température courante : 0 t ou tdt
  void RepercuteChangeTemperature(Enum_dure temps); 
		 
private :
  // déclaration des variables internes nécessaires pour les passages 2D - 3D
  // -- on définit des conteneurs pour le stockage des résultats des métriques, dimentionnés par défaut non vide
  // on utilise des pointeurs pour dimentionner après les variables internes
  Met_abstraite::Expli_t_tdt*  expli_3D;
  Met_abstraite::Impli* impli_3D;
  Met_abstraite::Umat_cont*  umat_cont_3D;
  
  // -- variables nécessaires pour la création de expli_3D, impli_3D et umat_cont_3D
  // certaines grandeurs sont associées à un pointeur qui peut soit être nulle soit pointer sur le conteneur
  // l'intérêt est que le fait d'avoir un pointeur nul est parfois utilisé pour éviter un calcul 
  BaseB giB_0_3D; 
  BaseH giH_0_3D; 
  BaseB giB_t_3D;
  BaseH giH_t_3D;
  BaseB  giB_tdt_3D;
  BaseH  giH_tdt_3D; 
  Tenseur3BB  gijBB_0_3D; 
  Tenseur3HH  gijHH_0_3D; 
  Tenseur3BB  gijBB_t_3D;
  Tenseur3HH  gijHH_t_3D;
  Tenseur3BB  gijBB_tdt_3D;
  Tenseur3HH  gijHH_tdt_3D;
  
  TenseurBB * gradVmoyBB_t_3D_P;		Tenseur_ns3BB  gradVmoyBB_t_3D;
  TenseurBB * gradVmoyBB_tdt_3D_P;	Tenseur_ns3BB  gradVmoyBB_tdt_3D;
  TenseurBB * gradVBB_tdt_3D_P;		Tenseur_ns3BB  gradVBB_tdt_3D;
  double jacobien_tdt_3D;double jacobien_t_3D;double jacobien_0_3D;  // pour les jacobiens on considère qu'ils existent toujours
  Vecteur d_jacobien_tdt_3D;
      // pour tous les tableaux de pointeurs, on double le tableau en déclarant un vrai tableau en //
  Tableau <BaseB>  d_giB_tdt_3D;
  Tableau <BaseH>  d_giH_tdt_3D;		
  Tableau <TenseurBB *> d_gijBB_tdt_3D_P;       Tableau <Tenseur3BB > d_gijBB_tdt_3D;
  Tableau2 <TenseurBB *>*  d2_gijBB_tdt_3D_P;    Tableau2 <Tenseur3BB >  d2_gijBB_tdt_3D; // a priori ne sera pas affecté, car ne sert
                                                                                          // dans les lois de comportement
  Tableau <TenseurHH *> d_gijHH_tdt_3D_P;       Tableau <Tenseur3HH > d_gijHH_tdt_3D;  
  Tableau <TenseurBB * >* d_gradVmoyBB_t_3D_P;   Tableau <Tenseur_ns3BB > d_gradVmoyBB_t_3D;
  Tableau <TenseurBB * >* d_gradVmoyBB_tdt_3D_P; Tableau <Tenseur_ns3BB > d_gradVmoyBB_tdt_3D;
  Tableau <TenseurBB * >* d_gradVBB_t_3D_P;      Tableau <Tenseur_ns3BB > d_gradVBB_t_3D;
  Tableau <TenseurBB * >* d_gradVBB_tdt_3D_P;    Tableau <Tenseur_ns3BB > d_gradVBB_tdt_3D;
								  
  // -- on définit les conteneurs pour les passages d'appels entrant de la loi 3D : donc en 3D par défaut
  Tenseur3HH  sig_HH_t_3D, sig_HH_3D ;
  Tenseur3BB  Deps_BB_3D, eps_BB_3D, delta_eps_BB_3D;
  Tableau <TenseurBB *> d_eps_BB_3D_P;          Tableau <Tenseur3BB > d_eps_BB_3D; // le tableau de pointeur puis les vrais grandeurs
  Tableau <TenseurHH *> d_sig_HH_3D_P;          Tableau <Tenseur3HH > d_sig_HH_3D; //  """"
  Tenseur3HHHH d_sigma_deps_3D;
  
  // cas d'un point d'intégration locale (méthode CalculGrandeurTravail par exemple)
  PtIntegMecaInterne ptintmeca;

  //---  méthodes internes
  // passage des grandeurs métriques de l'ordre 2 à 3: cas implicite
  void Passage_metrique_ordre2_vers_3(const Met_abstraite::Impli& ex);
  // passage des grandeurs métriques de l'ordre 2 à 3: cas explicite
  void Passage_metrique_ordre2_vers_3(const Met_abstraite::Expli_t_tdt& ex);
  // passage des informations liées à la déformation  de 2 vers 3, et variation de volume éventuelle
  // si le pointeur d_jacobien_tdt est non nul
  // idem pour d_epsBB
  void Passage_deformation_volume_ordre2_vers_3(TenseurBB& DepsBB
                    ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>* d_epsBB
								            ,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	               ,Vecteur* d_jacobien_tdt);


};
/// @}  // end of group

#endif		

		

