

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        10/04/2004                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:  Définir une loi telle que la contrainte résultante soit la *
 *           sommme pondérée : 1-alpha, et alpha , de contraintes       *
 *        élémentaires, eux-même définies à  partir de lois quelconques.*
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
// FICHIER : LoiDesMelangesEnSigma.h
// CLASSE : LoiDesMelangesEnSigma

#ifndef LOIDESMELANGESENSIGMA_H
#define LOIDESMELANGESENSIGMA_H

#include "Loi_comp_abstraite.h"
#include "Ponderation.h"


/// @addtogroup Les_lois_combinees
///  @{
///

class LoiDesMelangesEnSigma : public Loi_comp_abstraite
{
	public :
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		LoiDesMelangesEnSigma ();
		
		// Constructeur de copie
		LoiDesMelangesEnSigma (const LoiDesMelangesEnSigma& loi) ;
		
		// DESTRUCTEUR :
		~LoiDesMelangesEnSigma ();
		
		
		
		// initialise les donnees particulieres a l'elements
		// de matiere traite ( c-a-dire au pt calcule)
		// Il y a creation d'une instance de SaveResul particuliere
		// a la loi concernee
		// la SaveResul classe est remplie par les instances heritantes
		// le pointeur de SaveResul est sauvegarde au niveau de l'element
		// c'a-d que les info particulieres au point considere sont stocke
		// au niveau de l'element et non de la loi.
		class SaveResul_LoiDesMelangesEnSigma: public SaveResul 
		 { public : 
	        SaveResul_LoiDesMelangesEnSigma(); // constructeur par défaut (a ne pas utiliser)
	        // le constructeur courant
	        SaveResul_LoiDesMelangesEnSigma(list <SaveResul*>& l_des_SaveResul
	                                     ,list <TenseurHH* >& l_siHH,list <TenseurHH* >& l_siHH_t
	                                     ,list <TenseurHH* >& J_siHH,list <TenseurHH* >& J_siHH_t
	                                     ,list <EnergieMeca >& l_energ,list <EnergieMeca >& l_energ_t
                                      ,int type_evol_proportion);
	        // constructeur de copie 
	        SaveResul_LoiDesMelangesEnSigma(const SaveResul_LoiDesMelangesEnSigma& sav );
	        // destructeur
	        ~SaveResul_LoiDesMelangesEnSigma();
		       // définition d'une nouvelle instance identique
		       // appelle du constructeur via new
		       SaveResul * Nevez_SaveResul() const {return (new SaveResul_LoiDesMelangesEnSigma(*this));};
         // affectation
         virtual SaveResul & operator = ( const SaveResul & a);
	        //============= lecture écriture dans base info ==========
            // cas donne le niveau de la récupération
            // = 1 : on récupère tout
            // = 2 : on récupère uniquement les données variables (supposées comme telles)
	        void Lecture_base_info (ifstream& ent,const int cas);
            // cas donne le niveau de sauvegarde
            // = 1 : on sauvegarde tout
            // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	        void Ecriture_base_info(ofstream& sort,const int cas);
	        
	        // mise à jour des informations transitoires en définitif s'il y a convergence 
	        // par exemple (pour la plasticité par exemple)
	        void TdtversT() ;
	        void TversTdt() ;
			  
			      // affichage à l'écran des infos
			      void Affiche() const;
    
         //changement de base de toutes les grandeurs internes tensorielles stockées
         // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
         // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
         // gpH(i) = gamma(i,j) * gH(j)
         virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma);
    
         // procedure permettant de completer éventuellement les données particulières
         // de la loi stockées
         // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
         // completer est appelé apres sa creation avec les donnees du bloc transmis
         // peut etre appeler plusieurs fois
         SaveResul* Complete_SaveResul(const BlocGen & bloc, const Tableau <Coordonnee>& tab_coor
                                       ,const Loi_comp_abstraite* loi);

	        // ---- récupération d'information: spécifique à certaine classe dérivée
	        double Deformation_plastique() ;
	        
	        // données protégées
	        // la liste des données protégées de chaque loi
	        list <SaveResul*> liste_des_SaveResul; 
	        // la liste des contraintes initiales particulières pour chaque loi
	        // 1) les contraintes qui servent d'entrée au calcul des lois élémentaires (non proportionnées)
	        list <TenseurHH* > l_sigoHH,l_sigoHH_t; // valeur courante, et valeur sauvegardée au pas précédent
	        // 2) les contraintes proportionnées 
	        list <TenseurHH* > J_sigoHH,J_sigoHH_t; // valeur courante, et valeur sauvegardée au pas précédent
	        // la liste des énergies pour chaque loi
	        list <EnergieMeca > l_energ,l_energ_t; // valeur courante, et valeur sauvegardée au pas précédent
	        // proportion: qui est a priori variable au cours de l'évolution
	        double proportion,proportion_t;
         int type_evolution_proportion; // = 0 : aucune particularité
                                        // = 1 : au premier incrément la proportion
                                        //       démarre à 1 et ensuite évolue de manière strictement
                                        //       décroissante
                                        // = 2 : pour chaque incrément
                                        //       à chaque mise à jour tdt_t ou l'inverse: la proportion
                                        //       démarre à 1 et ensuite évolue de manière strictement
                                        //       décroissante
         bool deja_actif_sur_iter1,deja_actif_sur_iter2;
   };

        // def d'une instance de données spécifiques, et initialisation
		SaveResul * New_et_Initialise() ;
 
  friend class SaveResul_LoiDesMelangesEnSigma;

	    // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();

  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);
  // récupération d'un module de compressibilité équivalent à la loi, ceci pour un chargement nul
  // il s'agit ici de la relation -pression = sigma_trace/3. = module de compressibilité * I_eps
  double Module_compressibilite_equivalent(Enum_dure temps,const Deformation & def,SaveResul * saveResul);
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const
    { cout << "\n LoiDesMelangesEnSigma::HsurH0(.. , methode non implante pour l'instant ";
      Sortie(1);
    };
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new LoiDesMelangesEnSigma(*this)); };
	
		// activation des données des noeuds et/ou elements nécessaires au fonctionnement de la loi
		// exemple: mise en service des ddl de température aux noeuds
		virtual void Activation_donnees(Tableau<Noeud *>& tabnoeud,bool dilatation,LesPtIntegMecaInterne& lesPtMecaInt);
  // récupération des grandeurs particulière (hors ddl )
  // correspondant à liTQ
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void Grandeur_particuliere
        (bool absolue,List_io<TypeQuelconque>& liTQ,Loi_comp_abstraite::SaveResul * saveDon,list<int>& decal) const;
  // récupération de la liste de tous les grandeurs particulières
  // ces grandeurs sont ajoutées à la liste passées en paramètres
  // absolue: indique si oui ou non on sort les tenseurs dans la base absolue ou une base particulière
  virtual void ListeGrandeurs_particulieres(bool absolue,List_io<TypeQuelconque>& liTQ) const;

  // indique le type Enum_comp_3D_CP_DP_1D correspondant à une  loi de comportement
  // la fonction est simple dans le cas d'une loi basique, par contre dans le cas
  // d'une loi combinée, la réponse dépend des lois internes donc c'est redéfini
  // dans les classes dérivées
  virtual Enum_comp_3D_CP_DP_1D  Comportement_3D_CP_DP_1D();
		  	  
  //----- lecture écriture de restart -----
  // cas donne le niveau de la récupération
  // = 1 : on récupère tout
  // = 2 : on récupère uniquement les données variables (supposées comme telles)
  void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);

  // cas donne le niveau de sauvegarde
  // = 1 : on sauvegarde tout
  // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
  void Ecriture_base_info_loi(ofstream& sort,const int cas);
          
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);  
	
	protected :

        // donnees protegees			
	    Loi_comp_abstraite * lois_internes1; // liste des lois constitutives
	    Loi_comp_abstraite * lois_internes2; // liste des lois constitutives
     // gestion du calcul conditionnel si prop non nulle
     Tableau <int> calcule_si_prop_non_nulle; // = 0 : pas actif, =1 : calcul uniquement si non nul
            // = 2 : calcul à partir du moment où il a été une fois non nul
            // = 3 : calcul à partir du moment où il a été une fois non nul, avec ré-init à chaque incrément
	    // tableau de travail
	    Tableau <TenseurHH *> d_sigtotalHH;
	    // tenseur du 4ième orde de travail
	    TenseurHHHH* d_sigma_deps_inter;
	    // grandeur qui sert de proportion
	    Ddl_enum_etendu type_grandeur;
	    // indique si la grandeur est calculée à partir des noeuds (valeur par défaut)
	    // ou directement à partir d'une valeur obtenue au point d'intégration
	    bool valeur_aux_noeuds; 
	    double proportion,aA; // aA est le maxi de la somme des deux lois
	    Courbe1D* c_proport; // courbe éventuelle permettant le calcul de la proportion en fonction de la grandeur
		                      // de pilotage de la proportion
     // a) contrôle via une fct nd avec une ou plusieurs grandeurs globales et autres grandeurs
     Fonction_nD* niveauF_grandeurND;
     // b) via éventuellement un ddl étendu
     Ponderation * niveauF_ddlEtendu;
     // c) via éventuellement le temps
     Ponderation_temps * niveauF_temps;
     // d) contrôle via une ou plusieurs grandeurs consultables
     Ponderation_Consultable* niveauF_grandeurConsultable;
 
	    // type de loi des mélanges
	    int type_melange; // = 1 : type historique: le mélange se fait sur la contrainte totale (par défaut)
         // = 2 : type historique: le mélange se fait sur l'accroissement de la contrainte
         // = 3 : nouveau type, fonctions complexes, mélange sur la contrainte totale (par défaut)
         // = 4 : nouveau type, fonctions complexes, mélange sur l'accroissement de la contrainte
	
	
  // codage des METHODES VIRTUELLES  protegees:
 // calcul des contraintes a t+dt
        // calcul des contraintes 
 void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
     ,TenseurBB & delta_epsBB_
     ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
     ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Expli_t_tdt& ex);

       // calcul des contraintes et de ses variations  a t+dt
 void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
     ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
     ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
     ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
     ,Tableau <TenseurBB *>& d_gijBB_tdt
		  	,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
		  	,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Impli& ex);
		  	
        // calcul des contraintes et ses variations  par rapport aux déformations a t+dt
        // en_base_orthonormee:  le tenseur de contrainte en entrée est  en orthonormee
        //                  le tenseur de déformation et son incrémentsont également en orthonormees
        //                 si = false: les bases transmises sont utilisées
        // ex: contient les éléments de métrique relativement au paramétrage matériel = X_(0)^a
 void Calcul_dsigma_deps (bool en_base_orthonormee, TenseurHH & sigHH_t,TenseurBB& DepsBB
     ,TenseurBB & epsBB_tdt,TenseurBB & delta_epsBB,double& jacobien_0,double& jacobien
		  	,TenseurHH& sigHH,TenseurHHHH& d_sigma_deps
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Umat_cont& ex) ; //= 0;

		  	
  // fonction surchargée dans les classes dérivée si besoin est
  virtual void CalculGrandeurTravail
          (const PtIntegMecaInterne& ptintmeca
          ,const Deformation & def,Enum_dure temps,const ThermoDonnee& dTP
          ,const Met_abstraite::Impli* ex_impli
          ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
          ,const Met_abstraite::Umat_cont* ex_umat
          ,const List_io<Ddl_etendu>* exclure_dd_etend
          ,const List_io<const TypeQuelconque *>* exclure_Q
         );
 
  // permet d'indiquer à la classe à quelle valeur de PtIntegMecaInterne il faut se référer
  // en particulier est utilisé par les lois additives,
  // par contre doit être utilisé avec prudence
  virtual void IndiquePtIntegMecaInterne(const PtIntegMecaInterne * ptintmeca)
    { lois_internes1->IndiquePtIntegMecaInterne(ptintmeca);
      lois_internes2->IndiquePtIntegMecaInterne(ptintmeca);
      // puis la classe mère
      Loi_comp_abstraite::IndiquePtIntegMecaInterne(ptintmeca);
    };


  // fonction interne utilisée par les classes dérivées de Loi_comp_abstraite
  // pour répercuter les modifications de la température
  // ici utiliser pour modifier la température des lois élémentaires
  // l'Enum_dure: indique quel est la température courante : 0 t ou tdt
  void RepercuteChangeTemperature(Enum_dure temps);

};
/// @}  // end of group

#endif		

		

