// FICHIER : Prandtl_Reuss.h
// CLASSE : Prandtl_Reuss


// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

/************************************************************************
 *     DATE:        19/01/2001                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   La classe Prandtl-Reuss permet de calculer la contrainte  *
 *     et ses derivees pour une loi élasto-plastique de type            *
 *     Prandtl-Reuss.                                                   *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/


#ifndef PRANDTL_REUSS_H
#define PRANDTL_REUSS_H


#include "Loi_comp_abstraite.h"
#include "Courbe1D.h"
#include "TypeConsTens.h"

/** @defgroup Les_lois_de_plasticite
*
*     BUT:   groupe des lois de plasticité
*
*
* \author    Gérard Rio
* \version   1.0
* \date       19/01/2001
* \brief       Définition des  lois de plasticité
*
*/

/// @addtogroup Les_lois_de_plasticite
///  @{
///


class Prandtl_Reuss : public Loi_comp_abstraite
{


	public :
	
	
		// CONSTRUCTEURS :
		
		// Constructeur par defaut
		Prandtl_Reuss ();
		
		
		// Constructeur de copie
		Prandtl_Reuss (const Prandtl_Reuss& loi) ;
		
		// DESTRUCTEUR :
		
		~Prandtl_Reuss ();
		
		// initialise les donnees particulieres a l'elements
		// de matiere traite ( c-a-dire au pt calcule)
		// Il y a creation d'une instance de SaveResul particuliere
		// a la loi concernee
		// la SaveResul classe est remplie par les instances heritantes
		// le pointeur de SaveResul est sauvegarde au niveau de l'element
		// c'a-d que les info particulieres au point considere sont stocke
		// au niveau de l'element et non de la loi.
		class SaveResulPrandtl_Reuss: public SaveResul 
		 { public : 
	        SaveResulPrandtl_Reuss() :
	          epsilon_barre(0),epsilon_barre_t(0),def_plasBB(),def_plasBB_t()
	           {}; 
	        SaveResulPrandtl_Reuss(const SaveResulPrandtl_Reuss& sav):
	          epsilon_barre(sav.epsilon_barre),epsilon_barre_t(sav.epsilon_barre_t)
	          ,def_plasBB(sav.def_plasBB),def_plasBB_t(sav.def_plasBB_t)
	           {}; 
		       // définition d'une nouvelle instance identique
		       // appelle du constructeur via new
		       SaveResul * Nevez_SaveResul() const{return (new SaveResulPrandtl_Reuss(*this));};
         // affectation
         virtual SaveResul & operator = ( const SaveResul & a);
	        //============= lecture écriture dans base info ==========
            // cas donne le niveau de la récupération
            // = 1 : on récupère tout
            // = 2 : on récupère uniquement les données variables (supposées comme telles)
	        void Lecture_base_info (ifstream& ent,const int cas);
            // cas donne le niveau de sauvegarde
            // = 1 : on sauvegarde tout
            // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	        void Ecriture_base_info(ofstream& sort,const int cas);
	        
	        // mise à jour des informations transitoires en définitif s'il y a convergence 
	        // par exemple (pour la plasticité par exemple)
	        void TdtversT() {epsilon_barre_t = epsilon_barre;def_plasBB_t = def_plasBB;};
	        void TversTdt() {epsilon_barre = epsilon_barre_t;def_plasBB = def_plasBB_t;};
			  
			      // affichage à l'écran des infos
			      void Affiche() const;
    
         //changement de base de toutes les grandeurs internes tensorielles stockées
         // beta(i,j) represente les coordonnees de la nouvelle base naturelle gpB dans l'ancienne gB
         // gpB(i) = beta(i,j) * gB(j), i indice de ligne, j indice de colonne
         // actuellement les tenseurs sont exprimés en absolu donc pas de chgt de base actif
         // gpH(i) = gamma(i,j) * gH(j)
         virtual void ChBase_des_grandeurs(const Mat_pleine& beta,const Mat_pleine& gamma) {};
 
         // procedure permettant de completer éventuellement les données particulières
         // de la loi stockées
         // au niveau du point d'intégration par exemple: exemple: un repère d'anisotropie
         // completer est appelé apres sa creation avec les donnees du bloc transmis
         // peut etre appeler plusieurs fois
         SaveResul* Complete_SaveResul(const BlocGen & bloc, const Tableau <Coordonnee>& tab_coor
                                       ,const Loi_comp_abstraite* loi) {};

	        // ---- récupération d'information: spécifique à certaine classe dérivée
	        double Deformation_plastique() {  return epsilon_barre_t;}
	        
	        // données protégées
	        double epsilon_barre; // déformation plastique cumulée
	        Tenseur3BB def_plasBB; // déformation plastique
	        double epsilon_barre_t; // la dernière enregistrée
	        Tenseur3BB def_plasBB_t; // déformation plastique, la dernière enregistrée
	        // les tenseurs def_plasBB sont exprimés en absolu
   };

		SaveResul * New_et_Initialise()  { return (new SaveResulPrandtl_Reuss());};
		
	    // Lecture des donnees de la classe sur fichier
		void LectureDonneesParticulieres (UtilLecture * ,LesCourbes1D& lesCourbes1D
                                             ,LesFonctions_nD& lesFonctionsnD);
		// affichage de la loi
		void Affiche() const ;
		// test si la loi est complete
		// = 1 tout est ok, =0 loi incomplete
		int TestComplet();
     
  // calcul d'un module d'young équivalent à la loi, ceci pour un
  // chargement nul
  double Module_young_equivalent(Enum_dure ,const Deformation & ,SaveResul * )
    { return E; };
 
  // récupération de la variation relative d'épaisseur calculée: h/h0
  //  cette variation n'est utile que pour des lois en contraintes planes
  // - pour les lois 3D : retour d'un nombre très grand, indiquant que cette fonction est invalide
  // - pour les lois 2D def planes: retour de 0
  // les infos nécessaires à la récupération , sont stockées dans saveResul
  // qui est le conteneur spécifique au point où a été calculé la loi
  virtual double HsurH0(SaveResul * saveResul) const {return ConstMath::tresgrand;};
 
  // création d'une loi à l'identique et ramène un pointeur sur la loi créée
  Loi_comp_abstraite* Nouvelle_loi_identique() const  { return (new Prandtl_Reuss(*this)); };
		
  // affichage et definition interactive des commandes particulières à chaques lois
  void Info_commande_LoisDeComp(UtilLecture& lec);
		  	       
	 //----- lecture écriture de restart -----
	 // cas donne le niveau de la récupération
     // = 1 : on récupère tout
     // = 2 : on récupère uniquement les données variables (supposées comme telles)
	 void Lecture_base_info_loi(ifstream& ent,const int cas,LesReferences& lesRef,LesCourbes1D& lesCourbes1D
                                           ,LesFonctions_nD& lesFonctionsnD);

     // cas donne le niveau de sauvegarde
     // = 1 : on sauvegarde tout
     // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	 void Ecriture_base_info_loi(ofstream& sort,const int cas);
     
	protected :

        // donnees protegees			
	    double E,nu; // paramètres de la partie élastique de la loi
	    Courbe1D* f_ecrouissage; // courbe d'écrouissage
	    // paramètres de l'algorithme
	    double  tolerance_plas; // tolérance sur la résolution de la plasticité
	    int nb_boucle_maxi; // le maximum d'itération de plasticité permis
	
        // codage des METHODES VIRTUELLES  protegees:
 // calcul des contraintes a t+dt
        // calcul des contraintes 
 void Calcul_SigmaHH (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,TenseurBB & gijBB_t,TenseurHH & gijHH_t,BaseB& giB,BaseH& gi_H, TenseurBB & epsBB_
     ,TenseurBB & delta_epsBB_
     ,TenseurBB & gijBB_,TenseurHH & gijHH_,Tableau <TenseurBB *>& d_gijBB_
     ,double& jacobien_0,double& jacobien,TenseurHH & sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Expli_t_tdt& ex);

       // calcul des contraintes et de ses variations  a t+dt
 void Calcul_DsigmaHH_tdt (TenseurHH & sigHH_t,TenseurBB& DepsBB,DdlElement & tab_ddl
     ,BaseB& giB_t,TenseurBB & gijBB_t,TenseurHH & gijHH_t
     ,BaseB& giB_tdt,Tableau <BaseB> & d_giB_tdt,BaseH& giH_tdt,Tableau <BaseH> & d_giH_tdt
     ,TenseurBB & epsBB_tdt,Tableau <TenseurBB *>& d_epsBB
     ,TenseurBB & delta_epsBB,TenseurBB & gijBB_tdt,TenseurHH & gijHH_tdt
     ,Tableau <TenseurBB *>& d_gijBB_tdt
		  	,Tableau <TenseurHH *>& d_gijHH_tdt,double& jacobien_0,double& jacobien
		  	,Vecteur& d_jacobien_tdt,TenseurHH& sigHH,Tableau <TenseurHH *>& d_sigHH
		  	,EnergieMeca & energ,const EnergieMeca & energ_t,double& module_compressibilite,double&  module_cisaillement
		  	,const Met_abstraite::Impli& ex);

 
        // fonction surchargée dans les classes dérivée si besoin est
        virtual void CalculGrandeurTravail
                      (const PtIntegMecaInterne& ,const Deformation &
                       ,Enum_dure,const ThermoDonnee&
                       ,const Met_abstraite::Impli* ex_impli
                       ,const Met_abstraite::Expli_t_tdt* ex_expli_tdt
                       ,const Met_abstraite::Umat_cont* ex_umat
                       ,const List_io<Ddl_etendu>* exclure_dd_etend
                       ,const List_io<const TypeQuelconque *>* exclure_Q
                       ) {};

};
/// @}  // end of group


#endif		

		

