

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


/************************************************************************
 *           LABORATOIRE DE GENIE MECANIQUE ET MATERIAUX (LG2M)         *
 * Centre de Recherche Rue de Saint Maudé - 56325 Lorient cedex         *
 * tel. 02.97.87.45.70 fax. 02.97.87.45.72 http://www-lg2m.univ-ubs.fr  *
 ************************************************************************
 *     DATE:        3/5/2002                                            *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerard.rio@univ-ubs.fr)             *
 *                  Tel 0297874571   fax : 02.97.87.45.72               *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:   Definition des classes derivees tenseur du 4ieme ordre    *
 *            de dimension3. Ici de nombreuses fonctions ne sont pas    *
 *            pas disponible du à la forme particulière de stockage     *
 *            par contre c'est économique en stockage.                  *
 *            Stockage: (ijkl) = avec 36 valeurs                        *
 *            en fait tenseur symétrique sur ij et sur kl tel que:      *
 *            A(i,j,k,l) = A(j,i,k,l) = A(i,j,l,k) = A(j,i,l,k)         *
 *    typiquement le produit tensoriel de deux tenseurs symétriques :   *
 *    A(i,j,k,l) = B(i,j) * C(k,l)                                      *
 *    Chaque tenseur symétrique comporte 6 composantes                  *
 * B(1,1)->1, B(2,2)->2, B(3,3)->3, B(2,1)->4, B(3,2)->5, B(3,1)->6     *
 *  du coup A est rangé dans un tableau 6x6                             *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
#ifndef TENSEURQ3_H
#define TENSEURQ3_H

#include <iostream> 
#include "TenseurQ.h"  
#include "PtTabRel.h"
#include "Tenseur3.h"
# include "Tableau2_T.h"

//------------------------------------------------------------------
//          cas des composantes 4 fois contravariantes 3HHHH
//------------------------------------------------------------------ 
class  TenseurBBHH; 
class  TenseurHHBB;
    
  
class Tenseur3HHHH : public TenseurHHHH
{ // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, Tenseur3HHHH &);
  // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const Tenseur3HHHH &);
 
  public :
    // Constructeur
    Tenseur3HHHH() ; // par défaut
    // initialisation de toutes les composantes a une meme valeur val 
    Tenseur3HHHH(const double val);  
    // initialisation à partir d'un produit tensoriel avec 2 cas
    // booleen = true : produit tensoriel normal 
    //                  *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
    // booleen = false : produit tensoriel barre, qui conserve les symétries
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
    Tenseur3HHHH(bool normal, const TenseurHH & aHH, const TenseurHH & bHH); 

    // DESTRUCTEUR :
    ~Tenseur3HHHH() ;
    // constructeur a partir d'une instance non differenciee  
    // dans le cas d'un tenseur quelconque, celui-ci
    // est converti à condition que les symétries existent sinon erreur en debug
    // opération longue dans ce cas !
    Tenseur3HHHH (const  TenseurHHHH &); 
    // constructeur de copie  
    Tenseur3HHHH (const  Tenseur3HHHH &); 
    
   // METHODES PUBLIQUES :
//2)    virtuelles    
    // initialise toutes les composantes à val
    void Inita(double val) ;   
    // operations 
    TenseurHHHH & operator + ( const TenseurHHHH &) const ;
    void operator += ( const TenseurHHHH &);
    TenseurHHHH & operator - () const ;  // oppose du tenseur
    TenseurHHHH & operator - ( const TenseurHHHH &) const ;
    void  operator -= ( const TenseurHHHH &);
    TenseurHHHH & operator = ( const TenseurHHHH &);
    TenseurHHHH & operator = ( const Tenseur3HHHH & B)
       { return this->operator=((TenseurHHHH &) B); };
    TenseurHHHH & operator * (const double &) const   ;
    void  operator *= ( const double &);
    TenseurHHHH & operator / ( const double &) const ;
    void  operator /= ( const double &);
    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    // A(i,j,k,l)*B(l,k)=A..B
    // on commence par contracter l'indice du milieu puis externe
    TenseurHH& operator && ( const TenseurBB & )  const ;
    // contraction verticale: A(i,j,k,l)*B(k,l)=A:B
    TenseurHH&  ContractionVerticale( const TenseurBB & )  const ;

    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
    // A(i,j,k,l)*B(l,k,o,p)=A....B
    TenseurHHHH& operator && ( const TenseurBBHH & )  const ;
    TenseurHHBB& operator && ( const TenseurBBBB & )  const ;

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bHH(k,l) gBi gBj gBk gBl
    static TenseurHHHH &  Prod_tensoriel(const TenseurHH & aHH, const TenseurHH & bHH) ;
    //fonctions définissant le produit tensoriel croisé de deux tenseurs
    // *this=aHH(i,k).bHH(j,l) gBi gBj gBk gBl
    static TenseurHHHH &  Prod_tensoriel_croise(const TenseurHH & aHH, const TenseurHH & bHH) ;
    //fonctions définissant le produit tensoriel croisé de deux tenseurs
    // *this=aHH(i,l).bHH(j,k) gBi gBj gBk gBl
    static TenseurHHHH &  Prod_tensoriel_croise_croise(const TenseurHH & aHH, const TenseurHH & bHH) ;
 
    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // concervant les symétries !!
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
    static TenseurHHHH &  Prod_tensoriel_barre(const TenseurHH & aHH, const TenseurHH & bHH) ;

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
    TenseurHHHH & Transpose1et2avec3et4() const  ; 
 
    // il s'agit ici de calculer la variation d'un tenseur dans une nouvelle base
    //  --> variation par rapport aux composantes covariantes d'un tenseur (ex: composantes eps_ij)
 
    //  d sigma^ij /  d eps_kl = d gamma^i_{.a} / d eps_kl . sigma^ab . gamma^j_{.b}
    //                          + gamma^i_{.a} . d sigma^ab / d eps_kl. gamma^j_{.b}
    //                          + gamma^i^_{.a} . sigma^ab . d gamma^j_{.b} / d eps_kl

    // connaissant sa variation dans la base actuelle
    // var_tensHHHH : en entrée: la variation du tenseur dans la base initiale qu'on appelle g_i
    //   ex: var_tensHHHH(ijkl) = d A^ij / d eps_kl
    //            : en sortie: la variation du tenseur dans la base finale qu'on appelle gp_i
    // gamma      : en entrée gpH(i) = gamma(i,j) * gH(j)
    // var_gamma   : en entrée : la variation de gamma
    //   ex: var_gamma(i,j,k,l) = d gamma^i_{.j} / d eps_kl
    // tensHH     : le tenseur dont on cherche la variation
    /// -- pour mémoire ---
        // changement de base (cf. théorie) : la matrice beta est telle que:
    // gpB(i) = beta(i,j) * gB(j) <==> gp_i = beta_i^j * g_j
    // et la matrice gamma telle que:
    // gamma(i,j) represente les coordonnees de la nouvelle base duale gpH dans l'ancienne gH
    // gpH(i) = gamma(i,j) * gH(j), i indice de ligne, j indice de colonne
    //   c-a-d= gp^i = gamma^i_j * g^j
    // rappel des différentes relations entre beta et gamma
    // [beta]^{-1} = [gamma]^T      ; [beta]^{-1T} = [gamma]
    // [beta] = [gamma]^{-1T}      ; [beta]^{T} = [gamma]^{-1}
    // changement de base pour un tenseur en  deux fois covariants:
    // [Ap^kl] = [gamma] * [A^ij] * [gamma]^T

    static TenseurHHHH &  Var_tenseur_dans_nouvelle_base
              (const Mat_pleine& gamma,Tenseur3HHHH& var_tensHHHH, const Tableau2 <Tenseur3HH>& var_gamma
              ,const Tenseur3HH& tensHH);
 

    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    virtual void Affectation_trans_dimension(const TenseurHHHH & B,bool plusZero);
 
    // transférer un tenseur général de même dimension accessible en indice, dans un tenseur 36 Tenseur3HHHH
    // il n'y a pas de symétrisation !, seule certaines composantes sont prises en compte
    void TransfertDunTenseurGeneral(const TenseurHHHH & aHHHH);
    
    // ---- manipulation d'indice ----
    // on baisse les deux premiers indices -> création d'un tenseur TenseurBBHH 
    TenseurBBHH& Baisse2premiersIndices();
    // on baisse les deux derniers indices -> création d'un tenseur TenseurHHBB 
    TenseurHHBB& Baisse2derniersIndices();
    
    // calcul des composantes locales du tenseur considéré s'exprimé dans une  base absolue
    // en argument : A -> une reference sur le tenseur résultat qui peut avoir une dimension
    // différente du tenseur courant suivant que la dimension absolue et la dimension locale
    // sont égales ou différentes ,  retour d'une reference sur A
    TenseurHHHH & Baselocale(TenseurHHHH & A,const BaseH & gi) const ;
 
    // changement des composantes du tenseur, retour donc dans la même variance
    // en argument : A -> une reference sur le tenseur résultat qui a la même dimension
    // retour d'une reference sur A
    // A = A^{ijkl) g_i rond g_j rond g_k rond g_l =  A'^{efgh) gp_i rond gpp_j rond g_k rond gp_l
    // g_i = beta_i^j gp_j --> A'^{efgh) = A^{ijkl) beta_i^e beta_j^f beta_k^g beta_l^h
    TenseurHHHH & ChangeBase(TenseurHHHH & A,const BaseB & gi) const;
    
    // test
    int operator == ( const TenseurHHHH &) const ;
    
    // change la composante i,j,k,l du tenseur
    // acces en  ecriture, 
   	void Change (int i, int j, int k, int l,const double& val) ;
    // en cumul : équivalent de +=
    void ChangePlus (int i, int j, int k, int l,const double& val);

    // Retourne la composante i,j,k,l du tenseur
    // acces en lecture seule
	   double  operator () (int i, int j, int k, int l) const ;
        
    // calcul du maximum en valeur absolu des composantes du tenseur
    double MaxiComposante() const;
             	 
	   // lecture et écriture de données
    istream & Lecture(istream & entree);
    ostream & Ecriture(ostream & sort) const ;
 
    // affichage sous forme de tableau bidim
    void Affiche_bidim(ostream & sort) const ;
       
  protected :
    // allocator dans la liste de data
	  listdouble36Iter ipointe;
	  // --- gestion de changement  d'index ----
	  class ChangementIndex
	   { public:
	     ChangementIndex(); 
	     // passage pour les index de la forme vecteur à la forme i,j
	     Tableau <int> idx_i,idx_j; 
	     // passage pour les index de la forme i,j à la forme vecteur
	     Tableau2 <int> odVect;  
	    };
  public :    
	   static const ChangementIndex  cdex3HHHH;
	
	   // fonction pour le poduit contracté à gauche
    TenseurHH& Prod_gauche( const TenseurBB & F) const;
        
  }; 
//  
//------------------------------------------------------------------
//          cas des composantes 4 fois covariantes
//------------------------------------------------------------------
class Tenseur3BBBB : public TenseurBBBB
{ // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, Tenseur3BBBB &);
  // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const Tenseur3BBBB &);
 
  public :
    // Constructeur
    Tenseur3BBBB() ; // par défaut
    // initialisation de toutes les composantes a une meme valeur val 
    Tenseur3BBBB(const double val);  
    // initialisation à partir d'un produit tensoriel avec 2 cas
    // booleen = true : produit tensoriel normal 
    //                  *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
    // booleen = false : produit tensoriel barre
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
    Tenseur3BBBB(bool normal, const TenseurBB & aBB, const TenseurBB & bBB); 

    // DESTRUCTEUR :
    ~Tenseur3BBBB() ;
    // constructeur a partir d'une instance non differenciee  
    // dans le cas d'un tenseur quelconque, celui-ci
    // est converti à condition que les symétries existent sinon erreur en debug
    // opération longue dans ce cas !
    Tenseur3BBBB (const  TenseurBBBB &); 
    // constructeur de copie  
    Tenseur3BBBB (const  Tenseur3BBBB &); 
    
   // METHODES PUBLIQUES :
//2)    virtuelles    
    // initialise toutes les composantes à val
    void Inita(double val) ;   
    // operations 
    TenseurBBBB & operator + ( const TenseurBBBB &) const ;
    void operator += ( const TenseurBBBB &);
    TenseurBBBB & operator - () const ;  // oppose du tenseur
    TenseurBBBB & operator - ( const TenseurBBBB &) const ;
    void  operator -= ( const TenseurBBBB &);
    TenseurBBBB & operator = ( const TenseurBBBB &);
    TenseurBBBB & operator = ( const Tenseur3HHHH & B)
       { return this->operator=((TenseurBBBB &) B); };
    TenseurBBBB & operator * (const double &) const   ;
    void  operator *= ( const double &);
    TenseurBBBB & operator / ( const double &) const ;
    void  operator /= ( const double &);
    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    // A(i,j,k,l)*B(l,k)=A..B
    // on commence par contracter l'indice du milieu puis externe
    TenseurBB& operator && ( const TenseurHH & )  const ;
    // contraction verticale: A(i,j,k,l)*B(k,l)=A:B
    TenseurBB&  ContractionVerticale( const TenseurHH & )  const ;
 
    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
    // A(i,j,k,l)*B(l,k,o,p)=A....B
    TenseurBBBB& operator && ( const TenseurHHBB & )  const ;
    TenseurBBHH& operator && ( const TenseurHHHH & )  const ;

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bBB(k,l) gHi gHj gHk gHl
    static TenseurBBBB &  Prod_tensoriel(const TenseurBB & aBB, const TenseurBB & bBB) ;
    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // concervant les symétries !!
    // *this(i,j,k,l) 
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
    static TenseurBBBB &  Prod_tensoriel_barre(const TenseurBB & aBB, const TenseurBB & bBB) ;

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
    TenseurBBBB & Transpose1et2avec3et4() const  ; 
    
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    virtual void Affectation_trans_dimension(const TenseurBBBB & B,bool plusZero);
 
    // transférer un tenseur général de même dimension accessible en indice, dans un tenseur 36 Tenseur3HHHH
    // il n'y a pas de symétrisation !, seule certaines composantes sont prises en compte
    void TransfertDunTenseurGeneral(const TenseurBBBB & aBBBB);
    
    // ---- manipulation d'indice ----
    // on monte les deux premiers indices -> création d'un tenseur TenseurHHBB 
    TenseurHHBB& Monte2premiersIndices();
    // on monte les deux derniers indices -> création d'un tenseur TenseurBBHH 
    TenseurBBHH& Monte2derniersIndices();    
    // on monte les 4 indices -> création d'un tenseur TenseurHHHH 
    TenseurHHHH& Monte4Indices();    
    
    // calcul des composantes locales du tenseur considéré s'exprimé dans une  base absolue
    // en argument : A -> une reference sur le tenseur résultat qui peut avoir une dimension
    // différente du tenseur courant suivant que la dimension absolue et la dimension locale
    // sont égales ou différentes ,  retour d'une reference sur A
    TenseurBBBB & Baselocale(TenseurBBBB & A,const BaseB & gi) const ;
    // changement des composantes du tenseur, retour donc dans la même variance
    // en argument : A -> une reference sur le tenseur résultat qui a la même dimension
    // retour d'une reference sur A
    // A = A_{ijkl) g^i rond g^j rond g^k rond g_l =  A'_{efgh) gp^i rond gpp^j rond g^k rond gp^l
    // g^i = gamma^i_j gp^j --> A'_{efgh) = A_{ijkl) gamma^i_e gamma^j_f gamma^k_g gamma^l_h
    TenseurBBBB & ChangeBase(TenseurBBBB & A,const BaseH & gi) const;
    
    
    // test
    int operator == ( const TenseurBBBB &) const ;
    
    // Retourne la composante i,j,k,l du tenseur
    // acces en  ecriture, 
 	  void Change (int i, int j, int k, int l,const double& val) ;
    // en cumul : équivalent de +=
    void ChangePlus (int i, int j, int k, int l,const double& val);

    // Retourne la composante i,j,k,l du tenseur
    // acces en lecture seule
	   double  operator () (int i, int j, int k, int l) const ;
        
    // calcul du maximum en valeur absolu des composantes du tenseur
    double MaxiComposante() const;
             	 
	   // lecture et écriture de données
    istream & Lecture(istream & entree);
    ostream & Ecriture(ostream & sort) const ;
 
    // affichage sous forme de tableau bidim
    void Affiche_bidim(ostream & sort) const ;

  protected :
    // allocator dans la liste de data
	  listdouble36Iter ipointe;  
	  // --- gestion de changement  d'index ----
	  class ChangementIndex
	   { public:
	     ChangementIndex(); 
	     // passage pour les index de la forme vecteur à la forme i,j
	     Tableau <int> idx_i,idx_j; 
	     // passage pour les index de la forme i,j à la forme vecteur
	     Tableau2 <int> odVect;  
	    };
  public :    
	   static const ChangementIndex  cdex3BBBB;
	
	   // fonction pour le poduit contracté à gauche
    TenseurBB& Prod_gauche( const TenseurHH & F) const;

  }; 

//  
//------------------------------------------------------------------
//          cas des composantes mixte 3BBHH
//------------------------------------------------------------------
      
class Tenseur3BBHH : public TenseurBBHH
{ // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, Tenseur3BBHH &);
  // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const Tenseur3BBHH &);
 
  public :
    // Constructeur
    Tenseur3BBHH() ; // par défaut
    // initialisation de toutes les composantes a une meme valeur val 
    Tenseur3BBHH(const double val);  
    // initialisation à partir d'un produit tensoriel 
    //                  *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl
    Tenseur3BBHH(const TenseurBB & aBB, const TenseurHH & bHH); 

    // DESTRUCTEUR :
    ~Tenseur3BBHH() ;
    // constructeur a partir d'une instance non differenciee  
    // dans le cas d'un tenseur quelconque, celui-ci
    // est converti à condition que les symétries existent sinon erreur en debug
    // opération longue dans ce cas !
    Tenseur3BBHH (const  TenseurBBHH &); 
    // constructeur de copie  
    Tenseur3BBHH (const  Tenseur3BBHH &); 
    
   // METHODES PUBLIQUES :
//2)    virtuelles    
    // initialise toutes les composantes à val
    void Inita(double val) ;   
    // operations 
    TenseurBBHH & operator + ( const TenseurBBHH &) const ;
    void operator += ( const TenseurBBHH &);
    TenseurBBHH & operator - () const ;  // oppose du tenseur
    TenseurBBHH & operator - ( const TenseurBBHH &) const ;
    void  operator -= ( const TenseurBBHH &);
    TenseurBBHH & operator = ( const TenseurBBHH &);
    TenseurBBHH & operator * (const double &) const   ;
    void  operator *= ( const double &);
    TenseurBBHH & operator / ( const double &) const ;
    void  operator /= ( const double &);
    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    // A(i,j,k,l)*B(l,k)=A..B
    // on commence par contracter l'indice du milieu puis externe
    TenseurBB& operator && ( const TenseurBB & )  const ;
    // contraction verticale: A(i,j,k,l)*B(k,l)=A:B
    TenseurBB&  ContractionVerticale( const TenseurBB & )  const ;

    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
    // A(i,j,k,l)*B(l,k,o,p)=A....B
    TenseurBBBB& operator && ( const TenseurBBBB & )  const ;
    TenseurBBHH& operator && ( const TenseurBBHH & )  const ;

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl
    static TenseurBBHH &  Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH) ;

    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // concervant les symétries !!
    // *this(i,j,k,l)
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
    static TenseurBBHH &  Prod_tensoriel_barre(const TenseurBB & aBB, const TenseurHH & bHH) ;

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
    TenseurHHBB & Transpose1et2avec3et4() const  ;
 
    // il s'agit ici de calculer la variation d'un tenseur dans une nouvelle base
    //  --> variation par rapport aux composantes covariantes d'un tenseur (ex: composantes eps_ij)
 
    //  d sigma_ij /  d eps_kl = d beta_i^{.a} / d eps_kl . sigma_ab . beta_j^{.b}
    //                          + beta_i^{.a} . d sigma_ab / d eps_kl. beta_j^{.b}
    //                          + beta_i^{.a} . sigma_ab . d beta_j^{.b} / d eps_kl

    // connaissant sa variation dans la base actuelle
    // var_tensBBHH : en entrée: la variation du tenseur dans la base initiale qu'on appelle g^i
    //   ex: var_tensBBHH(ijkl) = d A_ij / d eps_kl
    //            : en sortie: la variation du tenseur dans la base finale qu'on appelle gp^i
    // beta       : en entrée gpB(i) = beta(i,j) * gB(j)
    // var_beta   : en entrée : la variation de beta
    //   ex: var_beta(i,j,k,l) = d beta_i^{.j} / d eps_kl
    // tensBB     : le tenseur dont on cherche la variation
    /// -- pour mémoire ---
        // changement de base (cf. théorie) : la matrice beta est telle que:
    // gpB(i) = beta(i,j) * gB(j) <==> gp_i = beta_i^j * g_j
    // et la matrice gamma telle que:
    // gamma(i,j) represente les coordonnees de la nouvelle base duale gpH dans l'ancienne gH
    // gpH(i) = gamma(i,j) * gH(j), i indice de ligne, j indice de colonne
    //   c-a-d= gp^i = gamma^i_j * g^j
    // rappel des différentes relations entre beta et gamma
    // [beta]^{-1} = [gamma]^T      ; [beta]^{-1T} = [gamma]
    // [beta] = [gamma]^{-1T}      ; [beta]^{T} = [gamma]^{-1}
    // changement de base pour un tenseur en  deux fois covariants:
    // [Ap_kl] = [beta] * [A_ij] * [beta]^T

    static TenseurBBHH &  Var_tenseur_dans_nouvelle_base
              (const Mat_pleine& beta,Tenseur3BBHH& var_tensBBHH, const Tableau2 <Tenseur3HH>& var_beta
              ,const Tenseur3BB& tensBB);
 
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    virtual void Affectation_trans_dimension(const TenseurBBHH & B,bool plusZero);
 
    // test
    int operator == ( const TenseurBBHH &) const ;
    
    // Retourne la composante i,j,k,l du tenseur
    // acces en  ecriture, 
 	  void Change (int i, int j, int k, int l,const double& val) ;
    // en cumul : équivalent de +=
    void ChangePlus (int i, int j, int k, int l,const double& val);

    // Retourne la composante i,j,k,l du tenseur
    // acces en lecture seule
	   double  operator () (int i, int j, int k, int l) const ;
        
    // calcul du maximum en valeur absolu des composantes du tenseur
    double MaxiComposante() const;
             	 
	   // lecture et écriture de données
    istream & Lecture(istream & entree);
    ostream & Ecriture(ostream & sort) const ;
 
    // affichage sous forme de tableau bidim
    void Affiche_bidim(ostream & sort) const ;

  protected :
    // allocator dans la liste de data
	  listdouble36Iter ipointe;  
	  // --- gestion de changement  d'index ----
	  class ChangementIndex
	   { public:
	     ChangementIndex(); 
	     // passage pour les index de la forme vecteur à la forme i,j
	     Tableau <int> idx_i,idx_j; 
	     // passage pour les index de la forme i,j à la forme vecteur
	     Tableau2 <int> odVect;  
	    };
  public :    
	   static const ChangementIndex  cdex3BBHH;
	    
	   // fonction pour le poduit contracté à gauche
    TenseurHH& Prod_gauche( const TenseurHH & F) const;   
};	   
//  
//------------------------------------------------------------------
//          cas des composantes mixte 3HHBB
//------------------------------------------------------------------
      
class Tenseur3HHBB : public TenseurHHBB
{ // surcharge de l'operator de lecture
  friend istream & operator >> (istream &, Tenseur3HHBB &);
  // surcharge de l'operator d'ecriture
  friend ostream & operator << (ostream &, const Tenseur3HHBB &);
 
  public :
    // Constructeur
    Tenseur3HHBB() ; // par défaut
    // initialisation de toutes les composantes a une meme valeur val 
    Tenseur3HHBB(const double val);  
    // initialisation à partir d'un produit tensoriel avec 1 cas
    // booleen = true : produit tensoriel normal 
    //                  *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl
    Tenseur3HHBB(const TenseurHH & aHH, const TenseurBB & bBB); 

    // DESTRUCTEUR :
    ~Tenseur3HHBB() ;
    // constructeur a partir d'une instance non differenciee  
    // dans le cas d'un tenseur quelconque, celui-ci
    // est converti à condition que les symétries existent sinon erreur en debug
    // opération longue dans ce cas !
    Tenseur3HHBB (const  TenseurHHBB &); 
    // constructeur de copie  
    Tenseur3HHBB (const  Tenseur3HHBB &); 
    
   // METHODES PUBLIQUES :
//2)    virtuelles    
    // initialise toutes les composantes à val
    void Inita(double val) ;   
    // operations 
    TenseurHHBB & operator + ( const TenseurHHBB &) const ;
    void operator += ( const TenseurHHBB &);
    TenseurHHBB & operator - () const ;  // oppose du tenseur
    TenseurHHBB & operator - ( const TenseurHHBB &) const ;
    void  operator -= ( const TenseurHHBB &);
    TenseurHHBB & operator = ( const TenseurHHBB &);
    TenseurHHBB & operator * (const double &) const   ;
    void  operator *= ( const double &);
    TenseurHHBB & operator / ( const double &) const ;
    void  operator /= ( const double &);
    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    // A(i,j,k,l)*B(l,k)=A..B
    // on commence par contracter l'indice du milieu puis externe
    TenseurHH& operator && ( const TenseurHH & )  const ;
    // contraction verticale: A(i,j,k,l)*B(k,l)=A:B
    TenseurHH&  ContractionVerticale( const TenseurHH & )  const ;

    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
    // A(i,j,k,l)*B(l,k,o,p)=A....B
    TenseurHHHH& operator && ( const TenseurHHHH & )  const ;
    TenseurHHBB& operator && ( const TenseurHHBB & )  const ;

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl
    static TenseurHHBB &  Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB) ;

    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // concervant les symétries !!
    // *this(i,j,k,l)
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
    static TenseurHHBB &  Prod_tensoriel_barre(const TenseurHH & aHH, const TenseurBB & bBB) ;

    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
    TenseurBBHH & Transpose1et2avec3et4() const  ; 
    
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
    virtual void Affectation_trans_dimension(const TenseurHHBB & B,bool plusZero);
 
    // test
    int operator == ( const TenseurHHBB &) const ;
    
    // Retourne la composante i,j,k,l du tenseur
    // acces en  ecriture, 
 	  void Change (int i, int j, int k, int l,const double& val) ;
    // en cumul : équivalent de +=
    void ChangePlus (int i, int j, int k, int l,const double& val);

    // Retourne la composante i,j,k,l du tenseur
    // acces en lecture seule
	   double  operator () (int i, int j, int k, int l) const ;
        
    // calcul du maximum en valeur absolu des composantes du tenseur
    double MaxiComposante() const;
             	 
	   // lecture et écriture de données
    istream & Lecture(istream & entree);
    ostream & Ecriture(ostream & sort) const ;
 
    // affichage sous forme de tableau bidim
    void Affiche_bidim(ostream & sort) const ;

  protected :
    // allocator dans la liste de data
	  listdouble36Iter ipointe;  
	  // --- gestion de changement  d'index ----
	  class ChangementIndex
	   { public:
	     ChangementIndex(); 
	     // passage pour les index de la forme vecteur à la forme i,j
	     Tableau <int> idx_i,idx_j; 
	     // passage pour les index de la forme i,j à la forme vecteur
	     Tableau2 <int> odVect;  
	    };
  public :    
	   static const ChangementIndex  cdex3HHBB;
	
	   // fonction pour le produit contracté à gauche
    TenseurBB& Prod_gauche( const TenseurBB & F) const;
};	   	   
     
#ifndef MISE_AU_POINT
  #include "TenseurQ3-1.cc"
  #include "TenseurQ3-2.cc"
  #define  TenseurQ3_H_deja_inclus
#endif
    
 
#endif  
