

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "TenseurQ2gene.h"
#include "ConstMath.h" 
#include "MathUtil.h"
#include "Tenseur2.h"
#include "CharUtil.h"

#ifndef  TenseurQ2gene_H_deja_inclus

// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal

//------------------------------------------------------------------
//          cas des composantes mixte 2BBHH
//------------------------------------------------------------------   

    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBHH::TenseurQ2geneBBHH() :
 ipointe() // par défaut
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<16;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBHH::TenseurQ2geneBBHH( const double val) :
 ipointe() 
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<16;i++) t[i]=val;
  };
  
    // initialisation à partir d'un produit tensoriel 
    //                  *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBHH::TenseurQ2geneBBHH(const TenseurBB & aBB, const TenseurHH & bHH) :
 ipointe()
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
    const Tenseur2HH & b2HH = *((Tenseur2HH*) &bHH); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2BB.Dimension()) != 2)
          Message(2,strcat(strcat("produit tensoriel a partir d'un premier tenseur non symétriques \n",
                   "TenseurQ2geneBBHH::TenseurQ2geneBBHH(bool normal, const"),
                    " TenseurBB & aBB, const TenseurHH & bHH);"));
       if (Dabs(b2HH.Dimension()) != 2)
          Message(2,strcat(strcat("produit tensoriel a partir d'un second tenseur non symétriques \n",
                   "TenseurQ2geneBBHH::TenseurQ2geneBBHH(bool normal, const"),
                    " TenseurBB & aBB, const TenseurHH & bHH);"));
    #endif
    for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
          t[8*i+4*j+2*k+l-15] = a2BB(i,j) * b2HH(k,l);
  }; 
       
    // initialisation à partir d'un produit tensoriel d'instance référencée 3 
    //                  *this=a2BB(i,j).b2HH(k,l) gHi gHj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBHH::TenseurQ2geneBBHH(const Tenseur2BB & a2BB, const Tenseur2HH & b2HH) :
 ipointe()
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
          t[8*i+4*j+2*k+l-15] = a2BB(i,j) * b2HH(k,l);
  };      

    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBHH::~TenseurQ2geneBBHH()
{ listdouble16.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBHH::TenseurQ2geneBBHH ( const TenseurBBHH & B) :
 ipointe()
  { dimension = B.dimension;
    #ifdef MISE_AU_POINT
    if (Dabs(dimension) != 20)
      { cout << "\n erreur de dimension, elle devrait etre = 20 ";
        cout << "\n TenseurQ2geneBBHH::TenseurQ2geneBBHH ( TenseurBBHH &) " << endl;
        Sortie(1);
      }  
    #endif
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 16;i++)
       t[i] = B.t[i];
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneBBHH::TenseurQ2geneBBHH (  const TenseurQ2geneBBHH & B) :
 ipointe()
  { this->dimension = B.dimension;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 16;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ2geneBBHH::Inita(double val)
	{ for (int i=0;i< 16;i++)
         t[i] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & TenseurQ2geneBBHH::operator + ( const TenseurBBHH & B) const
  {  TenseurBBHH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 20) Message(2,"TenseurQ2geneBBHH::operator + ( etc..");
     #endif
     res =  new TenseurQ2geneBBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ2geneBBHH::operator += ( const TenseurBBHH & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneBBHH::operator += ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
       this->t[i] += B.t[i];
     LesMaillonsBBHH::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & TenseurQ2geneBBHH::operator - () const
  {  TenseurBBHH * res;
     res =  new TenseurQ2geneBBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & TenseurQ2geneBBHH::operator - ( const TenseurBBHH & B) const
  {  TenseurBBHH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneBBHH::operator - ( etc..");
     #endif
     res =  new TenseurQ2geneBBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2geneBBHH::operator -= ( const TenseurBBHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneBBHH::operator -= ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
       this->t[i] -= B.t[i];
     LesMaillonsBBHH::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & TenseurQ2geneBBHH::operator = ( const TenseurBBHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneBBHH::operator = ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
            this->t[i] = B.t[i];
    LesMaillonsBBHH::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & TenseurQ2geneBBHH::operator * ( const double & b) const
  {  TenseurBBHH * res;
     res =  new TenseurQ2geneBBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2geneBBHH::operator *= ( const double & b)
  {for (int i = 0; i< 16; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & TenseurQ2geneBBHH::operator / ( const double & b) const
  {  TenseurBBHH * res;
     res =  new TenseurQ2geneBBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ2geneBBHH::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i< 16; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2geneBBHH::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ2geneBBHH::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i< 16; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& TenseurQ2geneBBHH::operator && ( const TenseurBB & aBB)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 2)
           Message(2,"TenseurQ2geneBBHH::operator && ( const TenseurBB & aBB)");
     #endif
     TenseurBB * res;
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
     // étant donné que le tenseur symétrique résultat stock la m^me grandeur en 1,2 et 2,1
     // il ne faut calculer que la moitié des composantes sinon on a le double dans le résultat 
     for (int i=1;i<3;i++) for (int j=1;j<=i;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
         res->Coor(i,j) += t[8*i+4*j+2*k+l-15] * a2BB(k,l);
     return *res ;
  };
    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre BBBB
TenseurBBBB& TenseurQ2geneBBHH::operator && ( const TenseurBBBB & aBBBB)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBBB.Dimension()) != 20)
           Message(2,"TenseurQ2geneBBHH::Prod_gauche( const TenseurBBBB & F)");
     #endif
     TenseurBBBB * res;
     res =  new TenseurQ2geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneBBBB & a2BBBB = *((TenseurQ2geneBBBB*) &aBBBB); // passage en dim 2
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       for(int e=1;e<3;e++) for (int f=1;f<3;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2BBBB.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre BBHH
TenseurBBHH& TenseurQ2geneBBHH::operator && ( const TenseurBBHH & aBBHH)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBHH.Dimension()) != 20)
           Message(2,"TenseurQ2geneBBHH::Prod_gauche( const TenseurBBHH & F)");
     #endif
     TenseurBBHH * res;
     res =  new TenseurQ2geneBBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneBBHH & a2BBHH = *((TenseurQ2geneBBHH*) &aBBHH); // passage en dim 2
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       for(int e=1;e<3;e++) for (int f=1;f<3;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2BBHH.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
  
//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& TenseurQ2geneBBHH::Prod_gauche( const TenseurHH & aHH) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 2)
           Message(2,"TenseurQ2geneBBHH::Prod_gauche( const TenseurHH & F)");
     #endif
     TenseurHH * res;
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
     // étant donné que le tenseur symétrique résultat stock la m^me grandeur en 1,2 et 2,1
     // il ne faut calculer que la moitié des composantes sinon on a le double dans le résultat 
     for (int i=1;i<3;i++) for (int j=1;j<=i;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
         res->Coor(i,j) +=  a2HH(k,l) * t[8*k+4*l+2*i+j-15] ;
     return *res ;
  };
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre BBHH
TenseurBBHH& TenseurQ2geneBBHH::Prod_gauche( const TenseurBBHH & aBBHH) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBHH.Dimension()) != 20)
           Message(2,"TenseurQ2geneBBHH::Prod_gauche( const TenseurBBHH & F)");
     #endif
     TenseurBBHH * res;
     res =  new TenseurQ2geneBBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneBBHH & a2BBHH = *((TenseurQ2geneBBHH*) &aBBHH); // passage en dim 2
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       for(int e=1;e<3;e++) for (int f=1;f<3;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2BBHH.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre HHHH
TenseurHHHH& TenseurQ2geneBBHH::Prod_gauche( const TenseurHHHH & aHHHH) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHHH.Dimension()) != 20)
           Message(2,"TenseurQ2geneBBHH::Prod_gauche( const TenseurHHHH & F)");
     #endif
     TenseurHHHH * res;
     res =  new TenseurQ2geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneHHHH & a2HHHH = *((TenseurQ2geneHHHH*) &aHHHH); // passage en dim 2
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       for(int e=1;e<3;e++) for (int f=1;f<3;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2HHHH.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
//=========== fin fonction protected ======================	

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH &  TenseurQ2geneBBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH)
  { TenseurBBHH * res;
    res =  new TenseurQ2geneBBHH;
    LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
    const Tenseur2HH & b2HH = *((Tenseur2HH*) &bHH); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2BB.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ2geneBBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH)";
            Sortie(2);
           }      
       if (Dabs(b2HH.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ2geneBBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] = a2BB(i,j) * b2HH(k,l);
    return *res;                         
  };      


    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & TenseurQ2geneBBHH::Transpose1et2avec3et4() const
   { TenseurHHBB * res;
     res =  new TenseurQ2geneHHBB;
     LesMaillonsHHBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
       res->t[8*k+4*l+2*i+j-15] = t[8*i+4*j+2*k+l-15];
     return *res;
   };
    
#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ2geneBBHH::Affectation_trans_dimension(const TenseurBBHH & aBBHH,bool plusZero)
   { switch (abs(aBBHH.Dimension()))
       { case 33 : case 30 : case 306 : case 22 :  case 206:
           //  on affecte
           for (int i=1;i<3;i++)
            for (int j=1;j<3;j++)
             for (int k=1;k<3;k++)
              for (int l=1;l<3;k++)
                t[8*k+4*l+2*i+j-15] = aBBHH(i,j,k,l);
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aBBHH(1,1,1,1);
           break;
         default:
          Message(2,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aBBHH.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ2geneBBHH::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ2geneBBHH::operator == ( const TenseurBBHH & B) const
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneBBHH::operator == ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
         return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ2geneBBHH::Change (int i, int j, int k, int l, const double& val)
  { t[8*i+4*j+2*k+l-15]  = val;};

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ2geneBBHH::ChangePlus (int i, int j, int k, int l, const double& val)
  { t[8*i+4*j+2*k+l-15]  += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ2geneBBHH::operator () (int i, int j, int k, int l) const
 { return t[8*i+4*j+2*k+l-15]; };
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ2geneBBHH::MaxiComposante() const
  { return DabsMaxiTab(t,16) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ2geneBBHH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ2geneBBHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 16; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ2geneBBHH::Ecriture(ostream & sort) const
  { // écriture du type
    sort << "TenseurQ2geneBBHH ";
    // puis les datas
     for (int i = 0; i< 16; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ2geneBBHH & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 20) A.Message(2,"operator >> (istream & entree, TenseurQ2geneBBHH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ2geneBBHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 16; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ2geneBBHH & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ2geneBBHH ";
    // puis les datas
    for (int i = 0; i< 16; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };

 
//------------------------------------------------------------------
//          cas des composantes mixte 2HHBB
//------------------------------------------------------------------   

    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHBB::TenseurQ2geneHHBB() :
 ipointe() // par défaut
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<16;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHBB::TenseurQ2geneHHBB( const double val) :
 ipointe() 
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<16;i++) t[i]=val;
  };
    // initialisation à partir d'un produit tensoriel 
    //                  *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHBB::TenseurQ2geneHHBB(const TenseurHH & aHH, const TenseurBB & bBB) :
 ipointe()
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
    const Tenseur2BB & b2BB = *((Tenseur2BB*) &bBB); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2HH.Dimension()) != 2)
          Message(2,strcat(strcat("produit tensoriel a partir d'un premier tenseur non symétriques \n",
                   "TenseurQ2geneHHBB::TenseurQ2geneHHBB(bool normal, const"),
                    " TenseurHH & aHH, const TenseurBB & bBB);"));
       if (Dabs(b2BB.Dimension()) != 2)
          Message(2,strcat(strcat("produit tensoriel a partir d'un second tenseur non symétriques \n",
                   "TenseurQ2geneHHBB::TenseurQ2geneHHBB(bool normal, const"),
                    " TenseurHH & aHH, const TenseurBB & bBB);"));
    #endif
    for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          t[8*i+4*j+2*k+l-15] = a2HH(i,j) * b2BB(k,l);
  };      

    // initialisation à partir d'un produit tensoriel d'instance référencée 2
    //                  *this=a2HH(i,j).b2BB(k,l) gBi gBj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHBB::TenseurQ2geneHHBB(const Tenseur2HH & a2HH, const Tenseur2BB & b2BB) :
 ipointe()
  { dimension = 20;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          t[8*i+4*j+2*k+l-15] = a2HH(i,j) * b2BB(k,l);
  };      

    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHBB::~TenseurQ2geneHHBB()
{ listdouble16.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHBB::TenseurQ2geneHHBB ( const TenseurHHBB & B) :
 ipointe()
  { dimension = B.dimension;
    #ifdef MISE_AU_POINT
    if (Dabs(dimension) != 20)
      { cout << "\n erreur de dimension, elle devrait etre = 20 ";
        cout << "\n TenseurQ2geneHHBB::TenseurQ2geneHHBB ( TenseurHHBB &) " << endl;
        Sortie(1);
      }  
    #endif
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 16;i++)
       t[i] = B.t[i];
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurQ2geneHHBB::TenseurQ2geneHHBB (  const TenseurQ2geneHHBB & B) :
 ipointe()
  { this->dimension = B.dimension;
    listdouble16.push_front(Reels16());  // allocation
    ipointe = listdouble16.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i< 16;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void TenseurQ2geneHHBB::Inita(double val)
	{ for (int i=0;i< 16;i++)
         t[i] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & TenseurQ2geneHHBB::operator + ( const TenseurHHBB & B) const
  {  TenseurHHBB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 20) Message(2,"TenseurQ2geneHHBB::operator + ( etc..");
     #endif
     res =  new TenseurQ2geneHHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void TenseurQ2geneHHBB::operator += ( const TenseurHHBB & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneHHBB::operator += ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
       this->t[i] += B.t[i];
     LesMaillonsHHBB::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & TenseurQ2geneHHBB::operator - () const
  {  TenseurHHBB * res;
     res =  new TenseurQ2geneHHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & TenseurQ2geneHHBB::operator - ( const TenseurHHBB & B) const
  {  TenseurHHBB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneHHBB::operator - ( etc..");
     #endif
     res =  new TenseurQ2geneHHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2geneHHBB::operator -= ( const TenseurHHBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneHHBB::operator -= ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
       this->t[i] -= B.t[i];
     LesMaillonsHHBB::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & TenseurQ2geneHHBB::operator = ( const TenseurHHBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneHHBB::operator = ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
            this->t[i] = B.t[i];
    LesMaillonsHHBB::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & TenseurQ2geneHHBB::operator * ( const double & b) const
  {  TenseurHHBB * res;
     res =  new TenseurQ2geneHHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i< 16; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2geneHHBB::operator *= ( const double & b)
  {for (int i = 0; i< 16; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & TenseurQ2geneHHBB::operator / ( const double & b) const
  {  TenseurHHBB * res;
     res =  new TenseurQ2geneHHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ2geneHHBB::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i< 16; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  TenseurQ2geneHHBB::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n TenseurQ2geneHHBB::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i< 16; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& TenseurQ2geneHHBB::operator && ( const TenseurHH & aHH)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 2)
           Message(2,"TenseurQ2geneHHBB::operator && ( const TenseurHH & aHH)");
     #endif
     TenseurHH * res;
     res =  new Tenseur2HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
     // étant donné que le tenseur symétrique résultat stock la m^me grandeur en 1,2 et 2,1
     // il ne faut calculer que la moitié des composantes sinon on a le double dans le résultat 
     for (int i=1;i<3;i++) for (int j=1;j<=i;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
         res->Coor(i,j) += t[8*i+4*j+2*k+l-15] * a2HH(k,l);
     return *res ;
  };
    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre HHBB
TenseurHHBB& TenseurQ2geneHHBB::operator && ( const TenseurHHBB & aHHBB)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHBB.Dimension()) != 20)
           Message(2,"TenseurQ2geneHHBB::Prod_gauche( const TenseurHHBB & F)");
     #endif
     TenseurHHBB * res;
     res =  new TenseurQ2geneHHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneHHBB & a2HHBB = *((TenseurQ2geneHHBB*) &aHHBB); // passage en dim 2
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       for(int e=1;e<3;e++) for (int f=1;f<3;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2HHBB.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre HHHH
TenseurHHHH& TenseurQ2geneHHBB::operator && ( const TenseurHHHH & aHHHH)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHHH.Dimension()) != 20)
           Message(2,"TenseurQ2geneHHBB::Prod_gauche( const TenseurHHHH & F)");
     #endif
     TenseurHHHH * res;
     res =  new TenseurQ2geneHHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneHHHH & a2HHHH = *((TenseurQ2geneHHHH*) &aHHHH); // passage en dim 2
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       for(int e=1;e<3;e++) for (int f=1;f<3;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2HHHH.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
  
//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& TenseurQ2geneHHBB::Prod_gauche( const TenseurBB & aBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 2)
           Message(2,"TenseurQ2geneHHBB::Prod_gauche( const TenseurBB & F)");
     #endif
     TenseurBB * res;
     res =  new Tenseur2BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur2BB & a2BB = *((Tenseur2BB*) &aBB); // passage en dim 2
     // étant donné que le tenseur symétrique résultat stock la m^me grandeur en 1,2 et 2,1
     // il ne faut calculer que la moitié des composantes sinon on a le double dans le résultat 
     for (int i=1;i<3;i++) for (int j=1;j<=i;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
         res->Coor(i,j) +=  a2BB(k,l) * t[8*k+4*l+2*i+j-15] ;
     return *res ;
  };
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre BBHH
TenseurBBBB& TenseurQ2geneHHBB::Prod_gauche( const TenseurBBBB & aBBBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBBB.Dimension()) != 20)
           Message(2,"TenseurQ2geneHHBB::Prod_gauche( const TenseurBBBB & F)");
     #endif
     TenseurBBBB * res;
     res =  new TenseurQ2geneBBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneBBBB & a2BBBB = *((TenseurQ2geneBBBB*) &aBBBB); // passage en dim 2
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       for(int e=1;e<3;e++) for (int f=1;f<3;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2BBBB.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
#ifndef MISE_AU_POINT
  inline 
#endif
// cas d'un tenseur d'ordre quatre HHHH
TenseurHHBB& TenseurQ2geneHHBB::Prod_gauche( const TenseurHHBB & aHHBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHBB.Dimension()) != 20)
           Message(2,"TenseurQ2geneHHBB::Prod_gauche( const TenseurHHBB & F)");
     #endif
     TenseurHHBB * res;
     res =  new TenseurQ2geneHHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const TenseurQ2geneHHBB & a2HHBB = *((TenseurQ2geneHHBB*) &aHHBB); // passage en dim 2
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       for(int e=1;e<3;e++) for (int f=1;f<3;f++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] += a2HHBB.t[8*l+4*k+2*e+f-15] * this->t[8*f+4*e+2*k+l-15];
     return *res;                         
  };
//=========== fin fonction protected ======================	

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB &  TenseurQ2geneHHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB)
  { TenseurHHBB * res;
    res =  new TenseurQ2geneHHBB;
    LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur2HH & a2HH = *((Tenseur2HH*) &aHH); // passage en dim 2
    const Tenseur2BB & b2BB = *((Tenseur2BB*) &bBB); // passage en dim 2
    #ifdef MISE_AU_POINT
       if (Dabs(a2HH.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "TenseurQ2geneHHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB)";
            Sortie(2);
           }      
       if (Dabs(b2BB.Dimension()) != 2)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "TenseurQ2geneHHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB)";
            Sortie(2);
           }      
    #endif
    for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
         // (((i-1)2+(j-1))2+(k-1))2+l-1
          res->t[8*i+4*j+2*k+l-15] = a2HH(i,j) * b2BB(k,l);
    return *res;                         
  };      


    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & TenseurQ2geneHHBB::Transpose1et2avec3et4() const
   { TenseurBBHH * res;
     res =  new TenseurQ2geneBBHH;
     LesMaillonsBBHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int i=1;i<3;i++) for (int j=1;j<3;j++) for (int k=1;k<3;k++) for (int l=1;l<3;l++)
       // (((i-1)2+(j-1))2+(k-1))2+l-1
       res->t[8*k+4*l+2*i+j-15] = t[8*i+4*j+2*k+l-15];
     return *res;
   };
    
#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void TenseurQ2geneHHBB::Affectation_trans_dimension(const TenseurHHBB & aHHBB,bool plusZero)
   { switch (abs(aHHBB.Dimension()))
       { case 33 : case 30 : case 306 : case 22 :  case 206:
           // on affecte
           for (int i=1;i<3;i++)
            for (int j=1;j<3;j++)
             for (int k=1;k<3;k++)
              for (int l=1;l<3;k++)
                t[8*k+4*l+2*i+j-15] = aHHBB(i,j,k,l);
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aHHBB(1,1,1,1);
           break;
         default:
          Message(2,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aHHBB.Dimension()))
                    +"n'est pas prise en compte \n TenseurQ2geneHHBB::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int TenseurQ2geneHHBB::operator == ( const TenseurHHBB & B) const
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 20) Message(2,"TenseurQ2geneHHBB::operator == ( etc..");
     #endif
     for (int i = 0; i< 16; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
         return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void TenseurQ2geneHHBB::Change (int i, int j, int k, int l,const double& val)
  { t[8*i+4*j+2*k+l-15]  = val;};

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void TenseurQ2geneHHBB::ChangePlus (int i, int j, int k, int l,const double& val)
  { t[8*i+4*j+2*k+l-15]  += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  TenseurQ2geneHHBB::operator () (int i, int j, int k, int l) const
 { return t[8*i+4*j+2*k+l-15]; };
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double TenseurQ2geneHHBB::MaxiComposante() const
  { return DabsMaxiTab(t,16) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & TenseurQ2geneHHBB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ2geneHHBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 16; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & TenseurQ2geneHHBB::Ecriture(ostream & sort) const
  { // écriture du type
    sort << "TenseurQ2geneHHBB ";
    // puis les datas
     for (int i = 0; i< 16; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };
       
#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, TenseurQ2geneHHBB & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 20) A.Message(2,"operator >> (istream & entree, TenseurQ2geneHHBB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "TenseurQ2geneHHBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i< 16; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const TenseurQ2geneHHBB & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "TenseurQ2geneHHBB ";
    // puis les datas
    for (int i = 0; i< 16; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };
  
#endif
