

// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.


//#include "Debug.h"
#include "TenseurQ-3.h" 
#include "ConstMath.h" 
#include "MathUtil.h"
#include "Tenseur3.h"
#include "CharUtil.h"

#ifndef  TenseurQ3_H_deja_inclus

// variables globales
// initialisation dans EnteteTenseur.h , utilisé dans le progr principal

//------------------------------------------------------------------
//          cas des composantes mixte 3BBHH
//------------------------------------------------------------------   
//    typiquement le produit tensoriel de deux tenseurs symétriques :
//    A(i,j,k,l) = B(i,j) * C(k,l)
//    Chaque tenseur symétrique comporte 6 composantes
// B(1,1)->1, B(2,2)->2, B(3,3)->3, B(2,1)->4, B(3,2)->5, B(3,1)->6
//  du coup A est rangé dans un tableau 6x6

	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBHH::ChangementIndex::ChangementIndex() :
  idx_i(6),idx_j(6),odVect(3)
  { idx_i(1)=1;idx_i(2)=2;idx_i(3)=3;idx_i(4)=2;idx_i(5)=3;idx_i(6)=3;
    idx_j(1)=1;idx_j(2)=2;idx_j(3)=3;idx_j(4)=1;idx_j(5)=2;idx_j(6)=1;
    odVect(1,1)=1;odVect(1,2)=4;odVect(1,3)=6;
    odVect(2,1)=4;odVect(2,2)=2;odVect(2,3)=5;
    odVect(3,1)=6;odVect(3,2)=5;odVect(3,3)=3;
  }; 


    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBHH::Tenseur3BBHH() :  
 ipointe() // par défaut
  { dimension = 33;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<36;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBHH::Tenseur3BBHH( const double val)  :  
 ipointe()
  { dimension = 33;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<36;i++) t[i]=val;
  };
    // initialisation à partir d'un produit tensoriel 
    //                  *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBHH::Tenseur3BBHH(const TenseurBB & aBB, const TenseurHH & bHH) :  
 ipointe()
  { dimension = 33;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
    const Tenseur3HH & b3HH = *((Tenseur3HH*) &bHH); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3BB.Dimension()) != 3) 
          Message(3,string("produit tensoriel a partir d'un premier tenseur non symétriques \n")
                   +"Tenseur3BBHH::Tenseur3BBHH(bool normal, const"
                    +" TenseurBB & aBB, const TenseurHH & bHH);");
       if (Dabs(b3HH.Dimension()) != 3) 
          Message(3,string("produit tensoriel a partir d'un second tenseur non symétriques \n")
                   +"Tenseur3BBHH::Tenseur3BBHH(bool normal, const"
                    " TenseurBB & aBB, const TenseurHH & bHH);");
    #endif
    for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          t[(ij-1)*6+kl-1] =   a3BB(cdex3BBHH.idx_i(ij),cdex3BBHH.idx_j(ij))
                             * b3HH(cdex3BBHH.idx_i(kl),cdex3BBHH.idx_j(kl));
  };      
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBHH::~Tenseur3BBHH() 
{ listdouble36.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBHH::Tenseur3BBHH ( const TenseurBBHH & B) :  
 ipointe()
  { dimension = 33;
//    #ifdef MISE_AU_POINT
//    if (Dabs(dimension) != 33)
//      { cout << "\n erreur de dimension, elle devrait etre = 33 ";
//        cout << "\n Tenseur3BBHH::Tenseur3BBHH ( TenseurBBHH &) " << endl;
//        Sortie(1);
//      }  
//    #endif
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    if (Dabs(B.dimension) == 33 ) // cas d'un tenseur du même type
      { for (int i=0;i<36;i++)
          t[i] = B.t[i];
      }
    else
      {// cas d'un tenseur quelconque
       double Z=B.MaxiComposante();
       for (int i=1;i < 4;i++)
         for (int j=1;j<=i;j++)
           for (int k=1;k < 4;k++)
             for (int l=1;l<=k;l++)
             	{// on teste les symétries et on affecte 
             	 double a = B(i,j,k,l);
              #ifdef MISE_AU_POINT 
                 if ((!diffpourcent(a,B(j,i,k,l),Z,ConstMath::unpeupetit)
                    && !diffpourcent(a,B(i,j,l,k),Z,ConstMath::unpeupetit))
                    || (Abs(Z) < ConstMath::trespetit) )
                 // erreur d'affectation
                 if (ParaGlob::NiveauImpression() > 5)
                    cout << "\n tenseurBBHH (ijkl= " << i << "," << j << "," << k << "," << l << ")= " 
                         << a << " " << B(j,i,k,l) << " " <<B(i,j,l,k) ; 
                 cout << "WARNING ** erreur constructeur, tenseur non symetrique, Tenseur3BBHH::Tenseur3BBHH(const TenseurBBHH & B)";
              #endif
                 // si il y a un pb normalement il y a eu un message
                 this->Change(i,j,k,l,a);
             	}
      };
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3BBHH::Tenseur3BBHH (  const Tenseur3BBHH & B) :  
 ipointe()
  { this->dimension = B.dimension;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<36;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void Tenseur3BBHH::Inita(double val)   
	{ for (int i=0;i< 36;i++)
         t[i] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur3BBHH::operator + ( const TenseurBBHH & B) const 
  {  TenseurBBHH * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 33) Message(3,"Tenseur3BBHH::operator + ( etc..");
     #endif
     res =  new Tenseur3BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<36; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur3BBHH::operator += ( const TenseurBBHH & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3BBHH::operator += ( etc..");
     #endif
     for (int i = 0; i<36; i++)
       this->t[i] += B.t[i];
     LesMaillonsBBHH::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur3BBHH::operator - () const 
  {  TenseurBBHH * res;
     res =  new Tenseur3BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<36; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur3BBHH::operator - ( const TenseurBBHH & B) const 
  {  TenseurBBHH * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3BBHH::operator - ( etc..");
     #endif
     res =  new Tenseur3BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<36; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur3BBHH::operator -= ( const TenseurBBHH & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3BBHH::operator -= ( etc..");
     #endif
     for (int i = 0; i<36; i++)
       this->t[i] -= B.t[i];
     LesMaillonsBBHH::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur3BBHH::operator = ( const TenseurBBHH & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3BBHH::operator = ( etc..");
     #endif
     for (int i = 0; i<36; i++)
            this->t[i] = B.t[i];
    LesMaillonsBBHH::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur3BBHH::operator * ( const double & b) const 
  {  TenseurBBHH * res;
     res =  new Tenseur3BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<36; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur3BBHH::operator *= ( const double & b)
  {for (int i = 0; i<36; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur3BBHH::operator / ( const double & b) const 
  {  TenseurBBHH * res;
     res =  new Tenseur3BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur3BBHH::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i<36; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur3BBHH::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur3BBHH::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i<36; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    // A(i,j,k,l)*B(l,k)=A..B
    // on commence par contracter l'indice du milieu puis externe
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& Tenseur3BBHH::operator && ( const TenseurBB & aBB)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 3) 
           Message(3,"Tenseur3BBHH::operator && ( const TenseurBB & aBB)");
     #endif
     TenseurBB * res;
     res =  new Tenseur3BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
      {for (int kl=1;kl < 4;kl++) // partie simple produit : la partie diagonale
         res->Coor(cdex3BBHH.idx_i(ij),cdex3BBHH.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                                             * a3BB(cdex3BBHH.idx_i(kl),cdex3BBHH.idx_j(kl));
       // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       for (int kl=4;kl < 7;kl++)
         res->Coor(cdex3BBHH.idx_i(ij),cdex3BBHH.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                       * (  a3BB(cdex3BBHH.idx_i(kl),cdex3BBHH.idx_j(kl))
                          + a3BB(cdex3BBHH.idx_j(kl),cdex3BBHH.idx_i(kl))
                         );
       };
   return *res ;
  };

    // contraction verticale: A(i,j,k,l)*B(k,l)=A:B
#ifndef MISE_AU_POINT
  inline
#endif
TenseurBB& Tenseur3BBHH::ContractionVerticale( const TenseurBB & aBB)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 3)
           Message(3,"Tenseur3BBHH::operator && ( const TenseurBB & aBB)");
     #endif
     TenseurBB * res;
     res =  new Tenseur3BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
      {for (int kl=1;kl < 4;kl++) // partie simple produit : la partie diagonale
         res->Coor(cdex3BBHH.idx_i(ij),cdex3BBHH.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                                             * a3BB(cdex3BBHH.idx_j(kl),cdex3BBHH.idx_i(kl));
       // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       for (int kl=4;kl < 7;kl++)
         res->Coor(cdex3BBHH.idx_i(ij),cdex3BBHH.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                       * (  a3BB(cdex3BBHH.idx_j(kl),cdex3BBHH.idx_i(kl))
                          + a3BB(cdex3BBHH.idx_i(kl),cdex3BBHH.idx_j(kl))
                         );
       };
   return *res ;
  };

    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline
#endif
// cas d'un tenseur d'ordre quatre BBHH
TenseurBBBB& Tenseur3BBHH::operator && ( const TenseurBBBB & aBBBB)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBBB.Dimension()) != 33)
           Message(3,"Tenseur3BBHH::operator && ( const TenseurBBBB & aBBBB)");
     #endif
     TenseurBBBB * res;
     res =  new Tenseur3BBBB;
     LesMaillonsBBBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3BBBB & a3BBBB = *((Tenseur3BBBB*) &aBBBB); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
       for (int kl=1;kl < 7;kl++)
        {double& resul= res->t[(ij-1)*6+kl-1];
         for (int n=1;n<4;n++) for (int m=1;m<4;m++)
           {int ef = cdex3BBHH.odVect(n,m);
            resul += this->t[(ij-1)*6+ef-1] * a3BBBB.t[(ef-1)*6+kl-1];
           };
//         for (int ef=1;ef < 7;ef++)
//           resul += this->t[(ij-1)*6+ef-1] * a3BBBB.t[(ef-1)*6+kl-1];
        };
     return *res;
  };


    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline
#endif
// cas d'un tenseur d'ordre quatre BBHH
TenseurBBHH& Tenseur3BBHH::operator && ( const TenseurBBHH & aBBHH)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBBHH.Dimension()) != 33)
           Message(3,"Tenseur3BBHH::operator && ( const TenseurBBHH & aBBHH)");
     #endif
     TenseurBBHH * res;
     res =  new Tenseur3BBHH;
     LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3BBHH & a3BBHH = *((Tenseur3BBHH*) &aBBHH); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
       for (int kl=1;kl < 7;kl++)
        {double& resul= res->t[(ij-1)*6+kl-1];
         for (int n=1;n<4;n++) for (int m=1;m<4;m++)
           {int ef = cdex3BBHH.odVect(n,m);
            resul += this->t[(ij-1)*6+ef-1] * a3BBHH.t[(ef-1)*6+kl-1];
           };
//         for (int ef=1;ef < 7;ef++)
//           resul += this->t[(ij-1)*6+ef-1] * a3BBHH.t[(ef-1)*6+kl-1];
        };
     return *res;
  };

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aBB(i,j).bHH(k,l) gHi gHj gBk gBl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH &  Tenseur3BBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH) 
  { TenseurBBHH * res;
    res =  new Tenseur3BBHH;
    LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
    const Tenseur3HH & b3HH = *((Tenseur3HH*) &bHH); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3BB.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur3BBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH)";
            Sortie(2);
           }      
       if (Dabs(b3HH.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur3BBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH)";
            Sortie(2);
           }      
    #endif
    for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          res->t[(ij-1)*6+kl-1] =   a3BB(cdex3BBHH.idx_i(ij),cdex3BBHH.idx_j(ij))
                                  * b3HH(cdex3BBHH.idx_i(kl),cdex3BBHH.idx_j(kl));
    return *res;                         
  };      

    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // concervant les symétries !!
    // *this(i,j,k,l)
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
#ifndef MISE_AU_POINT
  inline
#endif
TenseurBBHH &  Tenseur3BBHH::Prod_tensoriel_barre(const TenseurBB & aBB, const TenseurHH & bHH)
  { TenseurBBHH * res;
    res =  new Tenseur3BBHH;
    LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
    const Tenseur3HH & b3HH = *((Tenseur3HH*) &bHH); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3BB.Dimension()) != 3)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symetriques \n"
                 << "Tenseur3BBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH)";
            Sortie(2);
           }
       if (Dabs(b3HH.Dimension()) != 3)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symetriques \n"
                 << "Tenseur3BBHH::Prod_tensoriel(const TenseurBB & aBB, const TenseurHH & bHH)";
            Sortie(2);
           }
    #endif
    for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
         { int i = cdex3BBHH.idx_i(ij); int j = cdex3BBHH.idx_j(ij);
           int k = cdex3BBHH.idx_i(kl); int l = cdex3BBHH.idx_j(kl);
           res->t[(ij-1)*6+kl-1] =  0.25 * (
                a3BB(i,k) * b3HH(j,l) + a3BB(j,k) * b3HH(i,l)
               +a3BB(i,l) * b3HH(j,k) + a3BB(j,l) * b3HH(i,k)
                                      );
         };
    return *res;
  };
    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur3BBHH::Transpose1et2avec3et4() const   
   { TenseurHHBB * res;
     res =  new Tenseur3HHBB;
     LesMaillonsHHBB::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          res->t[(kl-1)*6+ij-1] = t[(ij-1)*6+kl-1] ;
     return *res;
   };           


    // il s'agit ici de calculer la variation d'un tenseur dans une nouvelle base
    //  --> variation par rapport aux composantes covariantes d'un tenseur (ex: composantes eps_ij)

    //  d sigma_ij /  d eps_kl = d beta_i^{.a} / d eps_kl . sigma_ab . beta_j^{.b}
    //                          + beta_i^{.a} . d sigma_ab / d eps_kl. beta_j^{.b}
    //                          + beta_i^{.a} . sigma_ab . d beta_j^{.b} / d eps_kl

    // connaissant sa variation dans la base actuelle
    // var_tensBBHH : en entrée: la variation du tenseur dans la base initiale qu'on appelle g^i
    //   ex: var_tensBBHH(i,j,k,l) = d sigma_ij / d eps_kl
    //            : en sortie: la variation du tenseur dans la base finale qu'on appelle gp^i
    // beta       : en entrée gpB(i) = beta(i,j) * gB(j)
    // var_beta   : en entrée : la variation de beta
    //   ex: var_beta(i,j,k,l) = d beta_i^{.j} / d eps_kl
    // tensBB     : le tenseur dont on cherche la variation
    /// -- pour mémoire ---
        // changement de base (cf. théorie) : la matrice beta est telle que:
    // gpB(i) = beta(i,j) * gB(j) <==> gp_i = beta_i^j * g_j
    // et la matrice gamma telle que:
    // gamma(i,j) represente les coordonnees de la nouvelle base duale gpH dans l'ancienne gH
    // gpH(i) = gamma(i,j) * gH(j), i indice de ligne, j indice de colonne
    //   c-a-d= gp^i = gamma^i_j * g^j
    // rappel des différentes relations entre beta et gamma
    // [beta]^{-1} = [gamma]^T      ; [beta]^{-1T} = [gamma]
    // [beta] = [gamma]^{-1T}      ; [beta]^{T} = [gamma]^{-1}
    // changement de base pour un tenseur en  deux fois covariants:
    // [Ap_kl] = [beta] * [A_ij] * [beta]^T

#ifndef MISE_AU_POINT
  inline
#endif
TenseurBBHH &  Tenseur3BBHH::Var_tenseur_dans_nouvelle_base
              (const Mat_pleine& beta,Tenseur3BBHH& var_tensBBHH, const Tableau2 <Tenseur3HH>& var_beta
              ,const Tenseur3BB& tensBB)
 {
 
    TenseurBBHH * res;
    res =  new Tenseur3BBHH;
    LesMaillonsBBHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    //  d sigma_ij /  d eps_kl = d beta_i^{.a} / d eps_kl . sigma_ab . beta_j^{.b}
    //                          + beta_i^{.a} . d sigma_ab / d eps_kl. beta_j^{.b}
    //                          + beta_i^{.a} . sigma_ab . d beta_j^{.b} / d eps_kl
    for (int i=1;i<4;i++)
     for (int j=1;j<4;j++)
      for (int k=1;k<4;k++)
       for (int l=1;l<4;l++)
        { double d_sig_ij_d_eps_kl = 0.;
          for (int a=1;a<4;a++)
           for (int b=1;b<4;b++)
             d_sig_ij_d_eps_kl += var_beta(i,a)(k,l) * tensBB(a,b) * beta(j,b)
                                  + beta(i,a) *  var_tensBBHH(a,b,k,l) * beta(j,b)
                                  + beta(i,a) * tensBB(a,b) * var_beta(j,b)(k,l);
          res->Change(i,j,k,l,d_sig_ij_d_eps_kl);
        };

    return *res;
};


    
#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void Tenseur3BBHH::Affectation_trans_dimension(const TenseurBBHH & aBBHH,bool plusZero)
   { switch (abs(aBBHH.Dimension()))
       { case 33 : case 30 : case 306 :
           for (int ij=1;ij < 7;ij++)
              for (int kl=1;kl < 7;kl++)
                t[(ij-1)*6+kl-1] = aBBHH(cdex3BBHH.idx_i(ij),cdex3BBHH.idx_j(ij)
                                         ,cdex3BBHH.idx_i(kl),cdex3BBHH.idx_j(kl));
           break;
          case 22 :  case 206:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           for (int ij=1;ij < 7;ij++)
              for (int kl=1;kl < 7;kl++)
               { if ((ij < 3) && (kl < 3)) // on affecte que les grandeurs qui existent
                   t[(ij-1)*6+kl-1] = aBBHH(cdex3BBHH.idx_i(ij),cdex3BBHH.idx_j(ij)
                                         ,cdex3BBHH.idx_i(kl),cdex3BBHH.idx_j(kl));
               };
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aBBHH(1,1,1,1);
           break;
         default:
          Message(3,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aBBHH.Dimension()))
                    +"n'est pas prise en compte \n Tenseur3BBHH::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur3BBHH::operator == ( const TenseurBBHH & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3BBHH::operator == ( etc..");
     #endif
     for (int i = 0; i<36; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
         return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void Tenseur3BBHH::Change (int i, int j, int k, int l, const double& val) 
  { t[(cdex3BBHH.odVect(i,j)-1)*6+cdex3BBHH.odVect(k,l)-1]  = val;}; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void Tenseur3BBHH::ChangePlus (int i, int j, int k, int l, const double& val)
  { t[(cdex3BBHH.odVect(i,j)-1)*6+cdex3BBHH.odVect(k,l)-1]  += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  Tenseur3BBHH::operator () (int i, int j, int k, int l) const 
 { return t[(cdex3BBHH.odVect(i,j)-1)*6+cdex3BBHH.odVect(k,l)-1]; };
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur3BBHH::MaxiComposante() const
  { return DabsMaxiTab(t,  36) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur3BBHH::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur3BBHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i < 36; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur3BBHH::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur3BBHH ";
    // puis les datas
     for (int i = 0; i < 36; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };

#ifndef MISE_AU_POINT
  inline
#endif
// affichage sous forme de tableau bidim
void Tenseur3BBHH::Affiche_bidim(ostream & sort) const
 {
    sort << "\n" ;
    for (int kl=1;kl < 7;kl++)
      sort << setw(15) << kl ;
    for (int ij=1;ij < 7;ij++)
      {sort << '\n'<< setw(4) << ij;
       for (int kl=1;kl < 7;kl++)
             { int i= cdex3BBHH.idx_i(ij); int j= cdex3BBHH.idx_j(ij);
               int k= cdex3BBHH.idx_i(kl); int l= cdex3BBHH.idx_j(kl);
               sort << setw(15) << setprecision(7)
                    << t[(cdex3BBHH.odVect(i,j)-1)*6+cdex3BBHH.odVect(k,l)-1];
             }
       sort << '\n';
      }
    cout << endl ;
 };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur3BBHH & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 33) A.Message(3,"operator >> (istream & entree, Tenseur3BBHH & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur3BBHH")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i < 36; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur3BBHH & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur3BBHH ";
    // puis les datas
    for (int i = 0; i < 36; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };
  
//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& Tenseur3BBHH::Prod_gauche( const TenseurHH & aHH) const
 {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 3) 
           Message(3,"Tenseur3BBHH::Prod_gauche( const TenseurHH & F)");
     #endif
     TenseurHH * res;
     res =  new Tenseur3HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
  
     for (int kl=1;kl < 7;kl++)
      {for (int ij=1;ij < 4;ij++)
         res->Coor(cdex3BBHH.idx_i(kl),cdex3BBHH.idx_j(kl)) +=
             a3HH(cdex3BBHH.idx_i(ij),cdex3BBHH.idx_j(ij)) * t[(ij-1)*6+kl-1];
       // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       for (int ij=4;ij < 7;ij++)
         res->Coor(cdex3BBHH.idx_i(kl),cdex3BBHH.idx_j(kl)) +=
                         (  a3HH(cdex3BBHH.idx_i(ij),cdex3BBHH.idx_j(ij))
                          + a3HH(cdex3BBHH.idx_j(ij),cdex3BBHH.idx_i(ij))
                         )
                       *  t[(ij-1)*6+kl-1] ;
       };
  
//     // pour vérif
//     for (int i=1;i<4;i++) for (int j=1;j<4;j++)
//      {res->Coor(i,j)=0;
//       for (int k=1;k<4;k++) for (int l=1;l<4;l++)
//         res->Coor(i,j) += a3BB(i,j) * (*this)(i,j,k,l) ;
//      };

     return *res ;
  };
//=========== fin fonction protected ======================	


 
//------------------------------------------------------------------
//          cas des composantes mixte 3HHBB
//------------------------------------------------------------------   
	  // --- gestion de changement  d'index ----
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHBB::ChangementIndex::ChangementIndex() :
  idx_i(6),idx_j(6),odVect(3)
  { idx_i(1)=1;idx_i(2)=2;idx_i(3)=3;idx_i(4)=2;idx_i(5)=3;idx_i(6)=3;
    idx_j(1)=1;idx_j(2)=2;idx_j(3)=3;idx_j(4)=1;idx_j(5)=2;idx_j(6)=1;
    odVect(1,1)=1;odVect(1,2)=4;odVect(1,3)=6;
    odVect(2,1)=4;odVect(2,2)=2;odVect(2,3)=5;
    odVect(3,1)=6;odVect(3,2)=5;odVect(3,3)=3;
  }; 
// variables globales
//Tenseur3HHBB::ChangementIndex  Tenseur3HHBB::cdex3HHBB;

    // Constructeur
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHBB::Tenseur3HHBB() :  
 ipointe() // par défaut
  { dimension = 33;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<36;i++) t[i]=0.;
  };
// initialisation de toutes les composantes a une meme valeur  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHBB::Tenseur3HHBB( const double val)  :  
 ipointe()
  { dimension = 33;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<36;i++) t[i]=val;
  };
    // initialisation à partir d'un produit tensoriel 
    //                  *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHBB::Tenseur3HHBB(const TenseurHH & aHH, const TenseurBB & bBB) :  
 ipointe()
  { dimension = 33;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
    const Tenseur3BB & b3BB = *((Tenseur3BB*) &bBB); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3HH.Dimension()) != 3) 
          Message(3,string("produit tensoriel a partir d'un premier tenseur non symétriques \n")
                   +"Tenseur3HHBB::Tenseur3HHBB(bool normal, const"
                    +" TenseurHH & aHH, const TenseurBB & bBB);");
       if (Dabs(b3BB.Dimension()) != 3) 
          Message(3,string("produit tensoriel a partir d'un second tenseur non symétriques \n")
                   +"Tenseur3HHBB::Tenseur3HHBB(bool normal, const"
                    +" TenseurHH & aHH, const TenseurBB & bBB);");
    #endif
    for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          t[(ij-1)*6+kl-1] =   a3HH(cdex3HHBB.idx_i(ij),cdex3HHBB.idx_j(ij))
                             * b3BB(cdex3HHBB.idx_i(kl),cdex3HHBB.idx_j(kl));
  };      
    // DESTRUCTEUR :
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHBB::~Tenseur3HHBB() 
{ listdouble36.erase(ipointe);} ; // suppression de l'élément de la liste
// constructeur a partir d'une instance non differenciee  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHBB::Tenseur3HHBB ( const TenseurHHBB & B) :  
 ipointe()
  { dimension = 33;
//    #ifdef MISE_AU_POINT
//    if (Dabs(dimension) != 33)
//      { cout << "\n erreur de dimension, elle devrait etre = 33 ";
//        cout << "\n Tenseur3HHBB::Tenseur3HHBB ( TenseurHHBB &) " << endl;
//        Sortie(1);
//      }  
//    #endif
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    if (Dabs(B.dimension) == 33 ) // cas d'un tenseur du même type
      { for (int i=0;i<36;i++)
          t[i] = B.t[i];
      }
    else
      {// cas d'un tenseur quelconque
       double Z=B.MaxiComposante();
       for (int i=1;i < 4;i++)
         for (int j=1;j<=i;j++)
           for (int k=1;k < 4;k++)
             for (int l=1;l<=k;l++)
             	{// on teste les symétries et on affecte 
             	 double a = B(i,j,k,l);
              #ifdef MISE_AU_POINT 
                 if ((!diffpourcent(a,B(j,i,k,l),Z,ConstMath::unpeupetit)
                    && !diffpourcent(a,B(i,j,l,k),Z,ConstMath::unpeupetit))
                    || (Abs(Z) < ConstMath::trespetit) )
                 // erreur d'affectation
                 if (ParaGlob::NiveauImpression() > 5)
                    cout << "\n tenseurHHBB (ijkl= " << i << "," << j << "," << k << "," << l << ")= " 
                         << a << " " << B(j,i,k,l) << " " <<B(i,j,l,k) ; 
                 cout << "WARNING ** erreur constructeur, tenseur non symetrique, Tenseur3HHBB::Tenseur3HHBB(const TenseurHHBB & B)";
              #endif
                 // si il y a un pb normalement il y a eu un message
                 this->Change(i,j,k,l,a);
             	}
      };
  };        
// constructeur de copie  
#ifndef MISE_AU_POINT
  inline 
#endif
Tenseur3HHBB::Tenseur3HHBB (  const Tenseur3HHBB & B) :  
 ipointe()
  { this->dimension = B.dimension;
    listdouble36.push_front(Reels36());  // allocation
    ipointe = listdouble36.begin(); // recup de la position de la maille dans la liste
    t = (ipointe)->donnees;    // recup de la position des datas dans la maille
    for (int i=0;i<36;i++)
       this->t[i] = B.t[i];
  };        
    // METHODES PUBLIQUES :
#ifndef MISE_AU_POINT
  inline 
#endif
// initialise toutes les composantes à val
void Tenseur3HHBB::Inita(double val)   
	{ for (int i=0;i< 36;i++)
         t[i] = val;
     };
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur3HHBB::operator + ( const TenseurHHBB & B) const 
  {  TenseurHHBB * res;
     #ifdef MISE_AU_POINT
       if (B.Dimension() != 33) Message(3,"Tenseur3HHBB::operator + ( etc..");
     #endif
     res =  new Tenseur3HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<36; i++)
        res->t[i] = this->t[i] + B.t[i]; //somme des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void Tenseur3HHBB::operator += ( const TenseurHHBB & B)
   { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3HHBB::operator += ( etc..");
     #endif
     for (int i = 0; i<36; i++)
       this->t[i] += B.t[i];
     LesMaillonsHHBB::Libere(); // destruction des tenseurs intermediaires
   }; //somme des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur3HHBB::operator - () const 
  {  TenseurHHBB * res;
     res =  new Tenseur3HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<36; i++)
        res->t[i] = - this->t[i]; //oppose
    return *res ;}; 
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur3HHBB::operator - ( const TenseurHHBB & B) const 
  {  TenseurHHBB * res;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3HHBB::operator - ( etc..");
     #endif
     res =  new Tenseur3HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<36; i++)
            res->t[i] = this->t[i] - B.t[i]; //soustraction des données
    return *res ;};  
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur3HHBB::operator -= ( const TenseurHHBB & B) 
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3HHBB::operator -= ( etc..");
     #endif
     for (int i = 0; i<36; i++)
       this->t[i] -= B.t[i];
     LesMaillonsHHBB::Libere(); // destruction des tenseurs intermediaires
    }; //soustraction des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur3HHBB::operator = ( const TenseurHHBB & B)
  { 
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3HHBB::operator = ( etc..");
     #endif
     for (int i = 0; i<36; i++)
            this->t[i] = B.t[i];
    LesMaillonsHHBB::Libere(); // destruction des tenseurs intermediaires
    return *this; 
    }; //affectation des données;
    
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur3HHBB::operator * ( const double & b) const 
  {  TenseurHHBB * res;
     res =  new Tenseur3HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     for (int i = 0; i<36; i++)
        res->t[i] = this->t[i] * b; //multiplication des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur3HHBB::operator *= ( const double & b)
  {for (int i = 0; i<36; i++)
       this->t[i] *= b ;}; //multiplication des données
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB & Tenseur3HHBB::operator / ( const double & b) const 
  {  TenseurHHBB * res;
     res =  new Tenseur3HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur3HHBB::operator / ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
     for (int i = 0; i<36; i++)
        res->t[i] = this->t[i] / b; //division des données
    return *res ;};
#ifndef MISE_AU_POINT
  inline 
#endif
void  Tenseur3HHBB::operator /= ( const double & b)
  { 
     #ifdef MISE_AU_POINT
     if (Dabs(b) < ConstMath::trespetit)
      { cout << "\n erreur le diviseur est trop petit = " << b;
        cout << "\n Tenseur3HHBB::operator /= ( const double & b) " << endl;
        Sortie(1);
      }  
     #endif
    for (int i = 0; i<36; i++)
       this->t[i] /= b ;}; //division des données

    
    // produit contracte à droite avec un tenseur du second ordre
    // différent à gauche !!
    // A(i,j,k,l)*B(l,k)=A..B
    // on commence par contracter l'indice du milieu puis externe
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHH& Tenseur3HHBB::operator && ( const TenseurHH & aHH)  const 
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 3) 
           Message(3,"Tenseur3HHBB::operator && ( const TenseurHH & aHH)");
     #endif
     TenseurHH * res;
     res =  new Tenseur3HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
      {for (int kl=1;kl < 4;kl++) // partie simple produit : la partie diagonale
         res->Coor(cdex3HHBB.idx_i(ij),cdex3HHBB.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                                             * a3HH(cdex3HHBB.idx_i(kl),cdex3HHBB.idx_j(kl));
       // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       for (int kl=4;kl < 7;kl++)
         res->Coor(cdex3HHBB.idx_i(ij),cdex3HHBB.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                       * (  a3HH(cdex3HHBB.idx_i(kl),cdex3HHBB.idx_j(kl))
                          + a3HH(cdex3HHBB.idx_j(kl),cdex3HHBB.idx_i(kl))
                         );
       };
   return *res ;
  };
    // contraction verticale: A(i,j,k,l)*B(k,l)=A:B
#ifndef MISE_AU_POINT
  inline
#endif
TenseurHH&  Tenseur3HHBB::ContractionVerticale( const TenseurHH & aHH )  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHH.Dimension()) != 3)
           Message(3,"Tenseur3HHBB::operator && ( const TenseurHH & aHH)");
     #endif
     TenseurHH * res;
     res =  new Tenseur3HH;
     LesMaillonsHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
      {for (int kl=1;kl < 4;kl++) // partie simple produit : la partie diagonale
         res->Coor(cdex3HHBB.idx_i(ij),cdex3HHBB.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                                             * a3HH(cdex3HHBB.idx_j(kl),cdex3HHBB.idx_i(kl));
       // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       for (int kl=4;kl < 7;kl++)
         res->Coor(cdex3HHBB.idx_i(ij),cdex3HHBB.idx_j(ij)) +=  t[(ij-1)*6+kl-1]
                       * (  a3HH(cdex3HHBB.idx_j(kl),cdex3HHBB.idx_i(kl))
                          + a3HH(cdex3HHBB.idx_i(kl),cdex3HHBB.idx_j(kl))
                         );
       };
   return *res ;
  };

    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline
#endif
// cas d'un tenseur d'ordre quatre HHBB
TenseurHHHH& Tenseur3HHBB::operator && ( const TenseurHHHH & aHHHH)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHHH.Dimension()) != 33)
           Message(3,"Tenseur3HHBB::operator && ( const TenseurHHHH & aHHHH)");
     #endif
     TenseurHHHH * res;
     res =  new Tenseur3HHHH;
     LesMaillonsHHHH::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3HHHH & a3HHHH = *((Tenseur3HHHH*) &aHHHH); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
       for (int kl=1;kl < 7;kl++)
        {double& resul= res->t[(ij-1)*6+kl-1];
         for (int n=1;n<4;n++) for (int m=1;m<4;m++)
           {int ef = cdex3HHBB.odVect(n,m);
            resul += this->t[(ij-1)*6+ef-1] * a3HHHH.t[(ef-1)*6+kl-1];
           };
//         for (int ef=1;ef < 7;ef++)
//           resul += this->t[(ij-1)*6+ef-1] * a3HHHH.t[(ef-1)*6+kl-1];
        };
     return *res;
  };

    // produit deux fois contracte à droite avec un tenseur du quatrième ordre
    // différent à gauche !! def dans TenseurQ.h à l'aide de la fonction privée Prod_gauche
#ifndef MISE_AU_POINT
  inline
#endif
// cas d'un tenseur d'ordre quatre HHBB
TenseurHHBB& Tenseur3HHBB::operator && ( const TenseurHHBB & aHHBB)  const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aHHBB.Dimension()) != 33)
           Message(3,"Tenseur3HHBB::operator && ( const TenseurHHBB & aHHBB)");
     #endif
     TenseurHHBB * res;
     res =  new Tenseur3HHBB;
     LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3HHBB & a3HHBB = *((Tenseur3HHBB*) &aHHBB); // passage en dim 3
     for (int ij=1;ij < 7;ij++)
       for (int kl=1;kl < 7;kl++)
        {double& resul= res->t[(ij-1)*6+kl-1];
         for (int n=1;n<4;n++) for (int m=1;m<4;m++)
           {int ef = cdex3HHBB.odVect(n,m);
            resul += this->t[(ij-1)*6+ef-1] * a3HHBB.t[(ef-1)*6+kl-1];
           };
//         for (int ef=1;ef < 7;ef++)
//           resul += this->t[(ij-1)*6+ef-1] * a3HHBB.t[(ef-1)*6+kl-1];
        };
     return *res;
  };

    //fonctions définissant le produit tensoriel normal de deux tenseurs
    // *this=aHH(i,j).bBB(k,l) gBi gBj gHk gHl
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurHHBB &  Tenseur3HHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB) 
  { TenseurHHBB * res;
    res =  new Tenseur3HHBB;
    LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
    const Tenseur3BB & b3BB = *((Tenseur3BB*) &bBB); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3HH.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur3HHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB)";
            Sortie(2);
           }      
       if (Dabs(b3BB.Dimension()) != 3) 
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur3HHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB)";
            Sortie(2);
           }      
    #endif
    for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          res->t[(ij-1)*6+kl-1] =   a3HH(cdex3HHBB.idx_i(ij),cdex3HHBB.idx_j(ij))
                                  * b3BB(cdex3HHBB.idx_i(kl),cdex3HHBB.idx_j(kl));
    return *res;                         
  };      

    //fonctions définissant le produit tensoriel barre de deux tenseurs
    // concervant les symétries !!
    // *this(i,j,k,l)
    //    = 1/4 * (a(i,k).b(j,l) + a(j,k).b(i,l) + a(i,l).b(j,k) + a(j,l).b(i,k))
#ifndef MISE_AU_POINT
  inline
#endif
TenseurHHBB &  Tenseur3HHBB::Prod_tensoriel_barre(const TenseurHH & aHH, const TenseurBB & bBB)
  { TenseurHHBB * res;
    res =  new Tenseur3HHBB;
    LesMaillonsHHBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
    const Tenseur3HH & a3HH = *((Tenseur3HH*) &aHH); // passage en dim 3
    const Tenseur3BB & b3BB = *((Tenseur3BB*) &bBB); // passage en dim 3
    #ifdef MISE_AU_POINT
       if (Dabs(a3HH.Dimension()) != 3)
          { cout << "\n produit tensoriel a partir d'un premier tenseur non symétriques \n"
                 << "Tenseur3HHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB)";
            Sortie(2);
           }
       if (Dabs(b3BB.Dimension()) != 3)
          { cout << "\n produit tensoriel a partir d'un second tenseur non symétriques \n"
                 << "Tenseur3HHBB::Prod_tensoriel(const TenseurHH & aHH, const TenseurBB & bBB)";
            Sortie(2);
           }
    #endif
    for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
         { int i = cdex3HHBB.idx_i(ij); int j = cdex3HHBB.idx_j(ij);
           int k = cdex3HHBB.idx_i(kl); int l = cdex3HHBB.idx_j(kl);
           res->t[(ij-1)*6+kl-1] =  0.25 * (
                a3HH(i,k) * b3BB(j,l) + a3HH(j,k) * b3BB(i,l)
               +a3HH(i,l) * b3BB(j,k) + a3HH(j,l) * b3BB(i,k)
                                      );
         };
    return *res;
  };
    // ATTENTION creation d'un tenseur transpose qui est supprime par Libere
    // les 2 premiers indices sont échangés avec les deux derniers indices
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBBHH & Tenseur3HHBB::Transpose1et2avec3et4() const   
   { TenseurBBHH * res;
     res =  new Tenseur3BBHH;
     LesMaillonsBBHH::NouveauMaillon(res); // ajout d'un tenseur intermediaire
     for (int ij=1;ij < 7;ij++)
        for (int kl=1;kl < 7;kl++)
          res->t[(kl-1)*6+ij-1] = t[(ij-1)*6+kl-1] ;
     return *res;
   };
    
#ifndef MISE_AU_POINT
  inline 
#endif
    // affectation de B dans this,  plusZero = false: les données manquantes sont inchangées,
    // plusZero = true: les données manquantes sont mises à 0
    // si au contraire la dimension de B est plus grande que *this, il y a uniquement affectation
    // des données possibles
void Tenseur3HHBB::Affectation_trans_dimension(const TenseurHHBB & aHHBB,bool plusZero)
   { switch (abs(aHHBB.Dimension()))
       { case 33 : case 30 : case 306 :
           for (int ij=1;ij < 7;ij++)
              for (int kl=1;kl < 7;kl++)
                t[(ij-1)*6+kl-1] = aHHBB(cdex3HHBB.idx_i(ij),cdex3HHBB.idx_j(ij)
                                         ,cdex3HHBB.idx_i(kl),cdex3HHBB.idx_j(kl));
           break;
         case 22 :  case 206:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           for (int ij=1;ij < 7;ij++)
              for (int kl=1;kl < 7;kl++)
               { if ((ij < 3) && (kl < 3)) // on affecte que les grandeurs qui existent
                   t[(ij-1)*6+kl-1] = aHHBB(cdex3HHBB.idx_i(ij),cdex3HHBB.idx_j(ij)
                                         ,cdex3HHBB.idx_i(kl),cdex3HHBB.idx_j(kl));
               };
           break;
         case 11 : case 10: case 106:
           if (plusZero)
             this->Inita(0.); // on commence par mettre à 0 si besoin
           // ensuite on affecte
           t[0] = aHHBB(1,1,1,1);
           break;
         default:
          Message(3,string(" *** erreur, la dimension: ")
                    + ChangeEntierSTring(abs(aHHBB.Dimension()))
                    +"n'est pas prise en compte \n Tenseur3HHBB::Affectation_trans_dimension(");
       };
   };

// test
#ifndef MISE_AU_POINT
  inline 
#endif
int Tenseur3HHBB::operator == ( const TenseurHHBB & B) const 
  { int res = 1;
     #ifdef MISE_AU_POINT
       if (Dabs(B.Dimension()) != 33) Message(3,"Tenseur3HHBB::operator == ( etc..");
     #endif
     for (int i = 0; i<36; i++)
         if (this->t[i] != B.t[i]) res = 0 ;
         return res;
   };  
   
#ifndef MISE_AU_POINT
  inline 
#endif
// change la composante i,j,k,l du tenseur
// acces en  ecriture, 
void Tenseur3HHBB::Change (int i, int j, int k, int l,const double& val) 
  { t[(cdex3HHBB.odVect(i,j)-1)*6+cdex3HHBB.odVect(k,l)-1]  = val;}; 

#ifndef MISE_AU_POINT
  inline
#endif
// change en cumulant la composante i,j,k,l du tenseur
// acces en  ecriture,
void Tenseur3HHBB::ChangePlus (int i, int j, int k, int l,const double& val)
  { t[(cdex3HHBB.odVect(i,j)-1)*6+cdex3HHBB.odVect(k,l)-1]  += val;};

#ifndef MISE_AU_POINT
  inline 
#endif
// Retourne la composante i,j,k,l du tenseur
// acces en lecture seule
double  Tenseur3HHBB::operator () (int i, int j, int k, int l) const 
 { return t[(cdex3HHBB.odVect(i,j)-1)*6+cdex3HHBB.odVect(k,l)-1]; };
     
// calcul du maximum en valeur absolu des composantes du tenseur
#ifndef MISE_AU_POINT
  inline 
#endif
double Tenseur3HHBB::MaxiComposante() const
  { return DabsMaxiTab(t,  36) ;
   };
             

    // lecture et écriture de données
#ifndef MISE_AU_POINT
  inline 
#endif
istream & Tenseur3HHBB::Lecture(istream & entree)
  { // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur3HHBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i < 36; i++)
        entree >> this->t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
ostream & Tenseur3HHBB::Ecriture(ostream & sort) const 
  { // écriture du type
    sort << "Tenseur3HHBB ";
    // puis les datas
     for (int i = 0; i < 36; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   this->t[i] << " ";
     return sort;      
  };

#ifndef MISE_AU_POINT
  inline
#endif
// affichage sous forme de tableau bidim
void Tenseur3HHBB::Affiche_bidim(ostream & sort) const
 {
    sort << "\n" ;
    for (int kl=1;kl < 7;kl++)
      sort << setw(15) << kl ;
    for (int ij=1;ij < 7;ij++)
      {sort << '\n'<< setw(4) << ij;
       for (int kl=1;kl < 7;kl++)
             { int i= cdex3HHBB.idx_i(ij); int j= cdex3HHBB.idx_j(ij);
               int k= cdex3HHBB.idx_i(kl); int l= cdex3HHBB.idx_j(kl);
               sort << setw(15) << setprecision(7)
                    << t[(cdex3HHBB.odVect(i,j)-1)*6+cdex3HHBB.odVect(k,l)-1];
             }
       sort << '\n';
      }
    cout << endl ;
 };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator de lecture
istream & operator >> (istream & entree, Tenseur3HHBB & A)
  { int dim = A.Dimension();
    #ifdef MISE_AU_POINT
       if (dim != 33) A.Message(3,"operator >> (istream & entree, Tenseur3HHBB & A)");
    #endif
    // lecture et vérification du type
    string nom_type;
    entree >> nom_type;
    if (nom_type != "Tenseur3HHBB")
      { Sortie(1);
        return entree;
       }
    // lecture des coordonnées    
    for (int i = 0; i < 36; i++)
        entree >> A.t[i];
    return entree;      
  };

#ifndef MISE_AU_POINT
  inline 
#endif
 // surcharge de l'operator d'ecriture
ostream & operator << (ostream & sort , const Tenseur3HHBB & A)
  { int dim = A.Dimension();
    // écriture du type
    sort << "Tenseur3HHBB ";
    // puis les datas
    for (int i = 0; i < 36; i++)
        sort  << setprecision(ParaGlob::NbdigdoCA()) <<   A.t[i] << " ";
    return sort;      
  };

//=========== fonction protected ======================	
	// fonction pour le produit contracté à gauche
#ifndef MISE_AU_POINT
  inline 
#endif
TenseurBB& Tenseur3HHBB::Prod_gauche( const TenseurBB & aBB) const
  {
     #ifdef MISE_AU_POINT
       if (Dabs(aBB.Dimension()) != 3) 
           Message(3,"Tenseur3HHBB::Prod_gauche( const TenseurBB & F)");
     #endif
     TenseurBB * res;
     res =  new Tenseur3BB;
     LesMaillonsBB::NouveauMaillon( res); // ajout d'un tenseur intermediaire
     const Tenseur3BB & a3BB = *((Tenseur3BB*) &aBB); // passage en dim 3
  
     for (int kl=1;kl < 7;kl++)
      {for (int ij=1;ij < 4;ij++)
         res->Coor(cdex3HHBB.idx_i(kl),cdex3HHBB.idx_j(kl)) +=
             a3BB(cdex3HHBB.idx_i(ij),cdex3HHBB.idx_j(ij)) * t[(ij-1)*6+kl-1];
       // partie produit doublée : partie extra-diagonale
       // comme le tenseur du second ordre n'est pas forcément symétrique on utilise les
       // 2 coordonnées de chaque coté de la diagonale,
       for (int ij=4;ij < 7;ij++)
         res->Coor(cdex3HHBB.idx_i(kl),cdex3HHBB.idx_j(kl)) +=
                         (  a3BB(cdex3HHBB.idx_i(ij),cdex3HHBB.idx_j(ij))
                          + a3BB(cdex3HHBB.idx_j(ij),cdex3HHBB.idx_i(ij))
                         )
                       *  t[(ij-1)*6+kl-1] ;
       };
  
//     // pour vérif
//     for (int i=1;i<4;i++) for (int j=1;j<4;j++)
//      {res->Coor(i,j)=0;
//       for (int k=1;k<4;k++) for (int l=1;l<4;l++)
//         res->Coor(i,j) += a3BB(i,j) * (*this)(i,j,k,l) ;
//      };

     return *res ;
  };
//=========== fin fonction protected ======================	
  
#endif
