// This file is part of the Herezh++ application.
//
// The finite element software Herezh++ is dedicated to the field
// of mechanics for large transformations of solid structures.
// It is developed by Gérard Rio (APP: IDDN.FR.010.0106078.000.R.P.2006.035.20600)
// INSTITUT DE RECHERCHE DUPUY DE LÔME (IRDL) <https://www.irdl.fr/>.
//
// Herezh++ is distributed under GPL 3 license ou ultérieure.
//
// Copyright (C) 1997-2021 Université Bretagne Sud (France)
// AUTHOR : Gérard Rio
// E-MAIL  : gerardrio56@free.fr
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.
//
// For more information, please consult: <https://herezh.irdl.fr/>.

// fichier : Courbe_expression_litterale_1D.h
// classe  : Courbe_expression_litterale_1D


/************************************************************************
 *     DATE:        07/04/2014                                          *
 *                                                                $     *
 *     AUTEUR:      G RIO   (mailto:gerardrio56@free.fr)                *
 *                                                                $     *
 *     PROJET:      Herezh++                                            *
 *                                                                $     *
 ************************************************************************
 *     BUT:    Classe permettant le calcul d'une fonction 1D            *
 *             de type : une expression littérale, exploitée ensuite    *
 *             par le parser : muparser                                 *
 *             avec un mini et/ou maxi éventuels                        *
 *             ainsi qu'un certain nombre d'information                 *
 *             supplémentaires telles que dérivées.                     *
 *             IMPORTANT: les dérivées sont calculées de manière        *
 *             numérique.                                               *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     * 
 *                                                                      *
 *     VERIFICATION:                                                    *
 *                                                                      *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *     !        !            !                                    !     *
 *                                                                $     *
 *     ''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''     *
 *     MODIFICATIONS:                                                   *
 *     !  date  !   auteur   !       but                          !     *
 *     ------------------------------------------------------------     *
 *                                                                $     *
 ************************************************************************/
 
#ifndef COURBE_EXPRESSION_LITTERALE_1_D_H
#define COURBE_EXPRESSION_LITTERALE_1_D_H

#include "Courbe1D.h"
#include "muParser.h"


/// @addtogroup Les_courbes_1D
///  @{
///

/**
*
*     BUT:    Classe permettant le calcul d'une fonction 1D
*             de type : une expression littérale, exploitée ensuite
*             par le parser : muparser
*             avec un mini et/ou maxi éventuels
*             ainsi qu'un certain nombre d'information
*             supplémentaires telles que dérivées.
*             IMPORTANT: les dérivées sont calculées de manière
*             numérique.
*
*
* \author    Gérard Rio
* \version   1.0
* \date      07/04/2014
* \brief       Classe permettant le calcul d'une fonction 1D de type : une expression littérale
*
*/

class Courbe_expression_litterale_1D : public Courbe1D
{
  public :

    // CONSTRUCTEURS :
    Courbe_expression_litterale_1D(string nom = "");
    
    // de copie
    Courbe_expression_litterale_1D(const Courbe_expression_litterale_1D& Co);
    Courbe_expression_litterale_1D(const Courbe1D& Co);
    
    // DESTRUCTEUR :
    ~Courbe_expression_litterale_1D();
    
    // METHODES PUBLIQUES :
    
    // --------- virtuelles ---------
    
    // affichage de la courbe
	   void Affiche() const ;
    // ramène true si ok, false sinon
    bool Complet_courbe()const;

    // Lecture des donnees de la classe sur fichier
    // le nom passé en paramètre est le nom de la courbe
    // s'il est vide c-a-d = "", la methode commence par lire le nom sinon
    // ce nom remplace le nom actuel
    void LectDonnParticulieres_courbes(const string& nom, UtilLecture * );
    
    // def info fichier de commande
    void Info_commande_Courbes1D(UtilLecture & entreePrinc) ;
    
    // ramène la valeur 
    
    double Valeur(double x)  ;

    // ramène la valeur et la dérivée en paramètre
    Courbe1D::ValDer Valeur_Et_derivee(double x)  ;
    
    // ramène la dérivée 
    double Derivee(double x)  ;
    
    // ramène la valeur et les dérivées première et seconde en paramètre
    Courbe1D::ValDer2 Valeur_Et_der12(double x);
        
    // ramène la dérivée seconde
    double Der_sec(double x);
    
    // ramène la valeur si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y
    // si supérieur au x maxi , ramène le valeur maximale possible de y
    Courbe1D::Valbool  Valeur_stricte(double x)  ;
    
    // ramène la valeur et la dérivée si dans le domaine strictement de définition
    // si c'est inférieur au x mini, ramène la valeur minimale possible de y et Y' correspondant
    // si supérieur au x maxi , ramène le valeur maximale possible de y et Y' correspondant
    Courbe1D::ValDerbool  Valeur_Et_derivee_stricte(double x)  ;
    
	//----- lecture écriture de restart -----
	// cas donne le niveau de la récupération
    // = 1 : on récupère tout
    // = 2 : on récupère uniquement les données variables (supposées comme telles)
	   void Lecture_base_info(ifstream& ent,const int cas);
    // cas donne le niveau de sauvegarde
    // = 1 : on sauvegarde tout
    // = 2 : on sauvegarde uniquement les données variables (supposées comme telles)
	   void Ecriture_base_info(ofstream& sort,const int cas);
    // sortie du schemaXML: en fonction de enu 
    void SchemaXML_Courbes1D(ofstream& sort,const Enum_IO_XML enu) ;
    
  protected :  
    // VARIABLES PROTEGEES :
    double ax,bx; // min max
 
    string expression_fonction; // l'expression littérale de la fonction
    mu::Parser p; // définition d'une instance de parser
    double fVal; // la variable interne
    double delta_xSur_x; // sert pour l'incrément pour le calcul des dérivées
//    int ordre_troncature; // l'ordre de troncature: 2 par défaut

 
    // METHODES PROTEGEES :
    // calcul de la dérivée première et seconde par différence finie à l'ordres 4
//    void Der_1_2(const double & x, double& der1, double& der2);
// muParser fourni déjà une dérivée à l'ordre 4 et on pourrait également adjoindre une dérivée
// seconde à l'ordre 4 cf. Numerical algorithms with C : page 356
// Gisela Engeln-Müllges et Frank Uhlig, Springer, ISBN 3_540-60530-4
 
 
 
 

 };
 /// @}  // end of group

#endif  
